# Generated by Django 4.2.13 on 2024-08-22 11:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moons', '0018_alter_moonfrack_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='oreprice',
            name='goo_only',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='oretaxrates',
            name='ignore_ores_in_refine',
            field=models.BooleanField(default=False),
        ),
    ]
