import{g as zi,_ as a1,r as w,a as ye,b as i1,c as Nl,u as Sl,T as jc,d as Yg,e as Xg,f as r1,Q as u1,h as o1}from"./@libs-rfllBq4_.js";import{u as s1,N as tr,O as vd,B as c1,R as f1,a as Xn,b as Sd}from"./@react-router-BiKtSVvI.js";(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const a of document.querySelectorAll('link[rel="modulepreload"]'))l(a);new MutationObserver(a=>{for(const i of a)if(i.type==="childList")for(const r of i.addedNodes)r.tagName==="LINK"&&r.rel==="modulepreload"&&l(r)}).observe(document,{childList:!0,subtree:!0});function n(a){const i={};return a.integrity&&(i.integrity=a.integrity),a.referrerPolicy&&(i.referrerPolicy=a.referrerPolicy),a.crossOrigin==="use-credentials"?i.credentials="include":a.crossOrigin==="anonymous"?i.credentials="omit":i.credentials="same-origin",i}function l(a){if(a.ep)return;a.ep=!0;const i=n(a);fetch(a.href,i)}})();var kg={exports:{}},Du={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var d1=Symbol.for("react.transitional.element"),p1=Symbol.for("react.fragment");function Qg(e,t,n){var l=null;if(n!==void 0&&(l=""+n),t.key!==void 0&&(l=""+t.key),"key"in t){n={};for(var a in t)a!=="key"&&(n[a]=t[a])}else n=t;return t=n.ref,{$$typeof:d1,type:e,key:l,ref:t!==void 0?t:null,props:n}}Du.Fragment=p1;Du.jsx=Qg;Du.jsxs=Qg;kg.exports=Du;var g=kg.exports;const g1="_refreshAnimate_o9fr8_10",h1="_spin_o9fr8_1",m1="_filterBtn_o9fr8_17",y1="_filterToggle_o9fr8_33",v1={refreshAnimate:g1,spin:h1,filterBtn:m1,filterToggle:y1},S1="_box_203nf_23",x1="_searchInput_203nf_28",w1="_radioWrapper_203nf_40",b1="_input_203nf_45",R1="_searchButton_203nf_55",E1="_searchWrapper_203nf_64",C1="_searchWrapperFrom_203nf_68",_1="_searchIcon_203nf_72",A1="_xIcon_203nf_73",O1="_dropDownIcon_203nf_99",de={box:S1,searchInput:x1,radioWrapper:w1,input:b1,searchButton:R1,searchWrapper:E1,searchWrapperFrom:C1,searchIcon:_1,xIcon:A1,dropDownIcon:O1};var Zg={exports:{}};/*!
	Copyright (c) 2018 Jed Watson.
	Licensed under the MIT License (MIT), see
	http://jedwatson.github.io/classnames
*/(function(e){(function(){var t={}.hasOwnProperty;function n(){for(var i="",r=0;r<arguments.length;r++){var u=arguments[r];u&&(i=a(i,l(u)))}return i}function l(i){if(typeof i=="string"||typeof i=="number")return i;if(typeof i!="object")return"";if(Array.isArray(i))return n.apply(null,i);if(i.toString!==Object.prototype.toString&&!i.toString.toString().includes("[native code]"))return i.toString();var r="";for(var u in i)t.call(i,u)&&i[u]&&(r=a(r,u));return r}function a(i,r){return r?i?i+" "+r:i+r:i}e.exports?(n.default=n,e.exports=n):window.classNames=n})()})(Zg);var T1=Zg.exports;const $=zi(T1);function Kg(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.includes(l))continue;n[l]=e[l]}return n}function xd(e){return"default"+e.charAt(0).toUpperCase()+e.substr(1)}function D1(e){var t=M1(e,"string");return typeof t=="symbol"?t:String(t)}function M1(e,t){if(typeof e!="object"||e===null)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var l=n.call(e,t);if(typeof l!="object")return l;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function Jg(e,t,n){var l=w.useRef(e!==void 0),a=w.useState(t),i=a[0],r=a[1],u=e!==void 0,o=l.current;return l.current=u,!u&&o&&i!==t&&r(t),[u?e:i,w.useCallback(function(s){for(var c=arguments.length,f=new Array(c>1?c-1:0),p=1;p<c;p++)f[p-1]=arguments[p];n&&n.apply(void 0,[s].concat(f)),r(s)},[n])]}function j1(e,t){return Object.keys(t).reduce(function(n,l){var a,i=n,r=i[xd(l)],u=i[l],o=Kg(i,[xd(l),l].map(D1)),s=t[l],c=Jg(u,r,e[s]),f=c[0],p=c[1];return a1({},o,(a={},a[l]=f,a[s]=p,a))},e)}const N1=["xxl","xl","lg","md","sm","xs"],z1="xs",Mu=w.createContext({prefixes:{},breakpoints:N1,minBreakpoint:z1});function Y(e,t){const{prefixes:n}=w.useContext(Mu);return e||n[t]||t}function U1(){const{breakpoints:e}=w.useContext(Mu);return e}function F1(){const{minBreakpoint:e}=w.useContext(Mu);return e}function Nc(){const{dir:e}=w.useContext(Mu);return e==="rtl"}function ju(e){return e&&e.ownerDocument||document}function B1(e){var t=ju(e);return t&&t.defaultView||window}function H1(e,t){return B1(e).getComputedStyle(e,t)}var L1=/([A-Z])/g;function V1(e){return e.replace(L1,"-$1").toLowerCase()}var $1=/^ms-/;function nr(e){return V1(e).replace($1,"-ms-")}var q1=/^((translate|rotate|scale)(X|Y|Z|3d)?|matrix(3d)?|perspective|skew(X|Y)?)$/i;function G1(e){return!!(e&&q1.test(e))}function Pg(e,t){var n="",l="";if(typeof t=="string")return e.style.getPropertyValue(nr(t))||H1(e).getPropertyValue(nr(t));Object.keys(t).forEach(function(a){var i=t[a];!i&&i!==0?e.style.removeProperty(nr(a)):G1(a)?l+=a+"("+i+") ":n+=nr(a)+": "+i+";"}),l&&(n+="transform: "+l+";"),e.style.cssText+=";"+n}var Ig={exports:{}},Y1="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED",X1=Y1,k1=X1;function Wg(){}function eh(){}eh.resetWarningCache=Wg;var Q1=function(){function e(l,a,i,r,u,o){if(o!==k1){var s=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw s.name="Invariant Violation",s}}e.isRequired=e;function t(){return e}var n={array:e,bigint:e,bool:e,func:e,number:e,object:e,string:e,symbol:e,any:e,arrayOf:t,element:e,elementType:e,instanceOf:t,node:e,objectOf:t,oneOf:t,oneOfType:t,shape:t,exact:t,checkPropTypes:eh,resetWarningCache:Wg};return n.PropTypes=n,n};Ig.exports=Q1();var Z1=Ig.exports;const z=zi(Z1),wd={disabled:!1},th=ye.createContext(null);var K1=function(t){return t.scrollTop},Va="unmounted",Zn="exited",hn="entering",Pn="entered",ps="exiting",fn=function(e){i1(t,e);function t(l,a){var i;i=e.call(this,l,a)||this;var r=a,u=r&&!r.isMounting?l.enter:l.appear,o;return i.appearStatus=null,l.in?u?(o=Zn,i.appearStatus=hn):o=Pn:l.unmountOnExit||l.mountOnEnter?o=Va:o=Zn,i.state={status:o},i.nextCallback=null,i}t.getDerivedStateFromProps=function(a,i){var r=a.in;return r&&i.status===Va?{status:Zn}:null};var n=t.prototype;return n.componentDidMount=function(){this.updateStatus(!0,this.appearStatus)},n.componentDidUpdate=function(a){var i=null;if(a!==this.props){var r=this.state.status;this.props.in?r!==hn&&r!==Pn&&(i=hn):(r===hn||r===Pn)&&(i=ps)}this.updateStatus(!1,i)},n.componentWillUnmount=function(){this.cancelNextCallback()},n.getTimeouts=function(){var a=this.props.timeout,i,r,u;return i=r=u=a,a!=null&&typeof a!="number"&&(i=a.exit,r=a.enter,u=a.appear!==void 0?a.appear:r),{exit:i,enter:r,appear:u}},n.updateStatus=function(a,i){if(a===void 0&&(a=!1),i!==null)if(this.cancelNextCallback(),i===hn){if(this.props.unmountOnExit||this.props.mountOnEnter){var r=this.props.nodeRef?this.props.nodeRef.current:Nl.findDOMNode(this);r&&K1(r)}this.performEnter(a)}else this.performExit();else this.props.unmountOnExit&&this.state.status===Zn&&this.setState({status:Va})},n.performEnter=function(a){var i=this,r=this.props.enter,u=this.context?this.context.isMounting:a,o=this.props.nodeRef?[u]:[Nl.findDOMNode(this),u],s=o[0],c=o[1],f=this.getTimeouts(),p=u?f.appear:f.enter;if(!a&&!r||wd.disabled){this.safeSetState({status:Pn},function(){i.props.onEntered(s)});return}this.props.onEnter(s,c),this.safeSetState({status:hn},function(){i.props.onEntering(s,c),i.onTransitionEnd(p,function(){i.safeSetState({status:Pn},function(){i.props.onEntered(s,c)})})})},n.performExit=function(){var a=this,i=this.props.exit,r=this.getTimeouts(),u=this.props.nodeRef?void 0:Nl.findDOMNode(this);if(!i||wd.disabled){this.safeSetState({status:Zn},function(){a.props.onExited(u)});return}this.props.onExit(u),this.safeSetState({status:ps},function(){a.props.onExiting(u),a.onTransitionEnd(r.exit,function(){a.safeSetState({status:Zn},function(){a.props.onExited(u)})})})},n.cancelNextCallback=function(){this.nextCallback!==null&&(this.nextCallback.cancel(),this.nextCallback=null)},n.safeSetState=function(a,i){i=this.setNextCallback(i),this.setState(a,i)},n.setNextCallback=function(a){var i=this,r=!0;return this.nextCallback=function(u){r&&(r=!1,i.nextCallback=null,a(u))},this.nextCallback.cancel=function(){r=!1},this.nextCallback},n.onTransitionEnd=function(a,i){this.setNextCallback(i);var r=this.props.nodeRef?this.props.nodeRef.current:Nl.findDOMNode(this),u=a==null&&!this.props.addEndListener;if(!r||u){setTimeout(this.nextCallback,0);return}if(this.props.addEndListener){var o=this.props.nodeRef?[this.nextCallback]:[r,this.nextCallback],s=o[0],c=o[1];this.props.addEndListener(s,c)}a!=null&&setTimeout(this.nextCallback,a)},n.render=function(){var a=this.state.status;if(a===Va)return null;var i=this.props,r=i.children;i.in,i.mountOnEnter,i.unmountOnExit,i.appear,i.enter,i.exit,i.timeout,i.addEndListener,i.onEnter,i.onEntering,i.onEntered,i.onExit,i.onExiting,i.onExited,i.nodeRef;var u=Kg(i,["children","in","mountOnEnter","unmountOnExit","appear","enter","exit","timeout","addEndListener","onEnter","onEntering","onEntered","onExit","onExiting","onExited","nodeRef"]);return ye.createElement(th.Provider,{value:null},typeof r=="function"?r(a,u):ye.cloneElement(ye.Children.only(r),u))},t}(ye.Component);fn.contextType=th;fn.propTypes={};function _l(){}fn.defaultProps={in:!1,mountOnEnter:!1,unmountOnExit:!1,appear:!1,enter:!0,exit:!0,onEnter:_l,onEntering:_l,onEntered:_l,onExit:_l,onExiting:_l,onExited:_l};fn.UNMOUNTED=Va;fn.EXITED=Zn;fn.ENTERING=hn;fn.ENTERED=Pn;fn.EXITING=ps;function J1(e){return e.code==="Escape"||e.keyCode===27}function P1(){const e=w.version.split(".");return{major:+e[0],minor:+e[1],patch:+e[2]}}function zc(e){if(!e||typeof e=="function")return null;const{major:t}=P1();return t>=19?e.props.ref:e.ref}const Uc=!!(typeof window<"u"&&window.document&&window.document.createElement);var gs=!1,hs=!1;try{var mo={get passive(){return gs=!0},get once(){return hs=gs=!0}};Uc&&(window.addEventListener("test",mo,mo),window.removeEventListener("test",mo,!0))}catch{}function nh(e,t,n,l){if(l&&typeof l!="boolean"&&!hs){var a=l.once,i=l.capture,r=n;!hs&&a&&(r=n.__once||function u(o){this.removeEventListener(t,u,i),n.call(this,o)},n.__once=r),e.addEventListener(t,r,gs?l:i)}e.addEventListener(t,n,l)}function I1(e,t,n,l){var a=l&&typeof l!="boolean"?l.capture:l;e.removeEventListener(t,n,a),n.__once&&e.removeEventListener(t,n.__once,a)}function Wn(e,t,n,l){return nh(e,t,n,l),function(){I1(e,t,n,l)}}function W1(e,t,n,l){if(l===void 0&&(l=!0),e){var a=document.createEvent("HTMLEvents");a.initEvent(t,n,l),e.dispatchEvent(a)}}function eS(e){var t=Pg(e,"transitionDuration")||"",n=t.indexOf("ms")===-1?1e3:1;return parseFloat(t)*n}function tS(e,t,n){n===void 0&&(n=5);var l=!1,a=setTimeout(function(){l||W1(e,"transitionend",!0)},t+n),i=Wn(e,"transitionend",function(){l=!0},{once:!0});return function(){clearTimeout(a),i()}}function nS(e,t,n,l){n==null&&(n=eS(e)||0);var a=tS(e,n,l),i=Wn(e,"transitionend",t);return function(){a(),i()}}function bd(e,t){const n=Pg(e,t)||"",l=n.indexOf("ms")===-1?1e3:1;return parseFloat(n)*l}function lS(e,t){const n=bd(e,"transitionDuration"),l=bd(e,"transitionDelay"),a=nS(e,i=>{i.target===e&&(a(),t(i))},n+l)}function aS(e){e.offsetHeight}const Rd=e=>!e||typeof e=="function"?e:t=>{e.current=t};function iS(e,t){const n=Rd(e),l=Rd(t);return a=>{n&&n(a),l&&l(a)}}function Ui(e,t){return w.useMemo(()=>iS(e,t),[e,t])}function Gr(e){return e&&"setState"in e?Nl.findDOMNode(e):e??null}const rS=ye.forwardRef(({onEnter:e,onEntering:t,onEntered:n,onExit:l,onExiting:a,onExited:i,addEndListener:r,children:u,childRef:o,...s},c)=>{const f=w.useRef(null),p=Ui(f,o),d=_=>{p(Gr(_))},h=_=>E=>{_&&f.current&&_(f.current,E)},S=w.useCallback(h(e),[e]),x=w.useCallback(h(t),[t]),y=w.useCallback(h(n),[n]),m=w.useCallback(h(l),[l]),v=w.useCallback(h(a),[a]),b=w.useCallback(h(i),[i]),R=w.useCallback(h(r),[r]);return g.jsx(fn,{ref:c,...s,onEnter:S,onEntered:y,onEntering:x,onExit:m,onExited:b,onExiting:v,addEndListener:R,nodeRef:f,children:typeof u=="function"?(_,E)=>u(_,{...E,ref:d}):ye.cloneElement(u,{ref:d})})});function uS(e){const t=w.useRef(e);return w.useEffect(()=>{t.current=e},[e]),t}function lh(e){const t=uS(e);return w.useCallback(function(...n){return t.current&&t.current(...n)},[t])}const ah=e=>w.forwardRef((t,n)=>g.jsx("div",{...t,ref:n,className:$(t.className,e)}));function ms(){return w.useState(null)}function oS(e){const t=w.useRef(e);return w.useEffect(()=>{t.current=e},[e]),t}function Rt(e){const t=oS(e);return w.useCallback(function(...n){return t.current&&t.current(...n)},[t])}function sS(e,t,n,l=!1){const a=Rt(n);w.useEffect(()=>{const i=typeof e=="function"?e():e;return i.addEventListener(t,a,l),()=>i.removeEventListener(t,a,l)},[e])}function cS(){const e=w.useRef(!0),t=w.useRef(()=>e.current);return w.useEffect(()=>(e.current=!0,()=>{e.current=!1}),[]),t.current}function fS(e){const t=w.useRef(null);return w.useEffect(()=>{t.current=e}),t.current}const dS=typeof global<"u"&&global.navigator&&global.navigator.product==="ReactNative",pS=typeof document<"u",Ed=pS||dS?w.useLayoutEffect:w.useEffect,gS=["as","disabled"];function hS(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}function mS(e){return!e||e.trim()==="#"}function Fc({tagName:e,disabled:t,href:n,target:l,rel:a,role:i,onClick:r,tabIndex:u=0,type:o}){e||(n!=null||l!=null||a!=null?e="a":e="button");const s={tagName:e};if(e==="button")return[{type:o||"button",disabled:t},s];const c=p=>{if((t||e==="a"&&mS(n))&&p.preventDefault(),t){p.stopPropagation();return}r==null||r(p)},f=p=>{p.key===" "&&(p.preventDefault(),c(p))};return e==="a"&&(n||(n="#"),t&&(n=void 0)),[{role:i??"button",disabled:void 0,tabIndex:t?void 0:u,href:n,target:e==="a"?l:void 0,"aria-disabled":t||void 0,rel:e==="a"?a:void 0,onClick:c,onKeyDown:f},s]}const ih=w.forwardRef((e,t)=>{let{as:n,disabled:l}=e,a=hS(e,gS);const[i,{tagName:r}]=Fc(Object.assign({tagName:n,disabled:l},a));return g.jsx(r,Object.assign({},a,i,{ref:t}))});ih.displayName="Button";const yS=["onKeyDown"];function vS(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}function SS(e){return!e||e.trim()==="#"}const rh=w.forwardRef((e,t)=>{let{onKeyDown:n}=e,l=vS(e,yS);const[a]=Fc(Object.assign({tagName:"a"},l)),i=Rt(r=>{a.onKeyDown(r),n==null||n(r)});return SS(l.href)||l.role==="button"?g.jsx("a",Object.assign({ref:t},l,a,{onKeyDown:i})):g.jsx("a",Object.assign({ref:t},l,{onKeyDown:n}))});rh.displayName="Anchor";const xS={[hn]:"show",[Pn]:"show"},ys=w.forwardRef(({className:e,children:t,transitionClasses:n={},onEnter:l,...a},i)=>{const r={in:!1,timeout:300,mountOnEnter:!1,unmountOnExit:!1,appear:!1,...a},u=w.useCallback((o,s)=>{aS(o),l==null||l(o,s)},[l]);return g.jsx(rS,{ref:i,addEndListener:lS,...r,onEnter:u,childRef:zc(t),children:(o,s)=>w.cloneElement(t,{...s,className:$("fade",e,t.props.className,xS[o],n[o])})})});ys.displayName="Fade";const Je=w.forwardRef(({bsPrefix:e,bg:t="primary",pill:n=!1,text:l,className:a,as:i="span",...r},u)=>{const o=Y(e,"badge");return g.jsx(i,{ref:u,...r,className:$(a,o,n&&"rounded-pill",l&&`text-${l}`,t&&`bg-${t}`)})});Je.displayName="Badge";const He=w.forwardRef(({as:e,bsPrefix:t,variant:n="primary",size:l,active:a=!1,disabled:i=!1,className:r,...u},o)=>{const s=Y(t,"btn"),[c,{tagName:f}]=Fc({tagName:e,disabled:i,...u}),p=f;return g.jsx(p,{...c,...u,ref:o,disabled:i,className:$(r,s,a&&"active",n&&`${s}-${n}`,l&&`${s}-${l}`,u.href&&i&&"disabled")})});He.displayName="Button";const Za=w.forwardRef(({bsPrefix:e,size:t,vertical:n=!1,className:l,role:a="group",as:i="div",...r},u)=>{const o=Y(e,"btn-group");let s=o;return n&&(s=`${o}-vertical`),g.jsx(i,{...r,ref:u,role:a,className:$(l,s,t&&`${o}-${t}`)})});Za.displayName="ButtonGroup";const uh=w.forwardRef(({bsPrefix:e,className:t,role:n="toolbar",...l},a)=>{const i=Y(e,"btn-toolbar");return g.jsx("div",{...l,ref:a,className:$(t,i),role:n})});uh.displayName="ButtonToolbar";const Bc=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"card-body"),g.jsx(n,{ref:a,className:$(e,t),...l})));Bc.displayName="CardBody";const oh=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"card-footer"),g.jsx(n,{ref:a,className:$(e,t),...l})));oh.displayName="CardFooter";const sh=w.createContext(null);sh.displayName="CardHeaderContext";const ch=w.forwardRef(({bsPrefix:e,className:t,as:n="div",...l},a)=>{const i=Y(e,"card-header"),r=w.useMemo(()=>({cardHeaderBsPrefix:i}),[i]);return g.jsx(sh.Provider,{value:r,children:g.jsx(n,{ref:a,...l,className:$(t,i)})})});ch.displayName="CardHeader";const fh=w.forwardRef(({bsPrefix:e,className:t,variant:n,as:l="img",...a},i)=>{const r=Y(e,"card-img");return g.jsx(l,{ref:i,className:$(n?`${r}-${n}`:r,t),...a})});fh.displayName="CardImg";const dh=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"card-img-overlay"),g.jsx(n,{ref:a,className:$(e,t),...l})));dh.displayName="CardImgOverlay";const ph=w.forwardRef(({className:e,bsPrefix:t,as:n="a",...l},a)=>(t=Y(t,"card-link"),g.jsx(n,{ref:a,className:$(e,t),...l})));ph.displayName="CardLink";const wS=ah("h6"),gh=w.forwardRef(({className:e,bsPrefix:t,as:n=wS,...l},a)=>(t=Y(t,"card-subtitle"),g.jsx(n,{ref:a,className:$(e,t),...l})));gh.displayName="CardSubtitle";const hh=w.forwardRef(({className:e,bsPrefix:t,as:n="p",...l},a)=>(t=Y(t,"card-text"),g.jsx(n,{ref:a,className:$(e,t),...l})));hh.displayName="CardText";const bS=ah("h5"),mh=w.forwardRef(({className:e,bsPrefix:t,as:n=bS,...l},a)=>(t=Y(t,"card-title"),g.jsx(n,{ref:a,className:$(e,t),...l})));mh.displayName="CardTitle";const yh=w.forwardRef(({bsPrefix:e,className:t,bg:n,text:l,border:a,body:i=!1,children:r,as:u="div",...o},s)=>{const c=Y(e,"card");return g.jsx(u,{ref:s,...o,className:$(t,c,n&&`bg-${n}`,l&&`text-${l}`,a&&`border-${a}`),children:i?g.jsx(Bc,{children:r}):r})});yh.displayName="Card";const lr=Object.assign(yh,{Img:fh,Title:mh,Subtitle:gh,Body:Bc,Link:ph,Text:hh,Header:ch,Footer:oh,ImgOverlay:dh});function RS(){const e=w.useRef(!0),t=w.useRef(()=>e.current);return w.useEffect(()=>(e.current=!0,()=>{e.current=!1}),[]),t.current}function ES(e){const t=w.useRef(e);return t.current=e,t}function CS(e){const t=ES(e);w.useEffect(()=>()=>t.current(),[])}const vs=2**31-1;function vh(e,t,n){const l=n-Date.now();e.current=l<=vs?setTimeout(t,l):setTimeout(()=>vh(e,t,n),vs)}function _S(){const e=RS(),t=w.useRef();return CS(()=>clearTimeout(t.current)),w.useMemo(()=>{const n=()=>clearTimeout(t.current);function l(a,i=0){e()&&(n(),i<=vs?t.current=setTimeout(a,i):vh(t,a,Date.now()+i))}return{set:l,clear:n,handleRef:t}},[])}function AS(e,t){let n=0;return w.Children.map(e,l=>w.isValidElement(l)?t(l,n++):l)}function OS(e,t){return w.Children.toArray(e).some(n=>w.isValidElement(n)&&n.type===t)}function TS({as:e,bsPrefix:t,className:n,...l}){t=Y(t,"col");const a=U1(),i=F1(),r=[],u=[];return a.forEach(o=>{const s=l[o];delete l[o];let c,f,p;typeof s=="object"&&s!=null?{span:c,offset:f,order:p}=s:c=s;const d=o!==i?`-${o}`:"";c&&r.push(c===!0?`${t}${d}`:`${t}${d}-${c}`),p!=null&&u.push(`order${d}-${p}`),f!=null&&u.push(`offset${d}-${f}`)}),[{...l,className:$(n,...r,...u)},{as:e,bsPrefix:t,spans:r}]}const Sh=w.forwardRef((e,t)=>{const[{className:n,...l},{as:a="div",bsPrefix:i,spans:r}]=TS(e);return g.jsx(a,{...l,ref:t,className:$(n,!r.length&&i)})});Sh.displayName="Col";var DS=Function.prototype.bind.call(Function.prototype.call,[].slice);function Cd(e,t){return DS(e.querySelectorAll(t))}function MS(e,t,n){const l=w.useRef(e!==void 0),[a,i]=w.useState(t),r=e!==void 0,u=l.current;return l.current=r,!r&&u&&a!==t&&i(t),[r?e:a,w.useCallback((...o)=>{const[s,...c]=o;let f=n==null?void 0:n(s,...c);return i(s),f},[n])]}function jS(){const[,e]=w.useReducer(t=>t+1,0);return e}const Nu=w.createContext(null);var _d=Object.prototype.hasOwnProperty;function Ad(e,t,n){for(n of e.keys())if(Ka(n,t))return n}function Ka(e,t){var n,l,a;if(e===t)return!0;if(e&&t&&(n=e.constructor)===t.constructor){if(n===Date)return e.getTime()===t.getTime();if(n===RegExp)return e.toString()===t.toString();if(n===Array){if((l=e.length)===t.length)for(;l--&&Ka(e[l],t[l]););return l===-1}if(n===Set){if(e.size!==t.size)return!1;for(l of e)if(a=l,a&&typeof a=="object"&&(a=Ad(t,a),!a)||!t.has(a))return!1;return!0}if(n===Map){if(e.size!==t.size)return!1;for(l of e)if(a=l[0],a&&typeof a=="object"&&(a=Ad(t,a),!a)||!Ka(l[1],t.get(a)))return!1;return!0}if(n===ArrayBuffer)e=new Uint8Array(e),t=new Uint8Array(t);else if(n===DataView){if((l=e.byteLength)===t.byteLength)for(;l--&&e.getInt8(l)===t.getInt8(l););return l===-1}if(ArrayBuffer.isView(e)){if((l=e.byteLength)===t.byteLength)for(;l--&&e[l]===t[l];);return l===-1}if(!n||typeof e=="object"){l=0;for(n in e)if(_d.call(e,n)&&++l&&!_d.call(t,n)||!(n in t)||!Ka(e[n],t[n]))return!1;return Object.keys(t).length===l}}return e!==e&&t!==t}function NS(e){const t=cS();return[e[0],w.useCallback(n=>{if(t())return e[1](n)},[t,e[1]])]}var Ve="top",yt="bottom",vt="right",$e="left",Hc="auto",Fi=[Ve,yt,vt,$e],ia="start",gi="end",zS="clippingParents",xh="viewport",Oa="popper",US="reference",Od=Fi.reduce(function(e,t){return e.concat([t+"-"+ia,t+"-"+gi])},[]),wh=[].concat(Fi,[Hc]).reduce(function(e,t){return e.concat([t,t+"-"+ia,t+"-"+gi])},[]),FS="beforeRead",BS="read",HS="afterRead",LS="beforeMain",VS="main",$S="afterMain",qS="beforeWrite",GS="write",YS="afterWrite",XS=[FS,BS,HS,LS,VS,$S,qS,GS,YS];function Nt(e){return e.split("-")[0]}function nt(e){if(e==null)return window;if(e.toString()!=="[object Window]"){var t=e.ownerDocument;return t&&t.defaultView||window}return e}function dl(e){var t=nt(e).Element;return e instanceof t||e instanceof Element}function zt(e){var t=nt(e).HTMLElement;return e instanceof t||e instanceof HTMLElement}function Lc(e){if(typeof ShadowRoot>"u")return!1;var t=nt(e).ShadowRoot;return e instanceof t||e instanceof ShadowRoot}var al=Math.max,Yr=Math.min,ra=Math.round;function Ss(){var e=navigator.userAgentData;return e!=null&&e.brands&&Array.isArray(e.brands)?e.brands.map(function(t){return t.brand+"/"+t.version}).join(" "):navigator.userAgent}function bh(){return!/^((?!chrome|android).)*safari/i.test(Ss())}function ua(e,t,n){t===void 0&&(t=!1),n===void 0&&(n=!1);var l=e.getBoundingClientRect(),a=1,i=1;t&&zt(e)&&(a=e.offsetWidth>0&&ra(l.width)/e.offsetWidth||1,i=e.offsetHeight>0&&ra(l.height)/e.offsetHeight||1);var r=dl(e)?nt(e):window,u=r.visualViewport,o=!bh()&&n,s=(l.left+(o&&u?u.offsetLeft:0))/a,c=(l.top+(o&&u?u.offsetTop:0))/i,f=l.width/a,p=l.height/i;return{width:f,height:p,top:c,right:s+f,bottom:c+p,left:s,x:s,y:c}}function Vc(e){var t=ua(e),n=e.offsetWidth,l=e.offsetHeight;return Math.abs(t.width-n)<=1&&(n=t.width),Math.abs(t.height-l)<=1&&(l=t.height),{x:e.offsetLeft,y:e.offsetTop,width:n,height:l}}function Rh(e,t){var n=t.getRootNode&&t.getRootNode();if(e.contains(t))return!0;if(n&&Lc(n)){var l=t;do{if(l&&e.isSameNode(l))return!0;l=l.parentNode||l.host}while(l)}return!1}function zn(e){return e?(e.nodeName||"").toLowerCase():null}function an(e){return nt(e).getComputedStyle(e)}function kS(e){return["table","td","th"].indexOf(zn(e))>=0}function Vn(e){return((dl(e)?e.ownerDocument:e.document)||window.document).documentElement}function zu(e){return zn(e)==="html"?e:e.assignedSlot||e.parentNode||(Lc(e)?e.host:null)||Vn(e)}function Td(e){return!zt(e)||an(e).position==="fixed"?null:e.offsetParent}function QS(e){var t=/firefox/i.test(Ss()),n=/Trident/i.test(Ss());if(n&&zt(e)){var l=an(e);if(l.position==="fixed")return null}var a=zu(e);for(Lc(a)&&(a=a.host);zt(a)&&["html","body"].indexOf(zn(a))<0;){var i=an(a);if(i.transform!=="none"||i.perspective!=="none"||i.contain==="paint"||["transform","perspective"].indexOf(i.willChange)!==-1||t&&i.willChange==="filter"||t&&i.filter&&i.filter!=="none")return a;a=a.parentNode}return null}function Bi(e){for(var t=nt(e),n=Td(e);n&&kS(n)&&an(n).position==="static";)n=Td(n);return n&&(zn(n)==="html"||zn(n)==="body"&&an(n).position==="static")?t:n||QS(e)||t}function $c(e){return["top","bottom"].indexOf(e)>=0?"x":"y"}function Ja(e,t,n){return al(e,Yr(t,n))}function ZS(e,t,n){var l=Ja(e,t,n);return l>n?n:l}function Eh(){return{top:0,right:0,bottom:0,left:0}}function Ch(e){return Object.assign({},Eh(),e)}function _h(e,t){return t.reduce(function(n,l){return n[l]=e,n},{})}var KS=function(t,n){return t=typeof t=="function"?t(Object.assign({},n.rects,{placement:n.placement})):t,Ch(typeof t!="number"?t:_h(t,Fi))};function JS(e){var t,n=e.state,l=e.name,a=e.options,i=n.elements.arrow,r=n.modifiersData.popperOffsets,u=Nt(n.placement),o=$c(u),s=[$e,vt].indexOf(u)>=0,c=s?"height":"width";if(!(!i||!r)){var f=KS(a.padding,n),p=Vc(i),d=o==="y"?Ve:$e,h=o==="y"?yt:vt,S=n.rects.reference[c]+n.rects.reference[o]-r[o]-n.rects.popper[c],x=r[o]-n.rects.reference[o],y=Bi(i),m=y?o==="y"?y.clientHeight||0:y.clientWidth||0:0,v=S/2-x/2,b=f[d],R=m-p[c]-f[h],_=m/2-p[c]/2+v,E=Ja(b,_,R),A=o;n.modifiersData[l]=(t={},t[A]=E,t.centerOffset=E-_,t)}}function PS(e){var t=e.state,n=e.options,l=n.element,a=l===void 0?"[data-popper-arrow]":l;a!=null&&(typeof a=="string"&&(a=t.elements.popper.querySelector(a),!a)||Rh(t.elements.popper,a)&&(t.elements.arrow=a))}const IS={name:"arrow",enabled:!0,phase:"main",fn:JS,effect:PS,requires:["popperOffsets"],requiresIfExists:["preventOverflow"]};function oa(e){return e.split("-")[1]}var WS={top:"auto",right:"auto",bottom:"auto",left:"auto"};function ex(e,t){var n=e.x,l=e.y,a=t.devicePixelRatio||1;return{x:ra(n*a)/a||0,y:ra(l*a)/a||0}}function Dd(e){var t,n=e.popper,l=e.popperRect,a=e.placement,i=e.variation,r=e.offsets,u=e.position,o=e.gpuAcceleration,s=e.adaptive,c=e.roundOffsets,f=e.isFixed,p=r.x,d=p===void 0?0:p,h=r.y,S=h===void 0?0:h,x=typeof c=="function"?c({x:d,y:S}):{x:d,y:S};d=x.x,S=x.y;var y=r.hasOwnProperty("x"),m=r.hasOwnProperty("y"),v=$e,b=Ve,R=window;if(s){var _=Bi(n),E="clientHeight",A="clientWidth";if(_===nt(n)&&(_=Vn(n),an(_).position!=="static"&&u==="absolute"&&(E="scrollHeight",A="scrollWidth")),_=_,a===Ve||(a===$e||a===vt)&&i===gi){b=yt;var N=f&&_===R&&R.visualViewport?R.visualViewport.height:_[E];S-=N-l.height,S*=o?1:-1}if(a===$e||(a===Ve||a===yt)&&i===gi){v=vt;var M=f&&_===R&&R.visualViewport?R.visualViewport.width:_[A];d-=M-l.width,d*=o?1:-1}}var j=Object.assign({position:u},s&&WS),V=c===!0?ex({x:d,y:S},nt(n)):{x:d,y:S};if(d=V.x,S=V.y,o){var H;return Object.assign({},j,(H={},H[b]=m?"0":"",H[v]=y?"0":"",H.transform=(R.devicePixelRatio||1)<=1?"translate("+d+"px, "+S+"px)":"translate3d("+d+"px, "+S+"px, 0)",H))}return Object.assign({},j,(t={},t[b]=m?S+"px":"",t[v]=y?d+"px":"",t.transform="",t))}function tx(e){var t=e.state,n=e.options,l=n.gpuAcceleration,a=l===void 0?!0:l,i=n.adaptive,r=i===void 0?!0:i,u=n.roundOffsets,o=u===void 0?!0:u,s={placement:Nt(t.placement),variation:oa(t.placement),popper:t.elements.popper,popperRect:t.rects.popper,gpuAcceleration:a,isFixed:t.options.strategy==="fixed"};t.modifiersData.popperOffsets!=null&&(t.styles.popper=Object.assign({},t.styles.popper,Dd(Object.assign({},s,{offsets:t.modifiersData.popperOffsets,position:t.options.strategy,adaptive:r,roundOffsets:o})))),t.modifiersData.arrow!=null&&(t.styles.arrow=Object.assign({},t.styles.arrow,Dd(Object.assign({},s,{offsets:t.modifiersData.arrow,position:"absolute",adaptive:!1,roundOffsets:o})))),t.attributes.popper=Object.assign({},t.attributes.popper,{"data-popper-placement":t.placement})}const nx={name:"computeStyles",enabled:!0,phase:"beforeWrite",fn:tx,data:{}};var ar={passive:!0};function lx(e){var t=e.state,n=e.instance,l=e.options,a=l.scroll,i=a===void 0?!0:a,r=l.resize,u=r===void 0?!0:r,o=nt(t.elements.popper),s=[].concat(t.scrollParents.reference,t.scrollParents.popper);return i&&s.forEach(function(c){c.addEventListener("scroll",n.update,ar)}),u&&o.addEventListener("resize",n.update,ar),function(){i&&s.forEach(function(c){c.removeEventListener("scroll",n.update,ar)}),u&&o.removeEventListener("resize",n.update,ar)}}const ax={name:"eventListeners",enabled:!0,phase:"write",fn:function(){},effect:lx,data:{}};var ix={left:"right",right:"left",bottom:"top",top:"bottom"};function _r(e){return e.replace(/left|right|bottom|top/g,function(t){return ix[t]})}var rx={start:"end",end:"start"};function Md(e){return e.replace(/start|end/g,function(t){return rx[t]})}function qc(e){var t=nt(e),n=t.pageXOffset,l=t.pageYOffset;return{scrollLeft:n,scrollTop:l}}function Gc(e){return ua(Vn(e)).left+qc(e).scrollLeft}function ux(e,t){var n=nt(e),l=Vn(e),a=n.visualViewport,i=l.clientWidth,r=l.clientHeight,u=0,o=0;if(a){i=a.width,r=a.height;var s=bh();(s||!s&&t==="fixed")&&(u=a.offsetLeft,o=a.offsetTop)}return{width:i,height:r,x:u+Gc(e),y:o}}function ox(e){var t,n=Vn(e),l=qc(e),a=(t=e.ownerDocument)==null?void 0:t.body,i=al(n.scrollWidth,n.clientWidth,a?a.scrollWidth:0,a?a.clientWidth:0),r=al(n.scrollHeight,n.clientHeight,a?a.scrollHeight:0,a?a.clientHeight:0),u=-l.scrollLeft+Gc(e),o=-l.scrollTop;return an(a||n).direction==="rtl"&&(u+=al(n.clientWidth,a?a.clientWidth:0)-i),{width:i,height:r,x:u,y:o}}function Yc(e){var t=an(e),n=t.overflow,l=t.overflowX,a=t.overflowY;return/auto|scroll|overlay|hidden/.test(n+a+l)}function Ah(e){return["html","body","#document"].indexOf(zn(e))>=0?e.ownerDocument.body:zt(e)&&Yc(e)?e:Ah(zu(e))}function Pa(e,t){var n;t===void 0&&(t=[]);var l=Ah(e),a=l===((n=e.ownerDocument)==null?void 0:n.body),i=nt(l),r=a?[i].concat(i.visualViewport||[],Yc(l)?l:[]):l,u=t.concat(r);return a?u:u.concat(Pa(zu(r)))}function xs(e){return Object.assign({},e,{left:e.x,top:e.y,right:e.x+e.width,bottom:e.y+e.height})}function sx(e,t){var n=ua(e,!1,t==="fixed");return n.top=n.top+e.clientTop,n.left=n.left+e.clientLeft,n.bottom=n.top+e.clientHeight,n.right=n.left+e.clientWidth,n.width=e.clientWidth,n.height=e.clientHeight,n.x=n.left,n.y=n.top,n}function jd(e,t,n){return t===xh?xs(ux(e,n)):dl(t)?sx(t,n):xs(ox(Vn(e)))}function cx(e){var t=Pa(zu(e)),n=["absolute","fixed"].indexOf(an(e).position)>=0,l=n&&zt(e)?Bi(e):e;return dl(l)?t.filter(function(a){return dl(a)&&Rh(a,l)&&zn(a)!=="body"}):[]}function fx(e,t,n,l){var a=t==="clippingParents"?cx(e):[].concat(t),i=[].concat(a,[n]),r=i[0],u=i.reduce(function(o,s){var c=jd(e,s,l);return o.top=al(c.top,o.top),o.right=Yr(c.right,o.right),o.bottom=Yr(c.bottom,o.bottom),o.left=al(c.left,o.left),o},jd(e,r,l));return u.width=u.right-u.left,u.height=u.bottom-u.top,u.x=u.left,u.y=u.top,u}function Oh(e){var t=e.reference,n=e.element,l=e.placement,a=l?Nt(l):null,i=l?oa(l):null,r=t.x+t.width/2-n.width/2,u=t.y+t.height/2-n.height/2,o;switch(a){case Ve:o={x:r,y:t.y-n.height};break;case yt:o={x:r,y:t.y+t.height};break;case vt:o={x:t.x+t.width,y:u};break;case $e:o={x:t.x-n.width,y:u};break;default:o={x:t.x,y:t.y}}var s=a?$c(a):null;if(s!=null){var c=s==="y"?"height":"width";switch(i){case ia:o[s]=o[s]-(t[c]/2-n[c]/2);break;case gi:o[s]=o[s]+(t[c]/2-n[c]/2);break}}return o}function hi(e,t){t===void 0&&(t={});var n=t,l=n.placement,a=l===void 0?e.placement:l,i=n.strategy,r=i===void 0?e.strategy:i,u=n.boundary,o=u===void 0?zS:u,s=n.rootBoundary,c=s===void 0?xh:s,f=n.elementContext,p=f===void 0?Oa:f,d=n.altBoundary,h=d===void 0?!1:d,S=n.padding,x=S===void 0?0:S,y=Ch(typeof x!="number"?x:_h(x,Fi)),m=p===Oa?US:Oa,v=e.rects.popper,b=e.elements[h?m:p],R=fx(dl(b)?b:b.contextElement||Vn(e.elements.popper),o,c,r),_=ua(e.elements.reference),E=Oh({reference:_,element:v,strategy:"absolute",placement:a}),A=xs(Object.assign({},v,E)),N=p===Oa?A:_,M={top:R.top-N.top+y.top,bottom:N.bottom-R.bottom+y.bottom,left:R.left-N.left+y.left,right:N.right-R.right+y.right},j=e.modifiersData.offset;if(p===Oa&&j){var V=j[a];Object.keys(M).forEach(function(H){var K=[vt,yt].indexOf(H)>=0?1:-1,P=[Ve,yt].indexOf(H)>=0?"y":"x";M[H]+=V[P]*K})}return M}function dx(e,t){t===void 0&&(t={});var n=t,l=n.placement,a=n.boundary,i=n.rootBoundary,r=n.padding,u=n.flipVariations,o=n.allowedAutoPlacements,s=o===void 0?wh:o,c=oa(l),f=c?u?Od:Od.filter(function(h){return oa(h)===c}):Fi,p=f.filter(function(h){return s.indexOf(h)>=0});p.length===0&&(p=f);var d=p.reduce(function(h,S){return h[S]=hi(e,{placement:S,boundary:a,rootBoundary:i,padding:r})[Nt(S)],h},{});return Object.keys(d).sort(function(h,S){return d[h]-d[S]})}function px(e){if(Nt(e)===Hc)return[];var t=_r(e);return[Md(e),t,Md(t)]}function gx(e){var t=e.state,n=e.options,l=e.name;if(!t.modifiersData[l]._skip){for(var a=n.mainAxis,i=a===void 0?!0:a,r=n.altAxis,u=r===void 0?!0:r,o=n.fallbackPlacements,s=n.padding,c=n.boundary,f=n.rootBoundary,p=n.altBoundary,d=n.flipVariations,h=d===void 0?!0:d,S=n.allowedAutoPlacements,x=t.options.placement,y=Nt(x),m=y===x,v=o||(m||!h?[_r(x)]:px(x)),b=[x].concat(v).reduce(function(Be,Qe){return Be.concat(Nt(Qe)===Hc?dx(t,{placement:Qe,boundary:c,rootBoundary:f,padding:s,flipVariations:h,allowedAutoPlacements:S}):Qe)},[]),R=t.rects.reference,_=t.rects.popper,E=new Map,A=!0,N=b[0],M=0;M<b.length;M++){var j=b[M],V=Nt(j),H=oa(j)===ia,K=[Ve,yt].indexOf(V)>=0,P=K?"width":"height",D=hi(t,{placement:j,boundary:c,rootBoundary:f,altBoundary:p,padding:s}),U=K?H?vt:$e:H?yt:Ve;R[P]>_[P]&&(U=_r(U));var L=_r(U),k=[];if(i&&k.push(D[V]<=0),u&&k.push(D[U]<=0,D[L]<=0),k.every(function(Be){return Be})){N=j,A=!1;break}E.set(j,k)}if(A)for(var Q=h?3:1,Fe=function(Qe){var Aa=b.find(function(Wi){var Gn=E.get(Wi);if(Gn)return Gn.slice(0,Qe).every(function(po){return po})});if(Aa)return N=Aa,"break"},it=Q;it>0;it--){var ke=Fe(it);if(ke==="break")break}t.placement!==N&&(t.modifiersData[l]._skip=!0,t.placement=N,t.reset=!0)}}const hx={name:"flip",enabled:!0,phase:"main",fn:gx,requiresIfExists:["offset"],data:{_skip:!1}};function Nd(e,t,n){return n===void 0&&(n={x:0,y:0}),{top:e.top-t.height-n.y,right:e.right-t.width+n.x,bottom:e.bottom-t.height+n.y,left:e.left-t.width-n.x}}function zd(e){return[Ve,vt,yt,$e].some(function(t){return e[t]>=0})}function mx(e){var t=e.state,n=e.name,l=t.rects.reference,a=t.rects.popper,i=t.modifiersData.preventOverflow,r=hi(t,{elementContext:"reference"}),u=hi(t,{altBoundary:!0}),o=Nd(r,l),s=Nd(u,a,i),c=zd(o),f=zd(s);t.modifiersData[n]={referenceClippingOffsets:o,popperEscapeOffsets:s,isReferenceHidden:c,hasPopperEscaped:f},t.attributes.popper=Object.assign({},t.attributes.popper,{"data-popper-reference-hidden":c,"data-popper-escaped":f})}const yx={name:"hide",enabled:!0,phase:"main",requiresIfExists:["preventOverflow"],fn:mx};function vx(e,t,n){var l=Nt(e),a=[$e,Ve].indexOf(l)>=0?-1:1,i=typeof n=="function"?n(Object.assign({},t,{placement:e})):n,r=i[0],u=i[1];return r=r||0,u=(u||0)*a,[$e,vt].indexOf(l)>=0?{x:u,y:r}:{x:r,y:u}}function Sx(e){var t=e.state,n=e.options,l=e.name,a=n.offset,i=a===void 0?[0,0]:a,r=wh.reduce(function(c,f){return c[f]=vx(f,t.rects,i),c},{}),u=r[t.placement],o=u.x,s=u.y;t.modifiersData.popperOffsets!=null&&(t.modifiersData.popperOffsets.x+=o,t.modifiersData.popperOffsets.y+=s),t.modifiersData[l]=r}const xx={name:"offset",enabled:!0,phase:"main",requires:["popperOffsets"],fn:Sx};function wx(e){var t=e.state,n=e.name;t.modifiersData[n]=Oh({reference:t.rects.reference,element:t.rects.popper,strategy:"absolute",placement:t.placement})}const bx={name:"popperOffsets",enabled:!0,phase:"read",fn:wx,data:{}};function Rx(e){return e==="x"?"y":"x"}function Ex(e){var t=e.state,n=e.options,l=e.name,a=n.mainAxis,i=a===void 0?!0:a,r=n.altAxis,u=r===void 0?!1:r,o=n.boundary,s=n.rootBoundary,c=n.altBoundary,f=n.padding,p=n.tether,d=p===void 0?!0:p,h=n.tetherOffset,S=h===void 0?0:h,x=hi(t,{boundary:o,rootBoundary:s,padding:f,altBoundary:c}),y=Nt(t.placement),m=oa(t.placement),v=!m,b=$c(y),R=Rx(b),_=t.modifiersData.popperOffsets,E=t.rects.reference,A=t.rects.popper,N=typeof S=="function"?S(Object.assign({},t.rects,{placement:t.placement})):S,M=typeof N=="number"?{mainAxis:N,altAxis:N}:Object.assign({mainAxis:0,altAxis:0},N),j=t.modifiersData.offset?t.modifiersData.offset[t.placement]:null,V={x:0,y:0};if(_){if(i){var H,K=b==="y"?Ve:$e,P=b==="y"?yt:vt,D=b==="y"?"height":"width",U=_[b],L=U+x[K],k=U-x[P],Q=d?-A[D]/2:0,Fe=m===ia?E[D]:A[D],it=m===ia?-A[D]:-E[D],ke=t.elements.arrow,Be=d&&ke?Vc(ke):{width:0,height:0},Qe=t.modifiersData["arrow#persistent"]?t.modifiersData["arrow#persistent"].padding:Eh(),Aa=Qe[K],Wi=Qe[P],Gn=Ja(0,E[D],Be[D]),po=v?E[D]/2-Q-Gn-Aa-M.mainAxis:Fe-Gn-Aa-M.mainAxis,Iv=v?-E[D]/2+Q+Gn+Wi+M.mainAxis:it+Gn+Wi+M.mainAxis,go=t.elements.arrow&&Bi(t.elements.arrow),Wv=go?b==="y"?go.clientTop||0:go.clientLeft||0:0,sd=(H=j==null?void 0:j[b])!=null?H:0,e1=U+po-sd-Wv,t1=U+Iv-sd,cd=Ja(d?Yr(L,e1):L,U,d?al(k,t1):k);_[b]=cd,V[b]=cd-U}if(u){var fd,n1=b==="x"?Ve:$e,l1=b==="x"?yt:vt,Yn=_[R],er=R==="y"?"height":"width",dd=Yn+x[n1],pd=Yn-x[l1],ho=[Ve,$e].indexOf(y)!==-1,gd=(fd=j==null?void 0:j[R])!=null?fd:0,hd=ho?dd:Yn-E[er]-A[er]-gd+M.altAxis,md=ho?Yn+E[er]+A[er]-gd-M.altAxis:pd,yd=d&&ho?ZS(hd,Yn,md):Ja(d?hd:dd,Yn,d?md:pd);_[R]=yd,V[R]=yd-Yn}t.modifiersData[l]=V}}const Cx={name:"preventOverflow",enabled:!0,phase:"main",fn:Ex,requiresIfExists:["offset"]};function _x(e){return{scrollLeft:e.scrollLeft,scrollTop:e.scrollTop}}function Ax(e){return e===nt(e)||!zt(e)?qc(e):_x(e)}function Ox(e){var t=e.getBoundingClientRect(),n=ra(t.width)/e.offsetWidth||1,l=ra(t.height)/e.offsetHeight||1;return n!==1||l!==1}function Tx(e,t,n){n===void 0&&(n=!1);var l=zt(t),a=zt(t)&&Ox(t),i=Vn(t),r=ua(e,a,n),u={scrollLeft:0,scrollTop:0},o={x:0,y:0};return(l||!l&&!n)&&((zn(t)!=="body"||Yc(i))&&(u=Ax(t)),zt(t)?(o=ua(t,!0),o.x+=t.clientLeft,o.y+=t.clientTop):i&&(o.x=Gc(i))),{x:r.left+u.scrollLeft-o.x,y:r.top+u.scrollTop-o.y,width:r.width,height:r.height}}function Dx(e){var t=new Map,n=new Set,l=[];e.forEach(function(i){t.set(i.name,i)});function a(i){n.add(i.name);var r=[].concat(i.requires||[],i.requiresIfExists||[]);r.forEach(function(u){if(!n.has(u)){var o=t.get(u);o&&a(o)}}),l.push(i)}return e.forEach(function(i){n.has(i.name)||a(i)}),l}function Mx(e){var t=Dx(e);return XS.reduce(function(n,l){return n.concat(t.filter(function(a){return a.phase===l}))},[])}function jx(e){var t;return function(){return t||(t=new Promise(function(n){Promise.resolve().then(function(){t=void 0,n(e())})})),t}}function Nx(e){var t=e.reduce(function(n,l){var a=n[l.name];return n[l.name]=a?Object.assign({},a,l,{options:Object.assign({},a.options,l.options),data:Object.assign({},a.data,l.data)}):l,n},{});return Object.keys(t).map(function(n){return t[n]})}var Ud={placement:"bottom",modifiers:[],strategy:"absolute"};function Fd(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return!t.some(function(l){return!(l&&typeof l.getBoundingClientRect=="function")})}function zx(e){e===void 0&&(e={});var t=e,n=t.defaultModifiers,l=n===void 0?[]:n,a=t.defaultOptions,i=a===void 0?Ud:a;return function(u,o,s){s===void 0&&(s=i);var c={placement:"bottom",orderedModifiers:[],options:Object.assign({},Ud,i),modifiersData:{},elements:{reference:u,popper:o},attributes:{},styles:{}},f=[],p=!1,d={state:c,setOptions:function(y){var m=typeof y=="function"?y(c.options):y;S(),c.options=Object.assign({},i,c.options,m),c.scrollParents={reference:dl(u)?Pa(u):u.contextElement?Pa(u.contextElement):[],popper:Pa(o)};var v=Mx(Nx([].concat(l,c.options.modifiers)));return c.orderedModifiers=v.filter(function(b){return b.enabled}),h(),d.update()},forceUpdate:function(){if(!p){var y=c.elements,m=y.reference,v=y.popper;if(Fd(m,v)){c.rects={reference:Tx(m,Bi(v),c.options.strategy==="fixed"),popper:Vc(v)},c.reset=!1,c.placement=c.options.placement,c.orderedModifiers.forEach(function(M){return c.modifiersData[M.name]=Object.assign({},M.data)});for(var b=0;b<c.orderedModifiers.length;b++){if(c.reset===!0){c.reset=!1,b=-1;continue}var R=c.orderedModifiers[b],_=R.fn,E=R.options,A=E===void 0?{}:E,N=R.name;typeof _=="function"&&(c=_({state:c,options:A,name:N,instance:d})||c)}}}},update:jx(function(){return new Promise(function(x){d.forceUpdate(),x(c)})}),destroy:function(){S(),p=!0}};if(!Fd(u,o))return d;d.setOptions(s).then(function(x){!p&&s.onFirstUpdate&&s.onFirstUpdate(x)});function h(){c.orderedModifiers.forEach(function(x){var y=x.name,m=x.options,v=m===void 0?{}:m,b=x.effect;if(typeof b=="function"){var R=b({state:c,name:y,instance:d,options:v}),_=function(){};f.push(R||_)}})}function S(){f.forEach(function(x){return x()}),f=[]}return d}}const Ux=zx({defaultModifiers:[yx,bx,nx,ax,xx,hx,Cx,IS]}),Fx=["enabled","placement","strategy","modifiers"];function Bx(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}const Hx={name:"applyStyles",enabled:!1,phase:"afterWrite",fn:()=>{}},Lx={name:"ariaDescribedBy",enabled:!0,phase:"afterWrite",effect:({state:e})=>()=>{const{reference:t,popper:n}=e.elements;if("removeAttribute"in t){const l=(t.getAttribute("aria-describedby")||"").split(",").filter(a=>a.trim()!==n.id);l.length?t.setAttribute("aria-describedby",l.join(",")):t.removeAttribute("aria-describedby")}},fn:({state:e})=>{var t;const{popper:n,reference:l}=e.elements,a=(t=n.getAttribute("role"))==null?void 0:t.toLowerCase();if(n.id&&a==="tooltip"&&"setAttribute"in l){const i=l.getAttribute("aria-describedby");if(i&&i.split(",").indexOf(n.id)!==-1)return;l.setAttribute("aria-describedby",i?`${i},${n.id}`:n.id)}}},Vx=[];function Th(e,t,n={}){let{enabled:l=!0,placement:a="bottom",strategy:i="absolute",modifiers:r=Vx}=n,u=Bx(n,Fx);const o=w.useRef(r),s=w.useRef(),c=w.useCallback(()=>{var x;(x=s.current)==null||x.update()},[]),f=w.useCallback(()=>{var x;(x=s.current)==null||x.forceUpdate()},[]),[p,d]=NS(w.useState({placement:a,update:c,forceUpdate:f,attributes:{},styles:{popper:{},arrow:{}}})),h=w.useMemo(()=>({name:"updateStateModifier",enabled:!0,phase:"write",requires:["computeStyles"],fn:({state:x})=>{const y={},m={};Object.keys(x.elements).forEach(v=>{y[v]=x.styles[v],m[v]=x.attributes[v]}),d({state:x,styles:y,attributes:m,update:c,forceUpdate:f,placement:x.placement})}}),[c,f,d]),S=w.useMemo(()=>(Ka(o.current,r)||(o.current=r),o.current),[r]);return w.useEffect(()=>{!s.current||!l||s.current.setOptions({placement:a,strategy:i,modifiers:[...S,h,Hx]})},[i,a,h,l,S]),w.useEffect(()=>{if(!(!l||e==null||t==null))return s.current=Ux(e,t,Object.assign({},u,{placement:a,strategy:i,modifiers:[...S,Lx,h]})),()=>{s.current!=null&&(s.current.destroy(),s.current=void 0,d(x=>Object.assign({},x,{attributes:{},styles:{popper:{}}})))}},[l,e,t]),p}function ws(e,t){if(e.contains)return e.contains(t);if(e.compareDocumentPosition)return e===t||!!(e.compareDocumentPosition(t)&16)}var $x=function(){},qx=$x;const Gx=zi(qx),Bd=()=>{};function Yx(e){return e.button===0}function Xx(e){return!!(e.metaKey||e.altKey||e.ctrlKey||e.shiftKey)}const Ar=e=>e&&("current"in e?e.current:e),Hd={click:"mousedown",mouseup:"mousedown",pointerup:"pointerdown"};function Dh(e,t=Bd,{disabled:n,clickTrigger:l="click"}={}){const a=w.useRef(!1),i=w.useRef(!1),r=w.useCallback(s=>{const c=Ar(e);Gx(!!c,"ClickOutside captured a close event but does not have a ref to compare it to. useClickOutside(), should be passed a ref that resolves to a DOM node"),a.current=!c||Xx(s)||!Yx(s)||!!ws(c,s.target)||i.current,i.current=!1},[e]),u=Rt(s=>{const c=Ar(e);c&&ws(c,s.target)?i.current=!0:i.current=!1}),o=Rt(s=>{a.current||t(s)});w.useEffect(()=>{var s,c;if(n||e==null)return;const f=ju(Ar(e)),p=f.defaultView||window;let d=(s=p.event)!=null?s:(c=p.parent)==null?void 0:c.event,h=null;Hd[l]&&(h=Wn(f,Hd[l],u,!0));const S=Wn(f,l,r,!0),x=Wn(f,l,m=>{if(m===d){d=void 0;return}o(m)});let y=[];return"ontouchstart"in f.documentElement&&(y=[].slice.call(f.body.children).map(m=>Wn(m,"mousemove",Bd))),()=>{h==null||h(),S(),x(),y.forEach(m=>m())}},[e,n,l,r,u,o])}function kx(e){const t={};return Array.isArray(e)?(e==null||e.forEach(n=>{t[n.name]=n}),t):e||t}function Qx(e={}){return Array.isArray(e)?e:Object.keys(e).map(t=>(e[t].name=t,e[t]))}function Mh({enabled:e,enableEvents:t,placement:n,flip:l,offset:a,fixed:i,containerPadding:r,arrowElement:u,popperConfig:o={}}){var s,c,f,p,d;const h=kx(o.modifiers);return Object.assign({},o,{placement:n,enabled:e,strategy:i?"fixed":o.strategy,modifiers:Qx(Object.assign({},h,{eventListeners:{enabled:t,options:(s=h.eventListeners)==null?void 0:s.options},preventOverflow:Object.assign({},h.preventOverflow,{options:r?Object.assign({padding:r},(c=h.preventOverflow)==null?void 0:c.options):(f=h.preventOverflow)==null?void 0:f.options}),offset:{options:Object.assign({offset:a},(p=h.offset)==null?void 0:p.options)},arrow:Object.assign({},h.arrow,{enabled:!!u,options:Object.assign({},(d=h.arrow)==null?void 0:d.options,{element:u})}),flip:Object.assign({enabled:!!l},h.flip)}))})}const Zx=["children","usePopper"];function Kx(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}const Jx=()=>{};function jh(e={}){const t=w.useContext(Nu),[n,l]=ms(),a=w.useRef(!1),{flip:i,offset:r,rootCloseEvent:u,fixed:o=!1,placement:s,popperConfig:c={},enableEventListeners:f=!0,usePopper:p=!!t}=e,d=(t==null?void 0:t.show)==null?!!e.show:t.show;d&&!a.current&&(a.current=!0);const h=_=>{t==null||t.toggle(!1,_)},{placement:S,setMenu:x,menuElement:y,toggleElement:m}=t||{},v=Th(m,y,Mh({placement:s||S||"bottom-start",enabled:p,enableEvents:f??d,offset:r,flip:i,fixed:o,arrowElement:n,popperConfig:c})),b=Object.assign({ref:x||Jx,"aria-labelledby":m==null?void 0:m.id},v.attributes.popper,{style:v.styles.popper}),R={show:d,placement:S,hasShown:a.current,toggle:t==null?void 0:t.toggle,popper:p?v:null,arrowProps:p?Object.assign({ref:l},v.attributes.arrow,{style:v.styles.arrow}):{}};return Dh(y,h,{clickTrigger:u,disabled:!d}),[b,R]}function Nh(e){let{children:t,usePopper:n=!0}=e,l=Kx(e,Zx);const[a,i]=jh(Object.assign({},l,{usePopper:n}));return g.jsx(g.Fragment,{children:t(a,i)})}Nh.displayName="DropdownMenu";const Xc={prefix:String(Math.round(Math.random()*1e10)),current:0},zh=ye.createContext(Xc),Px=ye.createContext(!1);let Ix=!!(typeof window<"u"&&window.document&&window.document.createElement),yo=new WeakMap;function Wx(e=!1){let t=w.useContext(zh),n=w.useRef(null);if(n.current===null&&!e){var l,a;let i=(a=ye.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED)===null||a===void 0||(l=a.ReactCurrentOwner)===null||l===void 0?void 0:l.current;if(i){let r=yo.get(i);r==null?yo.set(i,{id:t.current,state:i.memoizedState}):i.memoizedState!==r.state&&(t.current=r.id,yo.delete(i))}n.current=++t.current}return n.current}function ew(e){let t=w.useContext(zh);t===Xc&&!Ix&&console.warn("When server rendering, you must wrap your application in an <SSRProvider> to ensure consistent ids are generated between the client and server.");let n=Wx(!!e),l=`react-aria${t.prefix}`;return e||`${l}-${n}`}function tw(e){let t=ye.useId(),[n]=w.useState(rw()),l=n?"react-aria":`react-aria${Xc.prefix}`;return e||`${l}-${t}`}const nw=typeof ye.useId=="function"?tw:ew;function lw(){return!1}function aw(){return!0}function iw(e){return()=>{}}function rw(){return typeof ye.useSyncExternalStore=="function"?ye.useSyncExternalStore(iw,lw,aw):w.useContext(Px)}const Uh=e=>{var t;return((t=e.getAttribute("role"))==null?void 0:t.toLowerCase())==="menu"},Ld=()=>{};function Fh(){const e=nw(),{show:t=!1,toggle:n=Ld,setToggle:l,menuElement:a}=w.useContext(Nu)||{},i=w.useCallback(u=>{n(!t,u)},[t,n]),r={id:e,ref:l||Ld,onClick:i,"aria-expanded":!!t};return a&&Uh(a)&&(r["aria-haspopup"]=!0),[r,{show:t,toggle:n}]}function Bh({children:e}){const[t,n]=Fh();return g.jsx(g.Fragment,{children:e(t,n)})}Bh.displayName="DropdownToggle";const bs=w.createContext(null),Vd=(e,t=null)=>e!=null?String(e):t||null,Hh=w.createContext(null);Hh.displayName="NavContext";const uw="data-rr-ui-";function Lh(e){return`${uw}${e}`}const ow=["eventKey","disabled","onClick","active","as"];function sw(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}function Vh({key:e,href:t,active:n,disabled:l,onClick:a}){const i=w.useContext(bs),r=w.useContext(Hh),{activeKey:u}=r||{},o=Vd(e,t),s=n==null&&e!=null?Vd(u)===o:n;return[{onClick:Rt(f=>{l||(a==null||a(f),i&&!f.isPropagationStopped()&&i(o,f))}),"aria-disabled":l||void 0,"aria-selected":s,[Lh("dropdown-item")]:""},{isActive:s}]}const $h=w.forwardRef((e,t)=>{let{eventKey:n,disabled:l,onClick:a,active:i,as:r=ih}=e,u=sw(e,ow);const[o]=Vh({key:n,href:u.href,disabled:l,onClick:a,active:i});return g.jsx(r,Object.assign({},u,{ref:t},o))});$h.displayName="DropdownItem";const qh=w.createContext(Uc?window:void 0);qh.Provider;function Gh(){return w.useContext(qh)}function $d(){const e=jS(),t=w.useRef(null),n=w.useCallback(l=>{t.current=l,e()},[e]);return[t,n]}function Hi({defaultShow:e,show:t,onSelect:n,onToggle:l,itemSelector:a=`* [${Lh("dropdown-item")}]`,focusFirstItemOnShow:i,placement:r="bottom-start",children:u}){const o=Gh(),[s,c]=MS(t,e,l),[f,p]=$d(),d=f.current,[h,S]=$d(),x=h.current,y=fS(s),m=w.useRef(null),v=w.useRef(!1),b=w.useContext(bs),R=w.useCallback((j,V,H=V==null?void 0:V.type)=>{c(j,{originalEvent:V,source:H})},[c]),_=Rt((j,V)=>{n==null||n(j,V),R(!1,V,"select"),V.isPropagationStopped()||b==null||b(j,V)}),E=w.useMemo(()=>({toggle:R,placement:r,show:s,menuElement:d,toggleElement:x,setMenu:p,setToggle:S}),[R,r,s,d,x,p,S]);d&&y&&!s&&(v.current=d.contains(d.ownerDocument.activeElement));const A=Rt(()=>{x&&x.focus&&x.focus()}),N=Rt(()=>{const j=m.current;let V=i;if(V==null&&(V=f.current&&Uh(f.current)?"keyboard":!1),V===!1||V==="keyboard"&&!/^key.+$/.test(j))return;const H=Cd(f.current,a)[0];H&&H.focus&&H.focus()});w.useEffect(()=>{s?N():v.current&&(v.current=!1,A())},[s,v,A,N]),w.useEffect(()=>{m.current=null});const M=(j,V)=>{if(!f.current)return null;const H=Cd(f.current,a);let K=H.indexOf(j)+V;return K=Math.max(0,Math.min(K,H.length)),H[K]};return sS(w.useCallback(()=>o.document,[o]),"keydown",j=>{var V,H;const{key:K}=j,P=j.target,D=(V=f.current)==null?void 0:V.contains(P),U=(H=h.current)==null?void 0:H.contains(P);if(/input|textarea/i.test(P.tagName)&&(K===" "||K!=="Escape"&&D||K==="Escape"&&P.type==="search")||!D&&!U||K==="Tab"&&(!f.current||!s))return;m.current=j.type;const k={originalEvent:j,source:j.type};switch(K){case"ArrowUp":{const Q=M(P,-1);Q&&Q.focus&&Q.focus(),j.preventDefault();return}case"ArrowDown":if(j.preventDefault(),!s)c(!0,k);else{const Q=M(P,1);Q&&Q.focus&&Q.focus()}return;case"Tab":nh(P.ownerDocument,"keyup",Q=>{var Fe;(Q.key==="Tab"&&!Q.target||!((Fe=f.current)!=null&&Fe.contains(Q.target)))&&c(!1,k)},{once:!0});break;case"Escape":K==="Escape"&&(j.preventDefault(),j.stopPropagation()),c(!1,k);break}}),g.jsx(bs.Provider,{value:_,children:g.jsx(Nu.Provider,{value:E,children:u})})}Hi.displayName="Dropdown";Hi.Menu=Nh;Hi.Toggle=Bh;Hi.Item=$h;const kc=w.createContext({});kc.displayName="DropdownContext";const Yh=w.forwardRef(({className:e,bsPrefix:t,as:n="hr",role:l="separator",...a},i)=>(t=Y(t,"dropdown-divider"),g.jsx(n,{ref:i,className:$(e,t),role:l,...a})));Yh.displayName="DropdownDivider";const Xh=w.forwardRef(({className:e,bsPrefix:t,as:n="div",role:l="heading",...a},i)=>(t=Y(t,"dropdown-header"),g.jsx(n,{ref:i,className:$(e,t),role:l,...a})));Xh.displayName="DropdownHeader";const kh=w.forwardRef(({bsPrefix:e,className:t,eventKey:n,disabled:l=!1,onClick:a,active:i,as:r=rh,...u},o)=>{const s=Y(e,"dropdown-item"),[c,f]=Vh({key:n,href:u.href,disabled:l,onClick:a,active:i});return g.jsx(r,{...u,...c,ref:o,className:$(t,s,f.isActive&&"active",l&&"disabled")})});kh.displayName="DropdownItem";const Qh=w.forwardRef(({className:e,bsPrefix:t,as:n="span",...l},a)=>(t=Y(t,"dropdown-item-text"),g.jsx(n,{ref:a,className:$(e,t),...l})));Qh.displayName="DropdownItemText";const cw=typeof global<"u"&&global.navigator&&global.navigator.product==="ReactNative",fw=typeof document<"u",Zh=fw||cw?w.useLayoutEffect:w.useEffect,Qc=w.createContext(null);Qc.displayName="InputGroupContext";const Kh=w.createContext(null);Kh.displayName="NavbarContext";function Jh(e,t){return e}const Al=z.oneOf(["start","end"]),dw=z.oneOfType([Al,z.shape({sm:Al}),z.shape({md:Al}),z.shape({lg:Al}),z.shape({xl:Al}),z.shape({xxl:Al}),z.object]);function Ph(e,t,n){const l=n?"top-end":"top-start",a=n?"top-start":"top-end",i=n?"bottom-end":"bottom-start",r=n?"bottom-start":"bottom-end",u=n?"right-start":"left-start",o=n?"right-end":"left-end",s=n?"left-start":"right-start",c=n?"left-end":"right-end";let f=e?r:i;return t==="up"?f=e?a:l:t==="end"?f=e?c:s:t==="start"?f=e?o:u:t==="down-centered"?f="bottom":t==="up-centered"&&(f="top"),f}const Ih=w.forwardRef(({bsPrefix:e,className:t,align:n,rootCloseEvent:l,flip:a=!0,show:i,renderOnMount:r,as:u="div",popperConfig:o,variant:s,...c},f)=>{let p=!1;const d=w.useContext(Kh),h=Y(e,"dropdown-menu"),{align:S,drop:x,isRTL:y}=w.useContext(kc);n=n||S;const m=w.useContext(Qc),v=[];if(n)if(typeof n=="object"){const j=Object.keys(n);if(j.length){const V=j[0],H=n[V];p=H==="start",v.push(`${h}-${V}-${H}`)}}else n==="end"&&(p=!0);const b=Ph(p,x,y),[R,{hasShown:_,popper:E,show:A,toggle:N}]=jh({flip:a,rootCloseEvent:l,show:i,usePopper:!d&&v.length===0,offset:[0,2],popperConfig:o,placement:b});if(R.ref=Ui(Jh(f),R.ref),Zh(()=>{A&&(E==null||E.update())},[A]),!_&&!r&&!m)return null;typeof u!="string"&&(R.show=A,R.close=()=>N==null?void 0:N(!1),R.align=n);let M=c.style;return E!=null&&E.placement&&(M={...c.style,...R.style},c["x-placement"]=E.placement),g.jsx(u,{...c,...R,style:M,...(v.length||d)&&{"data-bs-popper":"static"},className:$(t,h,A&&"show",p&&`${h}-end`,s&&`${h}-${s}`,...v)})});Ih.displayName="DropdownMenu";const Wh=w.forwardRef(({bsPrefix:e,split:t,className:n,childBsPrefix:l,as:a=He,...i},r)=>{const u=Y(e,"dropdown-toggle"),o=w.useContext(Nu);l!==void 0&&(i.bsPrefix=l);const[s]=Fh();return s.ref=Ui(s.ref,Jh(r)),g.jsx(a,{className:$(n,u,t&&`${u}-split`,(o==null?void 0:o.show)&&"show"),...s,...i})});Wh.displayName="DropdownToggle";const em=w.forwardRef((e,t)=>{const{bsPrefix:n,drop:l="down",show:a,className:i,align:r="start",onSelect:u,onToggle:o,focusFirstItemOnShow:s,as:c="div",navbar:f,autoClose:p=!0,...d}=j1(e,{show:"onToggle"}),h=w.useContext(Qc),S=Y(n,"dropdown"),x=Nc(),y=E=>p===!1?E==="click":p==="inside"?E!=="rootClose":p==="outside"?E!=="select":!0,m=lh((E,A)=>{var N;!((N=A.originalEvent)==null||(N=N.target)==null)&&N.classList.contains("dropdown-toggle")&&A.source==="mousedown"||(A.originalEvent.currentTarget===document&&(A.source!=="keydown"||A.originalEvent.key==="Escape")&&(A.source="rootClose"),y(A.source)&&(o==null||o(E,A)))}),b=Ph(r==="end",l,x),R=w.useMemo(()=>({align:r,drop:l,isRTL:x}),[r,l,x]),_={down:S,"down-centered":`${S}-center`,up:"dropup","up-centered":"dropup-center dropup",end:"dropend",start:"dropstart"};return g.jsx(kc.Provider,{value:R,children:g.jsx(Hi,{placement:b,show:a,onSelect:u,onToggle:m,focusFirstItemOnShow:s,itemSelector:`.${S}-item:not(.disabled):not(:disabled)`,children:h?d.children:g.jsx(c,{...d,ref:t,className:$(i,a&&"show",_[l])})})})});em.displayName="Dropdown";const bn=Object.assign(em,{Toggle:Wh,Menu:Ih,Item:kh,ItemText:Qh,Divider:Yh,Header:Xh}),pw={type:z.string,tooltip:z.bool,as:z.elementType},Uu=w.forwardRef(({as:e="div",className:t,type:n="valid",tooltip:l=!1,...a},i)=>g.jsx(e,{...a,ref:i,className:$(t,`${n}-${l?"tooltip":"feedback"}`)}));Uu.displayName="Feedback";Uu.propTypes=pw;const rn=w.createContext({}),Zc=w.forwardRef(({id:e,bsPrefix:t,className:n,type:l="checkbox",isValid:a=!1,isInvalid:i=!1,as:r="input",...u},o)=>{const{controlId:s}=w.useContext(rn);return t=Y(t,"form-check-input"),g.jsx(r,{...u,ref:o,type:l,id:e||s,className:$(n,t,a&&"is-valid",i&&"is-invalid")})});Zc.displayName="FormCheckInput";const Xr=w.forwardRef(({bsPrefix:e,className:t,htmlFor:n,...l},a)=>{const{controlId:i}=w.useContext(rn);return e=Y(e,"form-check-label"),g.jsx("label",{...l,ref:a,htmlFor:n||i,className:$(t,e)})});Xr.displayName="FormCheckLabel";const tm=w.forwardRef(({id:e,bsPrefix:t,bsSwitchPrefix:n,inline:l=!1,reverse:a=!1,disabled:i=!1,isValid:r=!1,isInvalid:u=!1,feedbackTooltip:o=!1,feedback:s,feedbackType:c,className:f,style:p,title:d="",type:h="checkbox",label:S,children:x,as:y="input",...m},v)=>{t=Y(t,"form-check"),n=Y(n,"form-switch");const{controlId:b}=w.useContext(rn),R=w.useMemo(()=>({controlId:e||b}),[b,e]),_=!x&&S!=null&&S!==!1||OS(x,Xr),E=g.jsx(Zc,{...m,type:h==="switch"?"checkbox":h,ref:v,isValid:r,isInvalid:u,disabled:i,as:y});return g.jsx(rn.Provider,{value:R,children:g.jsx("div",{style:p,className:$(f,_&&t,l&&`${t}-inline`,a&&`${t}-reverse`,h==="switch"&&n),children:x||g.jsxs(g.Fragment,{children:[E,_&&g.jsx(Xr,{title:d,children:S}),s&&g.jsx(Uu,{type:c,tooltip:o,children:s})]})})})});tm.displayName="FormCheck";const kr=Object.assign(tm,{Input:Zc,Label:Xr}),nm=w.forwardRef(({bsPrefix:e,type:t,size:n,htmlSize:l,id:a,className:i,isValid:r=!1,isInvalid:u=!1,plaintext:o,readOnly:s,as:c="input",...f},p)=>{const{controlId:d}=w.useContext(rn);return e=Y(e,"form-control"),g.jsx(c,{...f,type:t,size:l,ref:p,readOnly:s,id:a||d,className:$(i,o?`${e}-plaintext`:e,n&&`${e}-${n}`,t==="color"&&`${e}-color`,r&&"is-valid",u&&"is-invalid")})});nm.displayName="FormControl";const gw=Object.assign(nm,{Feedback:Uu}),lm=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"form-floating"),g.jsx(n,{ref:a,className:$(e,t),...l})));lm.displayName="FormFloating";const Kc=w.forwardRef(({controlId:e,as:t="div",...n},l)=>{const a=w.useMemo(()=>({controlId:e}),[e]);return g.jsx(rn.Provider,{value:a,children:g.jsx(t,{...n,ref:l})})});Kc.displayName="FormGroup";const am=w.forwardRef(({as:e="label",bsPrefix:t,column:n=!1,visuallyHidden:l=!1,className:a,htmlFor:i,...r},u)=>{const{controlId:o}=w.useContext(rn);t=Y(t,"form-label");let s="col-form-label";typeof n=="string"&&(s=`${s} ${s}-${n}`);const c=$(a,t,l&&"visually-hidden",n&&s);return i=i||o,n?g.jsx(Sh,{ref:u,as:"label",className:c,htmlFor:i,...r}):g.jsx(e,{ref:u,className:c,htmlFor:i,...r})});am.displayName="FormLabel";const im=w.forwardRef(({bsPrefix:e,className:t,id:n,...l},a)=>{const{controlId:i}=w.useContext(rn);return e=Y(e,"form-range"),g.jsx("input",{...l,type:"range",ref:a,className:$(t,e),id:n||i})});im.displayName="FormRange";const rm=w.forwardRef(({bsPrefix:e,size:t,htmlSize:n,className:l,isValid:a=!1,isInvalid:i=!1,id:r,...u},o)=>{const{controlId:s}=w.useContext(rn);return e=Y(e,"form-select"),g.jsx("select",{...u,size:n,ref:o,className:$(l,e,t&&`${e}-${t}`,a&&"is-valid",i&&"is-invalid"),id:r||s})});rm.displayName="FormSelect";const um=w.forwardRef(({bsPrefix:e,className:t,as:n="small",muted:l,...a},i)=>(e=Y(e,"form-text"),g.jsx(n,{...a,ref:i,className:$(t,e,l&&"text-muted")})));um.displayName="FormText";const om=w.forwardRef((e,t)=>g.jsx(kr,{...e,ref:t,type:"switch"}));om.displayName="Switch";const hw=Object.assign(om,{Input:kr.Input,Label:kr.Label}),sm=w.forwardRef(({bsPrefix:e,className:t,children:n,controlId:l,label:a,...i},r)=>(e=Y(e,"form-floating"),g.jsxs(Kc,{ref:r,className:$(t,e),controlId:l,...i,children:[n,g.jsx("label",{htmlFor:l,children:a})]})));sm.displayName="FloatingLabel";const mw={_ref:z.any,validated:z.bool,as:z.elementType},Jc=w.forwardRef(({className:e,validated:t,as:n="form",...l},a)=>g.jsx(n,{...l,ref:a,className:$(e,t&&"was-validated")}));Jc.displayName="Form";Jc.propTypes=mw;const Zl=Object.assign(Jc,{Group:Kc,Control:gw,Floating:lm,Check:kr,Switch:hw,Label:am,Text:um,Range:im,Select:rm,FloatingLabel:sm}),qd=e=>!e||typeof e=="function"?e:t=>{e.current=t};function yw(e,t){const n=qd(e),l=qd(t);return a=>{n&&n(a),l&&l(a)}}function Fu(e,t){return w.useMemo(()=>yw(e,t),[e,t])}const vo=(e,t)=>Uc?e==null?(t||ju()).body:(typeof e=="function"&&(e=e()),e&&"current"in e&&(e=e.current),e&&("nodeType"in e||e.getBoundingClientRect)?e:null):null;function Gd(e,t){const n=Gh(),[l,a]=w.useState(()=>vo(e,n==null?void 0:n.document));if(!l){const i=vo(e);i&&a(i)}return w.useEffect(()=>{},[t,l]),w.useEffect(()=>{const i=vo(e);i!==l&&a(i)},[e,l]),l}function vw({children:e,in:t,onExited:n,mountOnEnter:l,unmountOnExit:a}){const i=w.useRef(null),r=w.useRef(t),u=Rt(n);w.useEffect(()=>{t?r.current=!0:u(i.current)},[t,u]);const o=Fu(i,e.ref),s=w.cloneElement(e,{ref:o});return t?s:a||!r.current&&l?null:s}const Sw=["onEnter","onEntering","onEntered","onExit","onExiting","onExited","addEndListener","children"];function xw(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}function ww(e){let{onEnter:t,onEntering:n,onEntered:l,onExit:a,onExiting:i,onExited:r,addEndListener:u,children:o}=e,s=xw(e,Sw);const c=w.useRef(null),f=Fu(c,zc(o)),p=b=>R=>{b&&c.current&&b(c.current,R)},d=w.useCallback(p(t),[t]),h=w.useCallback(p(n),[n]),S=w.useCallback(p(l),[l]),x=w.useCallback(p(a),[a]),y=w.useCallback(p(i),[i]),m=w.useCallback(p(r),[r]),v=w.useCallback(p(u),[u]);return Object.assign({},s,{nodeRef:c},t&&{onEnter:d},n&&{onEntering:h},l&&{onEntered:S},a&&{onExit:x},i&&{onExiting:y},r&&{onExited:m},u&&{addEndListener:v},{children:typeof o=="function"?(b,R)=>o(b,Object.assign({},R,{ref:f})):w.cloneElement(o,{ref:f})})}const bw=["component"];function Rw(e,t){if(e==null)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}const Ew=w.forwardRef((e,t)=>{let{component:n}=e,l=Rw(e,bw);const a=ww(l);return g.jsx(n,Object.assign({ref:t},a))});function Cw({in:e,onTransition:t}){const n=w.useRef(null),l=w.useRef(!0),a=Rt(t);return Ed(()=>{if(!n.current)return;let i=!1;return a({in:e,element:n.current,initial:l.current,isStale:()=>i}),()=>{i=!0}},[e,a]),Ed(()=>(l.current=!1,()=>{l.current=!0}),[]),n}function _w({children:e,in:t,onExited:n,onEntered:l,transition:a}){const[i,r]=w.useState(!t);t&&i&&r(!1);const u=Cw({in:!!t,onTransition:s=>{const c=()=>{s.isStale()||(s.in?l==null||l(s.element,s.initial):(r(!0),n==null||n(s.element)))};Promise.resolve(a(s)).then(c,f=>{throw s.in||r(!0),f})}}),o=Fu(u,e.ref);return i&&!t?null:w.cloneElement(e,{ref:o})}function Aw(e,t,n){return e?g.jsx(Ew,Object.assign({},n,{component:e})):t?g.jsx(_w,Object.assign({},n,{transition:t})):g.jsx(vw,Object.assign({},n))}function Yd(e,t){return e.classList?!!t&&e.classList.contains(t):(" "+(e.className.baseVal||e.className)+" ").indexOf(" "+t+" ")!==-1}const Ow=()=>{};function Tw(e,t,{disabled:n,clickTrigger:l}={}){const a=t||Ow;Dh(e,a,{disabled:n,clickTrigger:l});const i=Rt(r=>{J1(r)&&a(r)});w.useEffect(()=>{if(n||e==null)return;const r=ju(Ar(e));let u=(r.defaultView||window).event;const o=Wn(r,"keyup",s=>{if(s===u){u=void 0;return}i(s)});return()=>{o()}},[e,n,i])}const cm=w.forwardRef((e,t)=>{const{flip:n,offset:l,placement:a,containerPadding:i,popperConfig:r={},transition:u,runTransition:o}=e,[s,c]=ms(),[f,p]=ms(),d=Fu(c,t),h=Gd(e.container),S=Gd(e.target),[x,y]=w.useState(!e.show),m=Th(S,s,Mh({placement:a,enableEvents:!!e.show,containerPadding:i||5,flip:n,offset:l,arrowElement:f,popperConfig:r}));e.show&&x&&y(!1);const v=(...j)=>{y(!0),e.onExited&&e.onExited(...j)},b=e.show||!x;if(Tw(s,e.onHide,{disabled:!e.rootClose||e.rootCloseDisabled,clickTrigger:e.rootCloseEvent}),!b)return null;const{onExit:R,onExiting:_,onEnter:E,onEntering:A,onEntered:N}=e;let M=e.children(Object.assign({},m.attributes.popper,{style:m.styles.popper,ref:d}),{popper:m,placement:a,show:!!e.show,arrowProps:Object.assign({},m.attributes.arrow,{style:m.styles.arrow,ref:p})});return M=Aw(u,o,{in:!!e.show,appear:!0,mountOnEnter:!0,unmountOnExit:!0,children:M,onExit:R,onExiting:_,onExited:v,onEnter:E,onEntering:A,onEntered:N}),h?Nl.createPortal(M,h):null});cm.displayName="Overlay";const fm=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"popover-header"),g.jsx(n,{ref:a,className:$(e,t),...l})));fm.displayName="PopoverHeader";const Pc=w.forwardRef(({className:e,bsPrefix:t,as:n="div",...l},a)=>(t=Y(t,"popover-body"),g.jsx(n,{ref:a,className:$(e,t),...l})));Pc.displayName="PopoverBody";function dm(e,t){let n=e;return e==="left"?n=t?"end":"start":e==="right"&&(n=t?"start":"end"),n}function pm(e="absolute"){return{position:e,top:"0",left:"0",opacity:"0",pointerEvents:"none"}}const Dw=w.forwardRef(({bsPrefix:e,placement:t="right",className:n,style:l,children:a,body:i,arrowProps:r,hasDoneInitialMeasure:u,popper:o,show:s,...c},f)=>{const p=Y(e,"popover"),d=Nc(),[h]=(t==null?void 0:t.split("-"))||[],S=dm(h,d);let x=l;return s&&!u&&(x={...l,...pm(o==null?void 0:o.strategy)}),g.jsxs("div",{ref:f,role:"tooltip",style:x,"x-placement":h,className:$(n,p,h&&`bs-popover-${S}`),...c,children:[g.jsx("div",{className:"popover-arrow",...r}),i?g.jsx(Pc,{children:a}):a]})}),Ic=Object.assign(Dw,{Header:fm,Body:Pc,POPPER_OFFSET:[0,8]}),gm=w.forwardRef(({bsPrefix:e,placement:t="right",className:n,style:l,children:a,arrowProps:i,hasDoneInitialMeasure:r,popper:u,show:o,...s},c)=>{e=Y(e,"tooltip");const f=Nc(),[p]=(t==null?void 0:t.split("-"))||[],d=dm(p,f);let h=l;return o&&!r&&(h={...l,...pm(u==null?void 0:u.strategy)}),g.jsxs("div",{ref:c,style:h,role:"tooltip","x-placement":p,className:$(n,e,`bs-tooltip-${d}`),...s,children:[g.jsx("div",{className:"tooltip-arrow",...i}),g.jsx("div",{className:`${e}-inner`,children:a})]})});gm.displayName="Tooltip";const hm=Object.assign(gm,{TOOLTIP_OFFSET:[0,6]});function Mw(e){const t=w.useRef(null),n=Y(void 0,"popover"),l=Y(void 0,"tooltip"),a=w.useMemo(()=>({name:"offset",options:{offset:()=>{if(e)return e;if(t.current){if(Yd(t.current,n))return Ic.POPPER_OFFSET;if(Yd(t.current,l))return hm.TOOLTIP_OFFSET}return[0,0]}}}),[e,n,l]);return[t,[a]]}function jw(e,t){const{ref:n}=e,{ref:l}=t;e.ref=n.__wrapped||(n.__wrapped=a=>n(Gr(a))),t.ref=l.__wrapped||(l.__wrapped=a=>l(Gr(a)))}const mm=w.forwardRef(({children:e,transition:t=ys,popperConfig:n={},rootClose:l=!1,placement:a="top",show:i=!1,...r},u)=>{const o=w.useRef({}),[s,c]=w.useState(null),[f,p]=Mw(r.offset),d=Ui(u,f),h=t===!0?ys:t||void 0,S=lh(x=>{c(x),n==null||n.onFirstUpdate==null||n.onFirstUpdate(x)});return Zh(()=>{s&&r.target&&(o.current.scheduleUpdate==null||o.current.scheduleUpdate())},[s,r.target]),w.useEffect(()=>{i||c(null)},[i]),g.jsx(cm,{...r,ref:d,popperConfig:{...n,modifiers:p.concat(n.modifiers||[]),onFirstUpdate:S},transition:h,rootClose:l,placement:a,show:i,children:(x,{arrowProps:y,popper:m,show:v})=>{var b;jw(x,y);const R=m==null?void 0:m.placement,_=Object.assign(o.current,{state:m==null?void 0:m.state,scheduleUpdate:m==null?void 0:m.update,placement:R,outOfBoundaries:(m==null||(b=m.state)==null||(b=b.modifiersData.hide)==null?void 0:b.isReferenceHidden)||!1,strategy:n.strategy}),E=!!s;return typeof e=="function"?e({...x,placement:R,show:v,...!t&&v&&{className:"show"},popper:_,arrowProps:y,hasDoneInitialMeasure:E}):w.cloneElement(e,{...x,placement:R,arrowProps:y,popper:_,hasDoneInitialMeasure:E,className:$(e.props.className,!t&&v&&"show"),style:{...e.props.style,...x.style}})}})});mm.displayName="Overlay";function Nw(e){return e&&typeof e=="object"?e:{show:e,hide:e}}function Xd(e,t,n){const[l]=t,a=l.currentTarget,i=l.relatedTarget||l.nativeEvent[n];(!i||i!==a)&&!ws(a,i)&&e(...t)}z.oneOf(["click","hover","focus"]);const mi=({trigger:e=["hover","focus"],overlay:t,children:n,popperConfig:l={},show:a,defaultShow:i=!1,onToggle:r,delay:u,placement:o,flip:s=o&&o.indexOf("auto")!==-1,...c})=>{const f=w.useRef(null),p=Ui(f,zc(n)),d=_S(),h=w.useRef(""),[S,x]=Jg(a,i,r),y=Nw(u),{onFocus:m,onBlur:v,onClick:b}=typeof n!="function"?w.Children.only(n).props:{},R=P=>{p(Gr(P))},_=w.useCallback(()=>{if(d.clear(),h.current="show",!y.show){x(!0);return}d.set(()=>{h.current==="show"&&x(!0)},y.show)},[y.show,x,d]),E=w.useCallback(()=>{if(d.clear(),h.current="hide",!y.hide){x(!1);return}d.set(()=>{h.current==="hide"&&x(!1)},y.hide)},[y.hide,x,d]),A=w.useCallback((...P)=>{_(),m==null||m(...P)},[_,m]),N=w.useCallback((...P)=>{E(),v==null||v(...P)},[E,v]),M=w.useCallback((...P)=>{x(!S),b==null||b(...P)},[b,x,S]),j=w.useCallback((...P)=>{Xd(_,P,"fromElement")},[_]),V=w.useCallback((...P)=>{Xd(E,P,"toElement")},[E]),H=e==null?[]:[].concat(e),K={ref:R};return H.indexOf("click")!==-1&&(K.onClick=M),H.indexOf("focus")!==-1&&(K.onFocus=A,K.onBlur=N),H.indexOf("hover")!==-1&&(K.onMouseOver=j,K.onMouseOut=V),g.jsxs(g.Fragment,{children:[typeof n=="function"?n(K):w.cloneElement(n,K),g.jsx(mm,{...c,show:S,onHide:E,flip:s,placement:o,popperConfig:l,target:f.current,children:t})]})},kd=1e3;function zw(e,t,n){const l=(e-t)/(n-t)*100;return Math.round(l*kd)/kd}function Qd({min:e,now:t,max:n,label:l,visuallyHidden:a,striped:i,animated:r,className:u,style:o,variant:s,bsPrefix:c,...f},p){return g.jsx("div",{ref:p,...f,role:"progressbar",className:$(u,`${c}-bar`,{[`bg-${s}`]:s,[`${c}-bar-animated`]:r,[`${c}-bar-striped`]:r||i}),style:{width:`${zw(t,e,n)}%`,...o},"aria-valuenow":t,"aria-valuemin":e,"aria-valuemax":n,children:a?g.jsx("span",{className:"visually-hidden",children:l}):l})}const Wc=w.forwardRef(({isChild:e=!1,...t},n)=>{const l={min:0,max:100,animated:!1,visuallyHidden:!1,striped:!1,...t};if(l.bsPrefix=Y(l.bsPrefix,"progress"),e)return Qd(l,n);const{min:a,now:i,max:r,label:u,visuallyHidden:o,striped:s,animated:c,bsPrefix:f,variant:p,className:d,children:h,...S}=l;return g.jsx("div",{ref:n,...S,className:$(d,f),children:h?AS(h,x=>w.cloneElement(x,{isChild:!0})):Qd({min:a,now:i,max:r,label:u,visuallyHidden:o,striped:s,animated:c,bsPrefix:f,variant:p},n)})});Wc.displayName="ProgressBar";const Uw={id:z.string,toggleLabel:z.string,href:z.string,target:z.string,onClick:z.func,title:z.node.isRequired,type:z.string,disabled:z.bool,align:dw,menuRole:z.string,renderMenuOnMount:z.bool,rootCloseEvent:z.string,flip:z.bool,bsPrefix:z.string,variant:z.string,size:z.string},ef=w.forwardRef(({id:e,bsPrefix:t,size:n,variant:l,title:a,type:i="button",toggleLabel:r="Toggle dropdown",children:u,onClick:o,href:s,target:c,menuRole:f,renderMenuOnMount:p,rootCloseEvent:d,flip:h,...S},x)=>g.jsxs(bn,{ref:x,...S,as:Za,children:[g.jsx(He,{size:n,variant:l,disabled:S.disabled,bsPrefix:t,href:s,target:c,onClick:o,type:i,children:a}),g.jsx(bn.Toggle,{split:!0,id:e,size:n,variant:l,disabled:S.disabled,childBsPrefix:t,children:g.jsx("span",{className:"visually-hidden",children:r})}),g.jsx(bn.Menu,{role:f,renderOnMount:p,rootCloseEvent:d,flip:h,children:u})]}));ef.propTypes=Uw;ef.displayName="SplitButton";const Fw=w.forwardRef(({bsPrefix:e,className:t,striped:n,bordered:l,borderless:a,hover:i,size:r,variant:u,responsive:o,...s},c)=>{const f=Y(e,"table"),p=$(t,f,u&&`${f}-${u}`,r&&`${f}-${r}`,n&&`${f}-${typeof n=="string"?`striped-${n}`:"striped"}`,l&&`${f}-bordered`,a&&`${f}-borderless`,i&&`${f}-hover`),d=g.jsx("table",{...s,className:p,ref:c});if(o){let h=`${f}-responsive`;return typeof o=="string"&&(h=`${h}-${o}`),g.jsx("div",{className:h,children:d})}return d}),Bw=RegExp.prototype.test.bind(/(<([^>]+)>)/i),Hw=({column:e})=>{const n=e.getFilterValue(),l=g.jsx(Ic,{id:"popover-positioned-top",children:g.jsxs("div",{className:"p-3",children:[g.jsx("input",{type:"number",value:(n==null?void 0:n[0])??"",onChange:a=>e.setFilterValue(i=>[a.target.value,i==null?void 0:i[1]]),placeholder:"Min",className:"form-control"}),g.jsx("p",{className:"text-center",children:"to"}),g.jsx("input",{type:"number",value:(n==null?void 0:n[1])??"",onChange:a=>e.setFilterValue(i=>[i==null?void 0:i[0],a.target.value]),placeholder:"Max",className:"form-control"}),g.jsx(He,{variant:"secondary",size:"sm",className:"w-100 mt-2",onClick:()=>document.body.click(),children:"Close"})]})});return g.jsx(mi,{trigger:"click",placement:"bottom",rootClose:!0,overlay:l,children:g.jsx("form",{className:de.searchWrapperFrom,onReset:()=>{e.setFilterValue(()=>[void 0,void 0])},children:g.jsxs("div",{className:de.searchWrapper,children:[g.jsx(Zl.Control,{className:de.searchInput,readOnly:!0,type:"text",placeholder:"Set Range",value:typeof(n==null?void 0:n[0])<"u"||typeof(n==null?void 0:n[1])<"u"?`${typeof(n==null?void 0:n[0])>"u"||(n==null?void 0:n[0])===""?"-∞":n==null?void 0:n[0].toLocaleString()} to ${typeof(n==null?void 0:n[1])>"u"||(n==null?void 0:n[1])===""?"∞":n==null?void 0:n[1].toLocaleString()}`:void 0}),g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 512 512",className:`${de.searchIcon} ${de.dropDownIcon}`,children:g.jsx("path",{fill:"currentColor",d:"M233.4 406.6c12.5 12.5 32.8 12.5 45.3 0l192-192c12.5-12.5 12.5-32.8 0-45.3s-32.8-12.5-45.3 0L256 338.7 86.6 169.4c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3l192 192z"})}),g.jsx("button",{type:"reset",children:g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.xIcon,children:g.jsx("path",{fill:"currentColor",d:"M17.3 18.7a1 1 0 0 0 1.4-1.4L13.42 12l5.3-5.3a1 1 0 0 0-1.42-1.4L12 10.58l-5.3-5.3a1 1 0 0 0-1.4 1.42L10.58 12l-5.3 5.3a1 1 0 1 0 1.42 1.4L12 13.42l5.3 5.3Z"})})})]})})})},Lw=({column:e})=>{const t=e.getFilterValue(),n=g.jsx(Ic,{id:"popover-positioned-top",children:g.jsxs("div",{className:`${de.radioWrapper} p-2`,children:[g.jsx(Zl.Check,{label:"True",name:"group1",type:"radio",id:"radio-true",onChange:()=>{e.setFilterValue(!0)}}),g.jsx(Zl.Check,{label:"False",name:"group1",type:"radio",id:"radio-false",onChange:()=>{e.setFilterValue(!1)}}),g.jsx(He,{variant:"secondary",size:"sm",className:"w-100",onClick:()=>document.body.click(),children:"Close"})]})});return g.jsx(mi,{trigger:"click",placement:"bottom",rootClose:!0,overlay:n,children:g.jsx("form",{className:de.searchWrapperFrom,onReset:()=>{e.setFilterValue(()=>{})},children:g.jsxs("div",{className:de.searchWrapper,children:[g.jsx(Zl.Control,{className:de.searchInput,readOnly:!0,type:"text",placeholder:"Filter",value:typeof t>"u"?void 0:t?"True":"False"}),g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 512 512",className:`${de.searchIcon} ${de.dropDownIcon}`,children:g.jsx("path",{fill:"currentColor",d:"M233.4 406.6c12.5 12.5 32.8 12.5 45.3 0l192-192c12.5-12.5 12.5-32.8 0-45.3s-32.8-12.5-45.3 0L256 338.7 86.6 169.4c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3l192 192z"})}),g.jsx("button",{type:"reset",children:g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.xIcon,children:g.jsx("path",{fill:"currentColor",d:"M17.3 18.7a1 1 0 0 0 1.4-1.4L13.42 12l5.3-5.3a1 1 0 0 0-1.42-1.4L12 10.58l-5.3-5.3a1 1 0 0 0-1.4 1.42L10.58 12l-5.3 5.3a1 1 0 1 0 1.42 1.4L12 13.42l5.3 5.3Z"})})})]})})})},Vw=({column:e})=>g.jsx("form",{onReset:()=>{e.setFilterValue(null)},children:g.jsxs("div",{className:de.searchWrapper,children:[g.jsx(Zl.Control,{className:de.searchInput,type:"text",placeholder:"Search",onChange:t=>{e.setFilterValue(t.target.value?t.target.value:"")}}),g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.searchIcon,children:g.jsx("path",{fill:"currentColor","fill-rule":"evenodd",d:"M15.62 17.03a9 9 0 1 1 1.41-1.41l4.68 4.67a1 1 0 0 1-1.42 1.42l-4.67-4.68ZM17 10a7 7 0 1 1-14 0 7 7 0 0 1 14 0Z","clip-rule":"evenodd"})}),g.jsx("button",{type:"reset",children:g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.xIcon,children:g.jsx("path",{fill:"currentColor",d:"M17.3 18.7a1 1 0 0 0 1.4-1.4L13.42 12l5.3-5.3a1 1 0 0 0-1.42-1.4L12 10.58l-5.3-5.3a1 1 0 0 0-1.4 1.42L10.58 12l-5.3 5.3a1 1 0 1 0 1.42 1.4L12 13.42l5.3 5.3Z"})})})]})}),$w=({column:e})=>{const t=Array.from(e.getFacetedUniqueValues().keys()).sort(),n=e.getFilterValue(),l=Bw(t==null?void 0:t[0])||typeof(t==null?void 0:t[0])=="object",a=t.reduce((i,r)=>(typeof r<"u"&&(l||(n===void 0||r!=null&&r.toLowerCase().includes(n==null?void 0:n.toLowerCase()))&&i.push({value:r,label:r})),i),[]).slice(0,10);return g.jsx(mi,{trigger:"click",placement:"bottom-start",rootClose:!0,overlay:g.jsx(bn,{show:!0,drop:"down-centered",children:g.jsx(bn.Menu,{children:g.jsx(g.Fragment,{children:a.length>0?a.map(i=>g.jsx(bn.Item,{eventKey:i.value,onClick:()=>{e.setFilterValue(i.value?i.value:""),document.body.click()},children:i.label})):g.jsx(bn.Item,{disabled:!0,children:"Start typing to search."})})})}),children:g.jsx("form",{className:de.searchWrapperFrom,onReset:()=>{e.setFilterValue(()=>{})},children:g.jsxs("div",{className:de.searchWrapper,children:[g.jsx(Zl.Control,{className:de.searchInput,type:"text",placeholder:"Search",value:typeof n>"u"?void 0:n,onChange:i=>{e.setFilterValue(i.target.value?i.target.value:"")}}),g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.searchIcon,children:g.jsx("path",{fill:"currentColor","fill-rule":"evenodd",d:"M15.62 17.03a9 9 0 1 1 1.41-1.41l4.68 4.67a1 1 0 0 1-1.42 1.42l-4.67-4.68ZM17 10a7 7 0 1 1-14 0 7 7 0 0 1 14 0Z","clip-rule":"evenodd"})}),g.jsx("button",{type:"reset",children:g.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",className:de.xIcon,children:g.jsx("path",{fill:"currentColor",d:"M17.3 18.7a1 1 0 0 0 1.4-1.4L13.42 12l5.3-5.3a1 1 0 0 0-1.42-1.4L12 10.58l-5.3-5.3a1 1 0 0 0-1.4 1.42L10.58 12l-5.3 5.3a1 1 0 1 0 1.42 1.4L12 13.42l5.3 5.3Z"})})})]})})})},qw=({column:e,table:t})=>{var l;const n=(l=t.getPreFilteredRowModel().flatRows[0])==null?void 0:l.getValue(e.id);return typeof n=="number"?g.jsx(Hw,{column:e}):typeof n=="boolean"?g.jsx(Lw,{column:e}):typeof n=="object"?g.jsx(Vw,{column:e}):g.jsx($w,{column:e})};/**
   * table-core
   *
   * Copyright (c) TanStack
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE.md file in the root directory of this source tree.
   *
   * @license MIT
   */function Bu(){return{accessor:(e,t)=>typeof e=="function"?{...t,accessorFn:e}:{...t,accessorKey:e},display:e=>e,group:e=>e}}function Rn(e,t){return typeof e=="function"?e(t):e}function lt(e,t){return n=>{t.setState(l=>({...l,[e]:Rn(n,l[e])}))}}function Hu(e){return e instanceof Function}function Gw(e){return Array.isArray(e)&&e.every(t=>typeof t=="number")}function Yw(e,t){const n=[],l=a=>{a.forEach(i=>{n.push(i);const r=t(i);r!=null&&r.length&&l(r)})};return l(e),n}function F(e,t,n){let l=[],a;return i=>{let r;n.key&&n.debug&&(r=Date.now());const u=e(i);if(!(u.length!==l.length||u.some((c,f)=>l[f]!==c)))return a;l=u;let s;if(n.key&&n.debug&&(s=Date.now()),a=t(...u),n==null||n.onChange==null||n.onChange(a),n.key&&n.debug&&n!=null&&n.debug()){const c=Math.round((Date.now()-r)*100)/100,f=Math.round((Date.now()-s)*100)/100,p=f/16,d=(h,S)=>{for(h=String(h);h.length<S;)h=" "+h;return h};console.info(`%c⏱ ${d(f,5)} /${d(c,5)} ms`,`
            font-size: .6rem;
            font-weight: bold;
            color: hsl(${Math.max(0,Math.min(120-120*p,120))}deg 100% 31%);`,n==null?void 0:n.key)}return a}}function B(e,t,n,l){return{debug:()=>{var a;return(a=e==null?void 0:e.debugAll)!=null?a:e[t]},key:!1,onChange:l}}function Xw(e,t,n,l){const a=()=>{var r;return(r=i.getValue())!=null?r:e.options.renderFallbackValue},i={id:`${t.id}_${n.id}`,row:t,column:n,getValue:()=>t.getValue(l),renderValue:a,getContext:F(()=>[e,n,t,i],(r,u,o,s)=>({table:r,column:u,row:o,cell:s,getValue:s.getValue,renderValue:s.renderValue}),B(e.options,"debugCells"))};return e._features.forEach(r=>{r.createCell==null||r.createCell(i,n,t,e)},{}),i}function kw(e,t,n,l){var a,i;const u={...e._getDefaultColumnDef(),...t},o=u.accessorKey;let s=(a=(i=u.id)!=null?i:o?typeof String.prototype.replaceAll=="function"?o.replaceAll(".","_"):o.replace(/\./g,"_"):void 0)!=null?a:typeof u.header=="string"?u.header:void 0,c;if(u.accessorFn?c=u.accessorFn:o&&(o.includes(".")?c=p=>{let d=p;for(const S of o.split(".")){var h;d=(h=d)==null?void 0:h[S]}return d}:c=p=>p[u.accessorKey]),!s)throw new Error;let f={id:`${String(s)}`,accessorFn:c,parent:l,depth:n,columnDef:u,columns:[],getFlatColumns:F(()=>[!0],()=>{var p;return[f,...(p=f.columns)==null?void 0:p.flatMap(d=>d.getFlatColumns())]},B(e.options,"debugColumns")),getLeafColumns:F(()=>[e._getOrderColumnsFn()],p=>{var d;if((d=f.columns)!=null&&d.length){let h=f.columns.flatMap(S=>S.getLeafColumns());return p(h)}return[f]},B(e.options,"debugColumns"))};for(const p of e._features)p.createColumn==null||p.createColumn(f,e);return f}const De="debugHeaders";function Zd(e,t,n){var l;let i={id:(l=n.id)!=null?l:t.id,column:t,index:n.index,isPlaceholder:!!n.isPlaceholder,placeholderId:n.placeholderId,depth:n.depth,subHeaders:[],colSpan:0,rowSpan:0,headerGroup:null,getLeafHeaders:()=>{const r=[],u=o=>{o.subHeaders&&o.subHeaders.length&&o.subHeaders.map(u),r.push(o)};return u(i),r},getContext:()=>({table:e,header:i,column:t})};return e._features.forEach(r=>{r.createHeader==null||r.createHeader(i,e)}),i}const Qw={createTable:e=>{e.getHeaderGroups=F(()=>[e.getAllColumns(),e.getVisibleLeafColumns(),e.getState().columnPinning.left,e.getState().columnPinning.right],(t,n,l,a)=>{var i,r;const u=(i=l==null?void 0:l.map(f=>n.find(p=>p.id===f)).filter(Boolean))!=null?i:[],o=(r=a==null?void 0:a.map(f=>n.find(p=>p.id===f)).filter(Boolean))!=null?r:[],s=n.filter(f=>!(l!=null&&l.includes(f.id))&&!(a!=null&&a.includes(f.id)));return ir(t,[...u,...s,...o],e)},B(e.options,De)),e.getCenterHeaderGroups=F(()=>[e.getAllColumns(),e.getVisibleLeafColumns(),e.getState().columnPinning.left,e.getState().columnPinning.right],(t,n,l,a)=>(n=n.filter(i=>!(l!=null&&l.includes(i.id))&&!(a!=null&&a.includes(i.id))),ir(t,n,e,"center")),B(e.options,De)),e.getLeftHeaderGroups=F(()=>[e.getAllColumns(),e.getVisibleLeafColumns(),e.getState().columnPinning.left],(t,n,l)=>{var a;const i=(a=l==null?void 0:l.map(r=>n.find(u=>u.id===r)).filter(Boolean))!=null?a:[];return ir(t,i,e,"left")},B(e.options,De)),e.getRightHeaderGroups=F(()=>[e.getAllColumns(),e.getVisibleLeafColumns(),e.getState().columnPinning.right],(t,n,l)=>{var a;const i=(a=l==null?void 0:l.map(r=>n.find(u=>u.id===r)).filter(Boolean))!=null?a:[];return ir(t,i,e,"right")},B(e.options,De)),e.getFooterGroups=F(()=>[e.getHeaderGroups()],t=>[...t].reverse(),B(e.options,De)),e.getLeftFooterGroups=F(()=>[e.getLeftHeaderGroups()],t=>[...t].reverse(),B(e.options,De)),e.getCenterFooterGroups=F(()=>[e.getCenterHeaderGroups()],t=>[...t].reverse(),B(e.options,De)),e.getRightFooterGroups=F(()=>[e.getRightHeaderGroups()],t=>[...t].reverse(),B(e.options,De)),e.getFlatHeaders=F(()=>[e.getHeaderGroups()],t=>t.map(n=>n.headers).flat(),B(e.options,De)),e.getLeftFlatHeaders=F(()=>[e.getLeftHeaderGroups()],t=>t.map(n=>n.headers).flat(),B(e.options,De)),e.getCenterFlatHeaders=F(()=>[e.getCenterHeaderGroups()],t=>t.map(n=>n.headers).flat(),B(e.options,De)),e.getRightFlatHeaders=F(()=>[e.getRightHeaderGroups()],t=>t.map(n=>n.headers).flat(),B(e.options,De)),e.getCenterLeafHeaders=F(()=>[e.getCenterFlatHeaders()],t=>t.filter(n=>{var l;return!((l=n.subHeaders)!=null&&l.length)}),B(e.options,De)),e.getLeftLeafHeaders=F(()=>[e.getLeftFlatHeaders()],t=>t.filter(n=>{var l;return!((l=n.subHeaders)!=null&&l.length)}),B(e.options,De)),e.getRightLeafHeaders=F(()=>[e.getRightFlatHeaders()],t=>t.filter(n=>{var l;return!((l=n.subHeaders)!=null&&l.length)}),B(e.options,De)),e.getLeafHeaders=F(()=>[e.getLeftHeaderGroups(),e.getCenterHeaderGroups(),e.getRightHeaderGroups()],(t,n,l)=>{var a,i,r,u,o,s;return[...(a=(i=t[0])==null?void 0:i.headers)!=null?a:[],...(r=(u=n[0])==null?void 0:u.headers)!=null?r:[],...(o=(s=l[0])==null?void 0:s.headers)!=null?o:[]].map(c=>c.getLeafHeaders()).flat()},B(e.options,De))}};function ir(e,t,n,l){var a,i;let r=0;const u=function(p,d){d===void 0&&(d=1),r=Math.max(r,d),p.filter(h=>h.getIsVisible()).forEach(h=>{var S;(S=h.columns)!=null&&S.length&&u(h.columns,d+1)},0)};u(e);let o=[];const s=(p,d)=>{const h={depth:d,id:[l,`${d}`].filter(Boolean).join("_"),headers:[]},S=[];p.forEach(x=>{const y=[...S].reverse()[0],m=x.column.depth===h.depth;let v,b=!1;if(m&&x.column.parent?v=x.column.parent:(v=x.column,b=!0),y&&(y==null?void 0:y.column)===v)y.subHeaders.push(x);else{const R=Zd(n,v,{id:[l,d,v.id,x==null?void 0:x.id].filter(Boolean).join("_"),isPlaceholder:b,placeholderId:b?`${S.filter(_=>_.column===v).length}`:void 0,depth:d,index:S.length});R.subHeaders.push(x),S.push(R)}h.headers.push(x),x.headerGroup=h}),o.push(h),d>0&&s(S,d-1)},c=t.map((p,d)=>Zd(n,p,{depth:r,index:d}));s(c,r-1),o.reverse();const f=p=>p.filter(h=>h.column.getIsVisible()).map(h=>{let S=0,x=0,y=[0];h.subHeaders&&h.subHeaders.length?(y=[],f(h.subHeaders).forEach(v=>{let{colSpan:b,rowSpan:R}=v;S+=b,y.push(R)})):S=1;const m=Math.min(...y);return x=x+m,h.colSpan=S,h.rowSpan=x,{colSpan:S,rowSpan:x}});return f((a=(i=o[0])==null?void 0:i.headers)!=null?a:[]),o}const tf=(e,t,n,l,a,i,r)=>{let u={id:t,index:l,original:n,depth:a,parentId:r,_valuesCache:{},_uniqueValuesCache:{},getValue:o=>{if(u._valuesCache.hasOwnProperty(o))return u._valuesCache[o];const s=e.getColumn(o);if(s!=null&&s.accessorFn)return u._valuesCache[o]=s.accessorFn(u.original,l),u._valuesCache[o]},getUniqueValues:o=>{if(u._uniqueValuesCache.hasOwnProperty(o))return u._uniqueValuesCache[o];const s=e.getColumn(o);if(s!=null&&s.accessorFn)return s.columnDef.getUniqueValues?(u._uniqueValuesCache[o]=s.columnDef.getUniqueValues(u.original,l),u._uniqueValuesCache[o]):(u._uniqueValuesCache[o]=[u.getValue(o)],u._uniqueValuesCache[o])},renderValue:o=>{var s;return(s=u.getValue(o))!=null?s:e.options.renderFallbackValue},subRows:[],getLeafRows:()=>Yw(u.subRows,o=>o.subRows),getParentRow:()=>u.parentId?e.getRow(u.parentId,!0):void 0,getParentRows:()=>{let o=[],s=u;for(;;){const c=s.getParentRow();if(!c)break;o.push(c),s=c}return o.reverse()},getAllCells:F(()=>[e.getAllLeafColumns()],o=>o.map(s=>Xw(e,u,s,s.id)),B(e.options,"debugRows")),_getAllCellsByColumnId:F(()=>[u.getAllCells()],o=>o.reduce((s,c)=>(s[c.column.id]=c,s),{}),B(e.options,"debugRows"))};for(let o=0;o<e._features.length;o++){const s=e._features[o];s==null||s.createRow==null||s.createRow(u,e)}return u},Zw={createColumn:(e,t)=>{e._getFacetedRowModel=t.options.getFacetedRowModel&&t.options.getFacetedRowModel(t,e.id),e.getFacetedRowModel=()=>e._getFacetedRowModel?e._getFacetedRowModel():t.getPreFilteredRowModel(),e._getFacetedUniqueValues=t.options.getFacetedUniqueValues&&t.options.getFacetedUniqueValues(t,e.id),e.getFacetedUniqueValues=()=>e._getFacetedUniqueValues?e._getFacetedUniqueValues():new Map,e._getFacetedMinMaxValues=t.options.getFacetedMinMaxValues&&t.options.getFacetedMinMaxValues(t,e.id),e.getFacetedMinMaxValues=()=>{if(e._getFacetedMinMaxValues)return e._getFacetedMinMaxValues()}}},ym=(e,t,n)=>{var l,a;const i=n==null||(l=n.toString())==null?void 0:l.toLowerCase();return!!(!((a=e.getValue(t))==null||(a=a.toString())==null||(a=a.toLowerCase())==null)&&a.includes(i))};ym.autoRemove=e=>Et(e);const vm=(e,t,n)=>{var l;return!!(!((l=e.getValue(t))==null||(l=l.toString())==null)&&l.includes(n))};vm.autoRemove=e=>Et(e);const Sm=(e,t,n)=>{var l;return((l=e.getValue(t))==null||(l=l.toString())==null?void 0:l.toLowerCase())===(n==null?void 0:n.toLowerCase())};Sm.autoRemove=e=>Et(e);const xm=(e,t,n)=>{var l;return(l=e.getValue(t))==null?void 0:l.includes(n)};xm.autoRemove=e=>Et(e)||!(e!=null&&e.length);const wm=(e,t,n)=>!n.some(l=>{var a;return!((a=e.getValue(t))!=null&&a.includes(l))});wm.autoRemove=e=>Et(e)||!(e!=null&&e.length);const bm=(e,t,n)=>n.some(l=>{var a;return(a=e.getValue(t))==null?void 0:a.includes(l)});bm.autoRemove=e=>Et(e)||!(e!=null&&e.length);const Rm=(e,t,n)=>e.getValue(t)===n;Rm.autoRemove=e=>Et(e);const Em=(e,t,n)=>e.getValue(t)==n;Em.autoRemove=e=>Et(e);const nf=(e,t,n)=>{let[l,a]=n;const i=e.getValue(t);return i>=l&&i<=a};nf.resolveFilterValue=e=>{let[t,n]=e,l=typeof t!="number"?parseFloat(t):t,a=typeof n!="number"?parseFloat(n):n,i=t===null||Number.isNaN(l)?-1/0:l,r=n===null||Number.isNaN(a)?1/0:a;if(i>r){const u=i;i=r,r=u}return[i,r]};nf.autoRemove=e=>Et(e)||Et(e[0])&&Et(e[1]);const kt={includesString:ym,includesStringSensitive:vm,equalsString:Sm,arrIncludes:xm,arrIncludesAll:wm,arrIncludesSome:bm,equals:Rm,weakEquals:Em,inNumberRange:nf};function Et(e){return e==null||e===""}const Kw={getDefaultColumnDef:()=>({filterFn:"auto"}),getInitialState:e=>({columnFilters:[],...e}),getDefaultOptions:e=>({onColumnFiltersChange:lt("columnFilters",e),filterFromLeafRows:!1,maxLeafRowFilterDepth:100}),createColumn:(e,t)=>{e.getAutoFilterFn=()=>{const n=t.getCoreRowModel().flatRows[0],l=n==null?void 0:n.getValue(e.id);return typeof l=="string"?kt.includesString:typeof l=="number"?kt.inNumberRange:typeof l=="boolean"||l!==null&&typeof l=="object"?kt.equals:Array.isArray(l)?kt.arrIncludes:kt.weakEquals},e.getFilterFn=()=>{var n,l;return Hu(e.columnDef.filterFn)?e.columnDef.filterFn:e.columnDef.filterFn==="auto"?e.getAutoFilterFn():(n=(l=t.options.filterFns)==null?void 0:l[e.columnDef.filterFn])!=null?n:kt[e.columnDef.filterFn]},e.getCanFilter=()=>{var n,l,a;return((n=e.columnDef.enableColumnFilter)!=null?n:!0)&&((l=t.options.enableColumnFilters)!=null?l:!0)&&((a=t.options.enableFilters)!=null?a:!0)&&!!e.accessorFn},e.getIsFiltered=()=>e.getFilterIndex()>-1,e.getFilterValue=()=>{var n;return(n=t.getState().columnFilters)==null||(n=n.find(l=>l.id===e.id))==null?void 0:n.value},e.getFilterIndex=()=>{var n,l;return(n=(l=t.getState().columnFilters)==null?void 0:l.findIndex(a=>a.id===e.id))!=null?n:-1},e.setFilterValue=n=>{t.setColumnFilters(l=>{const a=e.getFilterFn(),i=l==null?void 0:l.find(c=>c.id===e.id),r=Rn(n,i?i.value:void 0);if(Kd(a,r,e)){var u;return(u=l==null?void 0:l.filter(c=>c.id!==e.id))!=null?u:[]}const o={id:e.id,value:r};if(i){var s;return(s=l==null?void 0:l.map(c=>c.id===e.id?o:c))!=null?s:[]}return l!=null&&l.length?[...l,o]:[o]})}},createRow:(e,t)=>{e.columnFilters={},e.columnFiltersMeta={}},createTable:e=>{e.setColumnFilters=t=>{const n=e.getAllLeafColumns(),l=a=>{var i;return(i=Rn(t,a))==null?void 0:i.filter(r=>{const u=n.find(o=>o.id===r.id);if(u){const o=u.getFilterFn();if(Kd(o,r.value,u))return!1}return!0})};e.options.onColumnFiltersChange==null||e.options.onColumnFiltersChange(l)},e.resetColumnFilters=t=>{var n,l;e.setColumnFilters(t?[]:(n=(l=e.initialState)==null?void 0:l.columnFilters)!=null?n:[])},e.getPreFilteredRowModel=()=>e.getCoreRowModel(),e.getFilteredRowModel=()=>(!e._getFilteredRowModel&&e.options.getFilteredRowModel&&(e._getFilteredRowModel=e.options.getFilteredRowModel(e)),e.options.manualFiltering||!e._getFilteredRowModel?e.getPreFilteredRowModel():e._getFilteredRowModel())}};function Kd(e,t,n){return(e&&e.autoRemove?e.autoRemove(t,n):!1)||typeof t>"u"||typeof t=="string"&&!t}const Jw=(e,t,n)=>n.reduce((l,a)=>{const i=a.getValue(e);return l+(typeof i=="number"?i:0)},0),Pw=(e,t,n)=>{let l;return n.forEach(a=>{const i=a.getValue(e);i!=null&&(l>i||l===void 0&&i>=i)&&(l=i)}),l},Iw=(e,t,n)=>{let l;return n.forEach(a=>{const i=a.getValue(e);i!=null&&(l<i||l===void 0&&i>=i)&&(l=i)}),l},Ww=(e,t,n)=>{let l,a;return n.forEach(i=>{const r=i.getValue(e);r!=null&&(l===void 0?r>=r&&(l=a=r):(l>r&&(l=r),a<r&&(a=r)))}),[l,a]},eb=(e,t)=>{let n=0,l=0;if(t.forEach(a=>{let i=a.getValue(e);i!=null&&(i=+i)>=i&&(++n,l+=i)}),n)return l/n},tb=(e,t)=>{if(!t.length)return;const n=t.map(i=>i.getValue(e));if(!Gw(n))return;if(n.length===1)return n[0];const l=Math.floor(n.length/2),a=n.sort((i,r)=>i-r);return n.length%2!==0?a[l]:(a[l-1]+a[l])/2},nb=(e,t)=>Array.from(new Set(t.map(n=>n.getValue(e))).values()),lb=(e,t)=>new Set(t.map(n=>n.getValue(e))).size,ab=(e,t)=>t.length,So={sum:Jw,min:Pw,max:Iw,extent:Ww,mean:eb,median:tb,unique:nb,uniqueCount:lb,count:ab},ib={getDefaultColumnDef:()=>({aggregatedCell:e=>{var t,n;return(t=(n=e.getValue())==null||n.toString==null?void 0:n.toString())!=null?t:null},aggregationFn:"auto"}),getInitialState:e=>({grouping:[],...e}),getDefaultOptions:e=>({onGroupingChange:lt("grouping",e),groupedColumnMode:"reorder"}),createColumn:(e,t)=>{e.toggleGrouping=()=>{t.setGrouping(n=>n!=null&&n.includes(e.id)?n.filter(l=>l!==e.id):[...n??[],e.id])},e.getCanGroup=()=>{var n,l;return((n=e.columnDef.enableGrouping)!=null?n:!0)&&((l=t.options.enableGrouping)!=null?l:!0)&&(!!e.accessorFn||!!e.columnDef.getGroupingValue)},e.getIsGrouped=()=>{var n;return(n=t.getState().grouping)==null?void 0:n.includes(e.id)},e.getGroupedIndex=()=>{var n;return(n=t.getState().grouping)==null?void 0:n.indexOf(e.id)},e.getToggleGroupingHandler=()=>{const n=e.getCanGroup();return()=>{n&&e.toggleGrouping()}},e.getAutoAggregationFn=()=>{const n=t.getCoreRowModel().flatRows[0],l=n==null?void 0:n.getValue(e.id);if(typeof l=="number")return So.sum;if(Object.prototype.toString.call(l)==="[object Date]")return So.extent},e.getAggregationFn=()=>{var n,l;if(!e)throw new Error;return Hu(e.columnDef.aggregationFn)?e.columnDef.aggregationFn:e.columnDef.aggregationFn==="auto"?e.getAutoAggregationFn():(n=(l=t.options.aggregationFns)==null?void 0:l[e.columnDef.aggregationFn])!=null?n:So[e.columnDef.aggregationFn]}},createTable:e=>{e.setGrouping=t=>e.options.onGroupingChange==null?void 0:e.options.onGroupingChange(t),e.resetGrouping=t=>{var n,l;e.setGrouping(t?[]:(n=(l=e.initialState)==null?void 0:l.grouping)!=null?n:[])},e.getPreGroupedRowModel=()=>e.getFilteredRowModel(),e.getGroupedRowModel=()=>(!e._getGroupedRowModel&&e.options.getGroupedRowModel&&(e._getGroupedRowModel=e.options.getGroupedRowModel(e)),e.options.manualGrouping||!e._getGroupedRowModel?e.getPreGroupedRowModel():e._getGroupedRowModel())},createRow:(e,t)=>{e.getIsGrouped=()=>!!e.groupingColumnId,e.getGroupingValue=n=>{if(e._groupingValuesCache.hasOwnProperty(n))return e._groupingValuesCache[n];const l=t.getColumn(n);return l!=null&&l.columnDef.getGroupingValue?(e._groupingValuesCache[n]=l.columnDef.getGroupingValue(e.original),e._groupingValuesCache[n]):e.getValue(n)},e._groupingValuesCache={}},createCell:(e,t,n,l)=>{e.getIsGrouped=()=>t.getIsGrouped()&&t.id===n.groupingColumnId,e.getIsPlaceholder=()=>!e.getIsGrouped()&&t.getIsGrouped(),e.getIsAggregated=()=>{var a;return!e.getIsGrouped()&&!e.getIsPlaceholder()&&!!((a=n.subRows)!=null&&a.length)}}};function rb(e,t,n){if(!(t!=null&&t.length)||!n)return e;const l=e.filter(i=>!t.includes(i.id));return n==="remove"?l:[...t.map(i=>e.find(r=>r.id===i)).filter(Boolean),...l]}const ub={getInitialState:e=>({columnOrder:[],...e}),getDefaultOptions:e=>({onColumnOrderChange:lt("columnOrder",e)}),createColumn:(e,t)=>{e.getIndex=F(n=>[Ia(t,n)],n=>n.findIndex(l=>l.id===e.id),B(t.options,"debugColumns")),e.getIsFirstColumn=n=>{var l;return((l=Ia(t,n)[0])==null?void 0:l.id)===e.id},e.getIsLastColumn=n=>{var l;const a=Ia(t,n);return((l=a[a.length-1])==null?void 0:l.id)===e.id}},createTable:e=>{e.setColumnOrder=t=>e.options.onColumnOrderChange==null?void 0:e.options.onColumnOrderChange(t),e.resetColumnOrder=t=>{var n;e.setColumnOrder(t?[]:(n=e.initialState.columnOrder)!=null?n:[])},e._getOrderColumnsFn=F(()=>[e.getState().columnOrder,e.getState().grouping,e.options.groupedColumnMode],(t,n,l)=>a=>{let i=[];if(!(t!=null&&t.length))i=a;else{const r=[...t],u=[...a];for(;u.length&&r.length;){const o=r.shift(),s=u.findIndex(c=>c.id===o);s>-1&&i.push(u.splice(s,1)[0])}i=[...i,...u]}return rb(i,n,l)},B(e.options,"debugTable"))}},xo=()=>({left:[],right:[]}),ob={getInitialState:e=>({columnPinning:xo(),...e}),getDefaultOptions:e=>({onColumnPinningChange:lt("columnPinning",e)}),createColumn:(e,t)=>{e.pin=n=>{const l=e.getLeafColumns().map(a=>a.id).filter(Boolean);t.setColumnPinning(a=>{var i,r;if(n==="right"){var u,o;return{left:((u=a==null?void 0:a.left)!=null?u:[]).filter(f=>!(l!=null&&l.includes(f))),right:[...((o=a==null?void 0:a.right)!=null?o:[]).filter(f=>!(l!=null&&l.includes(f))),...l]}}if(n==="left"){var s,c;return{left:[...((s=a==null?void 0:a.left)!=null?s:[]).filter(f=>!(l!=null&&l.includes(f))),...l],right:((c=a==null?void 0:a.right)!=null?c:[]).filter(f=>!(l!=null&&l.includes(f)))}}return{left:((i=a==null?void 0:a.left)!=null?i:[]).filter(f=>!(l!=null&&l.includes(f))),right:((r=a==null?void 0:a.right)!=null?r:[]).filter(f=>!(l!=null&&l.includes(f)))}})},e.getCanPin=()=>e.getLeafColumns().some(l=>{var a,i,r;return((a=l.columnDef.enablePinning)!=null?a:!0)&&((i=(r=t.options.enableColumnPinning)!=null?r:t.options.enablePinning)!=null?i:!0)}),e.getIsPinned=()=>{const n=e.getLeafColumns().map(u=>u.id),{left:l,right:a}=t.getState().columnPinning,i=n.some(u=>l==null?void 0:l.includes(u)),r=n.some(u=>a==null?void 0:a.includes(u));return i?"left":r?"right":!1},e.getPinnedIndex=()=>{var n,l;const a=e.getIsPinned();return a?(n=(l=t.getState().columnPinning)==null||(l=l[a])==null?void 0:l.indexOf(e.id))!=null?n:-1:0}},createRow:(e,t)=>{e.getCenterVisibleCells=F(()=>[e._getAllVisibleCells(),t.getState().columnPinning.left,t.getState().columnPinning.right],(n,l,a)=>{const i=[...l??[],...a??[]];return n.filter(r=>!i.includes(r.column.id))},B(t.options,"debugRows")),e.getLeftVisibleCells=F(()=>[e._getAllVisibleCells(),t.getState().columnPinning.left],(n,l)=>(l??[]).map(i=>n.find(r=>r.column.id===i)).filter(Boolean).map(i=>({...i,position:"left"})),B(t.options,"debugRows")),e.getRightVisibleCells=F(()=>[e._getAllVisibleCells(),t.getState().columnPinning.right],(n,l)=>(l??[]).map(i=>n.find(r=>r.column.id===i)).filter(Boolean).map(i=>({...i,position:"right"})),B(t.options,"debugRows"))},createTable:e=>{e.setColumnPinning=t=>e.options.onColumnPinningChange==null?void 0:e.options.onColumnPinningChange(t),e.resetColumnPinning=t=>{var n,l;return e.setColumnPinning(t?xo():(n=(l=e.initialState)==null?void 0:l.columnPinning)!=null?n:xo())},e.getIsSomeColumnsPinned=t=>{var n;const l=e.getState().columnPinning;if(!t){var a,i;return!!((a=l.left)!=null&&a.length||(i=l.right)!=null&&i.length)}return!!((n=l[t])!=null&&n.length)},e.getLeftLeafColumns=F(()=>[e.getAllLeafColumns(),e.getState().columnPinning.left],(t,n)=>(n??[]).map(l=>t.find(a=>a.id===l)).filter(Boolean),B(e.options,"debugColumns")),e.getRightLeafColumns=F(()=>[e.getAllLeafColumns(),e.getState().columnPinning.right],(t,n)=>(n??[]).map(l=>t.find(a=>a.id===l)).filter(Boolean),B(e.options,"debugColumns")),e.getCenterLeafColumns=F(()=>[e.getAllLeafColumns(),e.getState().columnPinning.left,e.getState().columnPinning.right],(t,n,l)=>{const a=[...n??[],...l??[]];return t.filter(i=>!a.includes(i.id))},B(e.options,"debugColumns"))}},rr={size:150,minSize:20,maxSize:Number.MAX_SAFE_INTEGER},wo=()=>({startOffset:null,startSize:null,deltaOffset:null,deltaPercentage:null,isResizingColumn:!1,columnSizingStart:[]}),sb={getDefaultColumnDef:()=>rr,getInitialState:e=>({columnSizing:{},columnSizingInfo:wo(),...e}),getDefaultOptions:e=>({columnResizeMode:"onEnd",columnResizeDirection:"ltr",onColumnSizingChange:lt("columnSizing",e),onColumnSizingInfoChange:lt("columnSizingInfo",e)}),createColumn:(e,t)=>{e.getSize=()=>{var n,l,a;const i=t.getState().columnSizing[e.id];return Math.min(Math.max((n=e.columnDef.minSize)!=null?n:rr.minSize,(l=i??e.columnDef.size)!=null?l:rr.size),(a=e.columnDef.maxSize)!=null?a:rr.maxSize)},e.getStart=F(n=>[n,Ia(t,n),t.getState().columnSizing],(n,l)=>l.slice(0,e.getIndex(n)).reduce((a,i)=>a+i.getSize(),0),B(t.options,"debugColumns")),e.getAfter=F(n=>[n,Ia(t,n),t.getState().columnSizing],(n,l)=>l.slice(e.getIndex(n)+1).reduce((a,i)=>a+i.getSize(),0),B(t.options,"debugColumns")),e.resetSize=()=>{t.setColumnSizing(n=>{let{[e.id]:l,...a}=n;return a})},e.getCanResize=()=>{var n,l;return((n=e.columnDef.enableResizing)!=null?n:!0)&&((l=t.options.enableColumnResizing)!=null?l:!0)},e.getIsResizing=()=>t.getState().columnSizingInfo.isResizingColumn===e.id},createHeader:(e,t)=>{e.getSize=()=>{let n=0;const l=a=>{if(a.subHeaders.length)a.subHeaders.forEach(l);else{var i;n+=(i=a.column.getSize())!=null?i:0}};return l(e),n},e.getStart=()=>{if(e.index>0){const n=e.headerGroup.headers[e.index-1];return n.getStart()+n.getSize()}return 0},e.getResizeHandler=n=>{const l=t.getColumn(e.column.id),a=l==null?void 0:l.getCanResize();return i=>{if(!l||!a||(i.persist==null||i.persist(),bo(i)&&i.touches&&i.touches.length>1))return;const r=e.getSize(),u=e?e.getLeafHeaders().map(y=>[y.column.id,y.column.getSize()]):[[l.id,l.getSize()]],o=bo(i)?Math.round(i.touches[0].clientX):i.clientX,s={},c=(y,m)=>{typeof m=="number"&&(t.setColumnSizingInfo(v=>{var b,R;const _=t.options.columnResizeDirection==="rtl"?-1:1,E=(m-((b=v==null?void 0:v.startOffset)!=null?b:0))*_,A=Math.max(E/((R=v==null?void 0:v.startSize)!=null?R:0),-.999999);return v.columnSizingStart.forEach(N=>{let[M,j]=N;s[M]=Math.round(Math.max(j+j*A,0)*100)/100}),{...v,deltaOffset:E,deltaPercentage:A}}),(t.options.columnResizeMode==="onChange"||y==="end")&&t.setColumnSizing(v=>({...v,...s})))},f=y=>c("move",y),p=y=>{c("end",y),t.setColumnSizingInfo(m=>({...m,isResizingColumn:!1,startOffset:null,startSize:null,deltaOffset:null,deltaPercentage:null,columnSizingStart:[]}))},d=n||typeof document<"u"?document:null,h={moveHandler:y=>f(y.clientX),upHandler:y=>{d==null||d.removeEventListener("mousemove",h.moveHandler),d==null||d.removeEventListener("mouseup",h.upHandler),p(y.clientX)}},S={moveHandler:y=>(y.cancelable&&(y.preventDefault(),y.stopPropagation()),f(y.touches[0].clientX),!1),upHandler:y=>{var m;d==null||d.removeEventListener("touchmove",S.moveHandler),d==null||d.removeEventListener("touchend",S.upHandler),y.cancelable&&(y.preventDefault(),y.stopPropagation()),p((m=y.touches[0])==null?void 0:m.clientX)}},x=cb()?{passive:!1}:!1;bo(i)?(d==null||d.addEventListener("touchmove",S.moveHandler,x),d==null||d.addEventListener("touchend",S.upHandler,x)):(d==null||d.addEventListener("mousemove",h.moveHandler,x),d==null||d.addEventListener("mouseup",h.upHandler,x)),t.setColumnSizingInfo(y=>({...y,startOffset:o,startSize:r,deltaOffset:0,deltaPercentage:0,columnSizingStart:u,isResizingColumn:l.id}))}}},createTable:e=>{e.setColumnSizing=t=>e.options.onColumnSizingChange==null?void 0:e.options.onColumnSizingChange(t),e.setColumnSizingInfo=t=>e.options.onColumnSizingInfoChange==null?void 0:e.options.onColumnSizingInfoChange(t),e.resetColumnSizing=t=>{var n;e.setColumnSizing(t?{}:(n=e.initialState.columnSizing)!=null?n:{})},e.resetHeaderSizeInfo=t=>{var n;e.setColumnSizingInfo(t?wo():(n=e.initialState.columnSizingInfo)!=null?n:wo())},e.getTotalSize=()=>{var t,n;return(t=(n=e.getHeaderGroups()[0])==null?void 0:n.headers.reduce((l,a)=>l+a.getSize(),0))!=null?t:0},e.getLeftTotalSize=()=>{var t,n;return(t=(n=e.getLeftHeaderGroups()[0])==null?void 0:n.headers.reduce((l,a)=>l+a.getSize(),0))!=null?t:0},e.getCenterTotalSize=()=>{var t,n;return(t=(n=e.getCenterHeaderGroups()[0])==null?void 0:n.headers.reduce((l,a)=>l+a.getSize(),0))!=null?t:0},e.getRightTotalSize=()=>{var t,n;return(t=(n=e.getRightHeaderGroups()[0])==null?void 0:n.headers.reduce((l,a)=>l+a.getSize(),0))!=null?t:0}}};let ur=null;function cb(){if(typeof ur=="boolean")return ur;let e=!1;try{const t={get passive(){return e=!0,!1}},n=()=>{};window.addEventListener("test",n,t),window.removeEventListener("test",n)}catch{e=!1}return ur=e,ur}function bo(e){return e.type==="touchstart"}const fb={getInitialState:e=>({columnVisibility:{},...e}),getDefaultOptions:e=>({onColumnVisibilityChange:lt("columnVisibility",e)}),createColumn:(e,t)=>{e.toggleVisibility=n=>{e.getCanHide()&&t.setColumnVisibility(l=>({...l,[e.id]:n??!e.getIsVisible()}))},e.getIsVisible=()=>{var n,l;const a=e.columns;return(n=a.length?a.some(i=>i.getIsVisible()):(l=t.getState().columnVisibility)==null?void 0:l[e.id])!=null?n:!0},e.getCanHide=()=>{var n,l;return((n=e.columnDef.enableHiding)!=null?n:!0)&&((l=t.options.enableHiding)!=null?l:!0)},e.getToggleVisibilityHandler=()=>n=>{e.toggleVisibility==null||e.toggleVisibility(n.target.checked)}},createRow:(e,t)=>{e._getAllVisibleCells=F(()=>[e.getAllCells(),t.getState().columnVisibility],n=>n.filter(l=>l.column.getIsVisible()),B(t.options,"debugRows")),e.getVisibleCells=F(()=>[e.getLeftVisibleCells(),e.getCenterVisibleCells(),e.getRightVisibleCells()],(n,l,a)=>[...n,...l,...a],B(t.options,"debugRows"))},createTable:e=>{const t=(n,l)=>F(()=>[l(),l().filter(a=>a.getIsVisible()).map(a=>a.id).join("_")],a=>a.filter(i=>i.getIsVisible==null?void 0:i.getIsVisible()),B(e.options,"debugColumns"));e.getVisibleFlatColumns=t("getVisibleFlatColumns",()=>e.getAllFlatColumns()),e.getVisibleLeafColumns=t("getVisibleLeafColumns",()=>e.getAllLeafColumns()),e.getLeftVisibleLeafColumns=t("getLeftVisibleLeafColumns",()=>e.getLeftLeafColumns()),e.getRightVisibleLeafColumns=t("getRightVisibleLeafColumns",()=>e.getRightLeafColumns()),e.getCenterVisibleLeafColumns=t("getCenterVisibleLeafColumns",()=>e.getCenterLeafColumns()),e.setColumnVisibility=n=>e.options.onColumnVisibilityChange==null?void 0:e.options.onColumnVisibilityChange(n),e.resetColumnVisibility=n=>{var l;e.setColumnVisibility(n?{}:(l=e.initialState.columnVisibility)!=null?l:{})},e.toggleAllColumnsVisible=n=>{var l;n=(l=n)!=null?l:!e.getIsAllColumnsVisible(),e.setColumnVisibility(e.getAllLeafColumns().reduce((a,i)=>({...a,[i.id]:n||!(i.getCanHide!=null&&i.getCanHide())}),{}))},e.getIsAllColumnsVisible=()=>!e.getAllLeafColumns().some(n=>!(n.getIsVisible!=null&&n.getIsVisible())),e.getIsSomeColumnsVisible=()=>e.getAllLeafColumns().some(n=>n.getIsVisible==null?void 0:n.getIsVisible()),e.getToggleAllColumnsVisibilityHandler=()=>n=>{var l;e.toggleAllColumnsVisible((l=n.target)==null?void 0:l.checked)}}};function Ia(e,t){return t?t==="center"?e.getCenterVisibleLeafColumns():t==="left"?e.getLeftVisibleLeafColumns():e.getRightVisibleLeafColumns():e.getVisibleLeafColumns()}const db={createTable:e=>{e._getGlobalFacetedRowModel=e.options.getFacetedRowModel&&e.options.getFacetedRowModel(e,"__global__"),e.getGlobalFacetedRowModel=()=>e.options.manualFiltering||!e._getGlobalFacetedRowModel?e.getPreFilteredRowModel():e._getGlobalFacetedRowModel(),e._getGlobalFacetedUniqueValues=e.options.getFacetedUniqueValues&&e.options.getFacetedUniqueValues(e,"__global__"),e.getGlobalFacetedUniqueValues=()=>e._getGlobalFacetedUniqueValues?e._getGlobalFacetedUniqueValues():new Map,e._getGlobalFacetedMinMaxValues=e.options.getFacetedMinMaxValues&&e.options.getFacetedMinMaxValues(e,"__global__"),e.getGlobalFacetedMinMaxValues=()=>{if(e._getGlobalFacetedMinMaxValues)return e._getGlobalFacetedMinMaxValues()}}},pb={getInitialState:e=>({globalFilter:void 0,...e}),getDefaultOptions:e=>({onGlobalFilterChange:lt("globalFilter",e),globalFilterFn:"auto",getColumnCanGlobalFilter:t=>{var n;const l=(n=e.getCoreRowModel().flatRows[0])==null||(n=n._getAllCellsByColumnId()[t.id])==null?void 0:n.getValue();return typeof l=="string"||typeof l=="number"}}),createColumn:(e,t)=>{e.getCanGlobalFilter=()=>{var n,l,a,i;return((n=e.columnDef.enableGlobalFilter)!=null?n:!0)&&((l=t.options.enableGlobalFilter)!=null?l:!0)&&((a=t.options.enableFilters)!=null?a:!0)&&((i=t.options.getColumnCanGlobalFilter==null?void 0:t.options.getColumnCanGlobalFilter(e))!=null?i:!0)&&!!e.accessorFn}},createTable:e=>{e.getGlobalAutoFilterFn=()=>kt.includesString,e.getGlobalFilterFn=()=>{var t,n;const{globalFilterFn:l}=e.options;return Hu(l)?l:l==="auto"?e.getGlobalAutoFilterFn():(t=(n=e.options.filterFns)==null?void 0:n[l])!=null?t:kt[l]},e.setGlobalFilter=t=>{e.options.onGlobalFilterChange==null||e.options.onGlobalFilterChange(t)},e.resetGlobalFilter=t=>{e.setGlobalFilter(t?void 0:e.initialState.globalFilter)}}},gb={getInitialState:e=>({expanded:{},...e}),getDefaultOptions:e=>({onExpandedChange:lt("expanded",e),paginateExpandedRows:!0}),createTable:e=>{let t=!1,n=!1;e._autoResetExpanded=()=>{var l,a;if(!t){e._queue(()=>{t=!0});return}if((l=(a=e.options.autoResetAll)!=null?a:e.options.autoResetExpanded)!=null?l:!e.options.manualExpanding){if(n)return;n=!0,e._queue(()=>{e.resetExpanded(),n=!1})}},e.setExpanded=l=>e.options.onExpandedChange==null?void 0:e.options.onExpandedChange(l),e.toggleAllRowsExpanded=l=>{l??!e.getIsAllRowsExpanded()?e.setExpanded(!0):e.setExpanded({})},e.resetExpanded=l=>{var a,i;e.setExpanded(l?{}:(a=(i=e.initialState)==null?void 0:i.expanded)!=null?a:{})},e.getCanSomeRowsExpand=()=>e.getPrePaginationRowModel().flatRows.some(l=>l.getCanExpand()),e.getToggleAllRowsExpandedHandler=()=>l=>{l.persist==null||l.persist(),e.toggleAllRowsExpanded()},e.getIsSomeRowsExpanded=()=>{const l=e.getState().expanded;return l===!0||Object.values(l).some(Boolean)},e.getIsAllRowsExpanded=()=>{const l=e.getState().expanded;return typeof l=="boolean"?l===!0:!(!Object.keys(l).length||e.getRowModel().flatRows.some(a=>!a.getIsExpanded()))},e.getExpandedDepth=()=>{let l=0;return(e.getState().expanded===!0?Object.keys(e.getRowModel().rowsById):Object.keys(e.getState().expanded)).forEach(i=>{const r=i.split(".");l=Math.max(l,r.length)}),l},e.getPreExpandedRowModel=()=>e.getSortedRowModel(),e.getExpandedRowModel=()=>(!e._getExpandedRowModel&&e.options.getExpandedRowModel&&(e._getExpandedRowModel=e.options.getExpandedRowModel(e)),e.options.manualExpanding||!e._getExpandedRowModel?e.getPreExpandedRowModel():e._getExpandedRowModel())},createRow:(e,t)=>{e.toggleExpanded=n=>{t.setExpanded(l=>{var a;const i=l===!0?!0:!!(l!=null&&l[e.id]);let r={};if(l===!0?Object.keys(t.getRowModel().rowsById).forEach(u=>{r[u]=!0}):r=l,n=(a=n)!=null?a:!i,!i&&n)return{...r,[e.id]:!0};if(i&&!n){const{[e.id]:u,...o}=r;return o}return l})},e.getIsExpanded=()=>{var n;const l=t.getState().expanded;return!!((n=t.options.getIsRowExpanded==null?void 0:t.options.getIsRowExpanded(e))!=null?n:l===!0||l!=null&&l[e.id])},e.getCanExpand=()=>{var n,l,a;return(n=t.options.getRowCanExpand==null?void 0:t.options.getRowCanExpand(e))!=null?n:((l=t.options.enableExpanding)!=null?l:!0)&&!!((a=e.subRows)!=null&&a.length)},e.getIsAllParentsExpanded=()=>{let n=!0,l=e;for(;n&&l.parentId;)l=t.getRow(l.parentId,!0),n=l.getIsExpanded();return n},e.getToggleExpandedHandler=()=>{const n=e.getCanExpand();return()=>{n&&e.toggleExpanded()}}}},Rs=0,Es=10,Ro=()=>({pageIndex:Rs,pageSize:Es}),hb={getInitialState:e=>({...e,pagination:{...Ro(),...e==null?void 0:e.pagination}}),getDefaultOptions:e=>({onPaginationChange:lt("pagination",e)}),createTable:e=>{let t=!1,n=!1;e._autoResetPageIndex=()=>{var l,a;if(!t){e._queue(()=>{t=!0});return}if((l=(a=e.options.autoResetAll)!=null?a:e.options.autoResetPageIndex)!=null?l:!e.options.manualPagination){if(n)return;n=!0,e._queue(()=>{e.resetPageIndex(),n=!1})}},e.setPagination=l=>{const a=i=>Rn(l,i);return e.options.onPaginationChange==null?void 0:e.options.onPaginationChange(a)},e.resetPagination=l=>{var a;e.setPagination(l?Ro():(a=e.initialState.pagination)!=null?a:Ro())},e.setPageIndex=l=>{e.setPagination(a=>{let i=Rn(l,a.pageIndex);const r=typeof e.options.pageCount>"u"||e.options.pageCount===-1?Number.MAX_SAFE_INTEGER:e.options.pageCount-1;return i=Math.max(0,Math.min(i,r)),{...a,pageIndex:i}})},e.resetPageIndex=l=>{var a,i;e.setPageIndex(l?Rs:(a=(i=e.initialState)==null||(i=i.pagination)==null?void 0:i.pageIndex)!=null?a:Rs)},e.resetPageSize=l=>{var a,i;e.setPageSize(l?Es:(a=(i=e.initialState)==null||(i=i.pagination)==null?void 0:i.pageSize)!=null?a:Es)},e.setPageSize=l=>{e.setPagination(a=>{const i=Math.max(1,Rn(l,a.pageSize)),r=a.pageSize*a.pageIndex,u=Math.floor(r/i);return{...a,pageIndex:u,pageSize:i}})},e.setPageCount=l=>e.setPagination(a=>{var i;let r=Rn(l,(i=e.options.pageCount)!=null?i:-1);return typeof r=="number"&&(r=Math.max(-1,r)),{...a,pageCount:r}}),e.getPageOptions=F(()=>[e.getPageCount()],l=>{let a=[];return l&&l>0&&(a=[...new Array(l)].fill(null).map((i,r)=>r)),a},B(e.options,"debugTable")),e.getCanPreviousPage=()=>e.getState().pagination.pageIndex>0,e.getCanNextPage=()=>{const{pageIndex:l}=e.getState().pagination,a=e.getPageCount();return a===-1?!0:a===0?!1:l<a-1},e.previousPage=()=>e.setPageIndex(l=>l-1),e.nextPage=()=>e.setPageIndex(l=>l+1),e.firstPage=()=>e.setPageIndex(0),e.lastPage=()=>e.setPageIndex(e.getPageCount()-1),e.getPrePaginationRowModel=()=>e.getExpandedRowModel(),e.getPaginationRowModel=()=>(!e._getPaginationRowModel&&e.options.getPaginationRowModel&&(e._getPaginationRowModel=e.options.getPaginationRowModel(e)),e.options.manualPagination||!e._getPaginationRowModel?e.getPrePaginationRowModel():e._getPaginationRowModel()),e.getPageCount=()=>{var l;return(l=e.options.pageCount)!=null?l:Math.ceil(e.getRowCount()/e.getState().pagination.pageSize)},e.getRowCount=()=>{var l;return(l=e.options.rowCount)!=null?l:e.getPrePaginationRowModel().rows.length}}},Eo=()=>({top:[],bottom:[]}),mb={getInitialState:e=>({rowPinning:Eo(),...e}),getDefaultOptions:e=>({onRowPinningChange:lt("rowPinning",e)}),createRow:(e,t)=>{e.pin=(n,l,a)=>{const i=l?e.getLeafRows().map(o=>{let{id:s}=o;return s}):[],r=a?e.getParentRows().map(o=>{let{id:s}=o;return s}):[],u=new Set([...r,e.id,...i]);t.setRowPinning(o=>{var s,c;if(n==="bottom"){var f,p;return{top:((f=o==null?void 0:o.top)!=null?f:[]).filter(S=>!(u!=null&&u.has(S))),bottom:[...((p=o==null?void 0:o.bottom)!=null?p:[]).filter(S=>!(u!=null&&u.has(S))),...Array.from(u)]}}if(n==="top"){var d,h;return{top:[...((d=o==null?void 0:o.top)!=null?d:[]).filter(S=>!(u!=null&&u.has(S))),...Array.from(u)],bottom:((h=o==null?void 0:o.bottom)!=null?h:[]).filter(S=>!(u!=null&&u.has(S)))}}return{top:((s=o==null?void 0:o.top)!=null?s:[]).filter(S=>!(u!=null&&u.has(S))),bottom:((c=o==null?void 0:o.bottom)!=null?c:[]).filter(S=>!(u!=null&&u.has(S)))}})},e.getCanPin=()=>{var n;const{enableRowPinning:l,enablePinning:a}=t.options;return typeof l=="function"?l(e):(n=l??a)!=null?n:!0},e.getIsPinned=()=>{const n=[e.id],{top:l,bottom:a}=t.getState().rowPinning,i=n.some(u=>l==null?void 0:l.includes(u)),r=n.some(u=>a==null?void 0:a.includes(u));return i?"top":r?"bottom":!1},e.getPinnedIndex=()=>{var n,l;const a=e.getIsPinned();if(!a)return-1;const i=(n=a==="top"?t.getTopRows():t.getBottomRows())==null?void 0:n.map(r=>{let{id:u}=r;return u});return(l=i==null?void 0:i.indexOf(e.id))!=null?l:-1}},createTable:e=>{e.setRowPinning=t=>e.options.onRowPinningChange==null?void 0:e.options.onRowPinningChange(t),e.resetRowPinning=t=>{var n,l;return e.setRowPinning(t?Eo():(n=(l=e.initialState)==null?void 0:l.rowPinning)!=null?n:Eo())},e.getIsSomeRowsPinned=t=>{var n;const l=e.getState().rowPinning;if(!t){var a,i;return!!((a=l.top)!=null&&a.length||(i=l.bottom)!=null&&i.length)}return!!((n=l[t])!=null&&n.length)},e._getPinnedRows=(t,n,l)=>{var a;return((a=e.options.keepPinnedRows)==null||a?(n??[]).map(r=>{const u=e.getRow(r,!0);return u.getIsAllParentsExpanded()?u:null}):(n??[]).map(r=>t.find(u=>u.id===r))).filter(Boolean).map(r=>({...r,position:l}))},e.getTopRows=F(()=>[e.getRowModel().rows,e.getState().rowPinning.top],(t,n)=>e._getPinnedRows(t,n,"top"),B(e.options,"debugRows")),e.getBottomRows=F(()=>[e.getRowModel().rows,e.getState().rowPinning.bottom],(t,n)=>e._getPinnedRows(t,n,"bottom"),B(e.options,"debugRows")),e.getCenterRows=F(()=>[e.getRowModel().rows,e.getState().rowPinning.top,e.getState().rowPinning.bottom],(t,n,l)=>{const a=new Set([...n??[],...l??[]]);return t.filter(i=>!a.has(i.id))},B(e.options,"debugRows"))}},yb={getInitialState:e=>({rowSelection:{},...e}),getDefaultOptions:e=>({onRowSelectionChange:lt("rowSelection",e),enableRowSelection:!0,enableMultiRowSelection:!0,enableSubRowSelection:!0}),createTable:e=>{e.setRowSelection=t=>e.options.onRowSelectionChange==null?void 0:e.options.onRowSelectionChange(t),e.resetRowSelection=t=>{var n;return e.setRowSelection(t?{}:(n=e.initialState.rowSelection)!=null?n:{})},e.toggleAllRowsSelected=t=>{e.setRowSelection(n=>{t=typeof t<"u"?t:!e.getIsAllRowsSelected();const l={...n},a=e.getPreGroupedRowModel().flatRows;return t?a.forEach(i=>{i.getCanSelect()&&(l[i.id]=!0)}):a.forEach(i=>{delete l[i.id]}),l})},e.toggleAllPageRowsSelected=t=>e.setRowSelection(n=>{const l=typeof t<"u"?t:!e.getIsAllPageRowsSelected(),a={...n};return e.getRowModel().rows.forEach(i=>{Cs(a,i.id,l,!0,e)}),a}),e.getPreSelectedRowModel=()=>e.getCoreRowModel(),e.getSelectedRowModel=F(()=>[e.getState().rowSelection,e.getCoreRowModel()],(t,n)=>Object.keys(t).length?Co(e,n):{rows:[],flatRows:[],rowsById:{}},B(e.options,"debugTable")),e.getFilteredSelectedRowModel=F(()=>[e.getState().rowSelection,e.getFilteredRowModel()],(t,n)=>Object.keys(t).length?Co(e,n):{rows:[],flatRows:[],rowsById:{}},B(e.options,"debugTable")),e.getGroupedSelectedRowModel=F(()=>[e.getState().rowSelection,e.getSortedRowModel()],(t,n)=>Object.keys(t).length?Co(e,n):{rows:[],flatRows:[],rowsById:{}},B(e.options,"debugTable")),e.getIsAllRowsSelected=()=>{const t=e.getFilteredRowModel().flatRows,{rowSelection:n}=e.getState();let l=!!(t.length&&Object.keys(n).length);return l&&t.some(a=>a.getCanSelect()&&!n[a.id])&&(l=!1),l},e.getIsAllPageRowsSelected=()=>{const t=e.getPaginationRowModel().flatRows.filter(a=>a.getCanSelect()),{rowSelection:n}=e.getState();let l=!!t.length;return l&&t.some(a=>!n[a.id])&&(l=!1),l},e.getIsSomeRowsSelected=()=>{var t;const n=Object.keys((t=e.getState().rowSelection)!=null?t:{}).length;return n>0&&n<e.getFilteredRowModel().flatRows.length},e.getIsSomePageRowsSelected=()=>{const t=e.getPaginationRowModel().flatRows;return e.getIsAllPageRowsSelected()?!1:t.filter(n=>n.getCanSelect()).some(n=>n.getIsSelected()||n.getIsSomeSelected())},e.getToggleAllRowsSelectedHandler=()=>t=>{e.toggleAllRowsSelected(t.target.checked)},e.getToggleAllPageRowsSelectedHandler=()=>t=>{e.toggleAllPageRowsSelected(t.target.checked)}},createRow:(e,t)=>{e.toggleSelected=(n,l)=>{const a=e.getIsSelected();t.setRowSelection(i=>{var r;if(n=typeof n<"u"?n:!a,e.getCanSelect()&&a===n)return i;const u={...i};return Cs(u,e.id,n,(r=l==null?void 0:l.selectChildren)!=null?r:!0,t),u})},e.getIsSelected=()=>{const{rowSelection:n}=t.getState();return lf(e,n)},e.getIsSomeSelected=()=>{const{rowSelection:n}=t.getState();return _s(e,n)==="some"},e.getIsAllSubRowsSelected=()=>{const{rowSelection:n}=t.getState();return _s(e,n)==="all"},e.getCanSelect=()=>{var n;return typeof t.options.enableRowSelection=="function"?t.options.enableRowSelection(e):(n=t.options.enableRowSelection)!=null?n:!0},e.getCanSelectSubRows=()=>{var n;return typeof t.options.enableSubRowSelection=="function"?t.options.enableSubRowSelection(e):(n=t.options.enableSubRowSelection)!=null?n:!0},e.getCanMultiSelect=()=>{var n;return typeof t.options.enableMultiRowSelection=="function"?t.options.enableMultiRowSelection(e):(n=t.options.enableMultiRowSelection)!=null?n:!0},e.getToggleSelectedHandler=()=>{const n=e.getCanSelect();return l=>{var a;n&&e.toggleSelected((a=l.target)==null?void 0:a.checked)}}}},Cs=(e,t,n,l,a)=>{var i;const r=a.getRow(t,!0);n?(r.getCanMultiSelect()||Object.keys(e).forEach(u=>delete e[u]),r.getCanSelect()&&(e[t]=!0)):delete e[t],l&&(i=r.subRows)!=null&&i.length&&r.getCanSelectSubRows()&&r.subRows.forEach(u=>Cs(e,u.id,n,l,a))};function Co(e,t){const n=e.getState().rowSelection,l=[],a={},i=function(r,u){return r.map(o=>{var s;const c=lf(o,n);if(c&&(l.push(o),a[o.id]=o),(s=o.subRows)!=null&&s.length&&(o={...o,subRows:i(o.subRows)}),c)return o}).filter(Boolean)};return{rows:i(t.rows),flatRows:l,rowsById:a}}function lf(e,t){var n;return(n=t[e.id])!=null?n:!1}function _s(e,t,n){var l;if(!((l=e.subRows)!=null&&l.length))return!1;let a=!0,i=!1;return e.subRows.forEach(r=>{if(!(i&&!a)&&(r.getCanSelect()&&(lf(r,t)?i=!0:a=!1),r.subRows&&r.subRows.length)){const u=_s(r,t);u==="all"?i=!0:(u==="some"&&(i=!0),a=!1)}}),a?"all":i?"some":!1}const As=/([0-9]+)/gm,vb=(e,t,n)=>Cm(Un(e.getValue(n)).toLowerCase(),Un(t.getValue(n)).toLowerCase()),Sb=(e,t,n)=>Cm(Un(e.getValue(n)),Un(t.getValue(n))),xb=(e,t,n)=>af(Un(e.getValue(n)).toLowerCase(),Un(t.getValue(n)).toLowerCase()),wb=(e,t,n)=>af(Un(e.getValue(n)),Un(t.getValue(n))),bb=(e,t,n)=>{const l=e.getValue(n),a=t.getValue(n);return l>a?1:l<a?-1:0},Rb=(e,t,n)=>af(e.getValue(n),t.getValue(n));function af(e,t){return e===t?0:e>t?1:-1}function Un(e){return typeof e=="number"?isNaN(e)||e===1/0||e===-1/0?"":String(e):typeof e=="string"?e:""}function Cm(e,t){const n=e.split(As).filter(Boolean),l=t.split(As).filter(Boolean);for(;n.length&&l.length;){const a=n.shift(),i=l.shift(),r=parseInt(a,10),u=parseInt(i,10),o=[r,u].sort();if(isNaN(o[0])){if(a>i)return 1;if(i>a)return-1;continue}if(isNaN(o[1]))return isNaN(r)?-1:1;if(r>u)return 1;if(u>r)return-1}return n.length-l.length}const Ta={alphanumeric:vb,alphanumericCaseSensitive:Sb,text:xb,textCaseSensitive:wb,datetime:bb,basic:Rb},Eb={getInitialState:e=>({sorting:[],...e}),getDefaultColumnDef:()=>({sortingFn:"auto",sortUndefined:1}),getDefaultOptions:e=>({onSortingChange:lt("sorting",e),isMultiSortEvent:t=>t.shiftKey}),createColumn:(e,t)=>{e.getAutoSortingFn=()=>{const n=t.getFilteredRowModel().flatRows.slice(10);let l=!1;for(const a of n){const i=a==null?void 0:a.getValue(e.id);if(Object.prototype.toString.call(i)==="[object Date]")return Ta.datetime;if(typeof i=="string"&&(l=!0,i.split(As).length>1))return Ta.alphanumeric}return l?Ta.text:Ta.basic},e.getAutoSortDir=()=>{const n=t.getFilteredRowModel().flatRows[0];return typeof(n==null?void 0:n.getValue(e.id))=="string"?"asc":"desc"},e.getSortingFn=()=>{var n,l;if(!e)throw new Error;return Hu(e.columnDef.sortingFn)?e.columnDef.sortingFn:e.columnDef.sortingFn==="auto"?e.getAutoSortingFn():(n=(l=t.options.sortingFns)==null?void 0:l[e.columnDef.sortingFn])!=null?n:Ta[e.columnDef.sortingFn]},e.toggleSorting=(n,l)=>{const a=e.getNextSortingOrder(),i=typeof n<"u"&&n!==null;t.setSorting(r=>{const u=r==null?void 0:r.find(d=>d.id===e.id),o=r==null?void 0:r.findIndex(d=>d.id===e.id);let s=[],c,f=i?n:a==="desc";if(r!=null&&r.length&&e.getCanMultiSort()&&l?u?c="toggle":c="add":r!=null&&r.length&&o!==r.length-1?c="replace":u?c="toggle":c="replace",c==="toggle"&&(i||a||(c="remove")),c==="add"){var p;s=[...r,{id:e.id,desc:f}],s.splice(0,s.length-((p=t.options.maxMultiSortColCount)!=null?p:Number.MAX_SAFE_INTEGER))}else c==="toggle"?s=r.map(d=>d.id===e.id?{...d,desc:f}:d):c==="remove"?s=r.filter(d=>d.id!==e.id):s=[{id:e.id,desc:f}];return s})},e.getFirstSortDir=()=>{var n,l;return((n=(l=e.columnDef.sortDescFirst)!=null?l:t.options.sortDescFirst)!=null?n:e.getAutoSortDir()==="desc")?"desc":"asc"},e.getNextSortingOrder=n=>{var l,a;const i=e.getFirstSortDir(),r=e.getIsSorted();return r?r!==i&&((l=t.options.enableSortingRemoval)==null||l)&&(!(n&&(a=t.options.enableMultiRemove)!=null)||a)?!1:r==="desc"?"asc":"desc":i},e.getCanSort=()=>{var n,l;return((n=e.columnDef.enableSorting)!=null?n:!0)&&((l=t.options.enableSorting)!=null?l:!0)&&!!e.accessorFn},e.getCanMultiSort=()=>{var n,l;return(n=(l=e.columnDef.enableMultiSort)!=null?l:t.options.enableMultiSort)!=null?n:!!e.accessorFn},e.getIsSorted=()=>{var n;const l=(n=t.getState().sorting)==null?void 0:n.find(a=>a.id===e.id);return l?l.desc?"desc":"asc":!1},e.getSortIndex=()=>{var n,l;return(n=(l=t.getState().sorting)==null?void 0:l.findIndex(a=>a.id===e.id))!=null?n:-1},e.clearSorting=()=>{t.setSorting(n=>n!=null&&n.length?n.filter(l=>l.id!==e.id):[])},e.getToggleSortingHandler=()=>{const n=e.getCanSort();return l=>{n&&(l.persist==null||l.persist(),e.toggleSorting==null||e.toggleSorting(void 0,e.getCanMultiSort()?t.options.isMultiSortEvent==null?void 0:t.options.isMultiSortEvent(l):!1))}}},createTable:e=>{e.setSorting=t=>e.options.onSortingChange==null?void 0:e.options.onSortingChange(t),e.resetSorting=t=>{var n,l;e.setSorting(t?[]:(n=(l=e.initialState)==null?void 0:l.sorting)!=null?n:[])},e.getPreSortedRowModel=()=>e.getGroupedRowModel(),e.getSortedRowModel=()=>(!e._getSortedRowModel&&e.options.getSortedRowModel&&(e._getSortedRowModel=e.options.getSortedRowModel(e)),e.options.manualSorting||!e._getSortedRowModel?e.getPreSortedRowModel():e._getSortedRowModel())}},Cb=[Qw,fb,ub,ob,Zw,Kw,db,pb,Eb,ib,gb,hb,mb,yb,sb];function _b(e){var t,n;const l=[...Cb,...(t=e._features)!=null?t:[]];let a={_features:l};const i=a._features.reduce((p,d)=>Object.assign(p,d.getDefaultOptions==null?void 0:d.getDefaultOptions(a)),{}),r=p=>a.options.mergeOptions?a.options.mergeOptions(i,p):{...i,...p};let o={...{},...(n=e.initialState)!=null?n:{}};a._features.forEach(p=>{var d;o=(d=p.getInitialState==null?void 0:p.getInitialState(o))!=null?d:o});const s=[];let c=!1;const f={_features:l,options:{...i,...e},initialState:o,_queue:p=>{s.push(p),c||(c=!0,Promise.resolve().then(()=>{for(;s.length;)s.shift()();c=!1}).catch(d=>setTimeout(()=>{throw d})))},reset:()=>{a.setState(a.initialState)},setOptions:p=>{const d=Rn(p,a.options);a.options=r(d)},getState:()=>a.options.state,setState:p=>{a.options.onStateChange==null||a.options.onStateChange(p)},_getRowId:(p,d,h)=>{var S;return(S=a.options.getRowId==null?void 0:a.options.getRowId(p,d,h))!=null?S:`${h?[h.id,d].join("."):d}`},getCoreRowModel:()=>(a._getCoreRowModel||(a._getCoreRowModel=a.options.getCoreRowModel(a)),a._getCoreRowModel()),getRowModel:()=>a.getPaginationRowModel(),getRow:(p,d)=>{let h=(d?a.getPrePaginationRowModel():a.getRowModel()).rowsById[p];if(!h&&(h=a.getCoreRowModel().rowsById[p],!h))throw new Error;return h},_getDefaultColumnDef:F(()=>[a.options.defaultColumn],p=>{var d;return p=(d=p)!=null?d:{},{header:h=>{const S=h.header.column.columnDef;return S.accessorKey?S.accessorKey:S.accessorFn?S.id:null},cell:h=>{var S,x;return(S=(x=h.renderValue())==null||x.toString==null?void 0:x.toString())!=null?S:null},...a._features.reduce((h,S)=>Object.assign(h,S.getDefaultColumnDef==null?void 0:S.getDefaultColumnDef()),{}),...p}},B(e,"debugColumns")),_getColumnDefs:()=>a.options.columns,getAllColumns:F(()=>[a._getColumnDefs()],p=>{const d=function(h,S,x){return x===void 0&&(x=0),h.map(y=>{const m=kw(a,y,x,S),v=y;return m.columns=v.columns?d(v.columns,m,x+1):[],m})};return d(p)},B(e,"debugColumns")),getAllFlatColumns:F(()=>[a.getAllColumns()],p=>p.flatMap(d=>d.getFlatColumns()),B(e,"debugColumns")),_getAllFlatColumnsById:F(()=>[a.getAllFlatColumns()],p=>p.reduce((d,h)=>(d[h.id]=h,d),{}),B(e,"debugColumns")),getAllLeafColumns:F(()=>[a.getAllColumns(),a._getOrderColumnsFn()],(p,d)=>{let h=p.flatMap(S=>S.getLeafColumns());return d(h)},B(e,"debugColumns")),getColumn:p=>a._getAllFlatColumnsById()[p]};Object.assign(a,f);for(let p=0;p<a._features.length;p++){const d=a._features[p];d==null||d.createTable==null||d.createTable(a)}return a}function Ab(){return e=>F(()=>[e.options.data],t=>{const n={rows:[],flatRows:[],rowsById:{}},l=function(a,i,r){i===void 0&&(i=0);const u=[];for(let s=0;s<a.length;s++){const c=tf(e,e._getRowId(a[s],s,r),a[s],s,i,void 0,r==null?void 0:r.id);if(n.flatRows.push(c),n.rowsById[c.id]=c,u.push(c),e.options.getSubRows){var o;c.originalSubRows=e.options.getSubRows(a[s],s),(o=c.originalSubRows)!=null&&o.length&&(c.subRows=l(c.originalSubRows,i+1,c))}}return u};return n.rows=l(t),n},B(e.options,"debugTable","getRowModel",()=>e._autoResetPageIndex()))}function Ob(e){const t=[],n=l=>{var a;t.push(l),(a=l.subRows)!=null&&a.length&&l.getIsExpanded()&&l.subRows.forEach(n)};return e.rows.forEach(n),{rows:t,flatRows:e.flatRows,rowsById:e.rowsById}}function Tb(){return(e,t)=>F(()=>{var n;return[(n=e.getColumn(t))==null?void 0:n.getFacetedRowModel()]},n=>{if(!n)return;const l=n.flatRows.flatMap(r=>{var u;return(u=r.getUniqueValues(t))!=null?u:[]}).map(Number).filter(r=>!Number.isNaN(r));if(!l.length)return;let a=l[0],i=l[l.length-1];for(const r of l)r<a?a=r:r>i&&(i=r);return[a,i]},B(e.options,"debugTable"))}function _m(e,t,n){return n.options.filterFromLeafRows?Db(e,t,n):Mb(e,t,n)}function Db(e,t,n){var l;const a=[],i={},r=(l=n.options.maxLeafRowFilterDepth)!=null?l:100,u=function(o,s){s===void 0&&(s=0);const c=[];for(let p=0;p<o.length;p++){var f;let d=o[p];const h=tf(n,d.id,d.original,d.index,d.depth,void 0,d.parentId);if(h.columnFilters=d.columnFilters,(f=d.subRows)!=null&&f.length&&s<r){if(h.subRows=u(d.subRows,s+1),d=h,t(d)&&!h.subRows.length){c.push(d),i[d.id]=d,a.push(d);continue}if(t(d)||h.subRows.length){c.push(d),i[d.id]=d,a.push(d);continue}}else d=h,t(d)&&(c.push(d),i[d.id]=d,a.push(d))}return c};return{rows:u(e),flatRows:a,rowsById:i}}function Mb(e,t,n){var l;const a=[],i={},r=(l=n.options.maxLeafRowFilterDepth)!=null?l:100,u=function(o,s){s===void 0&&(s=0);const c=[];for(let p=0;p<o.length;p++){let d=o[p];if(t(d)){var f;if((f=d.subRows)!=null&&f.length&&s<r){const S=tf(n,d.id,d.original,d.index,d.depth,void 0,d.parentId);S.subRows=u(d.subRows,s+1),d=S}c.push(d),a.push(d),i[d.id]=d}}return c};return{rows:u(e),flatRows:a,rowsById:i}}function jb(){return(e,t)=>F(()=>[e.getPreFilteredRowModel(),e.getState().columnFilters,e.getState().globalFilter,e.getFilteredRowModel()],(n,l,a)=>{if(!n.rows.length||!(l!=null&&l.length)&&!a)return n;const i=[...l.map(u=>u.id).filter(u=>u!==t),a?"__global__":void 0].filter(Boolean),r=u=>{for(let o=0;o<i.length;o++)if(u.columnFilters[i[o]]===!1)return!1;return!0};return _m(n.rows,r,e)},B(e.options,"debugTable"))}function Nb(){return(e,t)=>F(()=>{var n;return[(n=e.getColumn(t))==null?void 0:n.getFacetedRowModel()]},n=>{if(!n)return new Map;let l=new Map;for(let i=0;i<n.flatRows.length;i++){const r=n.flatRows[i].getUniqueValues(t);for(let u=0;u<r.length;u++){const o=r[u];if(l.has(o)){var a;l.set(o,((a=l.get(o))!=null?a:0)+1)}else l.set(o,1)}}return l},B(e.options,"debugTable"))}function zb(){return e=>F(()=>[e.getPreFilteredRowModel(),e.getState().columnFilters,e.getState().globalFilter],(t,n,l)=>{if(!t.rows.length||!(n!=null&&n.length)&&!l){for(let p=0;p<t.flatRows.length;p++)t.flatRows[p].columnFilters={},t.flatRows[p].columnFiltersMeta={};return t}const a=[],i=[];(n??[]).forEach(p=>{var d;const h=e.getColumn(p.id);if(!h)return;const S=h.getFilterFn();S&&a.push({id:p.id,filterFn:S,resolvedValue:(d=S.resolveFilterValue==null?void 0:S.resolveFilterValue(p.value))!=null?d:p.value})});const r=(n??[]).map(p=>p.id),u=e.getGlobalFilterFn(),o=e.getAllLeafColumns().filter(p=>p.getCanGlobalFilter());l&&u&&o.length&&(r.push("__global__"),o.forEach(p=>{var d;i.push({id:p.id,filterFn:u,resolvedValue:(d=u.resolveFilterValue==null?void 0:u.resolveFilterValue(l))!=null?d:l})}));let s,c;for(let p=0;p<t.flatRows.length;p++){const d=t.flatRows[p];if(d.columnFilters={},a.length)for(let h=0;h<a.length;h++){s=a[h];const S=s.id;d.columnFilters[S]=s.filterFn(d,S,s.resolvedValue,x=>{d.columnFiltersMeta[S]=x})}if(i.length){for(let h=0;h<i.length;h++){c=i[h];const S=c.id;if(c.filterFn(d,S,c.resolvedValue,x=>{d.columnFiltersMeta[S]=x})){d.columnFilters.__global__=!0;break}}d.columnFilters.__global__!==!0&&(d.columnFilters.__global__=!1)}}const f=p=>{for(let d=0;d<r.length;d++)if(p.columnFilters[r[d]]===!1)return!1;return!0};return _m(t.rows,f,e)},B(e.options,"debugTable","getFilteredRowModel",()=>e._autoResetPageIndex()))}function Ub(e){return t=>F(()=>[t.getState().pagination,t.getPrePaginationRowModel(),t.options.paginateExpandedRows?void 0:t.getState().expanded],(n,l)=>{if(!l.rows.length)return l;const{pageSize:a,pageIndex:i}=n;let{rows:r,flatRows:u,rowsById:o}=l;const s=a*i,c=s+a;r=r.slice(s,c);let f;t.options.paginateExpandedRows?f={rows:r,flatRows:u,rowsById:o}:f=Ob({rows:r,flatRows:u,rowsById:o}),f.flatRows=[];const p=d=>{f.flatRows.push(d),d.subRows.length&&d.subRows.forEach(p)};return f.rows.forEach(p),f},B(t.options,"debugTable"))}function Fb(){return e=>F(()=>[e.getState().sorting,e.getPreSortedRowModel()],(t,n)=>{if(!n.rows.length||!(t!=null&&t.length))return n;const l=e.getState().sorting,a=[],i=l.filter(o=>{var s;return(s=e.getColumn(o.id))==null?void 0:s.getCanSort()}),r={};i.forEach(o=>{const s=e.getColumn(o.id);s&&(r[o.id]={sortUndefined:s.columnDef.sortUndefined,invertSorting:s.columnDef.invertSorting,sortingFn:s.getSortingFn()})});const u=o=>{const s=o.map(c=>({...c}));return s.sort((c,f)=>{for(let d=0;d<i.length;d+=1){var p;const h=i[d],S=r[h.id],x=S.sortUndefined,y=(p=h==null?void 0:h.desc)!=null?p:!1;let m=0;if(x){const v=c.getValue(h.id),b=f.getValue(h.id),R=v===void 0,_=b===void 0;if(R||_){if(x==="first")return R?-1:1;if(x==="last")return R?1:-1;m=R&&_?0:R?x:-x}}if(m===0&&(m=S.sortingFn(c,f,h.id)),m!==0)return y&&(m*=-1),S.invertSorting&&(m*=-1),m}return c.index-f.index}),s.forEach(c=>{var f;a.push(c),(f=c.subRows)!=null&&f.length&&(c.subRows=u(c.subRows))}),s};return{rows:u(n.rows),flatRows:a,rowsById:n.rowsById}},B(e.options,"debugTable","getSortedRowModel",()=>e._autoResetPageIndex()))}/**
   * react-table
   *
   * Copyright (c) TanStack
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE.md file in the root directory of this source tree.
   *
   * @license MIT
   */function Jd(e,t){return e?Bb(e)?w.createElement(e,t):e:null}function Bb(e){return Hb(e)||typeof e=="function"||Lb(e)}function Hb(e){return typeof e=="function"&&(()=>{const t=Object.getPrototypeOf(e);return t.prototype&&t.prototype.isReactComponent})()}function Lb(e){return typeof e=="object"&&typeof e.$$typeof=="symbol"&&["react.memo","react.forward_ref"].includes(e.$$typeof.description)}function Vb(e){const t={state:{},onStateChange:()=>{},renderFallbackValue:null,...e},[n]=w.useState(()=>({current:_b(t)})),[l,a]=w.useState(()=>n.current.initialState);return n.current.setOptions(i=>({...i,...e,state:{...l,...e.state},onStateChange:r=>{a(r),e.onStateChange==null||e.onStateChange(r)}})),n.current}var Pd=typeof global<"u"?global:typeof self<"u"?self:typeof window<"u"?window:{},Tt=[],ot=[],$b=typeof Uint8Array<"u"?Uint8Array:Array,rf=!1;function Am(){rf=!0;for(var e="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",t=0,n=e.length;t<n;++t)Tt[t]=e[t],ot[e.charCodeAt(t)]=t;ot[45]=62,ot[95]=63}function qb(e){rf||Am();var t,n,l,a,i,r,u=e.length;if(u%4>0)throw new Error("Invalid string. Length must be a multiple of 4");i=e[u-2]==="="?2:e[u-1]==="="?1:0,r=new $b(u*3/4-i),l=i>0?u-4:u;var o=0;for(t=0,n=0;t<l;t+=4,n+=3)a=ot[e.charCodeAt(t)]<<18|ot[e.charCodeAt(t+1)]<<12|ot[e.charCodeAt(t+2)]<<6|ot[e.charCodeAt(t+3)],r[o++]=a>>16&255,r[o++]=a>>8&255,r[o++]=a&255;return i===2?(a=ot[e.charCodeAt(t)]<<2|ot[e.charCodeAt(t+1)]>>4,r[o++]=a&255):i===1&&(a=ot[e.charCodeAt(t)]<<10|ot[e.charCodeAt(t+1)]<<4|ot[e.charCodeAt(t+2)]>>2,r[o++]=a>>8&255,r[o++]=a&255),r}function Gb(e){return Tt[e>>18&63]+Tt[e>>12&63]+Tt[e>>6&63]+Tt[e&63]}function Yb(e,t,n){for(var l,a=[],i=t;i<n;i+=3)l=(e[i]<<16)+(e[i+1]<<8)+e[i+2],a.push(Gb(l));return a.join("")}function Id(e){rf||Am();for(var t,n=e.length,l=n%3,a="",i=[],r=16383,u=0,o=n-l;u<o;u+=r)i.push(Yb(e,u,u+r>o?o:u+r));return l===1?(t=e[n-1],a+=Tt[t>>2],a+=Tt[t<<4&63],a+="=="):l===2&&(t=(e[n-2]<<8)+e[n-1],a+=Tt[t>>10],a+=Tt[t>>4&63],a+=Tt[t<<2&63],a+="="),i.push(a),i.join("")}function Lu(e,t,n,l,a){var i,r,u=a*8-l-1,o=(1<<u)-1,s=o>>1,c=-7,f=n?a-1:0,p=n?-1:1,d=e[t+f];for(f+=p,i=d&(1<<-c)-1,d>>=-c,c+=u;c>0;i=i*256+e[t+f],f+=p,c-=8);for(r=i&(1<<-c)-1,i>>=-c,c+=l;c>0;r=r*256+e[t+f],f+=p,c-=8);if(i===0)i=1-s;else{if(i===o)return r?NaN:(d?-1:1)*(1/0);r=r+Math.pow(2,l),i=i-s}return(d?-1:1)*r*Math.pow(2,i-l)}function Om(e,t,n,l,a,i){var r,u,o,s=i*8-a-1,c=(1<<s)-1,f=c>>1,p=a===23?Math.pow(2,-24)-Math.pow(2,-77):0,d=l?0:i-1,h=l?1:-1,S=t<0||t===0&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(u=isNaN(t)?1:0,r=c):(r=Math.floor(Math.log(t)/Math.LN2),t*(o=Math.pow(2,-r))<1&&(r--,o*=2),r+f>=1?t+=p/o:t+=p*Math.pow(2,1-f),t*o>=2&&(r++,o/=2),r+f>=c?(u=0,r=c):r+f>=1?(u=(t*o-1)*Math.pow(2,a),r=r+f):(u=t*Math.pow(2,f-1)*Math.pow(2,a),r=0));a>=8;e[n+d]=u&255,d+=h,u/=256,a-=8);for(r=r<<a|u,s+=a;s>0;e[n+d]=r&255,d+=h,r/=256,s-=8);e[n+d-h]|=S*128}var Xb={}.toString,Tm=Array.isArray||function(e){return Xb.call(e)=="[object Array]"},kb=50;T.TYPED_ARRAY_SUPPORT=Pd.TYPED_ARRAY_SUPPORT!==void 0?Pd.TYPED_ARRAY_SUPPORT:!0;Qr();function Qr(){return T.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function Jt(e,t){if(Qr()<t)throw new RangeError("Invalid typed array length");return T.TYPED_ARRAY_SUPPORT?(e=new Uint8Array(t),e.__proto__=T.prototype):(e===null&&(e=new T(t)),e.length=t),e}function T(e,t,n){if(!T.TYPED_ARRAY_SUPPORT&&!(this instanceof T))return new T(e,t,n);if(typeof e=="number"){if(typeof t=="string")throw new Error("If encoding is specified then the first argument must be a string");return uf(this,e)}return Dm(this,e,t,n)}T.poolSize=8192;T._augment=function(e){return e.__proto__=T.prototype,e};function Dm(e,t,n,l){if(typeof t=="number")throw new TypeError('"value" argument must not be a number');return typeof ArrayBuffer<"u"&&t instanceof ArrayBuffer?Kb(e,t,n,l):typeof t=="string"?Zb(e,t,n):Jb(e,t)}T.from=function(e,t,n){return Dm(null,e,t,n)};T.TYPED_ARRAY_SUPPORT&&(T.prototype.__proto__=Uint8Array.prototype,T.__proto__=Uint8Array,typeof Symbol<"u"&&Symbol.species&&T[Symbol.species]);function Mm(e){if(typeof e!="number")throw new TypeError('"size" argument must be a number');if(e<0)throw new RangeError('"size" argument must not be negative')}function Qb(e,t,n,l){return Mm(t),t<=0?Jt(e,t):n!==void 0?typeof l=="string"?Jt(e,t).fill(n,l):Jt(e,t).fill(n):Jt(e,t)}T.alloc=function(e,t,n){return Qb(null,e,t,n)};function uf(e,t){if(Mm(t),e=Jt(e,t<0?0:of(t)|0),!T.TYPED_ARRAY_SUPPORT)for(var n=0;n<t;++n)e[n]=0;return e}T.allocUnsafe=function(e){return uf(null,e)};T.allocUnsafeSlow=function(e){return uf(null,e)};function Zb(e,t,n){if((typeof n!="string"||n==="")&&(n="utf8"),!T.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var l=jm(t,n)|0;e=Jt(e,l);var a=e.write(t,n);return a!==l&&(e=e.slice(0,a)),e}function Os(e,t){var n=t.length<0?0:of(t.length)|0;e=Jt(e,n);for(var l=0;l<n;l+=1)e[l]=t[l]&255;return e}function Kb(e,t,n,l){if(t.byteLength,n<0||t.byteLength<n)throw new RangeError("'offset' is out of bounds");if(t.byteLength<n+(l||0))throw new RangeError("'length' is out of bounds");return n===void 0&&l===void 0?t=new Uint8Array(t):l===void 0?t=new Uint8Array(t,n):t=new Uint8Array(t,n,l),T.TYPED_ARRAY_SUPPORT?(e=t,e.__proto__=T.prototype):e=Os(e,t),e}function Jb(e,t){if(Ht(t)){var n=of(t.length)|0;return e=Jt(e,n),e.length===0||t.copy(e,0,0,n),e}if(t){if(typeof ArrayBuffer<"u"&&t.buffer instanceof ArrayBuffer||"length"in t)return typeof t.length!="number"||hR(t.length)?Jt(e,0):Os(e,t);if(t.type==="Buffer"&&Tm(t.data))return Os(e,t.data)}throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}function of(e){if(e>=Qr())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+Qr().toString(16)+" bytes");return e|0}T.isBuffer=zl;function Ht(e){return!!(e!=null&&e._isBuffer)}T.compare=function(t,n){if(!Ht(t)||!Ht(n))throw new TypeError("Arguments must be Buffers");if(t===n)return 0;for(var l=t.length,a=n.length,i=0,r=Math.min(l,a);i<r;++i)if(t[i]!==n[i]){l=t[i],a=n[i];break}return l<a?-1:a<l?1:0};T.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}};T.concat=function(t,n){if(!Tm(t))throw new TypeError('"list" argument must be an Array of Buffers');if(t.length===0)return T.alloc(0);var l;if(n===void 0)for(n=0,l=0;l<t.length;++l)n+=t[l].length;var a=T.allocUnsafe(n),i=0;for(l=0;l<t.length;++l){var r=t[l];if(!Ht(r))throw new TypeError('"list" argument must be an Array of Buffers');r.copy(a,i),i+=r.length}return a};function jm(e,t){if(Ht(e))return e.length;if(typeof ArrayBuffer<"u"&&typeof ArrayBuffer.isView=="function"&&(ArrayBuffer.isView(e)||e instanceof ArrayBuffer))return e.byteLength;typeof e!="string"&&(e=""+e);var n=e.length;if(n===0)return 0;for(var l=!1;;)switch(t){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return Zr(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return n*2;case"hex":return n>>>1;case"base64":return Lm(e).length;default:if(l)return Zr(e).length;t=(""+t).toLowerCase(),l=!0}}T.byteLength=jm;function Pb(e,t,n){var l=!1;if((t===void 0||t<0)&&(t=0),t>this.length||((n===void 0||n>this.length)&&(n=this.length),n<=0)||(n>>>=0,t>>>=0,n<=t))return"";for(e||(e="utf8");;)switch(e){case"hex":return uR(this,t,n);case"utf8":case"utf-8":return Um(this,t,n);case"ascii":return iR(this,t,n);case"latin1":case"binary":return rR(this,t,n);case"base64":return lR(this,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return oR(this,t,n);default:if(l)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),l=!0}}T.prototype._isBuffer=!0;function el(e,t,n){var l=e[t];e[t]=e[n],e[n]=l}T.prototype.swap16=function(){var t=this.length;if(t%2!==0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var n=0;n<t;n+=2)el(this,n,n+1);return this};T.prototype.swap32=function(){var t=this.length;if(t%4!==0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var n=0;n<t;n+=4)el(this,n,n+3),el(this,n+1,n+2);return this};T.prototype.swap64=function(){var t=this.length;if(t%8!==0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var n=0;n<t;n+=8)el(this,n,n+7),el(this,n+1,n+6),el(this,n+2,n+5),el(this,n+3,n+4);return this};T.prototype.toString=function(){var t=this.length|0;return t===0?"":arguments.length===0?Um(this,0,t):Pb.apply(this,arguments)};T.prototype.equals=function(t){if(!Ht(t))throw new TypeError("Argument must be a Buffer");return this===t?!0:T.compare(this,t)===0};T.prototype.inspect=function(){var t="",n=kb;return this.length>0&&(t=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(t+=" ... ")),"<Buffer "+t+">"};T.prototype.compare=function(t,n,l,a,i){if(!Ht(t))throw new TypeError("Argument must be a Buffer");if(n===void 0&&(n=0),l===void 0&&(l=t?t.length:0),a===void 0&&(a=0),i===void 0&&(i=this.length),n<0||l>t.length||a<0||i>this.length)throw new RangeError("out of range index");if(a>=i&&n>=l)return 0;if(a>=i)return-1;if(n>=l)return 1;if(n>>>=0,l>>>=0,a>>>=0,i>>>=0,this===t)return 0;for(var r=i-a,u=l-n,o=Math.min(r,u),s=this.slice(a,i),c=t.slice(n,l),f=0;f<o;++f)if(s[f]!==c[f]){r=s[f],u=c[f];break}return r<u?-1:u<r?1:0};function Nm(e,t,n,l,a){if(e.length===0)return-1;if(typeof n=="string"?(l=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=a?0:e.length-1),n<0&&(n=e.length+n),n>=e.length){if(a)return-1;n=e.length-1}else if(n<0)if(a)n=0;else return-1;if(typeof t=="string"&&(t=T.from(t,l)),Ht(t))return t.length===0?-1:Wd(e,t,n,l,a);if(typeof t=="number")return t=t&255,T.TYPED_ARRAY_SUPPORT&&typeof Uint8Array.prototype.indexOf=="function"?a?Uint8Array.prototype.indexOf.call(e,t,n):Uint8Array.prototype.lastIndexOf.call(e,t,n):Wd(e,[t],n,l,a);throw new TypeError("val must be string, number or Buffer")}function Wd(e,t,n,l,a){var i=1,r=e.length,u=t.length;if(l!==void 0&&(l=String(l).toLowerCase(),l==="ucs2"||l==="ucs-2"||l==="utf16le"||l==="utf-16le")){if(e.length<2||t.length<2)return-1;i=2,r/=2,u/=2,n/=2}function o(d,h){return i===1?d[h]:d.readUInt16BE(h*i)}var s;if(a){var c=-1;for(s=n;s<r;s++)if(o(e,s)===o(t,c===-1?0:s-c)){if(c===-1&&(c=s),s-c+1===u)return c*i}else c!==-1&&(s-=s-c),c=-1}else for(n+u>r&&(n=r-u),s=n;s>=0;s--){for(var f=!0,p=0;p<u;p++)if(o(e,s+p)!==o(t,p)){f=!1;break}if(f)return s}return-1}T.prototype.includes=function(t,n,l){return this.indexOf(t,n,l)!==-1};T.prototype.indexOf=function(t,n,l){return Nm(this,t,n,l,!0)};T.prototype.lastIndexOf=function(t,n,l){return Nm(this,t,n,l,!1)};function Ib(e,t,n,l){n=Number(n)||0;var a=e.length-n;l?(l=Number(l),l>a&&(l=a)):l=a;var i=t.length;if(i%2!==0)throw new TypeError("Invalid hex string");l>i/2&&(l=i/2);for(var r=0;r<l;++r){var u=parseInt(t.substr(r*2,2),16);if(isNaN(u))return r;e[n+r]=u}return r}function Wb(e,t,n,l){return qu(Zr(t,e.length-n),e,n,l)}function zm(e,t,n,l){return qu(pR(t),e,n,l)}function eR(e,t,n,l){return zm(e,t,n,l)}function tR(e,t,n,l){return qu(Lm(t),e,n,l)}function nR(e,t,n,l){return qu(gR(t,e.length-n),e,n,l)}T.prototype.write=function(t,n,l,a){if(n===void 0)a="utf8",l=this.length,n=0;else if(l===void 0&&typeof n=="string")a=n,l=this.length,n=0;else if(isFinite(n))n=n|0,isFinite(l)?(l=l|0,a===void 0&&(a="utf8")):(a=l,l=void 0);else throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");var i=this.length-n;if((l===void 0||l>i)&&(l=i),t.length>0&&(l<0||n<0)||n>this.length)throw new RangeError("Attempt to write outside buffer bounds");a||(a="utf8");for(var r=!1;;)switch(a){case"hex":return Ib(this,t,n,l);case"utf8":case"utf-8":return Wb(this,t,n,l);case"ascii":return zm(this,t,n,l);case"latin1":case"binary":return eR(this,t,n,l);case"base64":return tR(this,t,n,l);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return nR(this,t,n,l);default:if(r)throw new TypeError("Unknown encoding: "+a);a=(""+a).toLowerCase(),r=!0}};T.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};function lR(e,t,n){return t===0&&n===e.length?Id(e):Id(e.slice(t,n))}function Um(e,t,n){n=Math.min(e.length,n);for(var l=[],a=t;a<n;){var i=e[a],r=null,u=i>239?4:i>223?3:i>191?2:1;if(a+u<=n){var o,s,c,f;switch(u){case 1:i<128&&(r=i);break;case 2:o=e[a+1],(o&192)===128&&(f=(i&31)<<6|o&63,f>127&&(r=f));break;case 3:o=e[a+1],s=e[a+2],(o&192)===128&&(s&192)===128&&(f=(i&15)<<12|(o&63)<<6|s&63,f>2047&&(f<55296||f>57343)&&(r=f));break;case 4:o=e[a+1],s=e[a+2],c=e[a+3],(o&192)===128&&(s&192)===128&&(c&192)===128&&(f=(i&15)<<18|(o&63)<<12|(s&63)<<6|c&63,f>65535&&f<1114112&&(r=f))}}r===null?(r=65533,u=1):r>65535&&(r-=65536,l.push(r>>>10&1023|55296),r=56320|r&1023),l.push(r),a+=u}return aR(l)}var ep=4096;function aR(e){var t=e.length;if(t<=ep)return String.fromCharCode.apply(String,e);for(var n="",l=0;l<t;)n+=String.fromCharCode.apply(String,e.slice(l,l+=ep));return n}function iR(e,t,n){var l="";n=Math.min(e.length,n);for(var a=t;a<n;++a)l+=String.fromCharCode(e[a]&127);return l}function rR(e,t,n){var l="";n=Math.min(e.length,n);for(var a=t;a<n;++a)l+=String.fromCharCode(e[a]);return l}function uR(e,t,n){var l=e.length;(!t||t<0)&&(t=0),(!n||n<0||n>l)&&(n=l);for(var a="",i=t;i<n;++i)a+=dR(e[i]);return a}function oR(e,t,n){for(var l=e.slice(t,n),a="",i=0;i<l.length;i+=2)a+=String.fromCharCode(l[i]+l[i+1]*256);return a}T.prototype.slice=function(t,n){var l=this.length;t=~~t,n=n===void 0?l:~~n,t<0?(t+=l,t<0&&(t=0)):t>l&&(t=l),n<0?(n+=l,n<0&&(n=0)):n>l&&(n=l),n<t&&(n=t);var a;if(T.TYPED_ARRAY_SUPPORT)a=this.subarray(t,n),a.__proto__=T.prototype;else{var i=n-t;a=new T(i,void 0);for(var r=0;r<i;++r)a[r]=this[r+t]}return a};function _e(e,t,n){if(e%1!==0||e<0)throw new RangeError("offset is not uint");if(e+t>n)throw new RangeError("Trying to access beyond buffer length")}T.prototype.readUIntLE=function(t,n,l){t=t|0,n=n|0,l||_e(t,n,this.length);for(var a=this[t],i=1,r=0;++r<n&&(i*=256);)a+=this[t+r]*i;return a};T.prototype.readUIntBE=function(t,n,l){t=t|0,n=n|0,l||_e(t,n,this.length);for(var a=this[t+--n],i=1;n>0&&(i*=256);)a+=this[t+--n]*i;return a};T.prototype.readUInt8=function(t,n){return n||_e(t,1,this.length),this[t]};T.prototype.readUInt16LE=function(t,n){return n||_e(t,2,this.length),this[t]|this[t+1]<<8};T.prototype.readUInt16BE=function(t,n){return n||_e(t,2,this.length),this[t]<<8|this[t+1]};T.prototype.readUInt32LE=function(t,n){return n||_e(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+this[t+3]*16777216};T.prototype.readUInt32BE=function(t,n){return n||_e(t,4,this.length),this[t]*16777216+(this[t+1]<<16|this[t+2]<<8|this[t+3])};T.prototype.readIntLE=function(t,n,l){t=t|0,n=n|0,l||_e(t,n,this.length);for(var a=this[t],i=1,r=0;++r<n&&(i*=256);)a+=this[t+r]*i;return i*=128,a>=i&&(a-=Math.pow(2,8*n)),a};T.prototype.readIntBE=function(t,n,l){t=t|0,n=n|0,l||_e(t,n,this.length);for(var a=n,i=1,r=this[t+--a];a>0&&(i*=256);)r+=this[t+--a]*i;return i*=128,r>=i&&(r-=Math.pow(2,8*n)),r};T.prototype.readInt8=function(t,n){return n||_e(t,1,this.length),this[t]&128?(255-this[t]+1)*-1:this[t]};T.prototype.readInt16LE=function(t,n){n||_e(t,2,this.length);var l=this[t]|this[t+1]<<8;return l&32768?l|4294901760:l};T.prototype.readInt16BE=function(t,n){n||_e(t,2,this.length);var l=this[t+1]|this[t]<<8;return l&32768?l|4294901760:l};T.prototype.readInt32LE=function(t,n){return n||_e(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24};T.prototype.readInt32BE=function(t,n){return n||_e(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]};T.prototype.readFloatLE=function(t,n){return n||_e(t,4,this.length),Lu(this,t,!0,23,4)};T.prototype.readFloatBE=function(t,n){return n||_e(t,4,this.length),Lu(this,t,!1,23,4)};T.prototype.readDoubleLE=function(t,n){return n||_e(t,8,this.length),Lu(this,t,!0,52,8)};T.prototype.readDoubleBE=function(t,n){return n||_e(t,8,this.length),Lu(this,t,!1,52,8)};function Ye(e,t,n,l,a,i){if(!Ht(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(t>a||t<i)throw new RangeError('"value" argument is out of bounds');if(n+l>e.length)throw new RangeError("Index out of range")}T.prototype.writeUIntLE=function(t,n,l,a){if(t=+t,n=n|0,l=l|0,!a){var i=Math.pow(2,8*l)-1;Ye(this,t,n,l,i,0)}var r=1,u=0;for(this[n]=t&255;++u<l&&(r*=256);)this[n+u]=t/r&255;return n+l};T.prototype.writeUIntBE=function(t,n,l,a){if(t=+t,n=n|0,l=l|0,!a){var i=Math.pow(2,8*l)-1;Ye(this,t,n,l,i,0)}var r=l-1,u=1;for(this[n+r]=t&255;--r>=0&&(u*=256);)this[n+r]=t/u&255;return n+l};T.prototype.writeUInt8=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,1,255,0),T.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),this[n]=t&255,n+1};function Vu(e,t,n,l){t<0&&(t=65535+t+1);for(var a=0,i=Math.min(e.length-n,2);a<i;++a)e[n+a]=(t&255<<8*(l?a:1-a))>>>(l?a:1-a)*8}T.prototype.writeUInt16LE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,2,65535,0),T.TYPED_ARRAY_SUPPORT?(this[n]=t&255,this[n+1]=t>>>8):Vu(this,t,n,!0),n+2};T.prototype.writeUInt16BE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,2,65535,0),T.TYPED_ARRAY_SUPPORT?(this[n]=t>>>8,this[n+1]=t&255):Vu(this,t,n,!1),n+2};function $u(e,t,n,l){t<0&&(t=4294967295+t+1);for(var a=0,i=Math.min(e.length-n,4);a<i;++a)e[n+a]=t>>>(l?a:3-a)*8&255}T.prototype.writeUInt32LE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,4,4294967295,0),T.TYPED_ARRAY_SUPPORT?(this[n+3]=t>>>24,this[n+2]=t>>>16,this[n+1]=t>>>8,this[n]=t&255):$u(this,t,n,!0),n+4};T.prototype.writeUInt32BE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,4,4294967295,0),T.TYPED_ARRAY_SUPPORT?(this[n]=t>>>24,this[n+1]=t>>>16,this[n+2]=t>>>8,this[n+3]=t&255):$u(this,t,n,!1),n+4};T.prototype.writeIntLE=function(t,n,l,a){if(t=+t,n=n|0,!a){var i=Math.pow(2,8*l-1);Ye(this,t,n,l,i-1,-i)}var r=0,u=1,o=0;for(this[n]=t&255;++r<l&&(u*=256);)t<0&&o===0&&this[n+r-1]!==0&&(o=1),this[n+r]=(t/u>>0)-o&255;return n+l};T.prototype.writeIntBE=function(t,n,l,a){if(t=+t,n=n|0,!a){var i=Math.pow(2,8*l-1);Ye(this,t,n,l,i-1,-i)}var r=l-1,u=1,o=0;for(this[n+r]=t&255;--r>=0&&(u*=256);)t<0&&o===0&&this[n+r+1]!==0&&(o=1),this[n+r]=(t/u>>0)-o&255;return n+l};T.prototype.writeInt8=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,1,127,-128),T.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),t<0&&(t=255+t+1),this[n]=t&255,n+1};T.prototype.writeInt16LE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,2,32767,-32768),T.TYPED_ARRAY_SUPPORT?(this[n]=t&255,this[n+1]=t>>>8):Vu(this,t,n,!0),n+2};T.prototype.writeInt16BE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,2,32767,-32768),T.TYPED_ARRAY_SUPPORT?(this[n]=t>>>8,this[n+1]=t&255):Vu(this,t,n,!1),n+2};T.prototype.writeInt32LE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,4,2147483647,-2147483648),T.TYPED_ARRAY_SUPPORT?(this[n]=t&255,this[n+1]=t>>>8,this[n+2]=t>>>16,this[n+3]=t>>>24):$u(this,t,n,!0),n+4};T.prototype.writeInt32BE=function(t,n,l){return t=+t,n=n|0,l||Ye(this,t,n,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),T.TYPED_ARRAY_SUPPORT?(this[n]=t>>>24,this[n+1]=t>>>16,this[n+2]=t>>>8,this[n+3]=t&255):$u(this,t,n,!1),n+4};function Fm(e,t,n,l,a,i){if(n+l>e.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function Bm(e,t,n,l,a){return a||Fm(e,t,n,4),Om(e,t,n,l,23,4),n+4}T.prototype.writeFloatLE=function(t,n,l){return Bm(this,t,n,!0,l)};T.prototype.writeFloatBE=function(t,n,l){return Bm(this,t,n,!1,l)};function Hm(e,t,n,l,a){return a||Fm(e,t,n,8),Om(e,t,n,l,52,8),n+8}T.prototype.writeDoubleLE=function(t,n,l){return Hm(this,t,n,!0,l)};T.prototype.writeDoubleBE=function(t,n,l){return Hm(this,t,n,!1,l)};T.prototype.copy=function(t,n,l,a){if(l||(l=0),!a&&a!==0&&(a=this.length),n>=t.length&&(n=t.length),n||(n=0),a>0&&a<l&&(a=l),a===l||t.length===0||this.length===0)return 0;if(n<0)throw new RangeError("targetStart out of bounds");if(l<0||l>=this.length)throw new RangeError("sourceStart out of bounds");if(a<0)throw new RangeError("sourceEnd out of bounds");a>this.length&&(a=this.length),t.length-n<a-l&&(a=t.length-n+l);var i=a-l,r;if(this===t&&l<n&&n<a)for(r=i-1;r>=0;--r)t[r+n]=this[r+l];else if(i<1e3||!T.TYPED_ARRAY_SUPPORT)for(r=0;r<i;++r)t[r+n]=this[r+l];else Uint8Array.prototype.set.call(t,this.subarray(l,l+i),n);return i};T.prototype.fill=function(t,n,l,a){if(typeof t=="string"){if(typeof n=="string"?(a=n,n=0,l=this.length):typeof l=="string"&&(a=l,l=this.length),t.length===1){var i=t.charCodeAt(0);i<256&&(t=i)}if(a!==void 0&&typeof a!="string")throw new TypeError("encoding must be a string");if(typeof a=="string"&&!T.isEncoding(a))throw new TypeError("Unknown encoding: "+a)}else typeof t=="number"&&(t=t&255);if(n<0||this.length<n||this.length<l)throw new RangeError("Out of range index");if(l<=n)return this;n=n>>>0,l=l===void 0?this.length:l>>>0,t||(t=0);var r;if(typeof t=="number")for(r=n;r<l;++r)this[r]=t;else{var u=Ht(t)?t:Zr(new T(t,a).toString()),o=u.length;for(r=0;r<l-n;++r)this[r+n]=u[r%o]}return this};var sR=/[^+\/0-9A-Za-z-_]/g;function cR(e){if(e=fR(e).replace(sR,""),e.length<2)return"";for(;e.length%4!==0;)e=e+"=";return e}function fR(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}function dR(e){return e<16?"0"+e.toString(16):e.toString(16)}function Zr(e,t){t=t||1/0;for(var n,l=e.length,a=null,i=[],r=0;r<l;++r){if(n=e.charCodeAt(r),n>55295&&n<57344){if(!a){if(n>56319){(t-=3)>-1&&i.push(239,191,189);continue}else if(r+1===l){(t-=3)>-1&&i.push(239,191,189);continue}a=n;continue}if(n<56320){(t-=3)>-1&&i.push(239,191,189),a=n;continue}n=(a-55296<<10|n-56320)+65536}else a&&(t-=3)>-1&&i.push(239,191,189);if(a=null,n<128){if((t-=1)<0)break;i.push(n)}else if(n<2048){if((t-=2)<0)break;i.push(n>>6|192,n&63|128)}else if(n<65536){if((t-=3)<0)break;i.push(n>>12|224,n>>6&63|128,n&63|128)}else if(n<1114112){if((t-=4)<0)break;i.push(n>>18|240,n>>12&63|128,n>>6&63|128,n&63|128)}else throw new Error("Invalid code point")}return i}function pR(e){for(var t=[],n=0;n<e.length;++n)t.push(e.charCodeAt(n)&255);return t}function gR(e,t){for(var n,l,a,i=[],r=0;r<e.length&&!((t-=2)<0);++r)n=e.charCodeAt(r),l=n>>8,a=n%256,i.push(a),i.push(l);return i}function Lm(e){return qb(cR(e))}function qu(e,t,n,l){for(var a=0;a<l&&!(a+n>=t.length||a>=e.length);++a)t[a+n]=e[a];return a}function hR(e){return e!==e}function zl(e){return e!=null&&(!!e._isBuffer||Vm(e)||mR(e))}function Vm(e){return!!e.constructor&&typeof e.constructor.isBuffer=="function"&&e.constructor.isBuffer(e)}function mR(e){return typeof e.readFloatLE=="function"&&typeof e.slice=="function"&&Vm(e.slice(0,0))}const yR=46,vR=/\\(\\)?/g,SR=RegExp(`[^.[\\]]+|\\[(?:([^"'][^[]*)|(["'])((?:(?!\\2)[^\\\\]|\\\\.)*?)\\2)\\]|(?=(?:\\.|\\[\\])(?:\\.|\\[\\]|$))`,"g"),xR=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,wR=/^\w*$/,bR=function(e){return Object.prototype.toString.call(e)},$m=function(e){const t=typeof e;return t==="symbol"||t==="object"&&e&&bR(e)==="[object Symbol]"},RR=function(e,t){if(Array.isArray(e))return!1;const n=typeof e;return n==="number"||n==="symbol"||n==="boolean"||!e||$m(e)?!0:wR.test(e)||!xR.test(e)||t!=null&&e in Object(t)},ER=function(e){const t=[];return e.charCodeAt(0)===yR&&t.push(""),e.replace(SR,function(n,l,a,i){let r=n;a?r=i.replace(vR,"$1"):l&&(r=l.trim()),t.push(r)}),t},CR=function(e,t){return Array.isArray(e)?e:RR(e,t)?[e]:ER(e)},_R=function(e){if(typeof e=="string"||$m(e))return e;const t=`${e}`;return t=="0"&&1/e==-INFINITY?"-0":t},AR=function(e,t){t=CR(t,e);let n=0;const l=t.length;for(;e!=null&&n<l;)e=e[_R(t[n++])];return n&&n===l?e:void 0},OR=function(e){return typeof e=="object"&&e!==null&&!Array.isArray(e)},qm=function(e){if(e==null)return[void 0,void 0];if(typeof e!="object")return[Error('Invalid option "columns": expect an array or an object')];if(Array.isArray(e)){const t=[];for(const n of e)if(typeof n=="string")t.push({key:n,header:n});else if(typeof n=="object"&&n!==null&&!Array.isArray(n)){if(!n.key)return[Error('Invalid column definition: property "key" is required')];n.header===void 0&&(n.header=n.key),t.push(n)}else return[Error("Invalid column definition: expect a string or an object")];e=t}else{const t=[];for(const n in e)t.push({key:n,header:e[n]});e=t}return[void 0,e]};class Ul extends Error{constructor(t,n,...l){Array.isArray(n)&&(n=n.join(" ")),super(n),Error.captureStackTrace!==void 0&&Error.captureStackTrace(this,Ul),this.code=t;for(const a of l)for(const i in a){const r=a[i];this[i]=zl(r)?r.toString():r==null?r:JSON.parse(JSON.stringify(r))}}}const TR=function(e){return e.replace(/([A-Z])/g,function(t,n){return"_"+n.toLowerCase()})},Gm=function(e){const t={};for(const a in e)t[TR(a)]=e[a];if(t.bom===void 0||t.bom===null||t.bom===!1)t.bom=!1;else if(t.bom!==!0)return[new Ul("CSV_OPTION_BOOLEAN_INVALID_TYPE",["option `bom` is optional and must be a boolean value,",`got ${JSON.stringify(t.bom)}`])];if(t.delimiter===void 0||t.delimiter===null)t.delimiter=",";else if(zl(t.delimiter))t.delimiter=t.delimiter.toString();else if(typeof t.delimiter!="string")return[new Ul("CSV_OPTION_DELIMITER_INVALID_TYPE",["option `delimiter` must be a buffer or a string,",`got ${JSON.stringify(t.delimiter)}`])];if(t.quote===void 0||t.quote===null)t.quote='"';else if(t.quote===!0)t.quote='"';else if(t.quote===!1)t.quote="";else if(zl(t.quote))t.quote=t.quote.toString();else if(typeof t.quote!="string")return[new Ul("CSV_OPTION_QUOTE_INVALID_TYPE",["option `quote` must be a boolean, a buffer or a string,",`got ${JSON.stringify(t.quote)}`])];if((t.quoted===void 0||t.quoted===null)&&(t.quoted=!1),t.escape_formulas===void 0||t.escape_formulas===null)t.escape_formulas=!1;else if(typeof t.escape_formulas!="boolean")return[new Ul("CSV_OPTION_ESCAPE_FORMULAS_INVALID_TYPE",["option `escape_formulas` must be a boolean,",`got ${JSON.stringify(t.escape_formulas)}`])];if((t.quoted_empty===void 0||t.quoted_empty===null)&&(t.quoted_empty=void 0),t.quoted_match===void 0||t.quoted_match===null||t.quoted_match===!1?t.quoted_match=null:Array.isArray(t.quoted_match)||(t.quoted_match=[t.quoted_match]),t.quoted_match)for(const a of t.quoted_match){const i=typeof a=="string",r=a instanceof RegExp;if(!i&&!r)return[Error(`Invalid Option: quoted_match must be a string or a regex, got ${JSON.stringify(a)}`)]}if((t.quoted_string===void 0||t.quoted_string===null)&&(t.quoted_string=!1),(t.eof===void 0||t.eof===null)&&(t.eof=!0),t.escape===void 0||t.escape===null)t.escape='"';else if(zl(t.escape))t.escape=t.escape.toString();else if(typeof t.escape!="string")return[Error(`Invalid Option: escape must be a buffer or a string, got ${JSON.stringify(t.escape)}`)];if(t.escape.length>1)return[Error(`Invalid Option: escape must be one character, got ${t.escape.length} characters`)];(t.header===void 0||t.header===null)&&(t.header=!1);const[n,l]=qm(t.columns);if(n!==void 0)return[n];if(t.columns=l,(t.quoted===void 0||t.quoted===null)&&(t.quoted=!1),(t.cast===void 0||t.cast===null)&&(t.cast={}),(t.cast.bigint===void 0||t.cast.bigint===null)&&(t.cast.bigint=a=>""+a),(t.cast.boolean===void 0||t.cast.boolean===null)&&(t.cast.boolean=a=>a?"1":""),(t.cast.date===void 0||t.cast.date===null)&&(t.cast.date=a=>""+a.getTime()),(t.cast.number===void 0||t.cast.number===null)&&(t.cast.number=a=>""+a),(t.cast.object===void 0||t.cast.object===null)&&(t.cast.object=a=>JSON.stringify(a)),(t.cast.string===void 0||t.cast.string===null)&&(t.cast.string=function(a){return a}),t.on_record!==void 0&&typeof t.on_record!="function")return[Error('Invalid Option: "on_record" must be a function.')];if(t.record_delimiter===void 0||t.record_delimiter===null)t.record_delimiter=`
`;else if(zl(t.record_delimiter))t.record_delimiter=t.record_delimiter.toString();else if(typeof t.record_delimiter!="string")return[Error(`Invalid Option: record_delimiter must be a buffer or a string, got ${JSON.stringify(t.record_delimiter)}`)];switch(t.record_delimiter){case"unix":t.record_delimiter=`
`;break;case"mac":t.record_delimiter="\r";break;case"windows":t.record_delimiter=`\r
`;break;case"ascii":t.record_delimiter="";break;case"unicode":t.record_delimiter="\u2028";break}return[void 0,t]},DR=T.from([239,187,191]),MR=function(e,t,n){return{options:e,state:t,info:n,__transform:function(l,a){if(!Array.isArray(l)&&typeof l!="object")return Error(`Invalid Record: expect an array or an object, got ${JSON.stringify(l)}`);if(this.info.records===0){if(Array.isArray(l)){if(this.options.header===!0&&this.options.columns===void 0)return Error("Undiscoverable Columns: header option requires column option or object records")}else if(this.options.columns===void 0){const[u,o]=qm(Object.keys(l));if(u)return;this.options.columns=o}}if(this.info.records===0){this.bom(a);const u=this.headers(a);if(u)return u}try{this.options.on_record&&this.options.on_record(l,this.info.records)}catch(u){return u}let i,r;if(this.options.eof){if([i,r]=this.stringify(l),i)return i;if(r===void 0)return;r=r+this.options.record_delimiter}else{if([i,r]=this.stringify(l),i)return i;if(r===void 0)return;(this.options.header||this.info.records)&&(r=this.options.record_delimiter+r)}this.info.records++,a(r)},stringify:function(l,a=!1){if(typeof l!="object")return[void 0,l];const{columns:i}=this.options,r=[];if(Array.isArray(l)){i&&l.splice(i.length);for(let o=0;o<l.length;o++){const s=l[o],[c,f]=this.__cast(s,{index:o,column:o,records:this.info.records,header:a});if(c)return[c];r[o]=[f,s]}}else for(let o=0;o<i.length;o++){const s=AR(l,i[o].key),[c,f]=this.__cast(s,{index:o,column:i[o].key,records:this.info.records,header:a});if(c)return[c];r[o]=[f,s]}let u="";for(let o=0;o<r.length;o++){let s,c,[f,p]=r[o];if(typeof f=="string")s=this.options;else if(OR(f)){if(s=f,f=s.value,delete s.value,typeof f!="string"&&f!==void 0&&f!==null&&c)return[Error(`Invalid Casting Value: returned value must return a string, null or undefined, got ${JSON.stringify(f)}`)];if(s={...this.options,...s},[c,s]=Gm(s),c!==void 0)return[c]}else if(f==null)s=this.options;else return[Error(`Invalid Casting Value: returned value must return a string, an object, null or undefined, got ${JSON.stringify(f)}`)];const{delimiter:d,escape:h,quote:S,quoted:x,quoted_empty:y,quoted_string:m,quoted_match:v,record_delimiter:b,escape_formulas:R}=s;if(f===""&&p===""){let _=v&&v.filter(A=>typeof A=="string"?f.indexOf(A)!==-1:A.test(f));_=_&&_.length>0,(_||y===!0||m===!0&&y!==!1)===!0&&(f=S+f+S),u+=f}else if(f){if(typeof f!="string")return[Error(`Formatter must return a string, null or undefined, got ${JSON.stringify(f)}`)];const _=d.length&&f.indexOf(d)>=0,E=S!==""&&f.indexOf(S)>=0,A=f.indexOf(h)>=0&&h!==S,N=f.indexOf(b)>=0,M=m&&typeof p=="string";let j=v&&v.filter(H=>typeof H=="string"?f.indexOf(H)!==-1:H.test(f));if(j=j&&j.length>0,R)switch(f[0]){case"=":case"+":case"-":case"@":case"	":case"\r":case"＝":case"＋":case"－":case"＠":f=`'${f}`;break}const V=E===!0||_||N||x||M||j;if(V===!0&&A===!0){const H=h==="\\"?new RegExp(h+h,"g"):new RegExp(h,"g");f=f.replace(H,h+h)}if(E===!0){const H=new RegExp(S,"g");f=f.replace(H,h+S)}V===!0&&(f=S+f+S),u+=f}else(y===!0||p===""&&m===!0&&y!==!1)&&(u+=S+S);o!==r.length-1&&(u+=d)}return[void 0,u]},bom:function(l){this.options.bom===!0&&l(DR)},headers:function(l){if(this.options.header===!1||this.options.columns===void 0)return;let a,i=this.options.columns.map(r=>r.header);if(this.options.eof?([a,i]=this.stringify(i,!0),i+=this.options.record_delimiter):[a,i]=this.stringify(i),a)return a;l(i)},__cast:function(l,a){const i=typeof l;try{return i==="string"?[void 0,this.options.cast.string(l,a)]:i==="bigint"?[void 0,this.options.cast.bigint(l,a)]:i==="number"?[void 0,this.options.cast.number(l,a)]:i==="boolean"?[void 0,this.options.cast.boolean(l,a)]:l instanceof Date?[void 0,this.options.cast.date(l,a)]:i==="object"&&l!==null?[void 0,this.options.cast.object(l,a)]:[void 0,l,l]}catch(r){return[r]}}}},jR=function(e,t={}){const n=[],[l,a]=Gm(t);if(l!==void 0)throw l;const u=MR(a,{stop:!1},{records:0});for(const o of e){const s=u.__transform(o,function(c){n.push(c)});if(s!==void 0)throw s}if(n.length===0){u.bom(s=>{n.push(s)});const o=u.headers(s=>{n.push(s)});if(o!==void 0)throw o}return n.join("")};function tp(e){return g.jsx(hm,{id:"character_tooltip",children:e})}const NR=(e,t)=>{const{rows:n}=e.getCoreRowModel(),l=e.getHeaderGroups().map(s=>s.headers.map(c=>c.column.columnDef.header)),a=n.map(s=>s.getVisibleCells().map(c=>c.getValue()));console.log(l,a);const i=jR([...l,...a]),r="csv",u=new Blob([i],{type:`text/${r};charset=utf8;`}),o=document.createElement("a");o.download=t,o.href=URL.createObjectURL(u),o.setAttribute("visibility","hidden"),document.body.appendChild(o),o.click(),document.body.removeChild(o)},Gu=({isFetching:e=!1,debugTable:t=!1,data:n=[],columns:l,striped:a=!1,hover:i=!1,initialState:r=void 0,exportFileName:u=void 0})=>{let o={pagination:{pageSize:15}};r!==void 0&&(o=r);const s=Vb({data:n,columns:l,getCoreRowModel:Ab(),getFilteredRowModel:zb(),getSortedRowModel:Fb(),getPaginationRowModel:Ub(),getFacetedRowModel:jb(),getFacetedUniqueValues:Nb(),getFacetedMinMaxValues:Tb(),debugTable:t,initialState:o});return g.jsx(zR,{table:s,data:n,columns:l,isFetching:e,debugTable:t,initialState:r,striped:a,hover:i,exportFileName:u})};function zR({table:e,isFetching:t=!1,striped:n=!1,hover:l=!1,debugTable:a=!1,exportFileName:i=void 0}){const{rows:r}=e.getRowModel(),u=s1(),o=typeof i<"u"?i:`ExportedData_${u.pathname}`;return g.jsxs(g.Fragment,{children:[g.jsxs(Fw,{striped:n,hover:l,children:[g.jsx("thead",{children:e.getHeaderGroups().map(s=>g.jsxs(g.Fragment,{children:[g.jsx("tr",{children:s.headers.map(c=>g.jsx("th",{colSpan:c.colSpan,children:c.isPlaceholder?null:g.jsxs("div",{className:c.column.getCanSort()?"d-flex align-items-center cursor-pointer select-none":"d-flex align-items-center",onClick:c.column.getToggleSortingHandler(),children:[c.column.getCanSort()?g.jsx("div",{children:{asc:g.jsx("i",{className:"fas fa-sort-down fa-fw"}),desc:g.jsx("i",{className:"fas fa-sort-up fa-fw"})}[c.column.getIsSorted()]??g.jsx("i",{className:"fas fa-sort fa-fw"})}):null,g.jsx("div",{children:Jd(c.column.columnDef.header,c.getContext())})]})},c.id))},`name-${s.id}`),g.jsx("tr",{children:s.headers.map(c=>g.jsx("th",{colSpan:c.colSpan,children:c.column.getCanFilter()&&r.length>=0?g.jsx(qw,{column:c.column,table:e}):g.jsx(g.Fragment,{})},c.id))},`filter-${s.id}`)]}))}),g.jsx("tbody",{children:r.map(s=>g.jsx("tr",{children:s.getVisibleCells().map(c=>g.jsx("td",{style:{verticalAlign:"middle"},children:Jd(c.column.columnDef.cell,c.getContext())},c.id))},s.id))})]}),g.jsxs("div",{className:"d-flex justify-content-between",children:[g.jsxs(Za,{children:[g.jsx(He,{active:!0,variant:"info",children:g.jsxs(g.Fragment,{children:[e.getState().pagination.pageIndex+1," of ",e.getPageCount()]})}),t?g.jsx(mi,{placement:"bottom",trigger:"focus",overlay:tp("Refreshing Data"),children:g.jsx(He,{variant:"info",children:g.jsx("i",{className:v1.refreshAnimate+" fas fa-sync"})})}):g.jsx(mi,{placement:"bottom",trigger:"focus",overlay:tp("Data Loaded: "+new Date().toLocaleString()),children:g.jsx(He,{variant:"info",children:g.jsx("i",{className:"far fa-check-circle"})})}),g.jsx(He,{active:!0,variant:"primary",onClick:()=>NR(e,o),children:"Export Table to CSV"})," "]}),g.jsxs(uh,{children:[g.jsxs(Za,{children:[g.jsx(He,{variant:"success",onClick:()=>e.setPageIndex(0),disabled:!e.getCanPreviousPage(),children:g.jsx("i",{className:"fas fa-angle-double-left"})})," ",g.jsx(He,{variant:"success",onClick:()=>e.previousPage(),disabled:!e.getCanPreviousPage(),children:g.jsx("i",{className:"fas fa-caret-left"})})," ",g.jsx(He,{variant:"success",onClick:()=>e.nextPage(),disabled:!e.getCanNextPage(),children:g.jsx("i",{className:"fas fa-caret-right"})}),g.jsx(He,{variant:"success",onClick:()=>e.setPageIndex(e.getPageCount()-1),disabled:!e.getCanNextPage(),children:g.jsx("i",{className:"fas fa-angle-double-right"})})]}),g.jsxs(Za,{className:"ms-1",children:[g.jsx(He,{active:!0,variant:"success",children:"Page Size:"})," ",g.jsx(ef,{id:"pageSizeDropdown",variant:"success",title:e.getState().pagination.pageSize,children:[15,30,60,100,1e6].map(s=>g.jsxs(bn.Item,{id:`${s}`,eventKey:s,onClick:c=>{console.log(c.target.id),e.setPageSize(Number(c.target.id))},children:["Show ",s]},s))})]})]})]}),a&&g.jsxs("div",{className:"col-xs-12",children:[g.jsxs("div",{children:[e.getRowModel().rows.length," Rows"]}),g.jsx("pre",{children:JSON.stringify(e.getState(),null,2)})]})]})}const UR="_flexContainer_ia7nm_1",FR="_errorAnim_ia7nm_33",BR="_shake_ia7nm_1",HR="_ldsDualRing_ia7nm_38",Kr={flexContainer:UR,errorAnim:FR,shake:BR,ldsDualRing:HR,"lds-dual-ring":"_lds-dual-ring_ia7nm_1"},LR=({className:e=""})=>g.jsx("div",{className:e+" "+Kr.ldsDualRing}),VR=(e={title:"Loading..."})=>g.jsx("div",{className:Kr.flexContainer,children:g.jsxs("div",{className:"text-center",children:[g.jsx(LR,{}),g.jsx("h3",{children:e.title&&e.title}),g.jsx("p",{children:e.message&&e.message})]})}),$R=(e={title:"Error Loading Component"})=>g.jsx("div",{className:Kr.flexContainer,children:g.jsxs("div",{className:"text-center",children:[g.jsx("div",{className:Kr.errorAnim,children:g.jsxs("svg",{xmlns:"http://www.w3.org/2000/svg",width:"100",height:"100",fill:"currentColor",className:"bi bi-exclamation-triangle",viewBox:"0 0 16 16",children:[g.jsx("path",{d:"M7.938 2.016A.13.13 0 0 1 8.002 2a.13.13 0 0 1 .063.016.146.146 0 0 1 .054.057l6.857 11.667c.036.06.035.124.002.183a.163.163 0 0 1-.054.06.116.116 0 0 1-.066.017H1.146a.115.115 0 0 1-.066-.017.163.163 0 0 1-.054-.06.176.176 0 0 1 .002-.183L7.884 2.073a.147.147 0 0 1 .054-.057zm1.044-.45a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L8.982 1.566z"}),g.jsx("path",{d:"M7.002 12a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 5.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 5.995z"})]})}),g.jsx("h3",{children:e.title&&e.title}),g.jsx("p",{children:e.message&&e.message}),g.jsx("p",{})]})});class Kl extends w.Component{constructor(t){super(t),this.state={hasError:!1}}componentDidCatch(t,n){console.error("ErrorBoundary caught an error: ",t,n),this.setState({hasError:!0})}render(){return this.state.hasError?g.jsx($R,{}):this.props.children}}function Ym(e,t){return function(){return e.apply(t,arguments)}}const{toString:qR}=Object.prototype,{getPrototypeOf:sf}=Object,Yu=(e=>t=>{const n=qR.call(t);return e[n]||(e[n]=n.slice(8,-1).toLowerCase())})(Object.create(null)),_t=e=>(e=e.toLowerCase(),t=>Yu(t)===e),Xu=e=>t=>typeof t===e,{isArray:Sa}=Array,yi=Xu("undefined");function GR(e){return e!==null&&!yi(e)&&e.constructor!==null&&!yi(e.constructor)&&Ie(e.constructor.isBuffer)&&e.constructor.isBuffer(e)}const Xm=_t("ArrayBuffer");function YR(e){let t;return typeof ArrayBuffer<"u"&&ArrayBuffer.isView?t=ArrayBuffer.isView(e):t=e&&e.buffer&&Xm(e.buffer),t}const XR=Xu("string"),Ie=Xu("function"),km=Xu("number"),ku=e=>e!==null&&typeof e=="object",kR=e=>e===!0||e===!1,Or=e=>{if(Yu(e)!=="object")return!1;const t=sf(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(Symbol.toStringTag in e)&&!(Symbol.iterator in e)},QR=_t("Date"),ZR=_t("File"),KR=_t("Blob"),JR=_t("FileList"),PR=e=>ku(e)&&Ie(e.pipe),IR=e=>{let t;return e&&(typeof FormData=="function"&&e instanceof FormData||Ie(e.append)&&((t=Yu(e))==="formdata"||t==="object"&&Ie(e.toString)&&e.toString()==="[object FormData]"))},WR=_t("URLSearchParams"),[eE,tE,nE,lE]=["ReadableStream","Request","Response","Headers"].map(_t),aE=e=>e.trim?e.trim():e.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"");function Li(e,t,{allOwnKeys:n=!1}={}){if(e===null||typeof e>"u")return;let l,a;if(typeof e!="object"&&(e=[e]),Sa(e))for(l=0,a=e.length;l<a;l++)t.call(null,e[l],l,e);else{const i=n?Object.getOwnPropertyNames(e):Object.keys(e),r=i.length;let u;for(l=0;l<r;l++)u=i[l],t.call(null,e[u],u,e)}}function Qm(e,t){t=t.toLowerCase();const n=Object.keys(e);let l=n.length,a;for(;l-- >0;)if(a=n[l],t===a.toLowerCase())return a;return null}const tl=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:global,Zm=e=>!yi(e)&&e!==tl;function Ts(){const{caseless:e}=Zm(this)&&this||{},t={},n=(l,a)=>{const i=e&&Qm(t,a)||a;Or(t[i])&&Or(l)?t[i]=Ts(t[i],l):Or(l)?t[i]=Ts({},l):Sa(l)?t[i]=l.slice():t[i]=l};for(let l=0,a=arguments.length;l<a;l++)arguments[l]&&Li(arguments[l],n);return t}const iE=(e,t,n,{allOwnKeys:l}={})=>(Li(t,(a,i)=>{n&&Ie(a)?e[i]=Ym(a,n):e[i]=a},{allOwnKeys:l}),e),rE=e=>(e.charCodeAt(0)===65279&&(e=e.slice(1)),e),uE=(e,t,n,l)=>{e.prototype=Object.create(t.prototype,l),e.prototype.constructor=e,Object.defineProperty(e,"super",{value:t.prototype}),n&&Object.assign(e.prototype,n)},oE=(e,t,n,l)=>{let a,i,r;const u={};if(t=t||{},e==null)return t;do{for(a=Object.getOwnPropertyNames(e),i=a.length;i-- >0;)r=a[i],(!l||l(r,e,t))&&!u[r]&&(t[r]=e[r],u[r]=!0);e=n!==!1&&sf(e)}while(e&&(!n||n(e,t))&&e!==Object.prototype);return t},sE=(e,t,n)=>{e=String(e),(n===void 0||n>e.length)&&(n=e.length),n-=t.length;const l=e.indexOf(t,n);return l!==-1&&l===n},cE=e=>{if(!e)return null;if(Sa(e))return e;let t=e.length;if(!km(t))return null;const n=new Array(t);for(;t-- >0;)n[t]=e[t];return n},fE=(e=>t=>e&&t instanceof e)(typeof Uint8Array<"u"&&sf(Uint8Array)),dE=(e,t)=>{const l=(e&&e[Symbol.iterator]).call(e);let a;for(;(a=l.next())&&!a.done;){const i=a.value;t.call(e,i[0],i[1])}},pE=(e,t)=>{let n;const l=[];for(;(n=e.exec(t))!==null;)l.push(n);return l},gE=_t("HTMLFormElement"),hE=e=>e.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,function(n,l,a){return l.toUpperCase()+a}),np=(({hasOwnProperty:e})=>(t,n)=>e.call(t,n))(Object.prototype),mE=_t("RegExp"),Km=(e,t)=>{const n=Object.getOwnPropertyDescriptors(e),l={};Li(n,(a,i)=>{let r;(r=t(a,i,e))!==!1&&(l[i]=r||a)}),Object.defineProperties(e,l)},yE=e=>{Km(e,(t,n)=>{if(Ie(e)&&["arguments","caller","callee"].indexOf(n)!==-1)return!1;const l=e[n];if(Ie(l)){if(t.enumerable=!1,"writable"in t){t.writable=!1;return}t.set||(t.set=()=>{throw Error("Can not rewrite read-only method '"+n+"'")})}})},vE=(e,t)=>{const n={},l=a=>{a.forEach(i=>{n[i]=!0})};return Sa(e)?l(e):l(String(e).split(t)),n},SE=()=>{},xE=(e,t)=>e!=null&&Number.isFinite(e=+e)?e:t,_o="abcdefghijklmnopqrstuvwxyz",lp="0123456789",Jm={DIGIT:lp,ALPHA:_o,ALPHA_DIGIT:_o+_o.toUpperCase()+lp},wE=(e=16,t=Jm.ALPHA_DIGIT)=>{let n="";const{length:l}=t;for(;e--;)n+=t[Math.random()*l|0];return n};function bE(e){return!!(e&&Ie(e.append)&&e[Symbol.toStringTag]==="FormData"&&e[Symbol.iterator])}const RE=e=>{const t=new Array(10),n=(l,a)=>{if(ku(l)){if(t.indexOf(l)>=0)return;if(!("toJSON"in l)){t[a]=l;const i=Sa(l)?[]:{};return Li(l,(r,u)=>{const o=n(r,a+1);!yi(o)&&(i[u]=o)}),t[a]=void 0,i}}return l};return n(e,0)},EE=_t("AsyncFunction"),CE=e=>e&&(ku(e)||Ie(e))&&Ie(e.then)&&Ie(e.catch),Pm=((e,t)=>e?setImmediate:t?((n,l)=>(tl.addEventListener("message",({source:a,data:i})=>{a===tl&&i===n&&l.length&&l.shift()()},!1),a=>{l.push(a),tl.postMessage(n,"*")}))(`axios@${Math.random()}`,[]):n=>setTimeout(n))(typeof setImmediate=="function",Ie(tl.postMessage)),_E=typeof queueMicrotask<"u"?queueMicrotask.bind(tl):typeof process<"u"&&process.nextTick||Pm,C={isArray:Sa,isArrayBuffer:Xm,isBuffer:GR,isFormData:IR,isArrayBufferView:YR,isString:XR,isNumber:km,isBoolean:kR,isObject:ku,isPlainObject:Or,isReadableStream:eE,isRequest:tE,isResponse:nE,isHeaders:lE,isUndefined:yi,isDate:QR,isFile:ZR,isBlob:KR,isRegExp:mE,isFunction:Ie,isStream:PR,isURLSearchParams:WR,isTypedArray:fE,isFileList:JR,forEach:Li,merge:Ts,extend:iE,trim:aE,stripBOM:rE,inherits:uE,toFlatObject:oE,kindOf:Yu,kindOfTest:_t,endsWith:sE,toArray:cE,forEachEntry:dE,matchAll:pE,isHTMLForm:gE,hasOwnProperty:np,hasOwnProp:np,reduceDescriptors:Km,freezeMethods:yE,toObjectSet:vE,toCamelCase:hE,noop:SE,toFiniteNumber:xE,findKey:Qm,global:tl,isContextDefined:Zm,ALPHABET:Jm,generateString:wE,isSpecCompliantForm:bE,toJSONObject:RE,isAsyncFn:EE,isThenable:CE,setImmediate:Pm,asap:_E};function q(e,t,n,l,a){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack,this.message=e,this.name="AxiosError",t&&(this.code=t),n&&(this.config=n),l&&(this.request=l),a&&(this.response=a,this.status=a.status?a.status:null)}C.inherits(q,Error,{toJSON:function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:C.toJSONObject(this.config),code:this.code,status:this.status}}});const Im=q.prototype,Wm={};["ERR_BAD_OPTION_VALUE","ERR_BAD_OPTION","ECONNABORTED","ETIMEDOUT","ERR_NETWORK","ERR_FR_TOO_MANY_REDIRECTS","ERR_DEPRECATED","ERR_BAD_RESPONSE","ERR_BAD_REQUEST","ERR_CANCELED","ERR_NOT_SUPPORT","ERR_INVALID_URL"].forEach(e=>{Wm[e]={value:e}});Object.defineProperties(q,Wm);Object.defineProperty(Im,"isAxiosError",{value:!0});q.from=(e,t,n,l,a,i)=>{const r=Object.create(Im);return C.toFlatObject(e,r,function(o){return o!==Error.prototype},u=>u!=="isAxiosError"),q.call(r,e.message,t,n,l,a),r.cause=e,r.name=e.name,i&&Object.assign(r,i),r};const AE=null;function Ds(e){return C.isPlainObject(e)||C.isArray(e)}function e0(e){return C.endsWith(e,"[]")?e.slice(0,-2):e}function ap(e,t,n){return e?e.concat(t).map(function(a,i){return a=e0(a),!n&&i?"["+a+"]":a}).join(n?".":""):t}function OE(e){return C.isArray(e)&&!e.some(Ds)}const TE=C.toFlatObject(C,{},null,function(t){return/^is[A-Z]/.test(t)});function Qu(e,t,n){if(!C.isObject(e))throw new TypeError("target must be an object");t=t||new FormData,n=C.toFlatObject(n,{metaTokens:!0,dots:!1,indexes:!1},!1,function(S,x){return!C.isUndefined(x[S])});const l=n.metaTokens,a=n.visitor||c,i=n.dots,r=n.indexes,o=(n.Blob||typeof Blob<"u"&&Blob)&&C.isSpecCompliantForm(t);if(!C.isFunction(a))throw new TypeError("visitor must be a function");function s(h){if(h===null)return"";if(C.isDate(h))return h.toISOString();if(!o&&C.isBlob(h))throw new q("Blob is not supported. Use a Buffer instead.");return C.isArrayBuffer(h)||C.isTypedArray(h)?o&&typeof Blob=="function"?new Blob([h]):Buffer.from(h):h}function c(h,S,x){let y=h;if(h&&!x&&typeof h=="object"){if(C.endsWith(S,"{}"))S=l?S:S.slice(0,-2),h=JSON.stringify(h);else if(C.isArray(h)&&OE(h)||(C.isFileList(h)||C.endsWith(S,"[]"))&&(y=C.toArray(h)))return S=e0(S),y.forEach(function(v,b){!(C.isUndefined(v)||v===null)&&t.append(r===!0?ap([S],b,i):r===null?S:S+"[]",s(v))}),!1}return Ds(h)?!0:(t.append(ap(x,S,i),s(h)),!1)}const f=[],p=Object.assign(TE,{defaultVisitor:c,convertValue:s,isVisitable:Ds});function d(h,S){if(!C.isUndefined(h)){if(f.indexOf(h)!==-1)throw Error("Circular reference detected in "+S.join("."));f.push(h),C.forEach(h,function(y,m){(!(C.isUndefined(y)||y===null)&&a.call(t,y,C.isString(m)?m.trim():m,S,p))===!0&&d(y,S?S.concat(m):[m])}),f.pop()}}if(!C.isObject(e))throw new TypeError("data must be an object");return d(e),t}function ip(e){const t={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+","%00":"\0"};return encodeURIComponent(e).replace(/[!'()~]|%20|%00/g,function(l){return t[l]})}function cf(e,t){this._pairs=[],e&&Qu(e,this,t)}const t0=cf.prototype;t0.append=function(t,n){this._pairs.push([t,n])};t0.toString=function(t){const n=t?function(l){return t.call(this,l,ip)}:ip;return this._pairs.map(function(a){return n(a[0])+"="+n(a[1])},"").join("&")};function DE(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}function n0(e,t,n){if(!t)return e;const l=n&&n.encode||DE;C.isFunction(n)&&(n={serialize:n});const a=n&&n.serialize;let i;if(a?i=a(t,n):i=C.isURLSearchParams(t)?t.toString():new cf(t,n).toString(l),i){const r=e.indexOf("#");r!==-1&&(e=e.slice(0,r)),e+=(e.indexOf("?")===-1?"?":"&")+i}return e}class rp{constructor(){this.handlers=[]}use(t,n,l){return this.handlers.push({fulfilled:t,rejected:n,synchronous:l?l.synchronous:!1,runWhen:l?l.runWhen:null}),this.handlers.length-1}eject(t){this.handlers[t]&&(this.handlers[t]=null)}clear(){this.handlers&&(this.handlers=[])}forEach(t){C.forEach(this.handlers,function(l){l!==null&&t(l)})}}const l0={silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},ME=typeof URLSearchParams<"u"?URLSearchParams:cf,jE=typeof FormData<"u"?FormData:null,NE=typeof Blob<"u"?Blob:null,zE={isBrowser:!0,classes:{URLSearchParams:ME,FormData:jE,Blob:NE},protocols:["http","https","file","blob","url","data"]},ff=typeof window<"u"&&typeof document<"u",Ms=typeof navigator=="object"&&navigator||void 0,UE=ff&&(!Ms||["ReactNative","NativeScript","NS"].indexOf(Ms.product)<0),FE=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope&&typeof self.importScripts=="function",BE=ff&&window.location.href||"http://localhost",HE=Object.freeze(Object.defineProperty({__proto__:null,hasBrowserEnv:ff,hasStandardBrowserEnv:UE,hasStandardBrowserWebWorkerEnv:FE,navigator:Ms,origin:BE},Symbol.toStringTag,{value:"Module"})),Me={...HE,...zE};function LE(e,t){return Qu(e,new Me.classes.URLSearchParams,Object.assign({visitor:function(n,l,a,i){return Me.isNode&&C.isBuffer(n)?(this.append(l,n.toString("base64")),!1):i.defaultVisitor.apply(this,arguments)}},t))}function VE(e){return C.matchAll(/\w+|\[(\w*)]/g,e).map(t=>t[0]==="[]"?"":t[1]||t[0])}function $E(e){const t={},n=Object.keys(e);let l;const a=n.length;let i;for(l=0;l<a;l++)i=n[l],t[i]=e[i];return t}function a0(e){function t(n,l,a,i){let r=n[i++];if(r==="__proto__")return!0;const u=Number.isFinite(+r),o=i>=n.length;return r=!r&&C.isArray(a)?a.length:r,o?(C.hasOwnProp(a,r)?a[r]=[a[r],l]:a[r]=l,!u):((!a[r]||!C.isObject(a[r]))&&(a[r]=[]),t(n,l,a[r],i)&&C.isArray(a[r])&&(a[r]=$E(a[r])),!u)}if(C.isFormData(e)&&C.isFunction(e.entries)){const n={};return C.forEachEntry(e,(l,a)=>{t(VE(l),a,n,0)}),n}return null}function qE(e,t,n){if(C.isString(e))try{return(t||JSON.parse)(e),C.trim(e)}catch(l){if(l.name!=="SyntaxError")throw l}return(0,JSON.stringify)(e)}const Vi={transitional:l0,adapter:["xhr","http","fetch"],transformRequest:[function(t,n){const l=n.getContentType()||"",a=l.indexOf("application/json")>-1,i=C.isObject(t);if(i&&C.isHTMLForm(t)&&(t=new FormData(t)),C.isFormData(t))return a?JSON.stringify(a0(t)):t;if(C.isArrayBuffer(t)||C.isBuffer(t)||C.isStream(t)||C.isFile(t)||C.isBlob(t)||C.isReadableStream(t))return t;if(C.isArrayBufferView(t))return t.buffer;if(C.isURLSearchParams(t))return n.setContentType("application/x-www-form-urlencoded;charset=utf-8",!1),t.toString();let u;if(i){if(l.indexOf("application/x-www-form-urlencoded")>-1)return LE(t,this.formSerializer).toString();if((u=C.isFileList(t))||l.indexOf("multipart/form-data")>-1){const o=this.env&&this.env.FormData;return Qu(u?{"files[]":t}:t,o&&new o,this.formSerializer)}}return i||a?(n.setContentType("application/json",!1),qE(t)):t}],transformResponse:[function(t){const n=this.transitional||Vi.transitional,l=n&&n.forcedJSONParsing,a=this.responseType==="json";if(C.isResponse(t)||C.isReadableStream(t))return t;if(t&&C.isString(t)&&(l&&!this.responseType||a)){const r=!(n&&n.silentJSONParsing)&&a;try{return JSON.parse(t)}catch(u){if(r)throw u.name==="SyntaxError"?q.from(u,q.ERR_BAD_RESPONSE,this,null,this.response):u}}return t}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,env:{FormData:Me.classes.FormData,Blob:Me.classes.Blob},validateStatus:function(t){return t>=200&&t<300},headers:{common:{Accept:"application/json, text/plain, */*","Content-Type":void 0}}};C.forEach(["delete","get","head","post","put","patch"],e=>{Vi.headers[e]={}});const GE=C.toObjectSet(["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"]),YE=e=>{const t={};let n,l,a;return e&&e.split(`
`).forEach(function(r){a=r.indexOf(":"),n=r.substring(0,a).trim().toLowerCase(),l=r.substring(a+1).trim(),!(!n||t[n]&&GE[n])&&(n==="set-cookie"?t[n]?t[n].push(l):t[n]=[l]:t[n]=t[n]?t[n]+", "+l:l)}),t},up=Symbol("internals");function Da(e){return e&&String(e).trim().toLowerCase()}function Tr(e){return e===!1||e==null?e:C.isArray(e)?e.map(Tr):String(e)}function XE(e){const t=Object.create(null),n=/([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;let l;for(;l=n.exec(e);)t[l[1]]=l[2];return t}const kE=e=>/^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(e.trim());function Ao(e,t,n,l,a){if(C.isFunction(l))return l.call(this,t,n);if(a&&(t=n),!!C.isString(t)){if(C.isString(l))return t.indexOf(l)!==-1;if(C.isRegExp(l))return l.test(t)}}function QE(e){return e.trim().toLowerCase().replace(/([a-z\d])(\w*)/g,(t,n,l)=>n.toUpperCase()+l)}function ZE(e,t){const n=C.toCamelCase(" "+t);["get","set","has"].forEach(l=>{Object.defineProperty(e,l+n,{value:function(a,i,r){return this[l].call(this,t,a,i,r)},configurable:!0})})}class qe{constructor(t){t&&this.set(t)}set(t,n,l){const a=this;function i(u,o,s){const c=Da(o);if(!c)throw new Error("header name must be a non-empty string");const f=C.findKey(a,c);(!f||a[f]===void 0||s===!0||s===void 0&&a[f]!==!1)&&(a[f||o]=Tr(u))}const r=(u,o)=>C.forEach(u,(s,c)=>i(s,c,o));if(C.isPlainObject(t)||t instanceof this.constructor)r(t,n);else if(C.isString(t)&&(t=t.trim())&&!kE(t))r(YE(t),n);else if(C.isHeaders(t))for(const[u,o]of t.entries())i(o,u,l);else t!=null&&i(n,t,l);return this}get(t,n){if(t=Da(t),t){const l=C.findKey(this,t);if(l){const a=this[l];if(!n)return a;if(n===!0)return XE(a);if(C.isFunction(n))return n.call(this,a,l);if(C.isRegExp(n))return n.exec(a);throw new TypeError("parser must be boolean|regexp|function")}}}has(t,n){if(t=Da(t),t){const l=C.findKey(this,t);return!!(l&&this[l]!==void 0&&(!n||Ao(this,this[l],l,n)))}return!1}delete(t,n){const l=this;let a=!1;function i(r){if(r=Da(r),r){const u=C.findKey(l,r);u&&(!n||Ao(l,l[u],u,n))&&(delete l[u],a=!0)}}return C.isArray(t)?t.forEach(i):i(t),a}clear(t){const n=Object.keys(this);let l=n.length,a=!1;for(;l--;){const i=n[l];(!t||Ao(this,this[i],i,t,!0))&&(delete this[i],a=!0)}return a}normalize(t){const n=this,l={};return C.forEach(this,(a,i)=>{const r=C.findKey(l,i);if(r){n[r]=Tr(a),delete n[i];return}const u=t?QE(i):String(i).trim();u!==i&&delete n[i],n[u]=Tr(a),l[u]=!0}),this}concat(...t){return this.constructor.concat(this,...t)}toJSON(t){const n=Object.create(null);return C.forEach(this,(l,a)=>{l!=null&&l!==!1&&(n[a]=t&&C.isArray(l)?l.join(", "):l)}),n}[Symbol.iterator](){return Object.entries(this.toJSON())[Symbol.iterator]()}toString(){return Object.entries(this.toJSON()).map(([t,n])=>t+": "+n).join(`
`)}get[Symbol.toStringTag](){return"AxiosHeaders"}static from(t){return t instanceof this?t:new this(t)}static concat(t,...n){const l=new this(t);return n.forEach(a=>l.set(a)),l}static accessor(t){const l=(this[up]=this[up]={accessors:{}}).accessors,a=this.prototype;function i(r){const u=Da(r);l[u]||(ZE(a,r),l[u]=!0)}return C.isArray(t)?t.forEach(i):i(t),this}}qe.accessor(["Content-Type","Content-Length","Accept","Accept-Encoding","User-Agent","Authorization"]);C.reduceDescriptors(qe.prototype,({value:e},t)=>{let n=t[0].toUpperCase()+t.slice(1);return{get:()=>e,set(l){this[n]=l}}});C.freezeMethods(qe);function Oo(e,t){const n=this||Vi,l=t||n,a=qe.from(l.headers);let i=l.data;return C.forEach(e,function(u){i=u.call(n,i,a.normalize(),t?t.status:void 0)}),a.normalize(),i}function i0(e){return!!(e&&e.__CANCEL__)}function xa(e,t,n){q.call(this,e??"canceled",q.ERR_CANCELED,t,n),this.name="CanceledError"}C.inherits(xa,q,{__CANCEL__:!0});function r0(e,t,n){const l=n.config.validateStatus;!n.status||!l||l(n.status)?e(n):t(new q("Request failed with status code "+n.status,[q.ERR_BAD_REQUEST,q.ERR_BAD_RESPONSE][Math.floor(n.status/100)-4],n.config,n.request,n))}function KE(e){const t=/^([-+\w]{1,25})(:?\/\/|:)/.exec(e);return t&&t[1]||""}function JE(e,t){e=e||10;const n=new Array(e),l=new Array(e);let a=0,i=0,r;return t=t!==void 0?t:1e3,function(o){const s=Date.now(),c=l[i];r||(r=s),n[a]=o,l[a]=s;let f=i,p=0;for(;f!==a;)p+=n[f++],f=f%e;if(a=(a+1)%e,a===i&&(i=(i+1)%e),s-r<t)return;const d=c&&s-c;return d?Math.round(p*1e3/d):void 0}}function PE(e,t){let n=0,l=1e3/t,a,i;const r=(s,c=Date.now())=>{n=c,a=null,i&&(clearTimeout(i),i=null),e.apply(null,s)};return[(...s)=>{const c=Date.now(),f=c-n;f>=l?r(s,c):(a=s,i||(i=setTimeout(()=>{i=null,r(a)},l-f)))},()=>a&&r(a)]}const Jr=(e,t,n=3)=>{let l=0;const a=JE(50,250);return PE(i=>{const r=i.loaded,u=i.lengthComputable?i.total:void 0,o=r-l,s=a(o),c=r<=u;l=r;const f={loaded:r,total:u,progress:u?r/u:void 0,bytes:o,rate:s||void 0,estimated:s&&u&&c?(u-r)/s:void 0,event:i,lengthComputable:u!=null,[t?"download":"upload"]:!0};e(f)},n)},op=(e,t)=>{const n=e!=null;return[l=>t[0]({lengthComputable:n,total:e,loaded:l}),t[1]]},sp=e=>(...t)=>C.asap(()=>e(...t)),IE=Me.hasStandardBrowserEnv?((e,t)=>n=>(n=new URL(n,Me.origin),e.protocol===n.protocol&&e.host===n.host&&(t||e.port===n.port)))(new URL(Me.origin),Me.navigator&&/(msie|trident)/i.test(Me.navigator.userAgent)):()=>!0,WE=Me.hasStandardBrowserEnv?{write(e,t,n,l,a,i){const r=[e+"="+encodeURIComponent(t)];C.isNumber(n)&&r.push("expires="+new Date(n).toGMTString()),C.isString(l)&&r.push("path="+l),C.isString(a)&&r.push("domain="+a),i===!0&&r.push("secure"),document.cookie=r.join("; ")},read(e){const t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove(e){this.write(e,"",Date.now()-864e5)}}:{write(){},read(){return null},remove(){}};function e2(e){return/^([a-z][a-z\d+\-.]*:)?\/\//i.test(e)}function t2(e,t){return t?e.replace(/\/?\/$/,"")+"/"+t.replace(/^\/+/,""):e}function u0(e,t){return e&&!e2(t)?t2(e,t):t}const cp=e=>e instanceof qe?{...e}:e;function pl(e,t){t=t||{};const n={};function l(s,c,f,p){return C.isPlainObject(s)&&C.isPlainObject(c)?C.merge.call({caseless:p},s,c):C.isPlainObject(c)?C.merge({},c):C.isArray(c)?c.slice():c}function a(s,c,f,p){if(C.isUndefined(c)){if(!C.isUndefined(s))return l(void 0,s,f,p)}else return l(s,c,f,p)}function i(s,c){if(!C.isUndefined(c))return l(void 0,c)}function r(s,c){if(C.isUndefined(c)){if(!C.isUndefined(s))return l(void 0,s)}else return l(void 0,c)}function u(s,c,f){if(f in t)return l(s,c);if(f in e)return l(void 0,s)}const o={url:i,method:i,data:i,baseURL:r,transformRequest:r,transformResponse:r,paramsSerializer:r,timeout:r,timeoutMessage:r,withCredentials:r,withXSRFToken:r,adapter:r,responseType:r,xsrfCookieName:r,xsrfHeaderName:r,onUploadProgress:r,onDownloadProgress:r,decompress:r,maxContentLength:r,maxBodyLength:r,beforeRedirect:r,transport:r,httpAgent:r,httpsAgent:r,cancelToken:r,socketPath:r,responseEncoding:r,validateStatus:u,headers:(s,c,f)=>a(cp(s),cp(c),f,!0)};return C.forEach(Object.keys(Object.assign({},e,t)),function(c){const f=o[c]||a,p=f(e[c],t[c],c);C.isUndefined(p)&&f!==u||(n[c]=p)}),n}const o0=e=>{const t=pl({},e);let{data:n,withXSRFToken:l,xsrfHeaderName:a,xsrfCookieName:i,headers:r,auth:u}=t;t.headers=r=qe.from(r),t.url=n0(u0(t.baseURL,t.url),e.params,e.paramsSerializer),u&&r.set("Authorization","Basic "+btoa((u.username||"")+":"+(u.password?unescape(encodeURIComponent(u.password)):"")));let o;if(C.isFormData(n)){if(Me.hasStandardBrowserEnv||Me.hasStandardBrowserWebWorkerEnv)r.setContentType(void 0);else if((o=r.getContentType())!==!1){const[s,...c]=o?o.split(";").map(f=>f.trim()).filter(Boolean):[];r.setContentType([s||"multipart/form-data",...c].join("; "))}}if(Me.hasStandardBrowserEnv&&(l&&C.isFunction(l)&&(l=l(t)),l||l!==!1&&IE(t.url))){const s=a&&i&&WE.read(i);s&&r.set(a,s)}return t},n2=typeof XMLHttpRequest<"u",l2=n2&&function(e){return new Promise(function(n,l){const a=o0(e);let i=a.data;const r=qe.from(a.headers).normalize();let{responseType:u,onUploadProgress:o,onDownloadProgress:s}=a,c,f,p,d,h;function S(){d&&d(),h&&h(),a.cancelToken&&a.cancelToken.unsubscribe(c),a.signal&&a.signal.removeEventListener("abort",c)}let x=new XMLHttpRequest;x.open(a.method.toUpperCase(),a.url,!0),x.timeout=a.timeout;function y(){if(!x)return;const v=qe.from("getAllResponseHeaders"in x&&x.getAllResponseHeaders()),R={data:!u||u==="text"||u==="json"?x.responseText:x.response,status:x.status,statusText:x.statusText,headers:v,config:e,request:x};r0(function(E){n(E),S()},function(E){l(E),S()},R),x=null}"onloadend"in x?x.onloadend=y:x.onreadystatechange=function(){!x||x.readyState!==4||x.status===0&&!(x.responseURL&&x.responseURL.indexOf("file:")===0)||setTimeout(y)},x.onabort=function(){x&&(l(new q("Request aborted",q.ECONNABORTED,e,x)),x=null)},x.onerror=function(){l(new q("Network Error",q.ERR_NETWORK,e,x)),x=null},x.ontimeout=function(){let b=a.timeout?"timeout of "+a.timeout+"ms exceeded":"timeout exceeded";const R=a.transitional||l0;a.timeoutErrorMessage&&(b=a.timeoutErrorMessage),l(new q(b,R.clarifyTimeoutError?q.ETIMEDOUT:q.ECONNABORTED,e,x)),x=null},i===void 0&&r.setContentType(null),"setRequestHeader"in x&&C.forEach(r.toJSON(),function(b,R){x.setRequestHeader(R,b)}),C.isUndefined(a.withCredentials)||(x.withCredentials=!!a.withCredentials),u&&u!=="json"&&(x.responseType=a.responseType),s&&([p,h]=Jr(s,!0),x.addEventListener("progress",p)),o&&x.upload&&([f,d]=Jr(o),x.upload.addEventListener("progress",f),x.upload.addEventListener("loadend",d)),(a.cancelToken||a.signal)&&(c=v=>{x&&(l(!v||v.type?new xa(null,e,x):v),x.abort(),x=null)},a.cancelToken&&a.cancelToken.subscribe(c),a.signal&&(a.signal.aborted?c():a.signal.addEventListener("abort",c)));const m=KE(a.url);if(m&&Me.protocols.indexOf(m)===-1){l(new q("Unsupported protocol "+m+":",q.ERR_BAD_REQUEST,e));return}x.send(i||null)})},a2=(e,t)=>{const{length:n}=e=e?e.filter(Boolean):[];if(t||n){let l=new AbortController,a;const i=function(s){if(!a){a=!0,u();const c=s instanceof Error?s:this.reason;l.abort(c instanceof q?c:new xa(c instanceof Error?c.message:c))}};let r=t&&setTimeout(()=>{r=null,i(new q(`timeout ${t} of ms exceeded`,q.ETIMEDOUT))},t);const u=()=>{e&&(r&&clearTimeout(r),r=null,e.forEach(s=>{s.unsubscribe?s.unsubscribe(i):s.removeEventListener("abort",i)}),e=null)};e.forEach(s=>s.addEventListener("abort",i));const{signal:o}=l;return o.unsubscribe=()=>C.asap(u),o}},i2=function*(e,t){let n=e.byteLength;if(n<t){yield e;return}let l=0,a;for(;l<n;)a=l+t,yield e.slice(l,a),l=a},r2=async function*(e,t){for await(const n of u2(e))yield*i2(n,t)},u2=async function*(e){if(e[Symbol.asyncIterator]){yield*e;return}const t=e.getReader();try{for(;;){const{done:n,value:l}=await t.read();if(n)break;yield l}}finally{await t.cancel()}},fp=(e,t,n,l)=>{const a=r2(e,t);let i=0,r,u=o=>{r||(r=!0,l&&l(o))};return new ReadableStream({async pull(o){try{const{done:s,value:c}=await a.next();if(s){u(),o.close();return}let f=c.byteLength;if(n){let p=i+=f;n(p)}o.enqueue(new Uint8Array(c))}catch(s){throw u(s),s}},cancel(o){return u(o),a.return()}},{highWaterMark:2})},Zu=typeof fetch=="function"&&typeof Request=="function"&&typeof Response=="function",s0=Zu&&typeof ReadableStream=="function",o2=Zu&&(typeof TextEncoder=="function"?(e=>t=>e.encode(t))(new TextEncoder):async e=>new Uint8Array(await new Response(e).arrayBuffer())),c0=(e,...t)=>{try{return!!e(...t)}catch{return!1}},s2=s0&&c0(()=>{let e=!1;const t=new Request(Me.origin,{body:new ReadableStream,method:"POST",get duplex(){return e=!0,"half"}}).headers.has("Content-Type");return e&&!t}),dp=64*1024,js=s0&&c0(()=>C.isReadableStream(new Response("").body)),Pr={stream:js&&(e=>e.body)};Zu&&(e=>{["text","arrayBuffer","blob","formData","stream"].forEach(t=>{!Pr[t]&&(Pr[t]=C.isFunction(e[t])?n=>n[t]():(n,l)=>{throw new q(`Response type '${t}' is not supported`,q.ERR_NOT_SUPPORT,l)})})})(new Response);const c2=async e=>{if(e==null)return 0;if(C.isBlob(e))return e.size;if(C.isSpecCompliantForm(e))return(await new Request(Me.origin,{method:"POST",body:e}).arrayBuffer()).byteLength;if(C.isArrayBufferView(e)||C.isArrayBuffer(e))return e.byteLength;if(C.isURLSearchParams(e)&&(e=e+""),C.isString(e))return(await o2(e)).byteLength},f2=async(e,t)=>{const n=C.toFiniteNumber(e.getContentLength());return n??c2(t)},d2=Zu&&(async e=>{let{url:t,method:n,data:l,signal:a,cancelToken:i,timeout:r,onDownloadProgress:u,onUploadProgress:o,responseType:s,headers:c,withCredentials:f="same-origin",fetchOptions:p}=o0(e);s=s?(s+"").toLowerCase():"text";let d=a2([a,i&&i.toAbortSignal()],r),h;const S=d&&d.unsubscribe&&(()=>{d.unsubscribe()});let x;try{if(o&&s2&&n!=="get"&&n!=="head"&&(x=await f2(c,l))!==0){let R=new Request(t,{method:"POST",body:l,duplex:"half"}),_;if(C.isFormData(l)&&(_=R.headers.get("content-type"))&&c.setContentType(_),R.body){const[E,A]=op(x,Jr(sp(o)));l=fp(R.body,dp,E,A)}}C.isString(f)||(f=f?"include":"omit");const y="credentials"in Request.prototype;h=new Request(t,{...p,signal:d,method:n.toUpperCase(),headers:c.normalize().toJSON(),body:l,duplex:"half",credentials:y?f:void 0});let m=await fetch(h);const v=js&&(s==="stream"||s==="response");if(js&&(u||v&&S)){const R={};["status","statusText","headers"].forEach(N=>{R[N]=m[N]});const _=C.toFiniteNumber(m.headers.get("content-length")),[E,A]=u&&op(_,Jr(sp(u),!0))||[];m=new Response(fp(m.body,dp,E,()=>{A&&A(),S&&S()}),R)}s=s||"text";let b=await Pr[C.findKey(Pr,s)||"text"](m,e);return!v&&S&&S(),await new Promise((R,_)=>{r0(R,_,{data:b,headers:qe.from(m.headers),status:m.status,statusText:m.statusText,config:e,request:h})})}catch(y){throw S&&S(),y&&y.name==="TypeError"&&/fetch/i.test(y.message)?Object.assign(new q("Network Error",q.ERR_NETWORK,e,h),{cause:y.cause||y}):q.from(y,y&&y.code,e,h)}}),Ns={http:AE,xhr:l2,fetch:d2};C.forEach(Ns,(e,t)=>{if(e){try{Object.defineProperty(e,"name",{value:t})}catch{}Object.defineProperty(e,"adapterName",{value:t})}});const pp=e=>`- ${e}`,p2=e=>C.isFunction(e)||e===null||e===!1,f0={getAdapter:e=>{e=C.isArray(e)?e:[e];const{length:t}=e;let n,l;const a={};for(let i=0;i<t;i++){n=e[i];let r;if(l=n,!p2(n)&&(l=Ns[(r=String(n)).toLowerCase()],l===void 0))throw new q(`Unknown adapter '${r}'`);if(l)break;a[r||"#"+i]=l}if(!l){const i=Object.entries(a).map(([u,o])=>`adapter ${u} `+(o===!1?"is not supported by the environment":"is not available in the build"));let r=t?i.length>1?`since :
`+i.map(pp).join(`
`):" "+pp(i[0]):"as no adapter specified";throw new q("There is no suitable adapter to dispatch the request "+r,"ERR_NOT_SUPPORT")}return l},adapters:Ns};function To(e){if(e.cancelToken&&e.cancelToken.throwIfRequested(),e.signal&&e.signal.aborted)throw new xa(null,e)}function gp(e){return To(e),e.headers=qe.from(e.headers),e.data=Oo.call(e,e.transformRequest),["post","put","patch"].indexOf(e.method)!==-1&&e.headers.setContentType("application/x-www-form-urlencoded",!1),f0.getAdapter(e.adapter||Vi.adapter)(e).then(function(l){return To(e),l.data=Oo.call(e,e.transformResponse,l),l.headers=qe.from(l.headers),l},function(l){return i0(l)||(To(e),l&&l.response&&(l.response.data=Oo.call(e,e.transformResponse,l.response),l.response.headers=qe.from(l.response.headers))),Promise.reject(l)})}const d0="1.7.9",Ku={};["object","boolean","number","function","string","symbol"].forEach((e,t)=>{Ku[e]=function(l){return typeof l===e||"a"+(t<1?"n ":" ")+e}});const hp={};Ku.transitional=function(t,n,l){function a(i,r){return"[Axios v"+d0+"] Transitional option '"+i+"'"+r+(l?". "+l:"")}return(i,r,u)=>{if(t===!1)throw new q(a(r," has been removed"+(n?" in "+n:"")),q.ERR_DEPRECATED);return n&&!hp[r]&&(hp[r]=!0,console.warn(a(r," has been deprecated since v"+n+" and will be removed in the near future"))),t?t(i,r,u):!0}};Ku.spelling=function(t){return(n,l)=>(console.warn(`${l} is likely a misspelling of ${t}`),!0)};function g2(e,t,n){if(typeof e!="object")throw new q("options must be an object",q.ERR_BAD_OPTION_VALUE);const l=Object.keys(e);let a=l.length;for(;a-- >0;){const i=l[a],r=t[i];if(r){const u=e[i],o=u===void 0||r(u,i,e);if(o!==!0)throw new q("option "+i+" must be "+o,q.ERR_BAD_OPTION_VALUE);continue}if(n!==!0)throw new q("Unknown option "+i,q.ERR_BAD_OPTION)}}const Dr={assertOptions:g2,validators:Ku},Ot=Dr.validators;class il{constructor(t){this.defaults=t,this.interceptors={request:new rp,response:new rp}}async request(t,n){try{return await this._request(t,n)}catch(l){if(l instanceof Error){let a={};Error.captureStackTrace?Error.captureStackTrace(a):a=new Error;const i=a.stack?a.stack.replace(/^.+\n/,""):"";try{l.stack?i&&!String(l.stack).endsWith(i.replace(/^.+\n.+\n/,""))&&(l.stack+=`
`+i):l.stack=i}catch{}}throw l}}_request(t,n){typeof t=="string"?(n=n||{},n.url=t):n=t||{},n=pl(this.defaults,n);const{transitional:l,paramsSerializer:a,headers:i}=n;l!==void 0&&Dr.assertOptions(l,{silentJSONParsing:Ot.transitional(Ot.boolean),forcedJSONParsing:Ot.transitional(Ot.boolean),clarifyTimeoutError:Ot.transitional(Ot.boolean)},!1),a!=null&&(C.isFunction(a)?n.paramsSerializer={serialize:a}:Dr.assertOptions(a,{encode:Ot.function,serialize:Ot.function},!0)),Dr.assertOptions(n,{baseUrl:Ot.spelling("baseURL"),withXsrfToken:Ot.spelling("withXSRFToken")},!0),n.method=(n.method||this.defaults.method||"get").toLowerCase();let r=i&&C.merge(i.common,i[n.method]);i&&C.forEach(["delete","get","head","post","put","patch","common"],h=>{delete i[h]}),n.headers=qe.concat(r,i);const u=[];let o=!0;this.interceptors.request.forEach(function(S){typeof S.runWhen=="function"&&S.runWhen(n)===!1||(o=o&&S.synchronous,u.unshift(S.fulfilled,S.rejected))});const s=[];this.interceptors.response.forEach(function(S){s.push(S.fulfilled,S.rejected)});let c,f=0,p;if(!o){const h=[gp.bind(this),void 0];for(h.unshift.apply(h,u),h.push.apply(h,s),p=h.length,c=Promise.resolve(n);f<p;)c=c.then(h[f++],h[f++]);return c}p=u.length;let d=n;for(f=0;f<p;){const h=u[f++],S=u[f++];try{d=h(d)}catch(x){S.call(this,x);break}}try{c=gp.call(this,d)}catch(h){return Promise.reject(h)}for(f=0,p=s.length;f<p;)c=c.then(s[f++],s[f++]);return c}getUri(t){t=pl(this.defaults,t);const n=u0(t.baseURL,t.url);return n0(n,t.params,t.paramsSerializer)}}C.forEach(["delete","get","head","options"],function(t){il.prototype[t]=function(n,l){return this.request(pl(l||{},{method:t,url:n,data:(l||{}).data}))}});C.forEach(["post","put","patch"],function(t){function n(l){return function(i,r,u){return this.request(pl(u||{},{method:t,headers:l?{"Content-Type":"multipart/form-data"}:{},url:i,data:r}))}}il.prototype[t]=n(),il.prototype[t+"Form"]=n(!0)});class df{constructor(t){if(typeof t!="function")throw new TypeError("executor must be a function.");let n;this.promise=new Promise(function(i){n=i});const l=this;this.promise.then(a=>{if(!l._listeners)return;let i=l._listeners.length;for(;i-- >0;)l._listeners[i](a);l._listeners=null}),this.promise.then=a=>{let i;const r=new Promise(u=>{l.subscribe(u),i=u}).then(a);return r.cancel=function(){l.unsubscribe(i)},r},t(function(i,r,u){l.reason||(l.reason=new xa(i,r,u),n(l.reason))})}throwIfRequested(){if(this.reason)throw this.reason}subscribe(t){if(this.reason){t(this.reason);return}this._listeners?this._listeners.push(t):this._listeners=[t]}unsubscribe(t){if(!this._listeners)return;const n=this._listeners.indexOf(t);n!==-1&&this._listeners.splice(n,1)}toAbortSignal(){const t=new AbortController,n=l=>{t.abort(l)};return this.subscribe(n),t.signal.unsubscribe=()=>this.unsubscribe(n),t.signal}static source(){let t;return{token:new df(function(a){t=a}),cancel:t}}}function h2(e){return function(n){return e.apply(null,n)}}function m2(e){return C.isObject(e)&&e.isAxiosError===!0}const zs={Continue:100,SwitchingProtocols:101,Processing:102,EarlyHints:103,Ok:200,Created:201,Accepted:202,NonAuthoritativeInformation:203,NoContent:204,ResetContent:205,PartialContent:206,MultiStatus:207,AlreadyReported:208,ImUsed:226,MultipleChoices:300,MovedPermanently:301,Found:302,SeeOther:303,NotModified:304,UseProxy:305,Unused:306,TemporaryRedirect:307,PermanentRedirect:308,BadRequest:400,Unauthorized:401,PaymentRequired:402,Forbidden:403,NotFound:404,MethodNotAllowed:405,NotAcceptable:406,ProxyAuthenticationRequired:407,RequestTimeout:408,Conflict:409,Gone:410,LengthRequired:411,PreconditionFailed:412,PayloadTooLarge:413,UriTooLong:414,UnsupportedMediaType:415,RangeNotSatisfiable:416,ExpectationFailed:417,ImATeapot:418,MisdirectedRequest:421,UnprocessableEntity:422,Locked:423,FailedDependency:424,TooEarly:425,UpgradeRequired:426,PreconditionRequired:428,TooManyRequests:429,RequestHeaderFieldsTooLarge:431,UnavailableForLegalReasons:451,InternalServerError:500,NotImplemented:501,BadGateway:502,ServiceUnavailable:503,GatewayTimeout:504,HttpVersionNotSupported:505,VariantAlsoNegotiates:506,InsufficientStorage:507,LoopDetected:508,NotExtended:510,NetworkAuthenticationRequired:511};Object.entries(zs).forEach(([e,t])=>{zs[t]=e});function p0(e){const t=new il(e),n=Ym(il.prototype.request,t);return C.extend(n,il.prototype,t,{allOwnKeys:!0}),C.extend(n,t,null,{allOwnKeys:!0}),n.create=function(a){return p0(pl(e,a))},n}const le=p0(Vi);le.Axios=il;le.CanceledError=xa;le.CancelToken=df;le.isCancel=i0;le.VERSION=d0;le.toFormData=Qu;le.AxiosError=q;le.Cancel=le.CanceledError;le.all=function(t){return Promise.all(t)};le.spread=h2;le.isAxiosError=m2;le.mergeConfig=pl;le.AxiosHeaders=qe;le.formToJSON=e=>a0(C.isHTMLForm(e)?new FormData(e):e);le.getAdapter=f0.getAdapter;le.HttpStatusCode=zs;le.default=le;le.defaults.xsrfHeaderName="X-CSRFToken";async function y2(e=3){return(await le.get("/m/api/extractions",{params:{past_days:e}})).data}async function v2(){return(await le.get("/m/api/extractions/past")).data}async function S2(){return(await le.get("/m/api/extractions/future")).data}async function x2(){return(await le.get("/m/api/user/permissions")).data}async function w2(){return(await le.get("/m/api/admin/list")).data}async function b2(){return(await le.get("/m/api/admin/explain")).data}async function R2(){return(await le.get("/m/api/admin/outstanding")).data}const E2=()=>{const{isLoading:e,isFetching:t,error:n,data:l}=Sl(["explain"],()=>b2(),{refetchOnWindowFocus:!0});return g.jsxs(g.Fragment,{children:[g.jsx(Wc,{striped:t,variant:n?"danger":t?"info":"success",now:100}),g.jsxs("pre",{children:[`#Tax Steps Explanation (Highest Rank First):
`,e?"Loading...":g.jsxs(g.Fragment,{children:[`---------------------------------------------------------------------------------------------------------
`,l==null?void 0:l.taxes.map(a=>g.jsxs(g.Fragment,{children:[console.log(a),`${a.name}
`,`  - Structures Captured in Tax Rank:
`,a.structures.map(i=>`     - ${i}
`),`---------------------------------------------------------------------------------------------------------
`]})),`
#All Structures Seen:
`,l==null?void 0:l.structures.map(a=>g.jsx(g.Fragment,{children:`  - ${a}
`}))]})]})]})},C2=()=>{const{isLoading:e,isFetching:t,error:n,data:l}=Sl(["outstanding"],()=>R2(),{refetchOnWindowFocus:!1});return g.jsxs(g.Fragment,{children:[g.jsx(Wc,{striped:t,variant:n?"danger":t?"info":"success",now:100}),g.jsxs("pre",{children:[`Mining Taxes:
`,e?"Loading...":"",l==null?void 0:l.map(a=>g.jsx(g.Fragment,{children:`${a}
`}))]})]})},_2=()=>{const{isLoading:e,isFetching:t,data:n}=Sl(["admin-corps"],()=>w2(),{initialData:[],refetchOnWindowFocus:!1}),l=Bu(),a=[l.accessor("name",{header:"Corporation"}),l.accessor("char_tokens",{header:"Character Level Tokens"}),l.accessor("corp_tokens",{header:"Corp Level Tokens"}),l.accessor("obs",{header:"Last Observation Update"}),l.accessor("frack",{header:"Last Frack Update"})];return g.jsxs(Kl,{children:[g.jsx(Gu,{isLoading:e,isFetching:t,data:n,columns:a}),g.jsx("br",{}),g.jsx(lr,{children:g.jsx(lr.Body,{children:g.jsx(Kl,{children:g.jsx(C2,{})})})}),g.jsx("br",{}),g.jsx(lr,{children:g.jsx(lr.Body,{children:g.jsx(Kl,{children:g.jsx(E2,{})})})})]})},pf=(e,t,n)=>{if(n){console.log(e,t,n);let l=e.getValue(t).reduce((a,i)=>a+"  "+i.type.cat+" "+i.type.name,"");return l?l.toLowerCase().includes(n.toLowerCase()):!1}else return!0},A2="_ore1884_hgwkx_1",O2="_ore1921_hgwkx_5",T2="_ore1920_hgwkx_9",D2="_ore1922_hgwkx_13",M2="_ore1923_hgwkx_17",Dt={ore1884:A2,ore1921:O2,ore1920:T2,ore1922:D2,ore1923:M2},Do={1884:Dt.ore1884,1921:Dt.ore1920,1920:Dt.ore1921,1922:Dt.ore1922,1923:Dt.ore1923},gf=()=>g.jsx("div",{className:"flex-container",children:g.jsxs("div",{className:"text-center",children:[g.jsx(Je,{className:`${Dt.ore1884} fw-normal mx-1`,children:"Ubiquitous Moon Asteroids"}),g.jsx(Je,{className:`${Dt.ore1920} fw-normal mx-1`,children:"Common Moon Asteroids"}),g.jsx(Je,{className:`${Dt.ore1921} fw-normal mx-1`,children:"Uncommon Moon Asteroids"}),g.jsx(Je,{className:`${Dt.ore1922} fw-normal mx-1`,children:"Rare Moon Asteroids"}),g.jsx(Je,{className:`${Dt.ore1923} fw-normal mx-1`,children:"Exceptional Moon Asteroids"})]})}),hf=({ore:e,percent:t,badgePercent:n,value:l=0,valueMessage:a=""})=>g.jsxs("div",{className:"d-flex align-items-center",children:[g.jsx("img",{alt:e.type.name,style:{width:"30px",height:"30px",margin:"15px",borderRadius:"30px"},className:`${Do[e.type.cat_id]}`,src:"https://images.evetech.net/types/"+e.type.id+"/icon"}),g.jsxs("div",{style:{flexGrow:1},children:[e.type.name," ",g.jsxs("div",{className:"float-end",children:[g.jsxs(Je,{pill:!0,className:"fw-normal",children:[n.toFixed(0),"% of Ore Composition"]}),l>0?g.jsxs(Je,{pill:!0,className:"fw-normal",children:["$",Number(l/1e9).toFixed(1),"B ",a]}):g.jsx(g.Fragment,{}),g.jsx(Je,{pill:!0,className:`pull-right fw-normal ${Do[e.type.cat_id]}`,children:e.type.cat})]}),g.jsx("div",{className:"progress my-1",children:g.jsxs("div",{className:`progress-bar progress-bar-striped bg-info fw-bold ${Do[e.type.cat_id]}`,style:{width:t+"%"},children:[t.toFixed(0),"%"]})})]})]});var j2=["date","verboseDate","tooltip","children"];function Us(){return Us=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var l in n)Object.prototype.hasOwnProperty.call(n,l)&&(e[l]=n[l])}return e},Us.apply(this,arguments)}function N2(e,t){if(e==null)return{};var n=z2(e,t),l,a;if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(a=0;a<i.length;a++)l=i[a],!(t.indexOf(l)>=0)&&Object.prototype.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}function z2(e,t){if(e==null)return{};var n={},l=Object.keys(e),a,i;for(i=0;i<l.length;i++)a=l[i],!(t.indexOf(a)>=0)&&(n[a]=e[a]);return n}function Ir(e,t){var n=e.date,l=e.verboseDate,a=e.tooltip,i=e.children,r=N2(e,j2),u=w.useMemo(function(){return n.toISOString()},[n]);return ye.createElement("time",Us({ref:t},r,{dateTime:u,title:a?l:void 0}),i)}Ir=ye.forwardRef(Ir);Ir.propTypes={date:z.instanceOf(Date).isRequired,verboseDate:z.string,tooltip:z.bool.isRequired,children:z.string.isRequired};var mp=Number.isNaN||function(t){return typeof t=="number"&&t!==t};function U2(e,t){return!!(e===t||mp(e)&&mp(t))}function F2(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(!U2(e[n],t[n]))return!1;return!0}function B2(e,t){t===void 0&&(t=F2);var n=null;function l(){for(var a=[],i=0;i<arguments.length;i++)a[i]=arguments[i];if(n&&n.lastThis===this&&t(a,n.lastArgs))return n.lastResult;var r=e.apply(this,a);return n={lastResult:r,lastArgs:a,lastThis:this},r}return l.clear=function(){n=null},l}function Fs(e){"@babel/helpers - typeof";return Fs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Fs(e)}function H2(e){if(g0())return Intl.DateTimeFormat.supportedLocalesOf(e)[0]}function g0(){var e=(typeof Intl>"u"?"undefined":Fs(Intl))==="object";return e&&typeof Intl.DateTimeFormat=="function"}function Wr(e){"@babel/helpers - typeof";return Wr=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Wr(e)}function L2(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function V2(e,t){for(var n=0;n<t.length;n++){var l=t[n];l.enumerable=l.enumerable||!1,l.configurable=!0,"value"in l&&(l.writable=!0),Object.defineProperty(e,l.key,l)}}function $2(e,t,n){return V2(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}var h0=function(){function e(){L2(this,e),this.cache={}}return $2(e,[{key:"get",value:function(){for(var n=this.cache,l=arguments.length,a=new Array(l),i=0;i<l;i++)a[i]=arguments[i];for(var r=0,u=a;r<u.length;r++){var o=u[r];if(Wr(n)!=="object")return;n=n[o]}return n}},{key:"put",value:function(){for(var n=arguments.length,l=new Array(n),a=0;a<n;a++)l[a]=arguments[a];for(var i=l.pop(),r=l.pop(),u=this.cache,o=0,s=l;o<s.length;o++){var c=s[o];Wr(u[c])!=="object"&&(u[c]={}),u=u[c]}return u[r]=i}}]),e}(),yp=new h0,q2=g0(),G2=function(t){return t.toString()};function Y2(e,t){if(!q2)return G2;var n=k2(e),l=JSON.stringify(t),a=yp.get(String(n),l)||yp.put(String(n),l,new Intl.DateTimeFormat(n,t));return function(i){return a.format(i)}}const X2=B2(Y2);var Mo={};function k2(e){var t=e.toString();return Mo[t]?Mo[t]:Mo[t]=H2(e)}function Bs(e){"@babel/helpers - typeof";return Bs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Bs(e)}function Q2(e){return Z2(e)?e:new Date(e)}function Z2(e){return e instanceof Date||K2(e)}function K2(e){return Bs(e)==="object"&&typeof e.getTime=="function"}var vp=new h0;function J2(e,t){var n=t.polyfill,l=String(e)+":"+String(n);return vp.get(l)||vp.put(l,new jc(e,{polyfill:n}))}var Ju={exports:{}},$a={exports:{}};(function(){var e,t,n,l,a,i;typeof performance<"u"&&performance!==null&&performance.now?$a.exports=function(){return performance.now()}:typeof process<"u"&&process!==null&&process.hrtime?($a.exports=function(){return(e()-a)/1e6},t=process.hrtime,e=function(){var r;return r=t(),r[0]*1e9+r[1]},l=e(),i=process.uptime()*1e9,a=l-i):Date.now?($a.exports=function(){return Date.now()-n},n=Date.now()):($a.exports=function(){return new Date().getTime()-n},n=new Date().getTime())}).call(Yg);var P2=$a.exports,I2=P2,nn=typeof window>"u"?Yg:window,or=["moz","webkit"],Jl="AnimationFrame",sa=nn["request"+Jl],vi=nn["cancel"+Jl]||nn["cancelRequest"+Jl];for(var Ma=0;!sa&&Ma<or.length;Ma++)sa=nn[or[Ma]+"Request"+Jl],vi=nn[or[Ma]+"Cancel"+Jl]||nn[or[Ma]+"CancelRequest"+Jl];if(!sa||!vi){var jo=0,Sp=0,kn=[],W2=1e3/60;sa=function(e){if(kn.length===0){var t=I2(),n=Math.max(0,W2-(t-jo));jo=n+t,setTimeout(function(){var l=kn.slice(0);kn.length=0;for(var a=0;a<l.length;a++)if(!l[a].cancelled)try{l[a].callback(jo)}catch(i){setTimeout(function(){throw i},0)}},Math.round(n))}return kn.push({handle:++Sp,callback:e,cancelled:!1}),Sp},vi=function(e){for(var t=0;t<kn.length;t++)kn[t].handle===e&&(kn[t].cancelled=!0)}}Ju.exports=function(e){return sa.call(nn,e)};Ju.exports.cancel=function(){vi.apply(nn,arguments)};Ju.exports.polyfill=function(e){e||(e=nn),e.requestAnimationFrame=sa,e.cancelAnimationFrame=vi};var eC=Ju.exports;const xp=zi(eC);function tC(e,t){if(e.length===0)return 0;for(var n=0,l=e.length-1,a;n<=l;){a=Math.floor((l+n)/2);var i=t(e[a]);if(i===0)return a;if(i<0){if(n=a+1,n>l)return n}else if(l=a-1,l<n)return n}}function nC(e,t){return rC(e)||iC(e,t)||aC(e,t)||lC()}function lC(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function aC(e,t){if(e){if(typeof e=="string")return wp(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return wp(e,t)}}function wp(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,l=new Array(t);n<t;n++)l[n]=e[n];return l}function iC(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var l=[],a=!0,i=!1,r,u;try{for(n=n.call(e);!(a=(r=n.next()).done)&&(l.push(r.value),!(t&&l.length===t));a=!0);}catch(o){i=!0,u=o}finally{try{!a&&n.return!=null&&n.return()}finally{if(i)throw u}}return l}}function rC(e){if(Array.isArray(e))return e}const uC={instances:[],add:function(t){var n=this,l=this.instances.length===0;return m0(this.instances,t),l&&this.start(),{stop:function(){y0(n.instances,t),n.instances.length===0&&n.stop()},forceUpdate:function(){bp(t,n.instances)}}},tick:function(){for(var t=Date.now();;){var n=this.instances[0];if(t>=n.nextUpdateTime)bp(n,this.instances);else break}},scheduleNextTick:function(){var t=this;this.scheduledTick=xp(function(){t.tick(),t.scheduleNextTick()})},start:function(){this.scheduleNextTick()},stop:function(){xp.cancel(this.scheduledTick)}};function oC(e){var t=e.getNextValue(),n=nC(t,2),l=n[0],a=n[1];e.setValue(l),e.nextUpdateTime=a}function bp(e,t){oC(e),y0(t,e),m0(t,e)}function m0(e,t){var n=sC(e,t);e.splice(n,0,t)}function y0(e,t){var n=e.indexOf(t);e.splice(n,1)}function sC(e,t){var n=t.nextUpdateTime;return tC(e,function(l){return l.nextUpdateTime===n?0:l.nextUpdateTime>n?1:-1})}function sr(e,t){return pC(e)||dC(e,t)||fC(e,t)||cC()}function cC(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function fC(e,t){if(e){if(typeof e=="string")return Rp(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return Rp(e,t)}}function Rp(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,l=new Array(t);n<t;n++)l[n]=e[n];return l}function dC(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var l=[],a=!0,i=!1,r,u;try{for(n=n.call(e);!(a=(r=n.next()).done)&&(l.push(r.value),!(t&&l.length===t));a=!0);}catch(o){i=!0,u=o}finally{try{!a&&n.return!=null&&n.return()}finally{if(i)throw u}}return l}}function pC(e){if(Array.isArray(e))return e}function gC(e){var t=e.date,n=e.future,l=e.locale,a=e.locales,i=e.timeStyle,r=e.round,u=e.minTimeLeft,o=e.formatVerboseDate,s=e.verboseDateFormat,c=s===void 0?yC:s,f=e.updateInterval,p=e.tick,d=p===void 0?!0:p,h=e.now,S=e.timeOffset,x=S===void 0?0:S,y=e.polyfill,m=w.useMemo(function(){return l&&(a=[l]),a.concat(jc.getDefaultLocale())},[l,a]),v=w.useMemo(function(){return J2(m,{polyfill:y})},[m,y]);t=w.useMemo(function(){return Q2(t)},[t]);var b=w.useCallback(function(){var U=(h||Date.now())-x,L;if(n&&U>=t.getTime()&&(U=t.getTime(),L=!0),u!==void 0){var k=t.getTime()-u*1e3;U>k&&(U=k,L=!0)}var Q=v.format(t,i,{getTimeToNextUpdate:!0,now:U,future:n,round:r}),Fe=sr(Q,2),it=Fe[0],ke=Fe[1];return L?ke=mC:ke=f||ke||60*1e3,[it,U+ke]},[t,n,i,f,r,u,v,h]),R=w.useRef();R.current=b;var _=w.useMemo(b,[]),E=sr(_,2),A=E[0],N=E[1],M=w.useState(A),j=sr(M,2),V=j[0],H=j[1],K=w.useRef();w.useEffect(function(){if(d)return K.current=uC.add({getNextValue:function(){return R.current()},setValue:H,nextUpdateTime:N}),function(){return K.current.stop()}},[d]),w.useEffect(function(){if(K.current)K.current.forceUpdate();else{var U=b(),L=sr(U,1),k=L[0];H(k)}},[b]);var P=w.useMemo(function(){return X2(m,c)},[m,c]),D=w.useMemo(function(){return o?o(t):P(t)},[t,o,P]);return{date:t,formattedDate:V,verboseDate:D}}var hC=365*24*60*60*1e3,mC=1e3*hC,yC={weekday:"long",day:"numeric",month:"long",year:"numeric",hour:"numeric",minute:"2-digit",second:"2-digit"},Hs=z.oneOfType,Ep=z.arrayOf,Fl=z.string,Cp=z.number,qa=z.shape,cr=z.func,vC=Hs([qa({minTime:Cp,formatAs:Fl.isRequired}),qa({test:cr,formatAs:Fl.isRequired}),qa({minTime:Cp,format:cr.isRequired}),qa({test:cr,format:cr.isRequired})]),SC=Hs([Fl,qa({steps:Ep(vC).isRequired,labels:Hs([Fl,Ep(Fl)]).isRequired,round:Fl})]),xC=["date","future","timeStyle","round","minTimeLeft","locale","locales","formatVerboseDate","verboseDateFormat","updateInterval","tick","now","timeOffset","polyfill","tooltip","component","container","wrapperComponent","wrapperProps"];function eu(){return eu=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var l in n)Object.prototype.hasOwnProperty.call(n,l)&&(e[l]=n[l])}return e},eu.apply(this,arguments)}function wC(e,t){if(e==null)return{};var n=bC(e,t),l,a;if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(a=0;a<i.length;a++)l=i[a],!(t.indexOf(l)>=0)&&Object.prototype.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}function bC(e,t){if(e==null)return{};var n={},l=Object.keys(e),a,i;for(i=0;i<l.length;i++)a=l[i],!(t.indexOf(a)>=0)&&(n[a]=e[a]);return n}function tu(e){var t=e.date,n=e.future,l=e.timeStyle,a=e.round,i=e.minTimeLeft,r=e.locale,u=e.locales,o=u===void 0?[]:u,s=e.formatVerboseDate,c=e.verboseDateFormat,f=e.updateInterval,p=e.tick,d=e.now,h=e.timeOffset,S=e.polyfill,x=e.tooltip,y=x===void 0?!0:x,m=e.component,v=m===void 0?Ir:m,b=e.container,R=e.wrapperComponent,_=e.wrapperProps,E=wC(e,xC),A=gC({date:t,future:n,timeStyle:l,round:a,minTimeLeft:i,locale:r,locales:o,formatVerboseDate:s,verboseDateFormat:c,updateInterval:f,tick:p,now:d,timeOffset:h,polyfill:S}),N=A.date,M=A.verboseDate,j=A.formattedDate,V=ye.createElement(v,eu({date:N,verboseDate:M,tooltip:y},E),j),H=R||b;return H?ye.createElement(H,eu({},_,{verboseDate:M}),V):V}tu.propTypes={date:z.oneOfType([z.instanceOf(Date),z.number]).isRequired,locale:z.string,locales:z.arrayOf(z.string),future:z.bool,timeStyle:SC,round:z.string,minTimeLeft:z.number,component:z.elementType,tooltip:z.bool,formatVerboseDate:z.func,verboseDateFormat:z.object,updateInterval:z.oneOfType([z.number,z.arrayOf(z.shape({threshold:z.number,interval:z.number.isRequired}))]),tick:z.bool,now:z.number,timeOffset:z.number,polyfill:z.bool,wrapperComponent:z.elementType,wrapperProps:z.object};tu=ye.memo(tu);const RC=Intl.DateTimeFormat("default",{year:"numeric",month:"long",day:"numeric"}),EC=Intl.DateTimeFormat("default",{hour:"numeric",minute:"numeric",hour12:!1}),mf=({stringDate:e})=>{const t=Date.parse(e);return g.jsxs("div",{className:"text-center",children:[RC.format(t)," ",EC.format(t),g.jsx("br",{}),g.jsx(Je,{className:"",children:g.jsx(tu,{date:t})})]})},CC=()=>{const{isFetching:e,data:t}=Sl({queryKey:["extractions"],queryFn:()=>y2(),initialData:[],refetchOnWindowFocus:!1}),n=Bu(),l=[n.accessor("extraction_end",{header:"Frack Arrival",cell:a=>g.jsx(mf,{stringDate:a.getValue()}),enableColumnFilter:!1}),n.accessor("moon.name",{header:"Moon",cell:a=>g.jsxs("div",{className:"text-center",children:[g.jsx("h6",{children:a.getValue()}),g.jsx("br",{}),g.jsx("p",{className:"m-0",children:a.cell.row.original.ObserverName}),g.jsxs("p",{children:[a.cell.row.original.constellation," - ",a.cell.row.original.region]}),a.cell.row.original.value>0&&g.jsxs(Je,{children:["Current Mined Value: $",Number(a.cell.row.original.value/1e9).toFixed(2),"B"]})]})}),n.accessor("jackpot",{header:"Jackpot",cell:a=>a.getValue()?g.jsx("div",{className:"text-center jackpot",children:g.jsx("i",{className:"fas fa-award",style:{fontSize:"64px"}})}):g.jsx(g.Fragment,{})}),n.accessor("mined_ore",{header:"Ore Composition",cell:a=>{var i;return a.getValue()?g.jsx(g.Fragment,{children:g.jsx("div",{className:"",children:(i=a.getValue())==null?void 0:i.map(r=>{let u=Number(r.volume/r.total_volume*100);return g.jsx(hf,{ore:r,percent:u,badgePercent:r.total_volume/a.cell.row.original.total_m3*100,value:r.value,valueMessage:"Mined"})})})}):g.jsx(g.Fragment,{})},filterFn:pf})];return g.jsxs(Kl,{children:[g.jsx("h5",{className:"text-center small",children:"Key:"}),g.jsx(gf,{}),g.jsx("br",{}),g.jsx(Gu,{isFetching:e,data:t,columns:l})]})},_C=()=>{const{isLoading:e,data:t}=Sl(["future-extractions"],()=>S2(),{initialData:[],refetchOnWindowFocus:!1}),n=Bu(),l=[n.accessor("extraction_end",{header:"Frack Arrival",cell:a=>g.jsx(mf,{stringDate:a.getValue()}),enableColumnFilter:!1}),n.accessor("moon.name",{header:"Moon",cell:a=>g.jsxs("div",{className:"text-center",children:[g.jsx("h6",{children:a.getValue()}),g.jsx("br",{}),g.jsx("p",{className:"m-0",children:a.cell.row.original.ObserverName}),g.jsxs("p",{children:[a.cell.row.original.constellation," - ",a.cell.row.original.region]}),a.cell.row.original.value>0&&g.jsxs(Je,{children:["Value Estimate: $",Number(a.cell.row.original.value/1e9).toFixed(2),"B"]})]})}),n.accessor("jackpot",{header:"Jackpot",cell:a=>a.getValue()?g.jsx("div",{className:"text-center jackpot",children:g.jsx("i",{className:"fas fa-award",style:{fontSize:"64px"}})}):g.jsx(g.Fragment,{})}),n.accessor("mined_ore",{header:"Ore Composition",cell:a=>{var i;return a.getValue()?g.jsx(g.Fragment,{children:g.jsx("div",{className:"align-items-center",children:(i=a.getValue())==null?void 0:i.map(r=>{let u=r.total_volume/a.cell.row.original.total_m3*100;return g.jsx(hf,{ore:r,percent:u,badgePercent:r.total_volume/a.cell.row.original.total_m3*100,value:r.value,valueMessage:"Estimated"})})})}):g.jsx(g.Fragment,{})},filterFn:pf})];return g.jsxs(Kl,{children:[g.jsx("h5",{className:"text-center small",children:"Key:"}),g.jsx(gf,{}),g.jsx("br",{}),g.jsx(Gu,{isLoading:e,data:t,columns:l})]})},_p=document.getElementById("nav-left"),AC=({futureExtractions:e=!1,limitedFutureExtractions:t=!1,admin:n=!1})=>_p?Xg.createPortal(g.jsxs(g.Fragment,{children:[g.jsx(tr,{to:"active",className:"nav-link",children:"Active Moons"}),(e||t)&&g.jsx(tr,{to:"future",className:"nav-link",children:"Future Extractions"}),e&&g.jsx(tr,{to:"past",className:"nav-link",children:"Past Extractions"}),n&&g.jsx(tr,{to:"admin",className:"nav-link",children:"Admin"})]}),_p):g.jsx(g.Fragment,{}),OC=()=>{const{isLoading:e,data:t}=Sl(["perms"],()=>x2(),{refetchOnWindowFocus:!1});return console.log("here"),e?g.jsxs(g.Fragment,{children:[g.jsx(VR,{}),g.jsx(vd,{})," "]}):g.jsxs(g.Fragment,{children:[g.jsx(AC,{futureExtractions:t.view_observations,limitedFutureExtractions:t.view_limited_future,admin:t.su}),g.jsx(vd,{})," "]})},TC=()=>{const{isLoading:e,data:t}=Sl(["extractions","Past"],()=>v2(),{initialData:[],refetchOnWindowFocus:!1}),n=Bu(),l=[n.accessor("extraction_end",{header:"Frack Arrival",cell:a=>g.jsx(mf,{stringDate:a.getValue()}),enableColumnFilter:!1}),n.accessor("moon.name",{header:"Moon",cell:a=>g.jsxs("div",{className:"text-center",children:[g.jsx("h6",{children:a.getValue()}),g.jsx("br",{}),g.jsx("p",{className:"m-0",children:a.cell.row.original.ObserverName}),g.jsxs("p",{children:[a.cell.row.original.constellation," - ",a.cell.row.original.region]}),a.cell.row.original.value>0&&g.jsxs(Je,{children:["Total Mined Value $",Number(a.cell.row.original.value/1e9).toFixed(2),"B"]})]})}),n.accessor("jackpot",{header:"Jackpot",cell:a=>a.getValue()?g.jsx("div",{className:"text-center jackpot",children:g.jsx("i",{className:"fas fa-award",style:{fontSize:"64px"}})}):g.jsx(g.Fragment,{})}),n.accessor("mined_ore",{header:"Ore Composition",cell:a=>{var i;return a.getValue()?g.jsx(g.Fragment,{children:g.jsx("div",{className:"",children:(i=a.getValue())==null?void 0:i.map(r=>{let u=Number(r.volume/r.total_volume*100);return g.jsx(hf,{ore:r,percent:u,badgePercent:r.total_volume/a.cell.row.original.total_m3*100,value:r.value,valueMessage:"Mined"})})})}):g.jsx(g.Fragment,{})},filterFn:pf})];return g.jsxs(Kl,{children:[g.jsx("h5",{className:"text-center small",children:"Key:"}),g.jsx(gf,{}),g.jsx("br",{}),g.jsx(Gu,{isLoading:e,data:t,columns:l})]})};jc.addDefaultLocale(r1);const DC=()=>g.jsx(g.Fragment,{children:g.jsx(c1,{children:g.jsxs(f1,{children:[g.jsx(Xn,{index:!0,element:g.jsx(Sd,{to:"m/r/",replace:!0})}),g.jsxs(Xn,{path:"m/r/",element:g.jsx(OC,{}),children:[g.jsx(Xn,{index:!0,element:g.jsx(Sd,{to:"active",replace:!0})}),g.jsx(Xn,{path:"active",element:g.jsx(CC,{})}),g.jsx(Xn,{path:"admin",element:g.jsx(_2,{})}),g.jsx(Xn,{path:"future",element:g.jsx(_C,{})}),g.jsx(Xn,{path:"past",element:g.jsx(TC,{})})]})]})})});var v0={exports:{}},Pu={},S0={exports:{}},x0={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */(function(e){function t(D,U){var L=D.length;D.push(U);e:for(;0<L;){var k=L-1>>>1,Q=D[k];if(0<a(Q,U))D[k]=U,D[L]=Q,L=k;else break e}}function n(D){return D.length===0?null:D[0]}function l(D){if(D.length===0)return null;var U=D[0],L=D.pop();if(L!==U){D[0]=L;e:for(var k=0,Q=D.length,Fe=Q>>>1;k<Fe;){var it=2*(k+1)-1,ke=D[it],Be=it+1,Qe=D[Be];if(0>a(ke,L))Be<Q&&0>a(Qe,ke)?(D[k]=Qe,D[Be]=L,k=Be):(D[k]=ke,D[it]=L,k=it);else if(Be<Q&&0>a(Qe,L))D[k]=Qe,D[Be]=L,k=Be;else break e}}return U}function a(D,U){var L=D.sortIndex-U.sortIndex;return L!==0?L:D.id-U.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var i=performance;e.unstable_now=function(){return i.now()}}else{var r=Date,u=r.now();e.unstable_now=function(){return r.now()-u}}var o=[],s=[],c=1,f=null,p=3,d=!1,h=!1,S=!1,x=typeof setTimeout=="function"?setTimeout:null,y=typeof clearTimeout=="function"?clearTimeout:null,m=typeof setImmediate<"u"?setImmediate:null;function v(D){for(var U=n(s);U!==null;){if(U.callback===null)l(s);else if(U.startTime<=D)l(s),U.sortIndex=U.expirationTime,t(o,U);else break;U=n(s)}}function b(D){if(S=!1,v(D),!h)if(n(o)!==null)h=!0,K();else{var U=n(s);U!==null&&P(b,U.startTime-D)}}var R=!1,_=-1,E=5,A=-1;function N(){return!(e.unstable_now()-A<E)}function M(){if(R){var D=e.unstable_now();A=D;var U=!0;try{e:{h=!1,S&&(S=!1,y(_),_=-1),d=!0;var L=p;try{t:{for(v(D),f=n(o);f!==null&&!(f.expirationTime>D&&N());){var k=f.callback;if(typeof k=="function"){f.callback=null,p=f.priorityLevel;var Q=k(f.expirationTime<=D);if(D=e.unstable_now(),typeof Q=="function"){f.callback=Q,v(D),U=!0;break t}f===n(o)&&l(o),v(D)}else l(o);f=n(o)}if(f!==null)U=!0;else{var Fe=n(s);Fe!==null&&P(b,Fe.startTime-D),U=!1}}break e}finally{f=null,p=L,d=!1}U=void 0}}finally{U?j():R=!1}}}var j;if(typeof m=="function")j=function(){m(M)};else if(typeof MessageChannel<"u"){var V=new MessageChannel,H=V.port2;V.port1.onmessage=M,j=function(){H.postMessage(null)}}else j=function(){x(M,0)};function K(){R||(R=!0,j())}function P(D,U){_=x(function(){D(e.unstable_now())},U)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(D){D.callback=null},e.unstable_continueExecution=function(){h||d||(h=!0,K())},e.unstable_forceFrameRate=function(D){0>D||125<D?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):E=0<D?Math.floor(1e3/D):5},e.unstable_getCurrentPriorityLevel=function(){return p},e.unstable_getFirstCallbackNode=function(){return n(o)},e.unstable_next=function(D){switch(p){case 1:case 2:case 3:var U=3;break;default:U=p}var L=p;p=U;try{return D()}finally{p=L}},e.unstable_pauseExecution=function(){},e.unstable_requestPaint=function(){},e.unstable_runWithPriority=function(D,U){switch(D){case 1:case 2:case 3:case 4:case 5:break;default:D=3}var L=p;p=D;try{return U()}finally{p=L}},e.unstable_scheduleCallback=function(D,U,L){var k=e.unstable_now();switch(typeof L=="object"&&L!==null?(L=L.delay,L=typeof L=="number"&&0<L?k+L:k):L=k,D){case 1:var Q=-1;break;case 2:Q=250;break;case 5:Q=1073741823;break;case 4:Q=1e4;break;default:Q=5e3}return Q=L+Q,D={id:c++,callback:U,priorityLevel:D,startTime:L,expirationTime:Q,sortIndex:-1},L>k?(D.sortIndex=L,t(s,D),n(o)===null&&D===n(s)&&(S?(y(_),_=-1):S=!0,P(b,L-k))):(D.sortIndex=Q,t(o,D),h||d||(h=!0,K())),D},e.unstable_shouldYield=N,e.unstable_wrapCallback=function(D){var U=p;return function(){var L=p;p=U;try{return D.apply(this,arguments)}finally{p=L}}}})(x0);S0.exports=x0;var MC=S0.exports;/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var we=MC,w0=w,jC=Xg;function O(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function b0(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}var NC=Symbol.for("react.element"),fr=Symbol.for("react.transitional.element"),Ga=Symbol.for("react.portal"),Bl=Symbol.for("react.fragment"),R0=Symbol.for("react.strict_mode"),Ls=Symbol.for("react.profiler"),zC=Symbol.for("react.provider"),E0=Symbol.for("react.consumer"),Pt=Symbol.for("react.context"),yf=Symbol.for("react.forward_ref"),Vs=Symbol.for("react.suspense"),$s=Symbol.for("react.suspense_list"),vf=Symbol.for("react.memo"),mn=Symbol.for("react.lazy"),C0=Symbol.for("react.offscreen"),UC=Symbol.for("react.memo_cache_sentinel"),Ap=Symbol.iterator;function ja(e){return e===null||typeof e!="object"?null:(e=Ap&&e[Ap]||e["@@iterator"],typeof e=="function"?e:null)}var FC=Symbol.for("react.client.reference");function qs(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===FC?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case Bl:return"Fragment";case Ga:return"Portal";case Ls:return"Profiler";case R0:return"StrictMode";case Vs:return"Suspense";case $s:return"SuspenseList"}if(typeof e=="object")switch(e.$$typeof){case Pt:return(e.displayName||"Context")+".Provider";case E0:return(e._context.displayName||"Context")+".Consumer";case yf:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case vf:return t=e.displayName||null,t!==null?t:qs(e.type)||"Memo";case mn:t=e._payload,e=e._init;try{return qs(e(t))}catch{}}return null}var G=w0.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,ue=Object.assign,No,Op;function Ya(e){if(No===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);No=t&&t[1]||"",Op=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+No+e+Op}var zo=!1;function Uo(e,t){if(!e||zo)return"";zo=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var l={DetermineComponentFrameRoot:function(){try{if(t){var f=function(){throw Error()};if(Object.defineProperty(f.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(f,[])}catch(d){var p=d}Reflect.construct(e,[],f)}else{try{f.call()}catch(d){p=d}e.call(f.prototype)}}else{try{throw Error()}catch(d){p=d}(f=e())&&typeof f.catch=="function"&&f.catch(function(){})}}catch(d){if(d&&p&&typeof d.stack=="string")return[d.stack,p.stack]}return[null,null]}};l.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var a=Object.getOwnPropertyDescriptor(l.DetermineComponentFrameRoot,"name");a&&a.configurable&&Object.defineProperty(l.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var i=l.DetermineComponentFrameRoot(),r=i[0],u=i[1];if(r&&u){var o=r.split(`
`),s=u.split(`
`);for(a=l=0;l<o.length&&!o[l].includes("DetermineComponentFrameRoot");)l++;for(;a<s.length&&!s[a].includes("DetermineComponentFrameRoot");)a++;if(l===o.length||a===s.length)for(l=o.length-1,a=s.length-1;1<=l&&0<=a&&o[l]!==s[a];)a--;for(;1<=l&&0<=a;l--,a--)if(o[l]!==s[a]){if(l!==1||a!==1)do if(l--,a--,0>a||o[l]!==s[a]){var c=`
`+o[l].replace(" at new "," at ");return e.displayName&&c.includes("<anonymous>")&&(c=c.replace("<anonymous>",e.displayName)),c}while(1<=l&&0<=a);break}}}finally{zo=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?Ya(n):""}function BC(e){switch(e.tag){case 26:case 27:case 5:return Ya(e.type);case 16:return Ya("Lazy");case 13:return Ya("Suspense");case 19:return Ya("SuspenseList");case 0:case 15:return e=Uo(e.type,!1),e;case 11:return e=Uo(e.type.render,!1),e;case 1:return e=Uo(e.type,!0),e;default:return""}}function Tp(e){try{var t="";do t+=BC(e),e=e.return;while(e);return t}catch(n){return`
Error generating stack: `+n.message+`
`+n.stack}}function wa(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,t.flags&4098&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function _0(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function Dp(e){if(wa(e)!==e)throw Error(O(188))}function HC(e){var t=e.alternate;if(!t){if(t=wa(e),t===null)throw Error(O(188));return t!==e?null:e}for(var n=e,l=t;;){var a=n.return;if(a===null)break;var i=a.alternate;if(i===null){if(l=a.return,l!==null){n=l;continue}break}if(a.child===i.child){for(i=a.child;i;){if(i===n)return Dp(a),e;if(i===l)return Dp(a),t;i=i.sibling}throw Error(O(188))}if(n.return!==l.return)n=a,l=i;else{for(var r=!1,u=a.child;u;){if(u===n){r=!0,n=a,l=i;break}if(u===l){r=!0,l=a,n=i;break}u=u.sibling}if(!r){for(u=i.child;u;){if(u===n){r=!0,n=i,l=a;break}if(u===l){r=!0,l=i,n=a;break}u=u.sibling}if(!r)throw Error(O(189))}}if(n.alternate!==l)throw Error(O(190))}if(n.tag!==3)throw Error(O(188));return n.stateNode.current===n?e:t}function A0(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=A0(e),t!==null)return t;e=e.sibling}return null}var Xa=Array.isArray,re=jC.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,rl={pending:!1,data:null,method:null,action:null},Gs=[],Hl=-1;function Vt(e){return{current:e}}function Ce(e){0>Hl||(e.current=Gs[Hl],Gs[Hl]=null,Hl--)}function oe(e,t){Hl++,Gs[Hl]=e.current,e.current=t}var Ut=Vt(null),Si=Vt(null),_n=Vt(null),nu=Vt(null);function lu(e,t){switch(oe(_n,t),oe(Si,e),oe(Ut,null),e=t.nodeType,e){case 9:case 11:t=(t=t.documentElement)&&(t=t.namespaceURI)?Mg(t):0;break;default:if(e=e===8?t.parentNode:t,t=e.tagName,e=e.namespaceURI)e=Mg(e),t=Lv(e,t);else switch(t){case"svg":t=1;break;case"math":t=2;break;default:t=0}}Ce(Ut),oe(Ut,t)}function ca(){Ce(Ut),Ce(Si),Ce(_n)}function Ys(e){e.memoizedState!==null&&oe(nu,e);var t=Ut.current,n=Lv(t,e.type);t!==n&&(oe(Si,e),oe(Ut,n))}function au(e){Si.current===e&&(Ce(Ut),Ce(Si)),nu.current===e&&(Ce(nu),Di._currentValue=rl)}var Xs=Object.prototype.hasOwnProperty,Sf=we.unstable_scheduleCallback,Fo=we.unstable_cancelCallback,LC=we.unstable_shouldYield,VC=we.unstable_requestPaint,Ft=we.unstable_now,$C=we.unstable_getCurrentPriorityLevel,O0=we.unstable_ImmediatePriority,T0=we.unstable_UserBlockingPriority,iu=we.unstable_NormalPriority,qC=we.unstable_LowPriority,D0=we.unstable_IdlePriority,GC=we.log,YC=we.unstable_setDisableYieldValue,$i=null,We=null;function XC(e){if(We&&typeof We.onCommitFiberRoot=="function")try{We.onCommitFiberRoot($i,e,void 0,(e.current.flags&128)===128)}catch{}}function En(e){if(typeof GC=="function"&&YC(e),We&&typeof We.setStrictMode=="function")try{We.setStrictMode($i,e)}catch{}}var et=Math.clz32?Math.clz32:ZC,kC=Math.log,QC=Math.LN2;function ZC(e){return e>>>=0,e===0?32:31-(kC(e)/QC|0)|0}var dr=128,pr=4194304;function Kn(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e&4194176;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function Iu(e,t){var n=e.pendingLanes;if(n===0)return 0;var l=0,a=e.suspendedLanes,i=e.pingedLanes,r=e.warmLanes;e=e.finishedLanes!==0;var u=n&134217727;return u!==0?(n=u&~a,n!==0?l=Kn(n):(i&=u,i!==0?l=Kn(i):e||(r=u&~r,r!==0&&(l=Kn(r))))):(u=n&~a,u!==0?l=Kn(u):i!==0?l=Kn(i):e||(r=n&~r,r!==0&&(l=Kn(r)))),l===0?0:t!==0&&t!==l&&!(t&a)&&(a=l&-l,r=t&-t,a>=r||a===32&&(r&4194176)!==0)?t:l}function qi(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function KC(e,t){switch(e){case 1:case 2:case 4:case 8:return t+250;case 16:case 32:case 64:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function M0(){var e=dr;return dr<<=1,!(dr&4194176)&&(dr=128),e}function j0(){var e=pr;return pr<<=1,!(pr&62914560)&&(pr=4194304),e}function Bo(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function Gi(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function JC(e,t,n,l,a,i){var r=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var u=e.entanglements,o=e.expirationTimes,s=e.hiddenUpdates;for(n=r&~n;0<n;){var c=31-et(n),f=1<<c;u[c]=0,o[c]=-1;var p=s[c];if(p!==null)for(s[c]=null,c=0;c<p.length;c++){var d=p[c];d!==null&&(d.lane&=-536870913)}n&=~f}l!==0&&N0(e,l,0),i!==0&&a===0&&e.tag!==0&&(e.suspendedLanes|=i&~(r&~t))}function N0(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var l=31-et(t);e.entangledLanes|=t,e.entanglements[l]=e.entanglements[l]|1073741824|n&4194218}function z0(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var l=31-et(n),a=1<<l;a&t|e[l]&t&&(e[l]|=t),n&=~a}}function U0(e){return e&=-e,2<e?8<e?e&134217727?32:268435456:8:2}function F0(){var e=re.p;return e!==0?e:(e=window.event,e===void 0?32:Kv(e.type))}function PC(e,t){var n=re.p;try{return re.p=e,t()}finally{re.p=n}}var $n=Math.random().toString(36).slice(2),je="__reactFiber$"+$n,Ge="__reactProps$"+$n,ba="__reactContainer$"+$n,ks="__reactEvents$"+$n,IC="__reactListeners$"+$n,WC="__reactHandles$"+$n,Mp="__reactResources$"+$n,xi="__reactMarker$"+$n;function xf(e){delete e[je],delete e[Ge],delete e[ks],delete e[IC],delete e[WC]}function nl(e){var t=e[je];if(t)return t;for(var n=e.parentNode;n;){if(t=n[ba]||n[je]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=Ng(e);e!==null;){if(n=e[je])return n;e=Ng(e)}return t}e=n,n=e.parentNode}return null}function Ra(e){if(e=e[je]||e[ba]){var t=e.tag;if(t===5||t===6||t===13||t===26||t===27||t===3)return e}return null}function ka(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(O(33))}function Pl(e){var t=e[Mp];return t||(t=e[Mp]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function Re(e){e[xi]=!0}var B0=new Set,H0={};function xl(e,t){fa(e,t),fa(e+"Capture",t)}function fa(e,t){for(H0[e]=t,e=0;e<t.length;e++)B0.add(t[e])}var un=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),e_=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),jp={},Np={};function t_(e){return Xs.call(Np,e)?!0:Xs.call(jp,e)?!1:e_.test(e)?Np[e]=!0:(jp[e]=!0,!1)}function Mr(e,t,n){if(t_(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var l=t.toLowerCase().slice(0,5);if(l!=="data-"&&l!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function gr(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function qt(e,t,n,l){if(l===null)e.removeAttribute(n);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+l)}}function ct(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function L0(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function n_(e){var t=L0(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),l=""+e[t];if(!e.hasOwnProperty(t)&&typeof n<"u"&&typeof n.get=="function"&&typeof n.set=="function"){var a=n.get,i=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return a.call(this)},set:function(r){l=""+r,i.call(this,r)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return l},setValue:function(r){l=""+r},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function ru(e){e._valueTracker||(e._valueTracker=n_(e))}function V0(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),l="";return e&&(l=L0(e)?e.checked?"true":"false":e.value),e=l,e!==n?(t.setValue(e),!0):!1}function uu(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var l_=/[\n"\\]/g;function pt(e){return e.replace(l_,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function Qs(e,t,n,l,a,i,r,u){e.name="",r!=null&&typeof r!="function"&&typeof r!="symbol"&&typeof r!="boolean"?e.type=r:e.removeAttribute("type"),t!=null?r==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+ct(t)):e.value!==""+ct(t)&&(e.value=""+ct(t)):r!=="submit"&&r!=="reset"||e.removeAttribute("value"),t!=null?Zs(e,r,ct(t)):n!=null?Zs(e,r,ct(n)):l!=null&&e.removeAttribute("value"),a==null&&i!=null&&(e.defaultChecked=!!i),a!=null&&(e.checked=a&&typeof a!="function"&&typeof a!="symbol"),u!=null&&typeof u!="function"&&typeof u!="symbol"&&typeof u!="boolean"?e.name=""+ct(u):e.removeAttribute("name")}function $0(e,t,n,l,a,i,r,u){if(i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"&&(e.type=i),t!=null||n!=null){if(!(i!=="submit"&&i!=="reset"||t!=null))return;n=n!=null?""+ct(n):"",t=t!=null?""+ct(t):n,u||t===e.value||(e.value=t),e.defaultValue=t}l=l??a,l=typeof l!="function"&&typeof l!="symbol"&&!!l,e.checked=u?e.checked:!!l,e.defaultChecked=!!l,r!=null&&typeof r!="function"&&typeof r!="symbol"&&typeof r!="boolean"&&(e.name=r)}function Zs(e,t,n){t==="number"&&uu(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function Il(e,t,n,l){if(e=e.options,t){t={};for(var a=0;a<n.length;a++)t["$"+n[a]]=!0;for(n=0;n<e.length;n++)a=t.hasOwnProperty("$"+e[n].value),e[n].selected!==a&&(e[n].selected=a),a&&l&&(e[n].defaultSelected=!0)}else{for(n=""+ct(n),t=null,a=0;a<e.length;a++){if(e[a].value===n){e[a].selected=!0,l&&(e[a].defaultSelected=!0);return}t!==null||e[a].disabled||(t=e[a])}t!==null&&(t.selected=!0)}}function q0(e,t,n){if(t!=null&&(t=""+ct(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+ct(n):""}function G0(e,t,n,l){if(t==null){if(l!=null){if(n!=null)throw Error(O(92));if(Xa(l)){if(1<l.length)throw Error(O(93));l=l[0]}n=l}n==null&&(n=""),t=n}n=ct(t),e.defaultValue=n,l=e.textContent,l===n&&l!==""&&l!==null&&(e.value=l)}function da(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var a_=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function zp(e,t,n){var l=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?l?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":l?e.setProperty(t,n):typeof n!="number"||n===0||a_.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function Y0(e,t,n){if(t!=null&&typeof t!="object")throw Error(O(62));if(e=e.style,n!=null){for(var l in n)!n.hasOwnProperty(l)||t!=null&&t.hasOwnProperty(l)||(l.indexOf("--")===0?e.setProperty(l,""):l==="float"?e.cssFloat="":e[l]="");for(var a in t)l=t[a],t.hasOwnProperty(a)&&n[a]!==l&&zp(e,a,l)}else for(var i in t)t.hasOwnProperty(i)&&zp(e,i,t[i])}function wf(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var i_=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),r_=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function jr(e){return r_.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}var Ks=null;function bf(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var Ll=null,Wl=null;function Up(e){var t=Ra(e);if(t&&(e=t.stateNode)){var n=e[Ge]||null;e:switch(e=t.stateNode,t.type){case"input":if(Qs(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+pt(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var l=n[t];if(l!==e&&l.form===e.form){var a=l[Ge]||null;if(!a)throw Error(O(90));Qs(l,a.value,a.defaultValue,a.defaultValue,a.checked,a.defaultChecked,a.type,a.name)}}for(t=0;t<n.length;t++)l=n[t],l.form===e.form&&V0(l)}break e;case"textarea":q0(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&Il(e,!!n.multiple,t,!1)}}}var Ho=!1;function X0(e,t,n){if(Ho)return e(t,n);Ho=!0;try{var l=e(t);return l}finally{if(Ho=!1,(Ll!==null||Wl!==null)&&(oo(),Ll&&(t=Ll,e=Wl,Wl=Ll=null,Up(t),e)))for(t=0;t<e.length;t++)Up(e[t])}}function wi(e,t){var n=e.stateNode;if(n===null)return null;var l=n[Ge]||null;if(l===null)return null;n=l[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(l=!l.disabled)||(e=e.type,l=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!l;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(O(231,t,typeof n));return n}var Js=!1;if(un)try{var Na={};Object.defineProperty(Na,"passive",{get:function(){Js=!0}}),window.addEventListener("test",Na,Na),window.removeEventListener("test",Na,Na)}catch{Js=!1}var Cn=null,Rf=null,Nr=null;function k0(){if(Nr)return Nr;var e,t=Rf,n=t.length,l,a="value"in Cn?Cn.value:Cn.textContent,i=a.length;for(e=0;e<n&&t[e]===a[e];e++);var r=n-e;for(l=1;l<=r&&t[n-l]===a[i-l];l++);return Nr=a.slice(e,1<l?1-l:void 0)}function zr(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function hr(){return!0}function Fp(){return!1}function Xe(e){function t(n,l,a,i,r){this._reactName=n,this._targetInst=a,this.type=l,this.nativeEvent=i,this.target=r,this.currentTarget=null;for(var u in e)e.hasOwnProperty(u)&&(n=e[u],this[u]=n?n(i):i[u]);return this.isDefaultPrevented=(i.defaultPrevented!=null?i.defaultPrevented:i.returnValue===!1)?hr:Fp,this.isPropagationStopped=Fp,this}return ue(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=hr)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=hr)},persist:function(){},isPersistent:hr}),t}var wl={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Wu=Xe(wl),Yi=ue({},wl,{view:0,detail:0}),u_=Xe(Yi),Lo,Vo,za,eo=ue({},Yi,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Ef,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==za&&(za&&e.type==="mousemove"?(Lo=e.screenX-za.screenX,Vo=e.screenY-za.screenY):Vo=Lo=0,za=e),Lo)},movementY:function(e){return"movementY"in e?e.movementY:Vo}}),Bp=Xe(eo),o_=ue({},eo,{dataTransfer:0}),s_=Xe(o_),c_=ue({},Yi,{relatedTarget:0}),$o=Xe(c_),f_=ue({},wl,{animationName:0,elapsedTime:0,pseudoElement:0}),d_=Xe(f_),p_=ue({},wl,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),g_=Xe(p_),h_=ue({},wl,{data:0}),Hp=Xe(h_),m_={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},y_={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},v_={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function S_(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=v_[e])?!!t[e]:!1}function Ef(){return S_}var x_=ue({},Yi,{key:function(e){if(e.key){var t=m_[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=zr(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?y_[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Ef,charCode:function(e){return e.type==="keypress"?zr(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?zr(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),w_=Xe(x_),b_=ue({},eo,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Lp=Xe(b_),R_=ue({},Yi,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Ef}),E_=Xe(R_),C_=ue({},wl,{propertyName:0,elapsedTime:0,pseudoElement:0}),__=Xe(C_),A_=ue({},eo,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),O_=Xe(A_),T_=ue({},wl,{newState:0,oldState:0}),D_=Xe(T_),M_=[9,13,27,32],Cf=un&&"CompositionEvent"in window,Wa=null;un&&"documentMode"in document&&(Wa=document.documentMode);var j_=un&&"TextEvent"in window&&!Wa,Q0=un&&(!Cf||Wa&&8<Wa&&11>=Wa),Vp=" ",$p=!1;function Z0(e,t){switch(e){case"keyup":return M_.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function K0(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Vl=!1;function N_(e,t){switch(e){case"compositionend":return K0(t);case"keypress":return t.which!==32?null:($p=!0,Vp);case"textInput":return e=t.data,e===Vp&&$p?null:e;default:return null}}function z_(e,t){if(Vl)return e==="compositionend"||!Cf&&Z0(e,t)?(e=k0(),Nr=Rf=Cn=null,Vl=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return Q0&&t.locale!=="ko"?null:t.data;default:return null}}var U_={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function qp(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!U_[e.type]:t==="textarea"}function J0(e,t,n,l){Ll?Wl?Wl.push(l):Wl=[l]:Ll=l,t=Ru(t,"onChange"),0<t.length&&(n=new Wu("onChange","change",null,n,l),e.push({event:n,listeners:t}))}var ei=null,bi=null;function F_(e){Fv(e,0)}function to(e){var t=ka(e);if(V0(t))return e}function Gp(e,t){if(e==="change")return t}var P0=!1;if(un){var qo;if(un){var Go="oninput"in document;if(!Go){var Yp=document.createElement("div");Yp.setAttribute("oninput","return;"),Go=typeof Yp.oninput=="function"}qo=Go}else qo=!1;P0=qo&&(!document.documentMode||9<document.documentMode)}function Xp(){ei&&(ei.detachEvent("onpropertychange",I0),bi=ei=null)}function I0(e){if(e.propertyName==="value"&&to(bi)){var t=[];J0(t,bi,e,bf(e)),X0(F_,t)}}function B_(e,t,n){e==="focusin"?(Xp(),ei=t,bi=n,ei.attachEvent("onpropertychange",I0)):e==="focusout"&&Xp()}function H_(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return to(bi)}function L_(e,t){if(e==="click")return to(t)}function V_(e,t){if(e==="input"||e==="change")return to(t)}function $_(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var at=typeof Object.is=="function"?Object.is:$_;function Ri(e,t){if(at(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),l=Object.keys(t);if(n.length!==l.length)return!1;for(l=0;l<n.length;l++){var a=n[l];if(!Xs.call(t,a)||!at(e[a],t[a]))return!1}return!0}function kp(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Qp(e,t){var n=kp(e);e=0;for(var l;n;){if(n.nodeType===3){if(l=e+n.textContent.length,e<=t&&l>=t)return{node:n,offset:t-e};e=l}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=kp(n)}}function W0(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?W0(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function ey(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=uu(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=uu(e.document)}return t}function _f(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}function q_(e,t){var n=ey(t);t=e.focusedElem;var l=e.selectionRange;if(n!==t&&t&&t.ownerDocument&&W0(t.ownerDocument.documentElement,t)){if(l!==null&&_f(t)){if(e=l.start,n=l.end,n===void 0&&(n=e),"selectionStart"in t)t.selectionStart=e,t.selectionEnd=Math.min(n,t.value.length);else if(n=(e=t.ownerDocument||document)&&e.defaultView||window,n.getSelection){n=n.getSelection();var a=t.textContent.length,i=Math.min(l.start,a);l=l.end===void 0?i:Math.min(l.end,a),!n.extend&&i>l&&(a=l,l=i,i=a),a=Qp(t,i);var r=Qp(t,l);a&&r&&(n.rangeCount!==1||n.anchorNode!==a.node||n.anchorOffset!==a.offset||n.focusNode!==r.node||n.focusOffset!==r.offset)&&(e=e.createRange(),e.setStart(a.node,a.offset),n.removeAllRanges(),i>l?(n.addRange(e),n.extend(r.node,r.offset)):(e.setEnd(r.node,r.offset),n.addRange(e)))}}for(e=[],n=t;n=n.parentNode;)n.nodeType===1&&e.push({element:n,left:n.scrollLeft,top:n.scrollTop});for(typeof t.focus=="function"&&t.focus(),t=0;t<e.length;t++)n=e[t],n.element.scrollLeft=n.left,n.element.scrollTop=n.top}}var G_=un&&"documentMode"in document&&11>=document.documentMode,$l=null,Ps=null,ti=null,Is=!1;function Zp(e,t,n){var l=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;Is||$l==null||$l!==uu(l)||(l=$l,"selectionStart"in l&&_f(l)?l={start:l.selectionStart,end:l.selectionEnd}:(l=(l.ownerDocument&&l.ownerDocument.defaultView||window).getSelection(),l={anchorNode:l.anchorNode,anchorOffset:l.anchorOffset,focusNode:l.focusNode,focusOffset:l.focusOffset}),ti&&Ri(ti,l)||(ti=l,l=Ru(Ps,"onSelect"),0<l.length&&(t=new Wu("onSelect","select",null,t,n),e.push({event:t,listeners:l}),t.target=$l)))}function Qn(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var ql={animationend:Qn("Animation","AnimationEnd"),animationiteration:Qn("Animation","AnimationIteration"),animationstart:Qn("Animation","AnimationStart"),transitionrun:Qn("Transition","TransitionRun"),transitionstart:Qn("Transition","TransitionStart"),transitioncancel:Qn("Transition","TransitionCancel"),transitionend:Qn("Transition","TransitionEnd")},Yo={},ty={};un&&(ty=document.createElement("div").style,"AnimationEvent"in window||(delete ql.animationend.animation,delete ql.animationiteration.animation,delete ql.animationstart.animation),"TransitionEvent"in window||delete ql.transitionend.transition);function bl(e){if(Yo[e])return Yo[e];if(!ql[e])return e;var t=ql[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in ty)return Yo[e]=t[n];return e}var ny=bl("animationend"),ly=bl("animationiteration"),ay=bl("animationstart"),Y_=bl("transitionrun"),X_=bl("transitionstart"),k_=bl("transitioncancel"),iy=bl("transitionend"),ry=new Map,Kp="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll scrollEnd toggle touchMove waiting wheel".split(" ");function At(e,t){ry.set(e,t),xl(t,[e])}var st=[],Gl=0,Af=0;function no(){for(var e=Gl,t=Af=Gl=0;t<e;){var n=st[t];st[t++]=null;var l=st[t];st[t++]=null;var a=st[t];st[t++]=null;var i=st[t];if(st[t++]=null,l!==null&&a!==null){var r=l.pending;r===null?a.next=a:(a.next=r.next,r.next=a),l.pending=a}i!==0&&uy(n,a,i)}}function lo(e,t,n,l){st[Gl++]=e,st[Gl++]=t,st[Gl++]=n,st[Gl++]=l,Af|=l,e.lanes|=l,e=e.alternate,e!==null&&(e.lanes|=l)}function Of(e,t,n,l){return lo(e,t,n,l),ou(e)}function Fn(e,t){return lo(e,null,null,t),ou(e)}function uy(e,t,n){e.lanes|=n;var l=e.alternate;l!==null&&(l.lanes|=n);for(var a=!1,i=e.return;i!==null;)i.childLanes|=n,l=i.alternate,l!==null&&(l.childLanes|=n),i.tag===22&&(e=i.stateNode,e===null||e._visibility&1||(a=!0)),e=i,i=i.return;a&&t!==null&&e.tag===3&&(i=e.stateNode,a=31-et(n),i=i.hiddenUpdates,e=i[a],e===null?i[a]=[t]:e.push(t),t.lane=n|536870912)}function ou(e){if(50<pi)throw pi=0,xc=null,Error(O(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var Yl={},Jp=new WeakMap;function gt(e,t){if(typeof e=="object"&&e!==null){var n=Jp.get(e);return n!==void 0?n:(t={value:e,source:t,stack:Tp(t)},Jp.set(e,t),t)}return{value:e,source:t,stack:Tp(t)}}var Xl=[],kl=0,su=null,cu=0,ft=[],dt=0,ul=null,It=1,Wt="";function Jn(e,t){Xl[kl++]=cu,Xl[kl++]=su,su=e,cu=t}function oy(e,t,n){ft[dt++]=It,ft[dt++]=Wt,ft[dt++]=ul,ul=e;var l=It;e=Wt;var a=32-et(l)-1;l&=~(1<<a),n+=1;var i=32-et(t)+a;if(30<i){var r=a-a%5;i=(l&(1<<r)-1).toString(32),l>>=r,a-=r,It=1<<32-et(t)+a|n<<a|l,Wt=i+e}else It=1<<i|n<<a|l,Wt=e}function Tf(e){e.return!==null&&(Jn(e,1),oy(e,1,0))}function Df(e){for(;e===su;)su=Xl[--kl],Xl[kl]=null,cu=Xl[--kl],Xl[kl]=null;for(;e===ul;)ul=ft[--dt],ft[dt]=null,Wt=ft[--dt],ft[dt]=null,It=ft[--dt],ft[dt]=null}var ze=null,Oe=null,W=!1,bt=null,Mt=!1,Ws=Error(O(519));function gl(e){var t=Error(O(418,""));throw Ei(gt(t,e)),Ws}function Pp(e){var t=e.stateNode,n=e.type,l=e.memoizedProps;switch(t[je]=e,t[Ge]=l,n){case"dialog":J("cancel",t),J("close",t);break;case"iframe":case"object":case"embed":J("load",t);break;case"video":case"audio":for(n=0;n<Ai.length;n++)J(Ai[n],t);break;case"source":J("error",t);break;case"img":case"image":case"link":J("error",t),J("load",t);break;case"details":J("toggle",t);break;case"input":J("invalid",t),$0(t,l.value,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name,!0),ru(t);break;case"select":J("invalid",t);break;case"textarea":J("invalid",t),G0(t,l.value,l.defaultValue,l.children),ru(t)}n=l.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||l.suppressHydrationWarning===!0||Hv(t.textContent,n)?(l.popover!=null&&(J("beforetoggle",t),J("toggle",t)),l.onScroll!=null&&J("scroll",t),l.onScrollEnd!=null&&J("scrollend",t),l.onClick!=null&&(t.onclick=co),t=!0):t=!1,t||gl(e)}function Ip(e){for(ze=e.return;ze;)switch(ze.tag){case 3:case 27:Mt=!0;return;case 5:case 13:Mt=!1;return;default:ze=ze.return}}function Ua(e){if(e!==ze)return!1;if(!W)return Ip(e),W=!0,!1;var t=!1,n;if((n=e.tag!==3&&e.tag!==27)&&((n=e.tag===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||Ac(e.type,e.memoizedProps)),n=!n),n&&(t=!0),t&&Oe&&gl(e),Ip(e),e.tag===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(O(317));e:{for(e=e.nextSibling,t=0;e;){if(e.nodeType===8)if(n=e.data,n==="/$"){if(t===0){Oe=Ct(e.nextSibling);break e}t--}else n!=="$"&&n!=="$!"&&n!=="$?"||t++;e=e.nextSibling}Oe=null}}else Oe=ze?Ct(e.stateNode.nextSibling):null;return!0}function Xi(){Oe=ze=null,W=!1}function Ei(e){bt===null?bt=[e]:bt.push(e)}var ni=Error(O(460)),sy=Error(O(474)),ec={then:function(){}};function Wp(e){return e=e.status,e==="fulfilled"||e==="rejected"}function mr(){}function cy(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(mr,mr),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,e===ni?Error(O(483)):e;default:if(typeof t.status=="string")t.then(mr,mr);else{if(e=ie,e!==null&&100<e.shellSuspendCounter)throw Error(O(482));e=t,e.status="pending",e.then(function(l){if(t.status==="pending"){var a=t;a.status="fulfilled",a.value=l}},function(l){if(t.status==="pending"){var a=t;a.status="rejected",a.reason=l}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,e===ni?Error(O(483)):e}throw li=t,ni}}var li=null;function eg(){if(li===null)throw Error(O(459));var e=li;return li=null,e}var ea=null,Ci=0;function yr(e){var t=Ci;return Ci+=1,ea===null&&(ea=[]),cy(ea,e,t)}function Fa(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function vr(e,t){throw t.$$typeof===NC?Error(O(525)):(e=Object.prototype.toString.call(t),Error(O(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function tg(e){var t=e._init;return t(e._payload)}function fy(e){function t(y,m){if(e){var v=y.deletions;v===null?(y.deletions=[m],y.flags|=16):v.push(m)}}function n(y,m){if(!e)return null;for(;m!==null;)t(y,m),m=m.sibling;return null}function l(y){for(var m=new Map;y!==null;)y.key!==null?m.set(y.key,y):m.set(y.index,y),y=y.sibling;return m}function a(y,m){return y=Tn(y,m),y.index=0,y.sibling=null,y}function i(y,m,v){return y.index=v,e?(v=y.alternate,v!==null?(v=v.index,v<m?(y.flags|=33554434,m):v):(y.flags|=33554434,m)):(y.flags|=1048576,m)}function r(y){return e&&y.alternate===null&&(y.flags|=33554434),y}function u(y,m,v,b){return m===null||m.tag!==6?(m=ns(v,y.mode,b),m.return=y,m):(m=a(m,v),m.return=y,m)}function o(y,m,v,b){var R=v.type;return R===Bl?c(y,m,v.props.children,b,v.key):m!==null&&(m.elementType===R||typeof R=="object"&&R!==null&&R.$$typeof===mn&&tg(R)===m.type)?(m=a(m,v.props),Fa(m,v),m.return=y,m):(m=Lr(v.type,v.key,v.props,null,y.mode,b),Fa(m,v),m.return=y,m)}function s(y,m,v,b){return m===null||m.tag!==4||m.stateNode.containerInfo!==v.containerInfo||m.stateNode.implementation!==v.implementation?(m=ls(v,y.mode,b),m.return=y,m):(m=a(m,v.children||[]),m.return=y,m)}function c(y,m,v,b,R){return m===null||m.tag!==7?(m=sl(v,y.mode,b,R),m.return=y,m):(m=a(m,v),m.return=y,m)}function f(y,m,v){if(typeof m=="string"&&m!==""||typeof m=="number"||typeof m=="bigint")return m=ns(""+m,y.mode,v),m.return=y,m;if(typeof m=="object"&&m!==null){switch(m.$$typeof){case fr:return v=Lr(m.type,m.key,m.props,null,y.mode,v),Fa(v,m),v.return=y,v;case Ga:return m=ls(m,y.mode,v),m.return=y,m;case mn:var b=m._init;return m=b(m._payload),f(y,m,v)}if(Xa(m)||ja(m))return m=sl(m,y.mode,v,null),m.return=y,m;if(typeof m.then=="function")return f(y,yr(m),v);if(m.$$typeof===Pt)return f(y,Sr(y,m),v);vr(y,m)}return null}function p(y,m,v,b){var R=m!==null?m.key:null;if(typeof v=="string"&&v!==""||typeof v=="number"||typeof v=="bigint")return R!==null?null:u(y,m,""+v,b);if(typeof v=="object"&&v!==null){switch(v.$$typeof){case fr:return v.key===R?o(y,m,v,b):null;case Ga:return v.key===R?s(y,m,v,b):null;case mn:return R=v._init,v=R(v._payload),p(y,m,v,b)}if(Xa(v)||ja(v))return R!==null?null:c(y,m,v,b,null);if(typeof v.then=="function")return p(y,m,yr(v),b);if(v.$$typeof===Pt)return p(y,m,Sr(y,v),b);vr(y,v)}return null}function d(y,m,v,b,R){if(typeof b=="string"&&b!==""||typeof b=="number"||typeof b=="bigint")return y=y.get(v)||null,u(m,y,""+b,R);if(typeof b=="object"&&b!==null){switch(b.$$typeof){case fr:return y=y.get(b.key===null?v:b.key)||null,o(m,y,b,R);case Ga:return y=y.get(b.key===null?v:b.key)||null,s(m,y,b,R);case mn:var _=b._init;return b=_(b._payload),d(y,m,v,b,R)}if(Xa(b)||ja(b))return y=y.get(v)||null,c(m,y,b,R,null);if(typeof b.then=="function")return d(y,m,v,yr(b),R);if(b.$$typeof===Pt)return d(y,m,v,Sr(m,b),R);vr(m,b)}return null}function h(y,m,v,b){for(var R=null,_=null,E=m,A=m=0,N=null;E!==null&&A<v.length;A++){E.index>A?(N=E,E=null):N=E.sibling;var M=p(y,E,v[A],b);if(M===null){E===null&&(E=N);break}e&&E&&M.alternate===null&&t(y,E),m=i(M,m,A),_===null?R=M:_.sibling=M,_=M,E=N}if(A===v.length)return n(y,E),W&&Jn(y,A),R;if(E===null){for(;A<v.length;A++)E=f(y,v[A],b),E!==null&&(m=i(E,m,A),_===null?R=E:_.sibling=E,_=E);return W&&Jn(y,A),R}for(E=l(E);A<v.length;A++)N=d(E,y,A,v[A],b),N!==null&&(e&&N.alternate!==null&&E.delete(N.key===null?A:N.key),m=i(N,m,A),_===null?R=N:_.sibling=N,_=N);return e&&E.forEach(function(j){return t(y,j)}),W&&Jn(y,A),R}function S(y,m,v,b){if(v==null)throw Error(O(151));for(var R=null,_=null,E=m,A=m=0,N=null,M=v.next();E!==null&&!M.done;A++,M=v.next()){E.index>A?(N=E,E=null):N=E.sibling;var j=p(y,E,M.value,b);if(j===null){E===null&&(E=N);break}e&&E&&j.alternate===null&&t(y,E),m=i(j,m,A),_===null?R=j:_.sibling=j,_=j,E=N}if(M.done)return n(y,E),W&&Jn(y,A),R;if(E===null){for(;!M.done;A++,M=v.next())M=f(y,M.value,b),M!==null&&(m=i(M,m,A),_===null?R=M:_.sibling=M,_=M);return W&&Jn(y,A),R}for(E=l(E);!M.done;A++,M=v.next())M=d(E,y,A,M.value,b),M!==null&&(e&&M.alternate!==null&&E.delete(M.key===null?A:M.key),m=i(M,m,A),_===null?R=M:_.sibling=M,_=M);return e&&E.forEach(function(V){return t(y,V)}),W&&Jn(y,A),R}function x(y,m,v,b){if(typeof v=="object"&&v!==null&&v.type===Bl&&v.key===null&&(v=v.props.children),typeof v=="object"&&v!==null){switch(v.$$typeof){case fr:e:{for(var R=v.key;m!==null;){if(m.key===R){if(R=v.type,R===Bl){if(m.tag===7){n(y,m.sibling),b=a(m,v.props.children),b.return=y,y=b;break e}}else if(m.elementType===R||typeof R=="object"&&R!==null&&R.$$typeof===mn&&tg(R)===m.type){n(y,m.sibling),b=a(m,v.props),Fa(b,v),b.return=y,y=b;break e}n(y,m);break}else t(y,m);m=m.sibling}v.type===Bl?(b=sl(v.props.children,y.mode,b,v.key),b.return=y,y=b):(b=Lr(v.type,v.key,v.props,null,y.mode,b),Fa(b,v),b.return=y,y=b)}return r(y);case Ga:e:{for(R=v.key;m!==null;){if(m.key===R)if(m.tag===4&&m.stateNode.containerInfo===v.containerInfo&&m.stateNode.implementation===v.implementation){n(y,m.sibling),b=a(m,v.children||[]),b.return=y,y=b;break e}else{n(y,m);break}else t(y,m);m=m.sibling}b=ls(v,y.mode,b),b.return=y,y=b}return r(y);case mn:return R=v._init,v=R(v._payload),x(y,m,v,b)}if(Xa(v))return h(y,m,v,b);if(ja(v)){if(R=ja(v),typeof R!="function")throw Error(O(150));return v=R.call(v),S(y,m,v,b)}if(typeof v.then=="function")return x(y,m,yr(v),b);if(v.$$typeof===Pt)return x(y,m,Sr(y,v),b);vr(y,v)}return typeof v=="string"&&v!==""||typeof v=="number"||typeof v=="bigint"?(v=""+v,m!==null&&m.tag===6?(n(y,m.sibling),b=a(m,v),b.return=y,y=b):(n(y,m),b=ns(v,y.mode,b),b.return=y,y=b),r(y)):n(y,m)}return function(y,m,v,b){try{Ci=0;var R=x(y,m,v,b);return ea=null,R}catch(E){if(E===ni)throw E;var _=ht(29,E,null,y.mode);return _.lanes=b,_.return=y,_}finally{}}}var hl=fy(!0),dy=fy(!1),pa=Vt(null),fu=Vt(0);function ng(e,t){e=cn,oe(fu,e),oe(pa,t),cn=e|t.baseLanes}function tc(){oe(fu,cn),oe(pa,pa.current)}function Mf(){cn=fu.current,Ce(pa),Ce(fu)}var St=Vt(null),Bt=null;function vn(e){var t=e.alternate;oe(xe,xe.current&1),oe(St,e),Bt===null&&(t===null||pa.current!==null||t.memoizedState!==null)&&(Bt=e)}function py(e){if(e.tag===22){if(oe(xe,xe.current),oe(St,e),Bt===null){var t=e.alternate;t!==null&&t.memoizedState!==null&&(Bt=e)}}else Sn()}function Sn(){oe(xe,xe.current),oe(St,St.current)}function en(e){Ce(St),Bt===e&&(Bt=null),Ce(xe)}var xe=Vt(0);function du(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||n.data==="$?"||n.data==="$!"))return t}else if(t.tag===19&&t.memoizedProps.revealOrder!==void 0){if(t.flags&128)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}var Q_=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,l){e.push(l)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},Z_=we.unstable_scheduleCallback,K_=we.unstable_NormalPriority,Se={$$typeof:Pt,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function jf(){return{controller:new Q_,data:new Map,refCount:0}}function ki(e){e.refCount--,e.refCount===0&&Z_(K_,function(){e.controller.abort()})}var ai=null,nc=0,ga=0,ta=null;function J_(e,t){if(ai===null){var n=ai=[];nc=0,ga=td(),ta={status:"pending",value:void 0,then:function(l){n.push(l)}}}return nc++,t.then(lg,lg),t}function lg(){if(--nc===0&&ai!==null){ta!==null&&(ta.status="fulfilled");var e=ai;ai=null,ga=0,ta=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function P_(e,t){var n=[],l={status:"pending",value:null,reason:null,then:function(a){n.push(a)}};return e.then(function(){l.status="fulfilled",l.value=t;for(var a=0;a<n.length;a++)(0,n[a])(t)},function(a){for(l.status="rejected",l.reason=a,a=0;a<n.length;a++)(0,n[a])(void 0)}),l}var ag=G.S;G.S=function(e,t){typeof t=="object"&&t!==null&&typeof t.then=="function"&&J_(e,t),ag!==null&&ag(e,t)};var ol=Vt(null);function Nf(){var e=ol.current;return e!==null?e:ie.pooledCache}function Ur(e,t){t===null?oe(ol,ol.current):oe(ol,t.pool)}function gy(){var e=Nf();return e===null?null:{parent:Se._currentValue,pool:e}}var Bn=0,X=null,te=null,me=null,pu=!1,na=!1,ml=!1,gu=0,_i=0,la=null,I_=0;function ge(){throw Error(O(321))}function zf(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!at(e[n],t[n]))return!1;return!0}function Uf(e,t,n,l,a,i){return Bn=i,X=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,G.H=e===null||e.memoizedState===null?Rl:qn,ml=!1,i=n(l,a),ml=!1,na&&(i=my(t,n,l,a)),hy(e),i}function hy(e){G.H=Lt;var t=te!==null&&te.next!==null;if(Bn=0,me=te=X=null,pu=!1,_i=0,la=null,t)throw Error(O(300));e===null||Ee||(e=e.dependencies,e!==null&&yu(e)&&(Ee=!0))}function my(e,t,n,l){X=e;var a=0;do{if(na&&(la=null),_i=0,na=!1,25<=a)throw Error(O(301));if(a+=1,me=te=null,e.updateQueue!=null){var i=e.updateQueue;i.lastEffect=null,i.events=null,i.stores=null,i.memoCache!=null&&(i.memoCache.index=0)}G.H=El,i=t(n,l)}while(na);return i}function W_(){var e=G.H,t=e.useState()[0];return t=typeof t.then=="function"?Qi(t):t,e=e.useState()[0],(te!==null?te.memoizedState:null)!==e&&(X.flags|=1024),t}function Ff(){var e=gu!==0;return gu=0,e}function Bf(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function Hf(e){if(pu){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}pu=!1}Bn=0,me=te=X=null,na=!1,_i=gu=0,la=null}function Le(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return me===null?X.memoizedState=me=e:me=me.next=e,me}function ve(){if(te===null){var e=X.alternate;e=e!==null?e.memoizedState:null}else e=te.next;var t=me===null?X.memoizedState:me.next;if(t!==null)me=t,te=e;else{if(e===null)throw X.alternate===null?Error(O(467)):Error(O(310));te=e,e={memoizedState:te.memoizedState,baseState:te.baseState,baseQueue:te.baseQueue,queue:te.queue,next:null},me===null?X.memoizedState=me=e:me=me.next=e}return me}var ao;ao=function(){return{lastEffect:null,events:null,stores:null,memoCache:null}};function Qi(e){var t=_i;return _i+=1,la===null&&(la=[]),e=cy(la,e,t),t=X,(me===null?t.memoizedState:me.next)===null&&(t=t.alternate,G.H=t===null||t.memoizedState===null?Rl:qn),e}function io(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return Qi(e);if(e.$$typeof===Pt)return Ne(e)}throw Error(O(438,String(e)))}function Lf(e){var t=null,n=X.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var l=X.alternate;l!==null&&(l=l.updateQueue,l!==null&&(l=l.memoCache,l!=null&&(t={data:l.data.map(function(a){return a.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=ao(),X.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),l=0;l<e;l++)n[l]=UC;return t.index++,n}function on(e,t){return typeof t=="function"?t(e):t}function Fr(e){var t=ve();return Vf(t,te,e)}function Vf(e,t,n){var l=e.queue;if(l===null)throw Error(O(311));l.lastRenderedReducer=n;var a=e.baseQueue,i=l.pending;if(i!==null){if(a!==null){var r=a.next;a.next=i.next,i.next=r}t.baseQueue=a=i,l.pending=null}if(i=e.baseState,a===null)e.memoizedState=i;else{t=a.next;var u=r=null,o=null,s=t,c=!1;do{var f=s.lane&-536870913;if(f!==s.lane?(I&f)===f:(Bn&f)===f){var p=s.revertLane;if(p===0)o!==null&&(o=o.next={lane:0,revertLane:0,action:s.action,hasEagerState:s.hasEagerState,eagerState:s.eagerState,next:null}),f===ga&&(c=!0);else if((Bn&p)===p){s=s.next,p===ga&&(c=!0);continue}else f={lane:0,revertLane:s.revertLane,action:s.action,hasEagerState:s.hasEagerState,eagerState:s.eagerState,next:null},o===null?(u=o=f,r=i):o=o.next=f,X.lanes|=p,Ln|=p;f=s.action,ml&&n(i,f),i=s.hasEagerState?s.eagerState:n(i,f)}else p={lane:f,revertLane:s.revertLane,action:s.action,hasEagerState:s.hasEagerState,eagerState:s.eagerState,next:null},o===null?(u=o=p,r=i):o=o.next=p,X.lanes|=f,Ln|=f;s=s.next}while(s!==null&&s!==t);if(o===null?r=i:o.next=u,!at(i,e.memoizedState)&&(Ee=!0,c&&(n=ta,n!==null)))throw n;e.memoizedState=i,e.baseState=r,e.baseQueue=o,l.lastRenderedState=i}return a===null&&(l.lanes=0),[e.memoizedState,l.dispatch]}function Xo(e){var t=ve(),n=t.queue;if(n===null)throw Error(O(311));n.lastRenderedReducer=e;var l=n.dispatch,a=n.pending,i=t.memoizedState;if(a!==null){n.pending=null;var r=a=a.next;do i=e(i,r.action),r=r.next;while(r!==a);at(i,t.memoizedState)||(Ee=!0),t.memoizedState=i,t.baseQueue===null&&(t.baseState=i),n.lastRenderedState=i}return[i,l]}function yy(e,t,n){var l=X,a=ve(),i=W;if(i){if(n===void 0)throw Error(O(407));n=n()}else n=t();var r=!at((te||a).memoizedState,n);if(r&&(a.memoizedState=n,Ee=!0),a=a.queue,$f(xy.bind(null,l,a,e),[e]),a.getSnapshot!==t||r||me!==null&&me.memoizedState.tag&1){if(l.flags|=2048,ha(9,Sy.bind(null,l,a,n,t),{destroy:void 0},null),ie===null)throw Error(O(349));i||Bn&60||vy(l,t,n)}return n}function vy(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=X.updateQueue,t===null?(t=ao(),X.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function Sy(e,t,n,l){t.value=n,t.getSnapshot=l,wy(t)&&by(e)}function xy(e,t,n){return n(function(){wy(t)&&by(e)})}function wy(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!at(e,n)}catch{return!0}}function by(e){var t=Fn(e,2);t!==null&&Ue(t,e,2)}function lc(e){var t=Le();if(typeof e=="function"){var n=e;if(e=n(),ml){En(!0);try{n()}finally{En(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:on,lastRenderedState:e},t}function Ry(e,t,n,l){return e.baseState=n,Vf(e,te,typeof l=="function"?l:on)}function eA(e,t,n,l,a){if(uo(e))throw Error(O(485));if(e=t.action,e!==null){var i={payload:a,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(r){i.listeners.push(r)}};G.T!==null?n(!0):i.isTransition=!1,l(i),n=t.pending,n===null?(i.next=t.pending=i,Ey(t,i)):(i.next=n.next,t.pending=n.next=i)}}function Ey(e,t){var n=t.action,l=t.payload,a=e.state;if(t.isTransition){var i=G.T,r={};G.T=r;try{var u=n(a,l),o=G.S;o!==null&&o(r,u),ig(e,t,u)}catch(s){ac(e,t,s)}finally{G.T=i}}else try{i=n(a,l),ig(e,t,i)}catch(s){ac(e,t,s)}}function ig(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(l){rg(e,t,l)},function(l){return ac(e,t,l)}):rg(e,t,n)}function rg(e,t,n){t.status="fulfilled",t.value=n,Cy(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,Ey(e,n)))}function ac(e,t,n){var l=e.pending;if(e.pending=null,l!==null){l=l.next;do t.status="rejected",t.reason=n,Cy(t),t=t.next;while(t!==l)}e.action=null}function Cy(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function _y(e,t){return t}function Ay(e,t){if(W){var n=ie.formState;if(n!==null){e:{var l=X;if(W){if(Oe){t:{for(var a=Oe,i=Mt;a.nodeType!==8;){if(!i){a=null;break t}if(a=Ct(a.nextSibling),a===null){a=null;break t}}i=a.data,a=i==="F!"||i==="F"?a:null}if(a){Oe=Ct(a.nextSibling),l=a.data==="F!";break e}}gl(l)}l=!1}l&&(t=n[0])}}return n=Le(),n.memoizedState=n.baseState=t,l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:_y,lastRenderedState:t},n.queue=l,n=Yy.bind(null,X,l),l.dispatch=n,l=lc(!1),i=Xf.bind(null,X,!1,l.queue),l=Le(),a={state:t,dispatch:null,action:e,pending:null},l.queue=a,n=eA.bind(null,X,a,i,n),a.dispatch=n,l.memoizedState=e,[t,n,!1]}function Oy(e){var t=ve();return Ty(t,te,e)}function Ty(e,t,n){t=Vf(e,t,_y)[0],e=Fr(on)[0],t=typeof t=="object"&&t!==null&&typeof t.then=="function"?Qi(t):t;var l=ve(),a=l.queue,i=a.dispatch;return n!==l.memoizedState&&(X.flags|=2048,ha(9,tA.bind(null,a,n),{destroy:void 0},null)),[t,i,e]}function tA(e,t){e.action=t}function Dy(e){var t=ve(),n=te;if(n!==null)return Ty(t,n,e);ve(),t=t.memoizedState,n=ve();var l=n.queue.dispatch;return n.memoizedState=e,[t,l,!1]}function ha(e,t,n,l){return e={tag:e,create:t,inst:n,deps:l,next:null},t=X.updateQueue,t===null&&(t=ao(),X.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(l=n.next,n.next=e,e.next=l,t.lastEffect=e),e}function My(){return ve().memoizedState}function Br(e,t,n,l){var a=Le();X.flags|=e,a.memoizedState=ha(1|t,n,{destroy:void 0},l===void 0?null:l)}function ro(e,t,n,l){var a=ve();l=l===void 0?null:l;var i=a.memoizedState.inst;te!==null&&l!==null&&zf(l,te.memoizedState.deps)?a.memoizedState=ha(t,n,i,l):(X.flags|=e,a.memoizedState=ha(1|t,n,i,l))}function ug(e,t){Br(8390656,8,e,t)}function $f(e,t){ro(2048,8,e,t)}function jy(e,t){return ro(4,2,e,t)}function Ny(e,t){return ro(4,4,e,t)}function zy(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function Uy(e,t,n){n=n!=null?n.concat([e]):null,ro(4,4,zy.bind(null,t,e),n)}function qf(){}function Fy(e,t){var n=ve();t=t===void 0?null:t;var l=n.memoizedState;return t!==null&&zf(t,l[1])?l[0]:(n.memoizedState=[e,t],e)}function By(e,t){var n=ve();t=t===void 0?null:t;var l=n.memoizedState;if(t!==null&&zf(t,l[1]))return l[0];if(l=e(),ml){En(!0);try{e()}finally{En(!1)}}return n.memoizedState=[l,t],l}function Gf(e,t,n){return n===void 0||Bn&1073741824?e.memoizedState=t:(e.memoizedState=n,e=Cv(),X.lanes|=e,Ln|=e,n)}function Hy(e,t,n,l){return at(n,t)?n:pa.current!==null?(e=Gf(e,n,l),at(e,t)||(Ee=!0),e):Bn&42?(e=Cv(),X.lanes|=e,Ln|=e,t):(Ee=!0,e.memoizedState=n)}function Ly(e,t,n,l,a){var i=re.p;re.p=i!==0&&8>i?i:8;var r=G.T,u={};G.T=u,Xf(e,!1,t,n);try{var o=a(),s=G.S;if(s!==null&&s(u,o),o!==null&&typeof o=="object"&&typeof o.then=="function"){var c=P_(o,l);ii(e,t,c,tt(e))}else ii(e,t,l,tt(e))}catch(f){ii(e,t,{then:function(){},status:"rejected",reason:f},tt())}finally{re.p=i,G.T=r}}function nA(){}function ic(e,t,n,l){if(e.tag!==5)throw Error(O(476));var a=Vy(e).queue;Ly(e,a,t,rl,n===null?nA:function(){return $y(e),n(l)})}function Vy(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:rl,baseState:rl,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:on,lastRenderedState:rl},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:on,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function $y(e){var t=Vy(e).next.queue;ii(e,t,{},tt())}function Yf(){return Ne(Di)}function qy(){return ve().memoizedState}function Gy(){return ve().memoizedState}function lA(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=tt();e=An(n);var l=On(t,e,n);l!==null&&(Ue(l,t,n),ui(l,t,n)),t={cache:jf()},e.payload=t;return}t=t.return}}function aA(e,t,n){var l=tt();n={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null},uo(e)?Xy(t,n):(n=Of(e,t,n,l),n!==null&&(Ue(n,e,l),ky(n,t,l)))}function Yy(e,t,n){var l=tt();ii(e,t,n,l)}function ii(e,t,n,l){var a={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null};if(uo(e))Xy(t,a);else{var i=e.alternate;if(e.lanes===0&&(i===null||i.lanes===0)&&(i=t.lastRenderedReducer,i!==null))try{var r=t.lastRenderedState,u=i(r,n);if(a.hasEagerState=!0,a.eagerState=u,at(u,r))return lo(e,t,a,0),ie===null&&no(),!1}catch{}finally{}if(n=Of(e,t,a,l),n!==null)return Ue(n,e,l),ky(n,t,l),!0}return!1}function Xf(e,t,n,l){if(l={lane:2,revertLane:td(),action:l,hasEagerState:!1,eagerState:null,next:null},uo(e)){if(t)throw Error(O(479))}else t=Of(e,n,l,2),t!==null&&Ue(t,e,2)}function uo(e){var t=e.alternate;return e===X||t!==null&&t===X}function Xy(e,t){na=pu=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function ky(e,t,n){if(n&4194176){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,z0(e,n)}}var Lt={readContext:Ne,use:io,useCallback:ge,useContext:ge,useEffect:ge,useImperativeHandle:ge,useLayoutEffect:ge,useInsertionEffect:ge,useMemo:ge,useReducer:ge,useRef:ge,useState:ge,useDebugValue:ge,useDeferredValue:ge,useTransition:ge,useSyncExternalStore:ge,useId:ge};Lt.useCacheRefresh=ge;Lt.useMemoCache=ge;Lt.useHostTransitionStatus=ge;Lt.useFormState=ge;Lt.useActionState=ge;Lt.useOptimistic=ge;var Rl={readContext:Ne,use:io,useCallback:function(e,t){return Le().memoizedState=[e,t===void 0?null:t],e},useContext:Ne,useEffect:ug,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,Br(4194308,4,zy.bind(null,t,e),n)},useLayoutEffect:function(e,t){return Br(4194308,4,e,t)},useInsertionEffect:function(e,t){Br(4,2,e,t)},useMemo:function(e,t){var n=Le();t=t===void 0?null:t;var l=e();if(ml){En(!0);try{e()}finally{En(!1)}}return n.memoizedState=[l,t],l},useReducer:function(e,t,n){var l=Le();if(n!==void 0){var a=n(t);if(ml){En(!0);try{n(t)}finally{En(!1)}}}else a=t;return l.memoizedState=l.baseState=a,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:a},l.queue=e,e=e.dispatch=aA.bind(null,X,e),[l.memoizedState,e]},useRef:function(e){var t=Le();return e={current:e},t.memoizedState=e},useState:function(e){e=lc(e);var t=e.queue,n=Yy.bind(null,X,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:qf,useDeferredValue:function(e,t){var n=Le();return Gf(n,e,t)},useTransition:function(){var e=lc(!1);return e=Ly.bind(null,X,e.queue,!0,!1),Le().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var l=X,a=Le();if(W){if(n===void 0)throw Error(O(407));n=n()}else{if(n=t(),ie===null)throw Error(O(349));I&60||vy(l,t,n)}a.memoizedState=n;var i={value:n,getSnapshot:t};return a.queue=i,ug(xy.bind(null,l,i,e),[e]),l.flags|=2048,ha(9,Sy.bind(null,l,i,n,t),{destroy:void 0},null),n},useId:function(){var e=Le(),t=ie.identifierPrefix;if(W){var n=Wt,l=It;n=(l&~(1<<32-et(l)-1)).toString(32)+n,t=":"+t+"R"+n,n=gu++,0<n&&(t+="H"+n.toString(32)),t+=":"}else n=I_++,t=":"+t+"r"+n.toString(32)+":";return e.memoizedState=t},useCacheRefresh:function(){return Le().memoizedState=lA.bind(null,X)}};Rl.useMemoCache=Lf;Rl.useHostTransitionStatus=Yf;Rl.useFormState=Ay;Rl.useActionState=Ay;Rl.useOptimistic=function(e){var t=Le();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=Xf.bind(null,X,!0,n),n.dispatch=t,[e,t]};var qn={readContext:Ne,use:io,useCallback:Fy,useContext:Ne,useEffect:$f,useImperativeHandle:Uy,useInsertionEffect:jy,useLayoutEffect:Ny,useMemo:By,useReducer:Fr,useRef:My,useState:function(){return Fr(on)},useDebugValue:qf,useDeferredValue:function(e,t){var n=ve();return Hy(n,te.memoizedState,e,t)},useTransition:function(){var e=Fr(on)[0],t=ve().memoizedState;return[typeof e=="boolean"?e:Qi(e),t]},useSyncExternalStore:yy,useId:qy};qn.useCacheRefresh=Gy;qn.useMemoCache=Lf;qn.useHostTransitionStatus=Yf;qn.useFormState=Oy;qn.useActionState=Oy;qn.useOptimistic=function(e,t){var n=ve();return Ry(n,te,e,t)};var El={readContext:Ne,use:io,useCallback:Fy,useContext:Ne,useEffect:$f,useImperativeHandle:Uy,useInsertionEffect:jy,useLayoutEffect:Ny,useMemo:By,useReducer:Xo,useRef:My,useState:function(){return Xo(on)},useDebugValue:qf,useDeferredValue:function(e,t){var n=ve();return te===null?Gf(n,e,t):Hy(n,te.memoizedState,e,t)},useTransition:function(){var e=Xo(on)[0],t=ve().memoizedState;return[typeof e=="boolean"?e:Qi(e),t]},useSyncExternalStore:yy,useId:qy};El.useCacheRefresh=Gy;El.useMemoCache=Lf;El.useHostTransitionStatus=Yf;El.useFormState=Dy;El.useActionState=Dy;El.useOptimistic=function(e,t){var n=ve();return te!==null?Ry(n,te,e,t):(n.baseState=e,[e,n.queue.dispatch])};function ko(e,t,n,l){t=e.memoizedState,n=n(l,t),n=n==null?t:ue({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var rc={isMounted:function(e){return(e=e._reactInternals)?wa(e)===e:!1},enqueueSetState:function(e,t,n){e=e._reactInternals;var l=tt(),a=An(l);a.payload=t,n!=null&&(a.callback=n),t=On(e,a,l),t!==null&&(Ue(t,e,l),ui(t,e,l))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var l=tt(),a=An(l);a.tag=1,a.payload=t,n!=null&&(a.callback=n),t=On(e,a,l),t!==null&&(Ue(t,e,l),ui(t,e,l))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=tt(),l=An(n);l.tag=2,t!=null&&(l.callback=t),t=On(e,l,n),t!==null&&(Ue(t,e,n),ui(t,e,n))}};function og(e,t,n,l,a,i,r){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(l,i,r):t.prototype&&t.prototype.isPureReactComponent?!Ri(n,l)||!Ri(a,i):!0}function sg(e,t,n,l){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,l),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,l),t.state!==e&&rc.enqueueReplaceState(t,t.state,null)}function yl(e,t){var n=t;if("ref"in t){n={};for(var l in t)l!=="ref"&&(n[l]=t[l])}if(e=e.defaultProps){n===t&&(n=ue({},n));for(var a in e)n[a]===void 0&&(n[a]=e[a])}return n}var hu=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)};function Qy(e){hu(e)}function Zy(e){console.error(e)}function Ky(e){hu(e)}function mu(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(l){setTimeout(function(){throw l})}}function cg(e,t,n){try{var l=e.onCaughtError;l(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(a){setTimeout(function(){throw a})}}function uc(e,t,n){return n=An(n),n.tag=3,n.payload={element:null},n.callback=function(){mu(e,t)},n}function Jy(e){return e=An(e),e.tag=3,e}function Py(e,t,n,l){var a=n.type.getDerivedStateFromError;if(typeof a=="function"){var i=l.value;e.payload=function(){return a(i)},e.callback=function(){cg(t,n,l)}}var r=n.stateNode;r!==null&&typeof r.componentDidCatch=="function"&&(e.callback=function(){cg(t,n,l),typeof a!="function"&&(Dn===null?Dn=new Set([this]):Dn.add(this));var u=l.stack;this.componentDidCatch(l.value,{componentStack:u!==null?u:""})})}function iA(e,t,n,l,a){if(n.flags|=32768,l!==null&&typeof l=="object"&&typeof l.then=="function"){if(t=n.alternate,t!==null&&Zi(t,n,a,!0),n=St.current,n!==null){switch(n.tag){case 13:return Bt===null?bc():n.alternate===null&&pe===0&&(pe=3),n.flags&=-257,n.flags|=65536,n.lanes=a,l===ec?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([l]):t.add(l),is(e,l,a)),!1;case 22:return n.flags|=65536,l===ec?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([l])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([l]):n.add(l)),is(e,l,a)),!1}throw Error(O(435,n.tag))}return is(e,l,a),bc(),!1}if(W)return t=St.current,t!==null?(!(t.flags&65536)&&(t.flags|=256),t.flags|=65536,t.lanes=a,l!==Ws&&(e=Error(O(422),{cause:l}),Ei(gt(e,n)))):(l!==Ws&&(t=Error(O(423),{cause:l}),Ei(gt(t,n))),e=e.current.alternate,e.flags|=65536,a&=-a,e.lanes|=a,l=gt(l,n),a=uc(e.stateNode,l,a),Io(e,a),pe!==4&&(pe=2)),!1;var i=Error(O(520),{cause:l});if(i=gt(i,n),fi===null?fi=[i]:fi.push(i),pe!==4&&(pe=2),t===null)return!0;l=gt(l,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=a&-a,n.lanes|=e,e=uc(n.stateNode,l,e),Io(n,e),!1;case 1:if(t=n.type,i=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||i!==null&&typeof i.componentDidCatch=="function"&&(Dn===null||!Dn.has(i))))return n.flags|=65536,a&=-a,n.lanes|=a,a=Jy(a),Py(a,e,n,l),Io(n,a),!1}n=n.return}while(n!==null);return!1}var Iy=Error(O(461)),Ee=!1;function Ae(e,t,n,l){t.child=e===null?dy(t,null,n,l):hl(t,e.child,n,l)}function fg(e,t,n,l,a){n=n.render;var i=t.ref;if("ref"in l){var r={};for(var u in l)u!=="ref"&&(r[u]=l[u])}else r=l;return vl(t),l=Uf(e,t,n,r,i,a),u=Ff(),e!==null&&!Ee?(Bf(e,t,a),sn(e,t,a)):(W&&u&&Tf(t),t.flags|=1,Ae(e,t,l,a),t.child)}function dg(e,t,n,l,a){if(e===null){var i=n.type;return typeof i=="function"&&!Jf(i)&&i.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=i,Wy(e,t,i,l,a)):(e=Lr(n.type,null,l,t,t.mode,a),e.ref=t.ref,e.return=t,t.child=e)}if(i=e.child,!kf(e,a)){var r=i.memoizedProps;if(n=n.compare,n=n!==null?n:Ri,n(r,l)&&e.ref===t.ref)return sn(e,t,a)}return t.flags|=1,e=Tn(i,l),e.ref=t.ref,e.return=t,t.child=e}function Wy(e,t,n,l,a){if(e!==null){var i=e.memoizedProps;if(Ri(i,l)&&e.ref===t.ref)if(Ee=!1,t.pendingProps=l=i,kf(e,a))e.flags&131072&&(Ee=!0);else return t.lanes=e.lanes,sn(e,t,a)}return oc(e,t,n,l,a)}function ev(e,t,n){var l=t.pendingProps,a=l.children,i=(t.stateNode._pendingVisibility&2)!==0,r=e!==null?e.memoizedState:null;if(ri(e,t),l.mode==="hidden"||i){if(t.flags&128){if(l=r!==null?r.baseLanes|n:n,e!==null){for(a=t.child=e.child,i=0;a!==null;)i=i|a.lanes|a.childLanes,a=a.sibling;t.childLanes=i&~l}else t.childLanes=0,t.child=null;return pg(e,t,l,n)}if(n&536870912)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&Ur(t,r!==null?r.cachePool:null),r!==null?ng(t,r):tc(),py(t);else return t.lanes=t.childLanes=536870912,pg(e,t,r!==null?r.baseLanes|n:n,n)}else r!==null?(Ur(t,r.cachePool),ng(t,r),Sn(),t.memoizedState=null):(e!==null&&Ur(t,null),tc(),Sn());return Ae(e,t,a,n),t.child}function pg(e,t,n,l){var a=Nf();return a=a===null?null:{parent:Se._currentValue,pool:a},t.memoizedState={baseLanes:n,cachePool:a},e!==null&&Ur(t,null),tc(),py(t),e!==null&&Zi(e,t,l,!0),null}function ri(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=2097664);else{if(typeof n!="function"&&typeof n!="object")throw Error(O(284));(e===null||e.ref!==n)&&(t.flags|=2097664)}}function oc(e,t,n,l,a){return vl(t),n=Uf(e,t,n,l,void 0,a),l=Ff(),e!==null&&!Ee?(Bf(e,t,a),sn(e,t,a)):(W&&l&&Tf(t),t.flags|=1,Ae(e,t,n,a),t.child)}function gg(e,t,n,l,a,i){return vl(t),t.updateQueue=null,n=my(t,l,n,a),hy(e),l=Ff(),e!==null&&!Ee?(Bf(e,t,i),sn(e,t,i)):(W&&l&&Tf(t),t.flags|=1,Ae(e,t,n,i),t.child)}function hg(e,t,n,l,a){if(vl(t),t.stateNode===null){var i=Yl,r=n.contextType;typeof r=="object"&&r!==null&&(i=Ne(r)),i=new n(l,i),t.memoizedState=i.state!==null&&i.state!==void 0?i.state:null,i.updater=rc,t.stateNode=i,i._reactInternals=t,i=t.stateNode,i.props=l,i.state=t.memoizedState,i.refs={},Qf(t),r=n.contextType,i.context=typeof r=="object"&&r!==null?Ne(r):Yl,i.state=t.memoizedState,r=n.getDerivedStateFromProps,typeof r=="function"&&(ko(t,n,r,l),i.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof i.getSnapshotBeforeUpdate=="function"||typeof i.UNSAFE_componentWillMount!="function"&&typeof i.componentWillMount!="function"||(r=i.state,typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount(),r!==i.state&&rc.enqueueReplaceState(i,i.state,null),si(t,l,i,a),oi(),i.state=t.memoizedState),typeof i.componentDidMount=="function"&&(t.flags|=4194308),l=!0}else if(e===null){i=t.stateNode;var u=t.memoizedProps,o=yl(n,u);i.props=o;var s=i.context,c=n.contextType;r=Yl,typeof c=="object"&&c!==null&&(r=Ne(c));var f=n.getDerivedStateFromProps;c=typeof f=="function"||typeof i.getSnapshotBeforeUpdate=="function",u=t.pendingProps!==u,c||typeof i.UNSAFE_componentWillReceiveProps!="function"&&typeof i.componentWillReceiveProps!="function"||(u||s!==r)&&sg(t,i,l,r),yn=!1;var p=t.memoizedState;i.state=p,si(t,l,i,a),oi(),s=t.memoizedState,u||p!==s||yn?(typeof f=="function"&&(ko(t,n,f,l),s=t.memoizedState),(o=yn||og(t,n,o,l,p,s,r))?(c||typeof i.UNSAFE_componentWillMount!="function"&&typeof i.componentWillMount!="function"||(typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount()),typeof i.componentDidMount=="function"&&(t.flags|=4194308)):(typeof i.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=l,t.memoizedState=s),i.props=l,i.state=s,i.context=r,l=o):(typeof i.componentDidMount=="function"&&(t.flags|=4194308),l=!1)}else{i=t.stateNode,gc(e,t),r=t.memoizedProps,c=yl(n,r),i.props=c,f=t.pendingProps,p=i.context,s=n.contextType,o=Yl,typeof s=="object"&&s!==null&&(o=Ne(s)),u=n.getDerivedStateFromProps,(s=typeof u=="function"||typeof i.getSnapshotBeforeUpdate=="function")||typeof i.UNSAFE_componentWillReceiveProps!="function"&&typeof i.componentWillReceiveProps!="function"||(r!==f||p!==o)&&sg(t,i,l,o),yn=!1,p=t.memoizedState,i.state=p,si(t,l,i,a),oi();var d=t.memoizedState;r!==f||p!==d||yn||e!==null&&e.dependencies!==null&&yu(e.dependencies)?(typeof u=="function"&&(ko(t,n,u,l),d=t.memoizedState),(c=yn||og(t,n,c,l,p,d,o)||e!==null&&e.dependencies!==null&&yu(e.dependencies))?(s||typeof i.UNSAFE_componentWillUpdate!="function"&&typeof i.componentWillUpdate!="function"||(typeof i.componentWillUpdate=="function"&&i.componentWillUpdate(l,d,o),typeof i.UNSAFE_componentWillUpdate=="function"&&i.UNSAFE_componentWillUpdate(l,d,o)),typeof i.componentDidUpdate=="function"&&(t.flags|=4),typeof i.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof i.componentDidUpdate!="function"||r===e.memoizedProps&&p===e.memoizedState||(t.flags|=4),typeof i.getSnapshotBeforeUpdate!="function"||r===e.memoizedProps&&p===e.memoizedState||(t.flags|=1024),t.memoizedProps=l,t.memoizedState=d),i.props=l,i.state=d,i.context=o,l=c):(typeof i.componentDidUpdate!="function"||r===e.memoizedProps&&p===e.memoizedState||(t.flags|=4),typeof i.getSnapshotBeforeUpdate!="function"||r===e.memoizedProps&&p===e.memoizedState||(t.flags|=1024),l=!1)}return i=l,ri(e,t),l=(t.flags&128)!==0,i||l?(i=t.stateNode,n=l&&typeof n.getDerivedStateFromError!="function"?null:i.render(),t.flags|=1,e!==null&&l?(t.child=hl(t,e.child,null,a),t.child=hl(t,null,n,a)):Ae(e,t,n,a),t.memoizedState=i.state,e=t.child):e=sn(e,t,a),e}function mg(e,t,n,l){return Xi(),t.flags|=256,Ae(e,t,n,l),t.child}var Qo={dehydrated:null,treeContext:null,retryLane:0};function Zo(e){return{baseLanes:e,cachePool:gy()}}function Ko(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=mt),e}function tv(e,t,n){var l=t.pendingProps,a=!1,i=(t.flags&128)!==0,r;if((r=i)||(r=e!==null&&e.memoizedState===null?!1:(xe.current&2)!==0),r&&(a=!0,t.flags&=-129),r=(t.flags&32)!==0,t.flags&=-33,e===null){if(W){if(a?vn(t):Sn(),W){var u=Oe,o;if(o=u){e:{for(o=u,u=Mt;o.nodeType!==8;){if(!u){u=null;break e}if(o=Ct(o.nextSibling),o===null){u=null;break e}}u=o}u!==null?(t.memoizedState={dehydrated:u,treeContext:ul!==null?{id:It,overflow:Wt}:null,retryLane:536870912},o=ht(18,null,null,0),o.stateNode=u,o.return=t,t.child=o,ze=t,Oe=null,o=!0):o=!1}o||gl(t)}if(u=t.memoizedState,u!==null&&(u=u.dehydrated,u!==null))return u.data==="$!"?t.lanes=16:t.lanes=536870912,null;en(t)}return u=l.children,l=l.fallback,a?(Sn(),a=t.mode,u=cc({mode:"hidden",children:u},a),l=sl(l,a,n,null),u.return=t,l.return=t,u.sibling=l,t.child=u,a=t.child,a.memoizedState=Zo(n),a.childLanes=Ko(e,r,n),t.memoizedState=Qo,l):(vn(t),sc(t,u))}if(o=e.memoizedState,o!==null&&(u=o.dehydrated,u!==null)){if(i)t.flags&256?(vn(t),t.flags&=-257,t=Jo(e,t,n)):t.memoizedState!==null?(Sn(),t.child=e.child,t.flags|=128,t=null):(Sn(),a=l.fallback,u=t.mode,l=cc({mode:"visible",children:l.children},u),a=sl(a,u,n,null),a.flags|=2,l.return=t,a.return=t,l.sibling=a,t.child=l,hl(t,e.child,null,n),l=t.child,l.memoizedState=Zo(n),l.childLanes=Ko(e,r,n),t.memoizedState=Qo,t=a);else if(vn(t),u.data==="$!"){if(r=u.nextSibling&&u.nextSibling.dataset,r)var s=r.dgst;r=s,l=Error(O(419)),l.stack="",l.digest=r,Ei({value:l,source:null,stack:null}),t=Jo(e,t,n)}else if(Ee||Zi(e,t,n,!1),r=(n&e.childLanes)!==0,Ee||r){if(r=ie,r!==null){if(l=n&-n,l&42)l=1;else switch(l){case 2:l=1;break;case 8:l=4;break;case 32:l=16;break;case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:l=64;break;case 268435456:l=134217728;break;default:l=0}if(l=l&(r.suspendedLanes|n)?0:l,l!==0&&l!==o.retryLane)throw o.retryLane=l,Fn(e,l),Ue(r,e,l),Iy}u.data==="$?"||bc(),t=Jo(e,t,n)}else u.data==="$?"?(t.flags|=128,t.child=e.child,t=xA.bind(null,e),u._reactRetry=t,t=null):(e=o.treeContext,Oe=Ct(u.nextSibling),ze=t,W=!0,bt=null,Mt=!1,e!==null&&(ft[dt++]=It,ft[dt++]=Wt,ft[dt++]=ul,It=e.id,Wt=e.overflow,ul=t),t=sc(t,l.children),t.flags|=4096);return t}return a?(Sn(),a=l.fallback,u=t.mode,o=e.child,s=o.sibling,l=Tn(o,{mode:"hidden",children:l.children}),l.subtreeFlags=o.subtreeFlags&31457280,s!==null?a=Tn(s,a):(a=sl(a,u,n,null),a.flags|=2),a.return=t,l.return=t,l.sibling=a,t.child=l,l=a,a=t.child,u=e.child.memoizedState,u===null?u=Zo(n):(o=u.cachePool,o!==null?(s=Se._currentValue,o=o.parent!==s?{parent:s,pool:s}:o):o=gy(),u={baseLanes:u.baseLanes|n,cachePool:o}),a.memoizedState=u,a.childLanes=Ko(e,r,n),t.memoizedState=Qo,l):(vn(t),n=e.child,e=n.sibling,n=Tn(n,{mode:"visible",children:l.children}),n.return=t,n.sibling=null,e!==null&&(r=t.deletions,r===null?(t.deletions=[e],t.flags|=16):r.push(e)),t.child=n,t.memoizedState=null,n)}function sc(e,t){return t=cc({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function cc(e,t){return Rv(e,t,0,null)}function Jo(e,t,n){return hl(t,e.child,null,n),e=sc(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function yg(e,t,n){e.lanes|=t;var l=e.alternate;l!==null&&(l.lanes|=t),dc(e.return,t,n)}function Po(e,t,n,l,a){var i=e.memoizedState;i===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:l,tail:n,tailMode:a}:(i.isBackwards=t,i.rendering=null,i.renderingStartTime=0,i.last=l,i.tail=n,i.tailMode=a)}function nv(e,t,n){var l=t.pendingProps,a=l.revealOrder,i=l.tail;if(Ae(e,t,l.children,n),l=xe.current,l&2)l=l&1|2,t.flags|=128;else{if(e!==null&&e.flags&128)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&yg(e,n,t);else if(e.tag===19)yg(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}l&=1}switch(oe(xe,l),a){case"forwards":for(n=t.child,a=null;n!==null;)e=n.alternate,e!==null&&du(e)===null&&(a=n),n=n.sibling;n=a,n===null?(a=t.child,t.child=null):(a=n.sibling,n.sibling=null),Po(t,!1,a,n,i);break;case"backwards":for(n=null,a=t.child,t.child=null;a!==null;){if(e=a.alternate,e!==null&&du(e)===null){t.child=a;break}e=a.sibling,a.sibling=n,n=a,a=e}Po(t,!0,n,null,i);break;case"together":Po(t,!1,null,null,void 0);break;default:t.memoizedState=null}return t.child}function sn(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),Ln|=t.lanes,!(n&t.childLanes))if(e!==null){if(Zi(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(O(153));if(t.child!==null){for(e=t.child,n=Tn(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=Tn(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function kf(e,t){return e.lanes&t?!0:(e=e.dependencies,!!(e!==null&&yu(e)))}function rA(e,t,n){switch(t.tag){case 3:lu(t,t.stateNode.containerInfo),xn(t,Se,e.memoizedState.cache),Xi();break;case 27:case 5:Ys(t);break;case 4:lu(t,t.stateNode.containerInfo);break;case 10:xn(t,t.type,t.memoizedProps.value);break;case 13:var l=t.memoizedState;if(l!==null)return l.dehydrated!==null?(vn(t),t.flags|=128,null):n&t.child.childLanes?tv(e,t,n):(vn(t),e=sn(e,t,n),e!==null?e.sibling:null);vn(t);break;case 19:var a=(e.flags&128)!==0;if(l=(n&t.childLanes)!==0,l||(Zi(e,t,n,!1),l=(n&t.childLanes)!==0),a){if(l)return nv(e,t,n);t.flags|=128}if(a=t.memoizedState,a!==null&&(a.rendering=null,a.tail=null,a.lastEffect=null),oe(xe,xe.current),l)break;return null;case 22:case 23:return t.lanes=0,ev(e,t,n);case 24:xn(t,Se,e.memoizedState.cache)}return sn(e,t,n)}function lv(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)Ee=!0;else{if(!kf(e,n)&&!(t.flags&128))return Ee=!1,rA(e,t,n);Ee=!!(e.flags&131072)}else Ee=!1,W&&t.flags&1048576&&oy(t,cu,t.index);switch(t.lanes=0,t.tag){case 16:e:{e=t.pendingProps;var l=t.elementType,a=l._init;if(l=a(l._payload),t.type=l,typeof l=="function")Jf(l)?(e=yl(l,e),t.tag=1,t=hg(null,t,l,e,n)):(t.tag=0,t=oc(null,t,l,e,n));else{if(l!=null){if(a=l.$$typeof,a===yf){t.tag=11,t=fg(null,t,l,e,n);break e}else if(a===vf){t.tag=14,t=dg(null,t,l,e,n);break e}}throw t=qs(l)||l,Error(O(306,t,""))}}return t;case 0:return oc(e,t,t.type,t.pendingProps,n);case 1:return l=t.type,a=yl(l,t.pendingProps),hg(e,t,l,a,n);case 3:e:{if(lu(t,t.stateNode.containerInfo),e===null)throw Error(O(387));var i=t.pendingProps;a=t.memoizedState,l=a.element,gc(e,t),si(t,i,null,n);var r=t.memoizedState;if(i=r.cache,xn(t,Se,i),i!==a.cache&&pc(t,[Se],n,!0),oi(),i=r.element,a.isDehydrated)if(a={element:i,isDehydrated:!1,cache:r.cache},t.updateQueue.baseState=a,t.memoizedState=a,t.flags&256){t=mg(e,t,i,n);break e}else if(i!==l){l=gt(Error(O(424)),t),Ei(l),t=mg(e,t,i,n);break e}else for(Oe=Ct(t.stateNode.containerInfo.firstChild),ze=t,W=!0,bt=null,Mt=!0,n=dy(t,null,i,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling;else{if(Xi(),i===l){t=sn(e,t,n);break e}Ae(e,t,i,n)}t=t.child}return t;case 26:return ri(e,t),e===null?(n=Ug(t.type,null,t.pendingProps,null))?t.memoizedState=n:W||(n=t.type,e=t.pendingProps,l=Eu(_n.current).createElement(n),l[je]=t,l[Ge]=e,Te(l,n,e),Re(l),t.stateNode=l):t.memoizedState=Ug(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return Ys(t),e===null&&W&&(l=t.stateNode=$v(t.type,t.pendingProps,_n.current),ze=t,Mt=!0,Oe=Ct(l.firstChild)),l=t.pendingProps.children,e!==null||W?Ae(e,t,l,n):t.child=hl(t,null,l,n),ri(e,t),t.child;case 5:return e===null&&W&&((a=l=Oe)&&(l=FA(l,t.type,t.pendingProps,Mt),l!==null?(t.stateNode=l,ze=t,Oe=Ct(l.firstChild),Mt=!1,a=!0):a=!1),a||gl(t)),Ys(t),a=t.type,i=t.pendingProps,r=e!==null?e.memoizedProps:null,l=i.children,Ac(a,i)?l=null:r!==null&&Ac(a,r)&&(t.flags|=32),t.memoizedState!==null&&(a=Uf(e,t,W_,null,null,n),Di._currentValue=a),ri(e,t),Ae(e,t,l,n),t.child;case 6:return e===null&&W&&((e=n=Oe)&&(n=BA(n,t.pendingProps,Mt),n!==null?(t.stateNode=n,ze=t,Oe=null,e=!0):e=!1),e||gl(t)),null;case 13:return tv(e,t,n);case 4:return lu(t,t.stateNode.containerInfo),l=t.pendingProps,e===null?t.child=hl(t,null,l,n):Ae(e,t,l,n),t.child;case 11:return fg(e,t,t.type,t.pendingProps,n);case 7:return Ae(e,t,t.pendingProps,n),t.child;case 8:return Ae(e,t,t.pendingProps.children,n),t.child;case 12:return Ae(e,t,t.pendingProps.children,n),t.child;case 10:return l=t.pendingProps,xn(t,t.type,l.value),Ae(e,t,l.children,n),t.child;case 9:return a=t.type._context,l=t.pendingProps.children,vl(t),a=Ne(a),l=l(a),t.flags|=1,Ae(e,t,l,n),t.child;case 14:return dg(e,t,t.type,t.pendingProps,n);case 15:return Wy(e,t,t.type,t.pendingProps,n);case 19:return nv(e,t,n);case 22:return ev(e,t,n);case 24:return vl(t),l=Ne(Se),e===null?(a=Nf(),a===null&&(a=ie,i=jf(),a.pooledCache=i,i.refCount++,i!==null&&(a.pooledCacheLanes|=n),a=i),t.memoizedState={parent:l,cache:a},Qf(t),xn(t,Se,a)):(e.lanes&n&&(gc(e,t),si(t,null,null,n),oi()),a=e.memoizedState,i=t.memoizedState,a.parent!==l?(a={parent:l,cache:l},t.memoizedState=a,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=a),xn(t,Se,l)):(l=i.cache,xn(t,Se,l),l!==a.cache&&pc(t,[Se],n,!0))),Ae(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(O(156,t.tag))}var fc=Vt(null),Cl=null,tn=null;function xn(e,t,n){oe(fc,t._currentValue),t._currentValue=n}function ln(e){e._currentValue=fc.current,Ce(fc)}function dc(e,t,n){for(;e!==null;){var l=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,l!==null&&(l.childLanes|=t)):l!==null&&(l.childLanes&t)!==t&&(l.childLanes|=t),e===n)break;e=e.return}}function pc(e,t,n,l){var a=e.child;for(a!==null&&(a.return=e);a!==null;){var i=a.dependencies;if(i!==null){var r=a.child;i=i.firstContext;e:for(;i!==null;){var u=i;i=a;for(var o=0;o<t.length;o++)if(u.context===t[o]){i.lanes|=n,u=i.alternate,u!==null&&(u.lanes|=n),dc(i.return,n,e),l||(r=null);break e}i=u.next}}else if(a.tag===18){if(r=a.return,r===null)throw Error(O(341));r.lanes|=n,i=r.alternate,i!==null&&(i.lanes|=n),dc(r,n,e),r=null}else r=a.child;if(r!==null)r.return=a;else for(r=a;r!==null;){if(r===e){r=null;break}if(a=r.sibling,a!==null){a.return=r.return,r=a;break}r=r.return}a=r}}function Zi(e,t,n,l){e=null;for(var a=t,i=!1;a!==null;){if(!i){if(a.flags&524288)i=!0;else if(a.flags&262144)break}if(a.tag===10){var r=a.alternate;if(r===null)throw Error(O(387));if(r=r.memoizedProps,r!==null){var u=a.type;at(a.pendingProps.value,r.value)||(e!==null?e.push(u):e=[u])}}else if(a===nu.current){if(r=a.alternate,r===null)throw Error(O(387));r.memoizedState.memoizedState!==a.memoizedState.memoizedState&&(e!==null?e.push(Di):e=[Di])}a=a.return}e!==null&&pc(t,e,n,l),t.flags|=262144}function yu(e){for(e=e.firstContext;e!==null;){if(!at(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function vl(e){Cl=e,tn=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function Ne(e){return av(Cl,e)}function Sr(e,t){return Cl===null&&vl(e),av(e,t)}function av(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},tn===null){if(e===null)throw Error(O(308));tn=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else tn=tn.next=t;return n}var yn=!1;function Qf(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function gc(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function An(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function On(e,t,n){var l=e.updateQueue;if(l===null)return null;if(l=l.shared,ce&2){var a=l.pending;return a===null?t.next=t:(t.next=a.next,a.next=t),l.pending=t,t=ou(e),uy(e,null,n),t}return lo(e,l,t,n),ou(e)}function ui(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194176)!==0)){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,z0(e,n)}}function Io(e,t){var n=e.updateQueue,l=e.alternate;if(l!==null&&(l=l.updateQueue,n===l)){var a=null,i=null;if(n=n.firstBaseUpdate,n!==null){do{var r={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};i===null?a=i=r:i=i.next=r,n=n.next}while(n!==null);i===null?a=i=t:i=i.next=t}else a=i=t;n={baseState:l.baseState,firstBaseUpdate:a,lastBaseUpdate:i,shared:l.shared,callbacks:l.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var hc=!1;function oi(){if(hc){var e=ta;if(e!==null)throw e}}function si(e,t,n,l){hc=!1;var a=e.updateQueue;yn=!1;var i=a.firstBaseUpdate,r=a.lastBaseUpdate,u=a.shared.pending;if(u!==null){a.shared.pending=null;var o=u,s=o.next;o.next=null,r===null?i=s:r.next=s,r=o;var c=e.alternate;c!==null&&(c=c.updateQueue,u=c.lastBaseUpdate,u!==r&&(u===null?c.firstBaseUpdate=s:u.next=s,c.lastBaseUpdate=o))}if(i!==null){var f=a.baseState;r=0,c=s=o=null,u=i;do{var p=u.lane&-536870913,d=p!==u.lane;if(d?(I&p)===p:(l&p)===p){p!==0&&p===ga&&(hc=!0),c!==null&&(c=c.next={lane:0,tag:u.tag,payload:u.payload,callback:null,next:null});e:{var h=e,S=u;p=t;var x=n;switch(S.tag){case 1:if(h=S.payload,typeof h=="function"){f=h.call(x,f,p);break e}f=h;break e;case 3:h.flags=h.flags&-65537|128;case 0:if(h=S.payload,p=typeof h=="function"?h.call(x,f,p):h,p==null)break e;f=ue({},f,p);break e;case 2:yn=!0}}p=u.callback,p!==null&&(e.flags|=64,d&&(e.flags|=8192),d=a.callbacks,d===null?a.callbacks=[p]:d.push(p))}else d={lane:p,tag:u.tag,payload:u.payload,callback:u.callback,next:null},c===null?(s=c=d,o=f):c=c.next=d,r|=p;if(u=u.next,u===null){if(u=a.shared.pending,u===null)break;d=u,u=d.next,d.next=null,a.lastBaseUpdate=d,a.shared.pending=null}}while(!0);c===null&&(o=f),a.baseState=o,a.firstBaseUpdate=s,a.lastBaseUpdate=c,i===null&&(a.shared.lanes=0),Ln|=r,e.lanes=r,e.memoizedState=f}}function iv(e,t){if(typeof e!="function")throw Error(O(191,e));e.call(t)}function rv(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)iv(n[e],t)}function Ki(e,t){try{var n=t.updateQueue,l=n!==null?n.lastEffect:null;if(l!==null){var a=l.next;n=a;do{if((n.tag&e)===e){l=void 0;var i=n.create,r=n.inst;l=i(),r.destroy=l}n=n.next}while(n!==a)}}catch(u){ne(t,t.return,u)}}function Hn(e,t,n){try{var l=t.updateQueue,a=l!==null?l.lastEffect:null;if(a!==null){var i=a.next;l=i;do{if((l.tag&e)===e){var r=l.inst,u=r.destroy;if(u!==void 0){r.destroy=void 0,a=t;var o=n;try{u()}catch(s){ne(a,o,s)}}}l=l.next}while(l!==i)}}catch(s){ne(t,t.return,s)}}function uv(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{rv(t,n)}catch(l){ne(e,e.return,l)}}}function ov(e,t,n){n.props=yl(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(l){ne(e,t,l)}}function ll(e,t){try{var n=e.ref;if(n!==null){var l=e.stateNode;switch(e.tag){case 26:case 27:case 5:var a=l;break;default:a=l}typeof n=="function"?e.refCleanup=n(a):n.current=a}}catch(i){ne(e,t,i)}}function Pe(e,t){var n=e.ref,l=e.refCleanup;if(n!==null)if(typeof l=="function")try{l()}catch(a){ne(e,t,a)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(a){ne(e,t,a)}else n.current=null}function sv(e){var t=e.type,n=e.memoizedProps,l=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&l.focus();break e;case"img":n.src?l.src=n.src:n.srcSet&&(l.srcset=n.srcSet)}}catch(a){ne(e,e.return,a)}}function vg(e,t,n){try{var l=e.stateNode;MA(l,e.type,n,t),l[Ge]=t}catch(a){ne(e,e.return,a)}}function cv(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27||e.tag===4}function Wo(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||cv(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==27&&e.tag!==18;){if(e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function mc(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?n.nodeType===8?n.parentNode.insertBefore(e,t):n.insertBefore(e,t):(n.nodeType===8?(t=n.parentNode,t.insertBefore(e,n)):(t=n,t.appendChild(e)),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=co));else if(l!==4&&l!==27&&(e=e.child,e!==null))for(mc(e,t,n),e=e.sibling;e!==null;)mc(e,t,n),e=e.sibling}function vu(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(l!==4&&l!==27&&(e=e.child,e!==null))for(vu(e,t,n),e=e.sibling;e!==null;)vu(e,t,n),e=e.sibling}var Qt=!1,fe=!1,es=!1,Sg=typeof WeakSet=="function"?WeakSet:Set,be=null,xg=!1;function uA(e,t){if(e=e.containerInfo,Cc=Ou,e=ey(e),_f(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var l=n.getSelection&&n.getSelection();if(l&&l.rangeCount!==0){n=l.anchorNode;var a=l.anchorOffset,i=l.focusNode;l=l.focusOffset;try{n.nodeType,i.nodeType}catch{n=null;break e}var r=0,u=-1,o=-1,s=0,c=0,f=e,p=null;t:for(;;){for(var d;f!==n||a!==0&&f.nodeType!==3||(u=r+a),f!==i||l!==0&&f.nodeType!==3||(o=r+l),f.nodeType===3&&(r+=f.nodeValue.length),(d=f.firstChild)!==null;)p=f,f=d;for(;;){if(f===e)break t;if(p===n&&++s===a&&(u=r),p===i&&++c===l&&(o=r),(d=f.nextSibling)!==null)break;f=p,p=f.parentNode}f=d}n=u===-1||o===-1?null:{start:u,end:o}}else n=null}n=n||{start:0,end:0}}else n=null;for(_c={focusedElem:e,selectionRange:n},Ou=!1,be=t;be!==null;)if(t=be,e=t.child,(t.subtreeFlags&1028)!==0&&e!==null)e.return=t,be=e;else for(;be!==null;){switch(t=be,i=t.alternate,e=t.flags,t.tag){case 0:break;case 11:case 15:break;case 1:if(e&1024&&i!==null){e=void 0,n=t,a=i.memoizedProps,i=i.memoizedState,l=n.stateNode;try{var h=yl(n.type,a,n.elementType===n.type);e=l.getSnapshotBeforeUpdate(h,i),l.__reactInternalSnapshotBeforeUpdate=e}catch(S){ne(n,n.return,S)}}break;case 3:if(e&1024){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)Oc(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":Oc(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if(e&1024)throw Error(O(163))}if(e=t.sibling,e!==null){e.return=t.return,be=e;break}be=t.return}return h=xg,xg=!1,h}function fv(e,t,n){var l=n.flags;switch(n.tag){case 0:case 11:case 15:Yt(e,n),l&4&&Ki(5,n);break;case 1:if(Yt(e,n),l&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(u){ne(n,n.return,u)}else{var a=yl(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(a,t,e.__reactInternalSnapshotBeforeUpdate)}catch(u){ne(n,n.return,u)}}l&64&&uv(n),l&512&&ll(n,n.return);break;case 3:if(Yt(e,n),l&64&&(l=n.updateQueue,l!==null)){if(e=null,n.child!==null)switch(n.child.tag){case 27:case 5:e=n.child.stateNode;break;case 1:e=n.child.stateNode}try{rv(l,e)}catch(u){ne(n,n.return,u)}}break;case 26:Yt(e,n),l&512&&ll(n,n.return);break;case 27:case 5:Yt(e,n),t===null&&l&4&&sv(n),l&512&&ll(n,n.return);break;case 12:Yt(e,n);break;case 13:Yt(e,n),l&4&&gv(e,n);break;case 22:if(a=n.memoizedState!==null||Qt,!a){t=t!==null&&t.memoizedState!==null||fe;var i=Qt,r=fe;Qt=a,(fe=t)&&!r?gn(e,n,(n.subtreeFlags&8772)!==0):Yt(e,n),Qt=i,fe=r}l&512&&(n.memoizedProps.mode==="manual"?ll(n,n.return):Pe(n,n.return));break;default:Yt(e,n)}}function dv(e){var t=e.alternate;t!==null&&(e.alternate=null,dv(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&xf(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var he=null,Ze=!1;function Gt(e,t,n){for(n=n.child;n!==null;)pv(e,t,n),n=n.sibling}function pv(e,t,n){if(We&&typeof We.onCommitFiberUnmount=="function")try{We.onCommitFiberUnmount($i,n)}catch{}switch(n.tag){case 26:fe||Pe(n,t),Gt(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:fe||Pe(n,t);var l=he,a=Ze;for(he=n.stateNode,Gt(e,t,n),n=n.stateNode,t=n.attributes;t.length;)n.removeAttributeNode(t[0]);xf(n),he=l,Ze=a;break;case 5:fe||Pe(n,t);case 6:a=he;var i=Ze;if(he=null,Gt(e,t,n),he=a,Ze=i,he!==null)if(Ze)try{e=he,l=n.stateNode,e.nodeType===8?e.parentNode.removeChild(l):e.removeChild(l)}catch(r){ne(n,t,r)}else try{he.removeChild(n.stateNode)}catch(r){ne(n,t,r)}break;case 18:he!==null&&(Ze?(t=he,n=n.stateNode,t.nodeType===8?ds(t.parentNode,n):t.nodeType===1&&ds(t,n),Ni(t)):ds(he,n.stateNode));break;case 4:l=he,a=Ze,he=n.stateNode.containerInfo,Ze=!0,Gt(e,t,n),he=l,Ze=a;break;case 0:case 11:case 14:case 15:fe||Hn(2,n,t),fe||Hn(4,n,t),Gt(e,t,n);break;case 1:fe||(Pe(n,t),l=n.stateNode,typeof l.componentWillUnmount=="function"&&ov(n,t,l)),Gt(e,t,n);break;case 21:Gt(e,t,n);break;case 22:fe||Pe(n,t),fe=(l=fe)||n.memoizedState!==null,Gt(e,t,n),fe=l;break;default:Gt(e,t,n)}}function gv(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{Ni(e)}catch(n){ne(t,t.return,n)}}function oA(e){switch(e.tag){case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new Sg),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new Sg),t;default:throw Error(O(435,e.tag))}}function ts(e,t){var n=oA(e);t.forEach(function(l){var a=wA.bind(null,e,l);n.has(l)||(n.add(l),l.then(a,a))})}function rt(e,t){var n=t.deletions;if(n!==null)for(var l=0;l<n.length;l++){var a=n[l],i=e,r=t,u=r;e:for(;u!==null;){switch(u.tag){case 27:case 5:he=u.stateNode,Ze=!1;break e;case 3:he=u.stateNode.containerInfo,Ze=!0;break e;case 4:he=u.stateNode.containerInfo,Ze=!0;break e}u=u.return}if(he===null)throw Error(O(160));pv(i,r,a),he=null,Ze=!1,i=a.alternate,i!==null&&(i.return=null),a.return=null}if(t.subtreeFlags&13878)for(t=t.child;t!==null;)hv(t,e),t=t.sibling}var wt=null;function hv(e,t){var n=e.alternate,l=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:rt(t,e),ut(e),l&4&&(Hn(3,e,e.return),Ki(3,e),Hn(5,e,e.return));break;case 1:rt(t,e),ut(e),l&512&&(fe||n===null||Pe(n,n.return)),l&64&&Qt&&(e=e.updateQueue,e!==null&&(l=e.callbacks,l!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?l:n.concat(l))));break;case 26:var a=wt;if(rt(t,e),ut(e),l&512&&(fe||n===null||Pe(n,n.return)),l&4){var i=n!==null?n.memoizedState:null;if(l=e.memoizedState,n===null)if(l===null)if(e.stateNode===null){e:{l=e.type,n=e.memoizedProps,a=a.ownerDocument||a;t:switch(l){case"title":i=a.getElementsByTagName("title")[0],(!i||i[xi]||i[je]||i.namespaceURI==="http://www.w3.org/2000/svg"||i.hasAttribute("itemprop"))&&(i=a.createElement(l),a.head.insertBefore(i,a.querySelector("head > title"))),Te(i,l,n),i[je]=e,Re(i),l=i;break e;case"link":var r=Bg("link","href",a).get(l+(n.href||""));if(r){for(var u=0;u<r.length;u++)if(i=r[u],i.getAttribute("href")===(n.href==null?null:n.href)&&i.getAttribute("rel")===(n.rel==null?null:n.rel)&&i.getAttribute("title")===(n.title==null?null:n.title)&&i.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){r.splice(u,1);break t}}i=a.createElement(l),Te(i,l,n),a.head.appendChild(i);break;case"meta":if(r=Bg("meta","content",a).get(l+(n.content||""))){for(u=0;u<r.length;u++)if(i=r[u],i.getAttribute("content")===(n.content==null?null:""+n.content)&&i.getAttribute("name")===(n.name==null?null:n.name)&&i.getAttribute("property")===(n.property==null?null:n.property)&&i.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&i.getAttribute("charset")===(n.charSet==null?null:n.charSet)){r.splice(u,1);break t}}i=a.createElement(l),Te(i,l,n),a.head.appendChild(i);break;default:throw Error(O(468,l))}i[je]=e,Re(i),l=i}e.stateNode=l}else Hg(a,e.type,e.stateNode);else e.stateNode=Fg(a,l,e.memoizedProps);else i!==l?(i===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):i.count--,l===null?Hg(a,e.type,e.stateNode):Fg(a,l,e.memoizedProps)):l===null&&e.stateNode!==null&&vg(e,e.memoizedProps,n.memoizedProps)}break;case 27:if(l&4&&e.alternate===null){a=e.stateNode,i=e.memoizedProps;try{for(var o=a.firstChild;o;){var s=o.nextSibling,c=o.nodeName;o[xi]||c==="HEAD"||c==="BODY"||c==="SCRIPT"||c==="STYLE"||c==="LINK"&&o.rel.toLowerCase()==="stylesheet"||a.removeChild(o),o=s}for(var f=e.type,p=a.attributes;p.length;)a.removeAttributeNode(p[0]);Te(a,f,i),a[je]=e,a[Ge]=i}catch(h){ne(e,e.return,h)}}case 5:if(rt(t,e),ut(e),l&512&&(fe||n===null||Pe(n,n.return)),e.flags&32){a=e.stateNode;try{da(a,"")}catch(h){ne(e,e.return,h)}}l&4&&e.stateNode!=null&&(a=e.memoizedProps,vg(e,a,n!==null?n.memoizedProps:a)),l&1024&&(es=!0);break;case 6:if(rt(t,e),ut(e),l&4){if(e.stateNode===null)throw Error(O(162));l=e.memoizedProps,n=e.stateNode;try{n.nodeValue=l}catch(h){ne(e,e.return,h)}}break;case 3:if($r=null,a=wt,wt=Cu(t.containerInfo),rt(t,e),wt=a,ut(e),l&4&&n!==null&&n.memoizedState.isDehydrated)try{Ni(t.containerInfo)}catch(h){ne(e,e.return,h)}es&&(es=!1,mv(e));break;case 4:l=wt,wt=Cu(e.stateNode.containerInfo),rt(t,e),ut(e),wt=l;break;case 12:rt(t,e),ut(e);break;case 13:rt(t,e),ut(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(Wf=Ft()),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,ts(e,l)));break;case 22:if(l&512&&(fe||n===null||Pe(n,n.return)),o=e.memoizedState!==null,s=n!==null&&n.memoizedState!==null,c=Qt,f=fe,Qt=c||o,fe=f||s,rt(t,e),fe=f,Qt=c,ut(e),t=e.stateNode,t._current=e,t._visibility&=-3,t._visibility|=t._pendingVisibility&2,l&8192&&(t._visibility=o?t._visibility&-2:t._visibility|1,o&&(t=Qt||fe,n===null||s||t||Dl(e)),e.memoizedProps===null||e.memoizedProps.mode!=="manual"))e:for(n=null,t=e;;){if(t.tag===5||t.tag===26||t.tag===27){if(n===null){s=n=t;try{if(a=s.stateNode,o)i=a.style,typeof i.setProperty=="function"?i.setProperty("display","none","important"):i.display="none";else{r=s.stateNode,u=s.memoizedProps.style;var d=u!=null&&u.hasOwnProperty("display")?u.display:null;r.style.display=d==null||typeof d=="boolean"?"":(""+d).trim()}}catch(h){ne(s,s.return,h)}}}else if(t.tag===6){if(n===null){s=t;try{s.stateNode.nodeValue=o?"":s.memoizedProps}catch(h){ne(s,s.return,h)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}l&4&&(l=e.updateQueue,l!==null&&(n=l.retryQueue,n!==null&&(l.retryQueue=null,ts(e,n))));break;case 19:rt(t,e),ut(e),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,ts(e,l)));break;case 21:break;default:rt(t,e),ut(e)}}function ut(e){var t=e.flags;if(t&2){try{if(e.tag!==27){e:{for(var n=e.return;n!==null;){if(cv(n)){var l=n;break e}n=n.return}throw Error(O(160))}switch(l.tag){case 27:var a=l.stateNode,i=Wo(e);vu(e,i,a);break;case 5:var r=l.stateNode;l.flags&32&&(da(r,""),l.flags&=-33);var u=Wo(e);vu(e,u,r);break;case 3:case 4:var o=l.stateNode.containerInfo,s=Wo(e);mc(e,s,o);break;default:throw Error(O(161))}}}catch(c){ne(e,e.return,c)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function mv(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;mv(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function Yt(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)fv(e,t.alternate,t),t=t.sibling}function Dl(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:Hn(4,t,t.return),Dl(t);break;case 1:Pe(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&ov(t,t.return,n),Dl(t);break;case 26:case 27:case 5:Pe(t,t.return),Dl(t);break;case 22:Pe(t,t.return),t.memoizedState===null&&Dl(t);break;default:Dl(t)}e=e.sibling}}function gn(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var l=t.alternate,a=e,i=t,r=i.flags;switch(i.tag){case 0:case 11:case 15:gn(a,i,n),Ki(4,i);break;case 1:if(gn(a,i,n),l=i,a=l.stateNode,typeof a.componentDidMount=="function")try{a.componentDidMount()}catch(s){ne(l,l.return,s)}if(l=i,a=l.updateQueue,a!==null){var u=l.stateNode;try{var o=a.shared.hiddenCallbacks;if(o!==null)for(a.shared.hiddenCallbacks=null,a=0;a<o.length;a++)iv(o[a],u)}catch(s){ne(l,l.return,s)}}n&&r&64&&uv(i),ll(i,i.return);break;case 26:case 27:case 5:gn(a,i,n),n&&l===null&&r&4&&sv(i),ll(i,i.return);break;case 12:gn(a,i,n);break;case 13:gn(a,i,n),n&&r&4&&gv(a,i);break;case 22:i.memoizedState===null&&gn(a,i,n),ll(i,i.return);break;default:gn(a,i,n)}t=t.sibling}}function Zf(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&ki(n))}function Kf(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&ki(e))}function pn(e,t,n,l){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)yv(e,t,n,l),t=t.sibling}function yv(e,t,n,l){var a=t.flags;switch(t.tag){case 0:case 11:case 15:pn(e,t,n,l),a&2048&&Ki(9,t);break;case 3:pn(e,t,n,l),a&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&ki(e)));break;case 12:if(a&2048){pn(e,t,n,l),e=t.stateNode;try{var i=t.memoizedProps,r=i.id,u=i.onPostCommit;typeof u=="function"&&u(r,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(o){ne(t,t.return,o)}}else pn(e,t,n,l);break;case 23:break;case 22:i=t.stateNode,t.memoizedState!==null?i._visibility&4?pn(e,t,n,l):ci(e,t):i._visibility&4?pn(e,t,n,l):(i._visibility|=4,Ml(e,t,n,l,(t.subtreeFlags&10256)!==0)),a&2048&&Zf(t.alternate,t);break;case 24:pn(e,t,n,l),a&2048&&Kf(t.alternate,t);break;default:pn(e,t,n,l)}}function Ml(e,t,n,l,a){for(a=a&&(t.subtreeFlags&10256)!==0,t=t.child;t!==null;){var i=e,r=t,u=n,o=l,s=r.flags;switch(r.tag){case 0:case 11:case 15:Ml(i,r,u,o,a),Ki(8,r);break;case 23:break;case 22:var c=r.stateNode;r.memoizedState!==null?c._visibility&4?Ml(i,r,u,o,a):ci(i,r):(c._visibility|=4,Ml(i,r,u,o,a)),a&&s&2048&&Zf(r.alternate,r);break;case 24:Ml(i,r,u,o,a),a&&s&2048&&Kf(r.alternate,r);break;default:Ml(i,r,u,o,a)}t=t.sibling}}function ci(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,l=t,a=l.flags;switch(l.tag){case 22:ci(n,l),a&2048&&Zf(l.alternate,l);break;case 24:ci(n,l),a&2048&&Kf(l.alternate,l);break;default:ci(n,l)}t=t.sibling}}var Qa=8192;function Ol(e){if(e.subtreeFlags&Qa)for(e=e.child;e!==null;)vv(e),e=e.sibling}function vv(e){switch(e.tag){case 26:Ol(e),e.flags&Qa&&e.memoizedState!==null&&JA(wt,e.memoizedState,e.memoizedProps);break;case 5:Ol(e);break;case 3:case 4:var t=wt;wt=Cu(e.stateNode.containerInfo),Ol(e),wt=t;break;case 22:e.memoizedState===null&&(t=e.alternate,t!==null&&t.memoizedState!==null?(t=Qa,Qa=16777216,Ol(e),Qa=t):Ol(e));break;default:Ol(e)}}function Sv(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function Ba(e){var t=e.deletions;if(e.flags&16){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];be=l,wv(l,e)}Sv(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)xv(e),e=e.sibling}function xv(e){switch(e.tag){case 0:case 11:case 15:Ba(e),e.flags&2048&&Hn(9,e,e.return);break;case 3:Ba(e);break;case 12:Ba(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&4&&(e.return===null||e.return.tag!==13)?(t._visibility&=-5,Hr(e)):Ba(e);break;default:Ba(e)}}function Hr(e){var t=e.deletions;if(e.flags&16){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];be=l,wv(l,e)}Sv(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:Hn(8,t,t.return),Hr(t);break;case 22:n=t.stateNode,n._visibility&4&&(n._visibility&=-5,Hr(t));break;default:Hr(t)}e=e.sibling}}function wv(e,t){for(;be!==null;){var n=be;switch(n.tag){case 0:case 11:case 15:Hn(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var l=n.memoizedState.cachePool.pool;l!=null&&l.refCount++}break;case 24:ki(n.memoizedState.cache)}if(l=n.child,l!==null)l.return=n,be=l;else e:for(n=e;be!==null;){l=be;var a=l.sibling,i=l.return;if(dv(l),l===n){be=null;break e}if(a!==null){a.return=i,be=a;break e}be=i}}}function sA(e,t,n,l){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=l,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function ht(e,t,n,l){return new sA(e,t,n,l)}function Jf(e){return e=e.prototype,!(!e||!e.isReactComponent)}function Tn(e,t){var n=e.alternate;return n===null?(n=ht(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&31457280,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function bv(e,t){e.flags&=31457282;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function Lr(e,t,n,l,a,i){var r=0;if(l=e,typeof e=="function")Jf(e)&&(r=1);else if(typeof e=="string")r=ZA(e,n,Ut.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case Bl:return sl(n.children,a,i,t);case R0:r=8,a|=24;break;case Ls:return e=ht(12,n,t,a|2),e.elementType=Ls,e.lanes=i,e;case Vs:return e=ht(13,n,t,a),e.elementType=Vs,e.lanes=i,e;case $s:return e=ht(19,n,t,a),e.elementType=$s,e.lanes=i,e;case C0:return Rv(n,a,i,t);default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case zC:case Pt:r=10;break e;case E0:r=9;break e;case yf:r=11;break e;case vf:r=14;break e;case mn:r=16,l=null;break e}r=29,n=Error(O(130,e===null?"null":typeof e,"")),l=null}return t=ht(r,n,t,a),t.elementType=e,t.type=l,t.lanes=i,t}function sl(e,t,n,l){return e=ht(7,e,l,t),e.lanes=n,e}function Rv(e,t,n,l){e=ht(22,e,l,t),e.elementType=C0,e.lanes=n;var a={_visibility:1,_pendingVisibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null,_current:null,detach:function(){var i=a._current;if(i===null)throw Error(O(456));if(!(a._pendingVisibility&2)){var r=Fn(i,2);r!==null&&(a._pendingVisibility|=2,Ue(r,i,2))}},attach:function(){var i=a._current;if(i===null)throw Error(O(456));if(a._pendingVisibility&2){var r=Fn(i,2);r!==null&&(a._pendingVisibility&=-3,Ue(r,i,2))}}};return e.stateNode=a,e}function ns(e,t,n){return e=ht(6,e,null,t),e.lanes=n,e}function ls(e,t,n){return t=ht(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}function Xt(e){e.flags|=4}function wg(e,t){if(t.type!=="stylesheet"||t.state.loading&4)e.flags&=-16777217;else if(e.flags|=16777216,!Yv(t)){if(t=St.current,t!==null&&((I&4194176)===I?Bt!==null:(I&62914560)!==I&&!(I&536870912)||t!==Bt))throw li=ec,sy;e.flags|=8192}}function xr(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?j0():536870912,e.lanes|=t,ma|=t)}function Ha(e,t){if(!W)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var l=null;n!==null;)n.alternate!==null&&(l=n),n=n.sibling;l===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:l.sibling=null}}function se(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,l=0;if(t)for(var a=e.child;a!==null;)n|=a.lanes|a.childLanes,l|=a.subtreeFlags&31457280,l|=a.flags&31457280,a.return=e,a=a.sibling;else for(a=e.child;a!==null;)n|=a.lanes|a.childLanes,l|=a.subtreeFlags,l|=a.flags,a.return=e,a=a.sibling;return e.subtreeFlags|=l,e.childLanes=n,t}function cA(e,t,n){var l=t.pendingProps;switch(Df(t),t.tag){case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return se(t),null;case 1:return se(t),null;case 3:return n=t.stateNode,l=null,e!==null&&(l=e.memoizedState.cache),t.memoizedState.cache!==l&&(t.flags|=2048),ln(Se),ca(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(Ua(t)?Xt(t):e===null||e.memoizedState.isDehydrated&&!(t.flags&256)||(t.flags|=1024,bt!==null&&(wc(bt),bt=null))),se(t),null;case 26:return n=t.memoizedState,e===null?(Xt(t),n!==null?(se(t),wg(t,n)):(se(t),t.flags&=-16777217)):n?n!==e.memoizedState?(Xt(t),se(t),wg(t,n)):(se(t),t.flags&=-16777217):(e.memoizedProps!==l&&Xt(t),se(t),t.flags&=-16777217),null;case 27:au(t),n=_n.current;var a=t.type;if(e!==null&&t.stateNode!=null)e.memoizedProps!==l&&Xt(t);else{if(!l){if(t.stateNode===null)throw Error(O(166));return se(t),null}e=Ut.current,Ua(t)?Pp(t):(e=$v(a,l,n),t.stateNode=e,Xt(t))}return se(t),null;case 5:if(au(t),n=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==l&&Xt(t);else{if(!l){if(t.stateNode===null)throw Error(O(166));return se(t),null}if(e=Ut.current,Ua(t))Pp(t);else{switch(a=Eu(_n.current),e){case 1:e=a.createElementNS("http://www.w3.org/2000/svg",n);break;case 2:e=a.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;default:switch(n){case"svg":e=a.createElementNS("http://www.w3.org/2000/svg",n);break;case"math":e=a.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;case"script":e=a.createElement("div"),e.innerHTML="<script><\/script>",e=e.removeChild(e.firstChild);break;case"select":e=typeof l.is=="string"?a.createElement("select",{is:l.is}):a.createElement("select"),l.multiple?e.multiple=!0:l.size&&(e.size=l.size);break;default:e=typeof l.is=="string"?a.createElement(n,{is:l.is}):a.createElement(n)}}e[je]=t,e[Ge]=l;e:for(a=t.child;a!==null;){if(a.tag===5||a.tag===6)e.appendChild(a.stateNode);else if(a.tag!==4&&a.tag!==27&&a.child!==null){a.child.return=a,a=a.child;continue}if(a===t)break e;for(;a.sibling===null;){if(a.return===null||a.return===t)break e;a=a.return}a.sibling.return=a.return,a=a.sibling}t.stateNode=e;e:switch(Te(e,n,l),n){case"button":case"input":case"select":case"textarea":e=!!l.autoFocus;break e;case"img":e=!0;break e;default:e=!1}e&&Xt(t)}}return se(t),t.flags&=-16777217,null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==l&&Xt(t);else{if(typeof l!="string"&&t.stateNode===null)throw Error(O(166));if(e=_n.current,Ua(t)){if(e=t.stateNode,n=t.memoizedProps,l=null,a=ze,a!==null)switch(a.tag){case 27:case 5:l=a.memoizedProps}e[je]=t,e=!!(e.nodeValue===n||l!==null&&l.suppressHydrationWarning===!0||Hv(e.nodeValue,n)),e||gl(t)}else e=Eu(e).createTextNode(l),e[je]=t,t.stateNode=e}return se(t),null;case 13:if(l=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(a=Ua(t),l!==null&&l.dehydrated!==null){if(e===null){if(!a)throw Error(O(318));if(a=t.memoizedState,a=a!==null?a.dehydrated:null,!a)throw Error(O(317));a[je]=t}else Xi(),!(t.flags&128)&&(t.memoizedState=null),t.flags|=4;se(t),a=!1}else bt!==null&&(wc(bt),bt=null),a=!0;if(!a)return t.flags&256?(en(t),t):(en(t),null)}if(en(t),t.flags&128)return t.lanes=n,t;if(n=l!==null,e=e!==null&&e.memoizedState!==null,n){l=t.child,a=null,l.alternate!==null&&l.alternate.memoizedState!==null&&l.alternate.memoizedState.cachePool!==null&&(a=l.alternate.memoizedState.cachePool.pool);var i=null;l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(i=l.memoizedState.cachePool.pool),i!==a&&(l.flags|=2048)}return n!==e&&n&&(t.child.flags|=8192),xr(t,t.updateQueue),se(t),null;case 4:return ca(),e===null&&nd(t.stateNode.containerInfo),se(t),null;case 10:return ln(t.type),se(t),null;case 19:if(Ce(xe),a=t.memoizedState,a===null)return se(t),null;if(l=(t.flags&128)!==0,i=a.rendering,i===null)if(l)Ha(a,!1);else{if(pe!==0||e!==null&&e.flags&128)for(e=t.child;e!==null;){if(i=du(e),i!==null){for(t.flags|=128,Ha(a,!1),e=i.updateQueue,t.updateQueue=e,xr(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)bv(n,e),n=n.sibling;return oe(xe,xe.current&1|2),t.child}e=e.sibling}a.tail!==null&&Ft()>Su&&(t.flags|=128,l=!0,Ha(a,!1),t.lanes=4194304)}else{if(!l)if(e=du(i),e!==null){if(t.flags|=128,l=!0,e=e.updateQueue,t.updateQueue=e,xr(t,e),Ha(a,!0),a.tail===null&&a.tailMode==="hidden"&&!i.alternate&&!W)return se(t),null}else 2*Ft()-a.renderingStartTime>Su&&n!==536870912&&(t.flags|=128,l=!0,Ha(a,!1),t.lanes=4194304);a.isBackwards?(i.sibling=t.child,t.child=i):(e=a.last,e!==null?e.sibling=i:t.child=i,a.last=i)}return a.tail!==null?(t=a.tail,a.rendering=t,a.tail=t.sibling,a.renderingStartTime=Ft(),t.sibling=null,e=xe.current,oe(xe,l?e&1|2:e&1),t):(se(t),null);case 22:case 23:return en(t),Mf(),l=t.memoizedState!==null,e!==null?e.memoizedState!==null!==l&&(t.flags|=8192):l&&(t.flags|=8192),l?n&536870912&&!(t.flags&128)&&(se(t),t.subtreeFlags&6&&(t.flags|=8192)):se(t),n=t.updateQueue,n!==null&&xr(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),l=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(l=t.memoizedState.cachePool.pool),l!==n&&(t.flags|=2048),e!==null&&Ce(ol),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),ln(Se),se(t),null;case 25:return null}throw Error(O(156,t.tag))}function fA(e,t){switch(Df(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return ln(Se),ca(),e=t.flags,e&65536&&!(e&128)?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return au(t),null;case 13:if(en(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(O(340));Xi()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return Ce(xe),null;case 4:return ca(),null;case 10:return ln(t.type),null;case 22:case 23:return en(t),Mf(),e!==null&&Ce(ol),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return ln(Se),null;case 25:return null;default:return null}}function Ev(e,t){switch(Df(t),t.tag){case 3:ln(Se),ca();break;case 26:case 27:case 5:au(t);break;case 4:ca();break;case 13:en(t);break;case 19:Ce(xe);break;case 10:ln(t.type);break;case 22:case 23:en(t),Mf(),e!==null&&Ce(ol);break;case 24:ln(Se)}}var dA={getCacheForType:function(e){var t=Ne(Se),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n}},pA=typeof WeakMap=="function"?WeakMap:Map,ce=0,ie=null,Z=null,I=0,ae=0,Ke=null,Zt=!1,Ea=!1,Pf=!1,cn=0,pe=0,Ln=0,cl=0,If=0,mt=0,ma=0,fi=null,jt=null,yc=!1,Wf=0,Su=1/0,xu=null,Dn=null,wr=!1,In=null,di=0,vc=0,Sc=null,pi=0,xc=null;function tt(){if(ce&2&&I!==0)return I&-I;if(G.T!==null){var e=ga;return e!==0?e:td()}return F0()}function Cv(){mt===0&&(mt=!(I&536870912)||W?M0():536870912);var e=St.current;return e!==null&&(e.flags|=32),mt}function Ue(e,t,n){(e===ie&&ae===2||e.cancelPendingCommit!==null)&&(ya(e,0),Kt(e,I,mt,!1)),Gi(e,n),(!(ce&2)||e!==ie)&&(e===ie&&(!(ce&2)&&(cl|=n),pe===4&&Kt(e,I,mt,!1)),$t(e))}function _v(e,t,n){if(ce&6)throw Error(O(327));var l=!n&&(t&60)===0&&(t&e.expiredLanes)===0||qi(e,t),a=l?mA(e,t):as(e,t,!0),i=l;do{if(a===0){Ea&&!l&&Kt(e,t,0,!1);break}else if(a===6)Kt(e,t,0,!Zt);else{if(n=e.current.alternate,i&&!gA(n)){a=as(e,t,!1),i=!1;continue}if(a===2){if(i=t,e.errorRecoveryDisabledLanes&i)var r=0;else r=e.pendingLanes&-536870913,r=r!==0?r:r&536870912?536870912:0;if(r!==0){t=r;e:{var u=e;a=fi;var o=u.current.memoizedState.isDehydrated;if(o&&(ya(u,r).flags|=256),r=as(u,r,!1),r!==2){if(Pf&&!o){u.errorRecoveryDisabledLanes|=i,cl|=i,a=4;break e}i=jt,jt=a,i!==null&&wc(i)}a=r}if(i=!1,a!==2)continue}}if(a===1){ya(e,0),Kt(e,t,0,!0);break}e:{switch(l=e,a){case 0:case 1:throw Error(O(345));case 4:if((t&4194176)===t){Kt(l,t,mt,!Zt);break e}break;case 2:jt=null;break;case 3:case 5:break;default:throw Error(O(329))}if(l.finishedWork=n,l.finishedLanes=t,(t&62914560)===t&&(i=Wf+300-Ft(),10<i)){if(Kt(l,t,mt,!Zt),Iu(l,0)!==0)break e;l.timeoutHandle=Vv(bg.bind(null,l,n,jt,xu,yc,t,mt,cl,ma,Zt,2,-0,0),i);break e}bg(l,n,jt,xu,yc,t,mt,cl,ma,Zt,0,-0,0)}}break}while(!0);$t(e)}function wc(e){jt===null?jt=e:jt.push.apply(jt,e)}function bg(e,t,n,l,a,i,r,u,o,s,c,f,p){var d=t.subtreeFlags;if((d&8192||(d&16785408)===16785408)&&(Ti={stylesheets:null,count:0,unsuspend:KA},vv(t),t=PA(),t!==null)){e.cancelPendingCommit=t(Eg.bind(null,e,n,l,a,r,u,o,1,f,p)),Kt(e,i,r,!s);return}Eg(e,n,l,a,r,u,o,c,f,p)}function gA(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var l=0;l<n.length;l++){var a=n[l],i=a.getSnapshot;a=a.value;try{if(!at(i(),a))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function Kt(e,t,n,l){t&=~If,t&=~cl,e.suspendedLanes|=t,e.pingedLanes&=~t,l&&(e.warmLanes|=t),l=e.expirationTimes;for(var a=t;0<a;){var i=31-et(a),r=1<<i;l[i]=-1,a&=~r}n!==0&&N0(e,n,t)}function oo(){return ce&6?!0:(Ji(0),!1)}function ed(){if(Z!==null){if(ae===0)var e=Z.return;else e=Z,tn=Cl=null,Hf(e),ea=null,Ci=0,e=Z;for(;e!==null;)Ev(e.alternate,e),e=e.return;Z=null}}function ya(e,t){e.finishedWork=null,e.finishedLanes=0;var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,NA(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),ed(),ie=e,Z=n=Tn(e.current,null),I=t,ae=0,Ke=null,Zt=!1,Ea=qi(e,t),Pf=!1,ma=mt=If=cl=Ln=pe=0,jt=fi=null,yc=!1,t&8&&(t|=t&32);var l=e.entangledLanes;if(l!==0)for(e=e.entanglements,l&=t;0<l;){var a=31-et(l),i=1<<a;t|=e[a],l&=~i}return cn=t,no(),n}function Av(e,t){X=null,G.H=Lt,t===ni?(t=eg(),ae=3):t===sy?(t=eg(),ae=4):ae=t===Iy?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,Ke=t,Z===null&&(pe=1,mu(e,gt(t,e.current)))}function Ov(){var e=G.H;return G.H=Lt,e===null?Lt:e}function Tv(){var e=G.A;return G.A=dA,e}function bc(){pe=4,Zt||(I&4194176)!==I&&St.current!==null||(Ea=!0),!(Ln&134217727)&&!(cl&134217727)||ie===null||Kt(ie,I,mt,!1)}function as(e,t,n){var l=ce;ce|=2;var a=Ov(),i=Tv();(ie!==e||I!==t)&&(xu=null,ya(e,t)),t=!1;var r=pe;e:do try{if(ae!==0&&Z!==null){var u=Z,o=Ke;switch(ae){case 8:ed(),r=6;break e;case 3:case 2:case 6:St.current===null&&(t=!0);var s=ae;if(ae=0,Ke=null,Ql(e,u,o,s),n&&Ea){r=0;break e}break;default:s=ae,ae=0,Ke=null,Ql(e,u,o,s)}}hA(),r=pe;break}catch(c){Av(e,c)}while(!0);return t&&e.shellSuspendCounter++,tn=Cl=null,ce=l,G.H=a,G.A=i,Z===null&&(ie=null,I=0,no()),r}function hA(){for(;Z!==null;)Dv(Z)}function mA(e,t){var n=ce;ce|=2;var l=Ov(),a=Tv();ie!==e||I!==t?(xu=null,Su=Ft()+500,ya(e,t)):Ea=qi(e,t);e:do try{if(ae!==0&&Z!==null){t=Z;var i=Ke;t:switch(ae){case 1:ae=0,Ke=null,Ql(e,t,i,1);break;case 2:if(Wp(i)){ae=0,Ke=null,Rg(t);break}t=function(){ae===2&&ie===e&&(ae=7),$t(e)},i.then(t,t);break e;case 3:ae=7;break e;case 4:ae=5;break e;case 7:Wp(i)?(ae=0,Ke=null,Rg(t)):(ae=0,Ke=null,Ql(e,t,i,7));break;case 5:var r=null;switch(Z.tag){case 26:r=Z.memoizedState;case 5:case 27:var u=Z;if(!r||Yv(r)){ae=0,Ke=null;var o=u.sibling;if(o!==null)Z=o;else{var s=u.return;s!==null?(Z=s,so(s)):Z=null}break t}}ae=0,Ke=null,Ql(e,t,i,5);break;case 6:ae=0,Ke=null,Ql(e,t,i,6);break;case 8:ed(),pe=6;break e;default:throw Error(O(462))}}yA();break}catch(c){Av(e,c)}while(!0);return tn=Cl=null,G.H=l,G.A=a,ce=n,Z!==null?0:(ie=null,I=0,no(),pe)}function yA(){for(;Z!==null&&!LC();)Dv(Z)}function Dv(e){var t=lv(e.alternate,e,cn);e.memoizedProps=e.pendingProps,t===null?so(e):Z=t}function Rg(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=gg(n,t,t.pendingProps,t.type,void 0,I);break;case 11:t=gg(n,t,t.pendingProps,t.type.render,t.ref,I);break;case 5:Hf(t);default:Ev(n,t),t=Z=bv(t,cn),t=lv(n,t,cn)}e.memoizedProps=e.pendingProps,t===null?so(e):Z=t}function Ql(e,t,n,l){tn=Cl=null,Hf(t),ea=null,Ci=0;var a=t.return;try{if(iA(e,a,t,n,I)){pe=1,mu(e,gt(n,e.current)),Z=null;return}}catch(i){if(a!==null)throw Z=a,i;pe=1,mu(e,gt(n,e.current)),Z=null;return}t.flags&32768?(W||l===1?e=!0:Ea||I&536870912?e=!1:(Zt=e=!0,(l===2||l===3||l===6)&&(l=St.current,l!==null&&l.tag===13&&(l.flags|=16384))),Mv(t,e)):so(t)}function so(e){var t=e;do{if(t.flags&32768){Mv(t,Zt);return}e=t.return;var n=cA(t.alternate,t,cn);if(n!==null){Z=n;return}if(t=t.sibling,t!==null){Z=t;return}Z=t=e}while(t!==null);pe===0&&(pe=5)}function Mv(e,t){do{var n=fA(e.alternate,e);if(n!==null){n.flags&=32767,Z=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){Z=e;return}Z=e=n}while(e!==null);pe=6,Z=null}function Eg(e,t,n,l,a,i,r,u,o,s){var c=G.T,f=re.p;try{re.p=2,G.T=null,vA(e,t,n,l,f,a,i,r,u,o,s)}finally{G.T=c,re.p=f}}function vA(e,t,n,l,a,i,r,u){do aa();while(In!==null);if(ce&6)throw Error(O(327));var o=e.finishedWork;if(l=e.finishedLanes,o===null)return null;if(e.finishedWork=null,e.finishedLanes=0,o===e.current)throw Error(O(177));e.callbackNode=null,e.callbackPriority=0,e.cancelPendingCommit=null;var s=o.lanes|o.childLanes;if(s|=Af,JC(e,l,s,i,r,u),e===ie&&(Z=ie=null,I=0),!(o.subtreeFlags&10256)&&!(o.flags&10256)||wr||(wr=!0,vc=s,Sc=n,bA(iu,function(){return aa(),null})),n=(o.flags&15990)!==0,o.subtreeFlags&15990||n?(n=G.T,G.T=null,i=re.p,re.p=2,r=ce,ce|=4,uA(e,o),hv(o,e),q_(_c,e.containerInfo),Ou=!!Cc,_c=Cc=null,e.current=o,fv(e,o.alternate,o),VC(),ce=r,re.p=i,G.T=n):e.current=o,wr?(wr=!1,In=e,di=l):jv(e,s),s=e.pendingLanes,s===0&&(Dn=null),XC(o.stateNode),$t(e),t!==null)for(a=e.onRecoverableError,o=0;o<t.length;o++)s=t[o],a(s.value,{componentStack:s.stack});return di&3&&aa(),s=e.pendingLanes,l&4194218&&s&42?e===xc?pi++:(pi=0,xc=e):pi=0,Ji(0),null}function jv(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,ki(t)))}function aa(){if(In!==null){var e=In,t=vc;vc=0;var n=U0(di),l=G.T,a=re.p;try{if(re.p=32>n?32:n,G.T=null,In===null)var i=!1;else{n=Sc,Sc=null;var r=In,u=di;if(In=null,di=0,ce&6)throw Error(O(331));var o=ce;if(ce|=4,xv(r.current),yv(r,r.current,u,n),ce=o,Ji(0,!1),We&&typeof We.onPostCommitFiberRoot=="function")try{We.onPostCommitFiberRoot($i,r)}catch{}i=!0}return i}finally{re.p=a,G.T=l,jv(e,t)}}return!1}function Cg(e,t,n){t=gt(n,t),t=uc(e.stateNode,t,2),e=On(e,t,2),e!==null&&(Gi(e,2),$t(e))}function ne(e,t,n){if(e.tag===3)Cg(e,e,n);else for(;t!==null;){if(t.tag===3){Cg(t,e,n);break}else if(t.tag===1){var l=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof l.componentDidCatch=="function"&&(Dn===null||!Dn.has(l))){e=gt(n,e),n=Jy(2),l=On(t,n,2),l!==null&&(Py(n,l,t,e),Gi(l,2),$t(l));break}}t=t.return}}function is(e,t,n){var l=e.pingCache;if(l===null){l=e.pingCache=new pA;var a=new Set;l.set(t,a)}else a=l.get(t),a===void 0&&(a=new Set,l.set(t,a));a.has(n)||(Pf=!0,a.add(n),e=SA.bind(null,e,t,n),t.then(e,e))}function SA(e,t,n){var l=e.pingCache;l!==null&&l.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,ie===e&&(I&n)===n&&(pe===4||pe===3&&(I&62914560)===I&&300>Ft()-Wf?!(ce&2)&&ya(e,0):If|=n,ma===I&&(ma=0)),$t(e)}function Nv(e,t){t===0&&(t=j0()),e=Fn(e,t),e!==null&&(Gi(e,t),$t(e))}function xA(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),Nv(e,n)}function wA(e,t){var n=0;switch(e.tag){case 13:var l=e.stateNode,a=e.memoizedState;a!==null&&(n=a.retryLane);break;case 19:l=e.stateNode;break;case 22:l=e.stateNode._retryCache;break;default:throw Error(O(314))}l!==null&&l.delete(t),Nv(e,n)}function bA(e,t){return Sf(e,t)}var wu=null,jl=null,Rc=!1,bu=!1,rs=!1,fl=0;function $t(e){e!==jl&&e.next===null&&(jl===null?wu=jl=e:jl=jl.next=e),bu=!0,Rc||(Rc=!0,EA(RA))}function Ji(e,t){if(!rs&&bu){rs=!0;do for(var n=!1,l=wu;l!==null;){if(e!==0){var a=l.pendingLanes;if(a===0)var i=0;else{var r=l.suspendedLanes,u=l.pingedLanes;i=(1<<31-et(42|e)+1)-1,i&=a&~(r&~u),i=i&201326677?i&201326677|1:i?i|2:0}i!==0&&(n=!0,_g(l,i))}else i=I,i=Iu(l,l===ie?i:0),!(i&3)||qi(l,i)||(n=!0,_g(l,i));l=l.next}while(n);rs=!1}}function RA(){bu=Rc=!1;var e=0;fl!==0&&(jA()&&(e=fl),fl=0);for(var t=Ft(),n=null,l=wu;l!==null;){var a=l.next,i=zv(l,t);i===0?(l.next=null,n===null?wu=a:n.next=a,a===null&&(jl=n)):(n=l,(e!==0||i&3)&&(bu=!0)),l=a}Ji(e)}function zv(e,t){for(var n=e.suspendedLanes,l=e.pingedLanes,a=e.expirationTimes,i=e.pendingLanes&-62914561;0<i;){var r=31-et(i),u=1<<r,o=a[r];o===-1?(!(u&n)||u&l)&&(a[r]=KC(u,t)):o<=t&&(e.expiredLanes|=u),i&=~u}if(t=ie,n=I,n=Iu(e,e===t?n:0),l=e.callbackNode,n===0||e===t&&ae===2||e.cancelPendingCommit!==null)return l!==null&&l!==null&&Fo(l),e.callbackNode=null,e.callbackPriority=0;if(!(n&3)||qi(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(l!==null&&Fo(l),U0(n)){case 2:case 8:n=T0;break;case 32:n=iu;break;case 268435456:n=D0;break;default:n=iu}return l=Uv.bind(null,e),n=Sf(n,l),e.callbackPriority=t,e.callbackNode=n,t}return l!==null&&l!==null&&Fo(l),e.callbackPriority=2,e.callbackNode=null,2}function Uv(e,t){var n=e.callbackNode;if(aa()&&e.callbackNode!==n)return null;var l=I;return l=Iu(e,e===ie?l:0),l===0?null:(_v(e,l,t),zv(e,Ft()),e.callbackNode!=null&&e.callbackNode===n?Uv.bind(null,e):null)}function _g(e,t){if(aa())return null;_v(e,t,!0)}function EA(e){zA(function(){ce&6?Sf(O0,e):e()})}function td(){return fl===0&&(fl=M0()),fl}function Ag(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:jr(""+e)}function Og(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function CA(e,t,n,l,a){if(t==="submit"&&n&&n.stateNode===a){var i=Ag((a[Ge]||null).action),r=l.submitter;r&&(t=(t=r[Ge]||null)?Ag(t.formAction):r.getAttribute("formAction"),t!==null&&(i=t,r=null));var u=new Wu("action","action",null,l,a);e.push({event:u,listeners:[{instance:null,listener:function(){if(l.defaultPrevented){if(fl!==0){var o=r?Og(a,r):new FormData(a);ic(n,{pending:!0,data:o,method:a.method,action:i},null,o)}}else typeof i=="function"&&(u.preventDefault(),o=r?Og(a,r):new FormData(a),ic(n,{pending:!0,data:o,method:a.method,action:i},i,o))},currentTarget:a}]})}}for(var us=0;us<Kp.length;us++){var os=Kp[us],_A=os.toLowerCase(),AA=os[0].toUpperCase()+os.slice(1);At(_A,"on"+AA)}At(ny,"onAnimationEnd");At(ly,"onAnimationIteration");At(ay,"onAnimationStart");At("dblclick","onDoubleClick");At("focusin","onFocus");At("focusout","onBlur");At(Y_,"onTransitionRun");At(X_,"onTransitionStart");At(k_,"onTransitionCancel");At(iy,"onTransitionEnd");fa("onMouseEnter",["mouseout","mouseover"]);fa("onMouseLeave",["mouseout","mouseover"]);fa("onPointerEnter",["pointerout","pointerover"]);fa("onPointerLeave",["pointerout","pointerover"]);xl("onChange","change click focusin focusout input keydown keyup selectionchange".split(" "));xl("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" "));xl("onBeforeInput",["compositionend","keypress","textInput","paste"]);xl("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" "));xl("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" "));xl("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Ai="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),OA=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Ai));function Fv(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var l=e[n],a=l.event;l=l.listeners;e:{var i=void 0;if(t)for(var r=l.length-1;0<=r;r--){var u=l[r],o=u.instance,s=u.currentTarget;if(u=u.listener,o!==i&&a.isPropagationStopped())break e;i=u,a.currentTarget=s;try{i(a)}catch(c){hu(c)}a.currentTarget=null,i=o}else for(r=0;r<l.length;r++){if(u=l[r],o=u.instance,s=u.currentTarget,u=u.listener,o!==i&&a.isPropagationStopped())break e;i=u,a.currentTarget=s;try{i(a)}catch(c){hu(c)}a.currentTarget=null,i=o}}}}function J(e,t){var n=t[ks];n===void 0&&(n=t[ks]=new Set);var l=e+"__bubble";n.has(l)||(Bv(t,e,2,!1),n.add(l))}function ss(e,t,n){var l=0;t&&(l|=4),Bv(n,e,l,t)}var br="_reactListening"+Math.random().toString(36).slice(2);function nd(e){if(!e[br]){e[br]=!0,B0.forEach(function(n){n!=="selectionchange"&&(OA.has(n)||ss(n,!1,e),ss(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[br]||(t[br]=!0,ss("selectionchange",!1,t))}}function Bv(e,t,n,l){switch(Kv(t)){case 2:var a=eO;break;case 8:a=tO;break;default:a=rd}n=a.bind(null,t,n,e),a=void 0,!Js||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(a=!0),l?a!==void 0?e.addEventListener(t,n,{capture:!0,passive:a}):e.addEventListener(t,n,!0):a!==void 0?e.addEventListener(t,n,{passive:a}):e.addEventListener(t,n,!1)}function cs(e,t,n,l,a){var i=l;if(!(t&1)&&!(t&2)&&l!==null)e:for(;;){if(l===null)return;var r=l.tag;if(r===3||r===4){var u=l.stateNode.containerInfo;if(u===a||u.nodeType===8&&u.parentNode===a)break;if(r===4)for(r=l.return;r!==null;){var o=r.tag;if((o===3||o===4)&&(o=r.stateNode.containerInfo,o===a||o.nodeType===8&&o.parentNode===a))return;r=r.return}for(;u!==null;){if(r=nl(u),r===null)return;if(o=r.tag,o===5||o===6||o===26||o===27){l=i=r;continue e}u=u.parentNode}}l=l.return}X0(function(){var s=i,c=bf(n),f=[];e:{var p=ry.get(e);if(p!==void 0){var d=Wu,h=e;switch(e){case"keypress":if(zr(n)===0)break e;case"keydown":case"keyup":d=w_;break;case"focusin":h="focus",d=$o;break;case"focusout":h="blur",d=$o;break;case"beforeblur":case"afterblur":d=$o;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":d=Bp;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":d=s_;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":d=E_;break;case ny:case ly:case ay:d=d_;break;case iy:d=__;break;case"scroll":case"scrollend":d=u_;break;case"wheel":d=O_;break;case"copy":case"cut":case"paste":d=g_;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":d=Lp;break;case"toggle":case"beforetoggle":d=D_}var S=(t&4)!==0,x=!S&&(e==="scroll"||e==="scrollend"),y=S?p!==null?p+"Capture":null:p;S=[];for(var m=s,v;m!==null;){var b=m;if(v=b.stateNode,b=b.tag,b!==5&&b!==26&&b!==27||v===null||y===null||(b=wi(m,y),b!=null&&S.push(Oi(m,b,v))),x)break;m=m.return}0<S.length&&(p=new d(p,h,null,n,c),f.push({event:p,listeners:S}))}}if(!(t&7)){e:{if(p=e==="mouseover"||e==="pointerover",d=e==="mouseout"||e==="pointerout",p&&n!==Ks&&(h=n.relatedTarget||n.fromElement)&&(nl(h)||h[ba]))break e;if((d||p)&&(p=c.window===c?c:(p=c.ownerDocument)?p.defaultView||p.parentWindow:window,d?(h=n.relatedTarget||n.toElement,d=s,h=h?nl(h):null,h!==null&&(x=wa(h),S=h.tag,h!==x||S!==5&&S!==27&&S!==6)&&(h=null)):(d=null,h=s),d!==h)){if(S=Bp,b="onMouseLeave",y="onMouseEnter",m="mouse",(e==="pointerout"||e==="pointerover")&&(S=Lp,b="onPointerLeave",y="onPointerEnter",m="pointer"),x=d==null?p:ka(d),v=h==null?p:ka(h),p=new S(b,m+"leave",d,n,c),p.target=x,p.relatedTarget=v,b=null,nl(c)===s&&(S=new S(y,m+"enter",h,n,c),S.target=v,S.relatedTarget=x,b=S),x=b,d&&h)t:{for(S=d,y=h,m=0,v=S;v;v=Tl(v))m++;for(v=0,b=y;b;b=Tl(b))v++;for(;0<m-v;)S=Tl(S),m--;for(;0<v-m;)y=Tl(y),v--;for(;m--;){if(S===y||y!==null&&S===y.alternate)break t;S=Tl(S),y=Tl(y)}S=null}else S=null;d!==null&&Tg(f,p,d,S,!1),h!==null&&x!==null&&Tg(f,x,h,S,!0)}}e:{if(p=s?ka(s):window,d=p.nodeName&&p.nodeName.toLowerCase(),d==="select"||d==="input"&&p.type==="file")var R=Gp;else if(qp(p))if(P0)R=V_;else{R=H_;var _=B_}else d=p.nodeName,!d||d.toLowerCase()!=="input"||p.type!=="checkbox"&&p.type!=="radio"?s&&wf(s.elementType)&&(R=Gp):R=L_;if(R&&(R=R(e,s))){J0(f,R,n,c);break e}_&&_(e,p,s),e==="focusout"&&s&&p.type==="number"&&s.memoizedProps.value!=null&&Zs(p,"number",p.value)}switch(_=s?ka(s):window,e){case"focusin":(qp(_)||_.contentEditable==="true")&&($l=_,Ps=s,ti=null);break;case"focusout":ti=Ps=$l=null;break;case"mousedown":Is=!0;break;case"contextmenu":case"mouseup":case"dragend":Is=!1,Zp(f,n,c);break;case"selectionchange":if(G_)break;case"keydown":case"keyup":Zp(f,n,c)}var E;if(Cf)e:{switch(e){case"compositionstart":var A="onCompositionStart";break e;case"compositionend":A="onCompositionEnd";break e;case"compositionupdate":A="onCompositionUpdate";break e}A=void 0}else Vl?Z0(e,n)&&(A="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(A="onCompositionStart");A&&(Q0&&n.locale!=="ko"&&(Vl||A!=="onCompositionStart"?A==="onCompositionEnd"&&Vl&&(E=k0()):(Cn=c,Rf="value"in Cn?Cn.value:Cn.textContent,Vl=!0)),_=Ru(s,A),0<_.length&&(A=new Hp(A,e,null,n,c),f.push({event:A,listeners:_}),E?A.data=E:(E=K0(n),E!==null&&(A.data=E)))),(E=j_?N_(e,n):z_(e,n))&&(A=Ru(s,"onBeforeInput"),0<A.length&&(_=new Hp("onBeforeInput","beforeinput",null,n,c),f.push({event:_,listeners:A}),_.data=E)),CA(f,e,s,n,c)}Fv(f,t)})}function Oi(e,t,n){return{instance:e,listener:t,currentTarget:n}}function Ru(e,t){for(var n=t+"Capture",l=[];e!==null;){var a=e,i=a.stateNode;a=a.tag,a!==5&&a!==26&&a!==27||i===null||(a=wi(e,n),a!=null&&l.unshift(Oi(e,a,i)),a=wi(e,t),a!=null&&l.push(Oi(e,a,i))),e=e.return}return l}function Tl(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function Tg(e,t,n,l,a){for(var i=t._reactName,r=[];n!==null&&n!==l;){var u=n,o=u.alternate,s=u.stateNode;if(u=u.tag,o!==null&&o===l)break;u!==5&&u!==26&&u!==27||s===null||(o=s,a?(s=wi(n,i),s!=null&&r.unshift(Oi(n,s,o))):a||(s=wi(n,i),s!=null&&r.push(Oi(n,s,o)))),n=n.return}r.length!==0&&e.push({event:t,listeners:r})}var TA=/\r\n?/g,DA=/\u0000|\uFFFD/g;function Dg(e){return(typeof e=="string"?e:""+e).replace(TA,`
`).replace(DA,"")}function Hv(e,t){return t=Dg(t),Dg(e)===t}function co(){}function ee(e,t,n,l,a,i){switch(n){case"children":typeof l=="string"?t==="body"||t==="textarea"&&l===""||da(e,l):(typeof l=="number"||typeof l=="bigint")&&t!=="body"&&da(e,""+l);break;case"className":gr(e,"class",l);break;case"tabIndex":gr(e,"tabindex",l);break;case"dir":case"role":case"viewBox":case"width":case"height":gr(e,n,l);break;case"style":Y0(e,l,i);break;case"data":if(t!=="object"){gr(e,"data",l);break}case"src":case"href":if(l===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(l==null||typeof l=="function"||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=jr(""+l),e.setAttribute(n,l);break;case"action":case"formAction":if(typeof l=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof i=="function"&&(n==="formAction"?(t!=="input"&&ee(e,t,"name",a.name,a,null),ee(e,t,"formEncType",a.formEncType,a,null),ee(e,t,"formMethod",a.formMethod,a,null),ee(e,t,"formTarget",a.formTarget,a,null)):(ee(e,t,"encType",a.encType,a,null),ee(e,t,"method",a.method,a,null),ee(e,t,"target",a.target,a,null)));if(l==null||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=jr(""+l),e.setAttribute(n,l);break;case"onClick":l!=null&&(e.onclick=co);break;case"onScroll":l!=null&&J("scroll",e);break;case"onScrollEnd":l!=null&&J("scrollend",e);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(O(61));if(n=l.__html,n!=null){if(a.children!=null)throw Error(O(60));e.innerHTML=n}}break;case"multiple":e.multiple=l&&typeof l!="function"&&typeof l!="symbol";break;case"muted":e.muted=l&&typeof l!="function"&&typeof l!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(l==null||typeof l=="function"||typeof l=="boolean"||typeof l=="symbol"){e.removeAttribute("xlink:href");break}n=jr(""+l),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""+l):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":l&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":l===!0?e.setAttribute(n,""):l!==!1&&l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,l):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":l!=null&&typeof l!="function"&&typeof l!="symbol"&&!isNaN(l)&&1<=l?e.setAttribute(n,l):e.removeAttribute(n);break;case"rowSpan":case"start":l==null||typeof l=="function"||typeof l=="symbol"||isNaN(l)?e.removeAttribute(n):e.setAttribute(n,l);break;case"popover":J("beforetoggle",e),J("toggle",e),Mr(e,"popover",l);break;case"xlinkActuate":qt(e,"http://www.w3.org/1999/xlink","xlink:actuate",l);break;case"xlinkArcrole":qt(e,"http://www.w3.org/1999/xlink","xlink:arcrole",l);break;case"xlinkRole":qt(e,"http://www.w3.org/1999/xlink","xlink:role",l);break;case"xlinkShow":qt(e,"http://www.w3.org/1999/xlink","xlink:show",l);break;case"xlinkTitle":qt(e,"http://www.w3.org/1999/xlink","xlink:title",l);break;case"xlinkType":qt(e,"http://www.w3.org/1999/xlink","xlink:type",l);break;case"xmlBase":qt(e,"http://www.w3.org/XML/1998/namespace","xml:base",l);break;case"xmlLang":qt(e,"http://www.w3.org/XML/1998/namespace","xml:lang",l);break;case"xmlSpace":qt(e,"http://www.w3.org/XML/1998/namespace","xml:space",l);break;case"is":Mr(e,"is",l);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=i_.get(n)||n,Mr(e,n,l))}}function Ec(e,t,n,l,a,i){switch(n){case"style":Y0(e,l,i);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(O(61));if(n=l.__html,n!=null){if(a.children!=null)throw Error(O(60));e.innerHTML=n}}break;case"children":typeof l=="string"?da(e,l):(typeof l=="number"||typeof l=="bigint")&&da(e,""+l);break;case"onScroll":l!=null&&J("scroll",e);break;case"onScrollEnd":l!=null&&J("scrollend",e);break;case"onClick":l!=null&&(e.onclick=co);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!H0.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(a=n.endsWith("Capture"),t=n.slice(2,a?n.length-7:void 0),i=e[Ge]||null,i=i!=null?i[n]:null,typeof i=="function"&&e.removeEventListener(t,i,a),typeof l=="function")){typeof i!="function"&&i!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,l,a);break e}n in e?e[n]=l:l===!0?e.setAttribute(n,""):Mr(e,n,l)}}}function Te(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":J("error",e),J("load",e);var l=!1,a=!1,i;for(i in n)if(n.hasOwnProperty(i)){var r=n[i];if(r!=null)switch(i){case"src":l=!0;break;case"srcSet":a=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(O(137,t));default:ee(e,t,i,r,n,null)}}a&&ee(e,t,"srcSet",n.srcSet,n,null),l&&ee(e,t,"src",n.src,n,null);return;case"input":J("invalid",e);var u=i=r=a=null,o=null,s=null;for(l in n)if(n.hasOwnProperty(l)){var c=n[l];if(c!=null)switch(l){case"name":a=c;break;case"type":r=c;break;case"checked":o=c;break;case"defaultChecked":s=c;break;case"value":i=c;break;case"defaultValue":u=c;break;case"children":case"dangerouslySetInnerHTML":if(c!=null)throw Error(O(137,t));break;default:ee(e,t,l,c,n,null)}}$0(e,i,u,o,s,r,a,!1),ru(e);return;case"select":J("invalid",e),l=r=i=null;for(a in n)if(n.hasOwnProperty(a)&&(u=n[a],u!=null))switch(a){case"value":i=u;break;case"defaultValue":r=u;break;case"multiple":l=u;default:ee(e,t,a,u,n,null)}t=i,n=r,e.multiple=!!l,t!=null?Il(e,!!l,t,!1):n!=null&&Il(e,!!l,n,!0);return;case"textarea":J("invalid",e),i=a=l=null;for(r in n)if(n.hasOwnProperty(r)&&(u=n[r],u!=null))switch(r){case"value":l=u;break;case"defaultValue":a=u;break;case"children":i=u;break;case"dangerouslySetInnerHTML":if(u!=null)throw Error(O(91));break;default:ee(e,t,r,u,n,null)}G0(e,l,a,i),ru(e);return;case"option":for(o in n)if(n.hasOwnProperty(o)&&(l=n[o],l!=null))switch(o){case"selected":e.selected=l&&typeof l!="function"&&typeof l!="symbol";break;default:ee(e,t,o,l,n,null)}return;case"dialog":J("cancel",e),J("close",e);break;case"iframe":case"object":J("load",e);break;case"video":case"audio":for(l=0;l<Ai.length;l++)J(Ai[l],e);break;case"image":J("error",e),J("load",e);break;case"details":J("toggle",e);break;case"embed":case"source":case"link":J("error",e),J("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(s in n)if(n.hasOwnProperty(s)&&(l=n[s],l!=null))switch(s){case"children":case"dangerouslySetInnerHTML":throw Error(O(137,t));default:ee(e,t,s,l,n,null)}return;default:if(wf(t)){for(c in n)n.hasOwnProperty(c)&&(l=n[c],l!==void 0&&Ec(e,t,c,l,n,void 0));return}}for(u in n)n.hasOwnProperty(u)&&(l=n[u],l!=null&&ee(e,t,u,l,n,null))}function MA(e,t,n,l){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var a=null,i=null,r=null,u=null,o=null,s=null,c=null;for(d in n){var f=n[d];if(n.hasOwnProperty(d)&&f!=null)switch(d){case"checked":break;case"value":break;case"defaultValue":o=f;default:l.hasOwnProperty(d)||ee(e,t,d,null,l,f)}}for(var p in l){var d=l[p];if(f=n[p],l.hasOwnProperty(p)&&(d!=null||f!=null))switch(p){case"type":i=d;break;case"name":a=d;break;case"checked":s=d;break;case"defaultChecked":c=d;break;case"value":r=d;break;case"defaultValue":u=d;break;case"children":case"dangerouslySetInnerHTML":if(d!=null)throw Error(O(137,t));break;default:d!==f&&ee(e,t,p,d,l,f)}}Qs(e,r,u,o,s,c,i,a);return;case"select":d=r=u=p=null;for(i in n)if(o=n[i],n.hasOwnProperty(i)&&o!=null)switch(i){case"value":break;case"multiple":d=o;default:l.hasOwnProperty(i)||ee(e,t,i,null,l,o)}for(a in l)if(i=l[a],o=n[a],l.hasOwnProperty(a)&&(i!=null||o!=null))switch(a){case"value":p=i;break;case"defaultValue":u=i;break;case"multiple":r=i;default:i!==o&&ee(e,t,a,i,l,o)}t=u,n=r,l=d,p!=null?Il(e,!!n,p,!1):!!l!=!!n&&(t!=null?Il(e,!!n,t,!0):Il(e,!!n,n?[]:"",!1));return;case"textarea":d=p=null;for(u in n)if(a=n[u],n.hasOwnProperty(u)&&a!=null&&!l.hasOwnProperty(u))switch(u){case"value":break;case"children":break;default:ee(e,t,u,null,l,a)}for(r in l)if(a=l[r],i=n[r],l.hasOwnProperty(r)&&(a!=null||i!=null))switch(r){case"value":p=a;break;case"defaultValue":d=a;break;case"children":break;case"dangerouslySetInnerHTML":if(a!=null)throw Error(O(91));break;default:a!==i&&ee(e,t,r,a,l,i)}q0(e,p,d);return;case"option":for(var h in n)if(p=n[h],n.hasOwnProperty(h)&&p!=null&&!l.hasOwnProperty(h))switch(h){case"selected":e.selected=!1;break;default:ee(e,t,h,null,l,p)}for(o in l)if(p=l[o],d=n[o],l.hasOwnProperty(o)&&p!==d&&(p!=null||d!=null))switch(o){case"selected":e.selected=p&&typeof p!="function"&&typeof p!="symbol";break;default:ee(e,t,o,p,l,d)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var S in n)p=n[S],n.hasOwnProperty(S)&&p!=null&&!l.hasOwnProperty(S)&&ee(e,t,S,null,l,p);for(s in l)if(p=l[s],d=n[s],l.hasOwnProperty(s)&&p!==d&&(p!=null||d!=null))switch(s){case"children":case"dangerouslySetInnerHTML":if(p!=null)throw Error(O(137,t));break;default:ee(e,t,s,p,l,d)}return;default:if(wf(t)){for(var x in n)p=n[x],n.hasOwnProperty(x)&&p!==void 0&&!l.hasOwnProperty(x)&&Ec(e,t,x,void 0,l,p);for(c in l)p=l[c],d=n[c],!l.hasOwnProperty(c)||p===d||p===void 0&&d===void 0||Ec(e,t,c,p,l,d);return}}for(var y in n)p=n[y],n.hasOwnProperty(y)&&p!=null&&!l.hasOwnProperty(y)&&ee(e,t,y,null,l,p);for(f in l)p=l[f],d=n[f],!l.hasOwnProperty(f)||p===d||p==null&&d==null||ee(e,t,f,p,l,d)}var Cc=null,_c=null;function Eu(e){return e.nodeType===9?e:e.ownerDocument}function Mg(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function Lv(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function Ac(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var fs=null;function jA(){var e=window.event;return e&&e.type==="popstate"?e===fs?!1:(fs=e,!0):(fs=null,!1)}var Vv=typeof setTimeout=="function"?setTimeout:void 0,NA=typeof clearTimeout=="function"?clearTimeout:void 0,jg=typeof Promise=="function"?Promise:void 0,zA=typeof queueMicrotask=="function"?queueMicrotask:typeof jg<"u"?function(e){return jg.resolve(null).then(e).catch(UA)}:Vv;function UA(e){setTimeout(function(){throw e})}function ds(e,t){var n=t,l=0;do{var a=n.nextSibling;if(e.removeChild(n),a&&a.nodeType===8)if(n=a.data,n==="/$"){if(l===0){e.removeChild(a),Ni(t);return}l--}else n!=="$"&&n!=="$?"&&n!=="$!"||l++;n=a}while(n);Ni(t)}function Oc(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":Oc(n),xf(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function FA(e,t,n,l){for(;e.nodeType===1;){var a=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!l&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(l){if(!e[xi])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(i=e.getAttribute("rel"),i==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(i!==a.rel||e.getAttribute("href")!==(a.href==null?null:a.href)||e.getAttribute("crossorigin")!==(a.crossOrigin==null?null:a.crossOrigin)||e.getAttribute("title")!==(a.title==null?null:a.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(i=e.getAttribute("src"),(i!==(a.src==null?null:a.src)||e.getAttribute("type")!==(a.type==null?null:a.type)||e.getAttribute("crossorigin")!==(a.crossOrigin==null?null:a.crossOrigin))&&i&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var i=a.name==null?null:""+a.name;if(a.type==="hidden"&&e.getAttribute("name")===i)return e}else return e;if(e=Ct(e.nextSibling),e===null)break}return null}function BA(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Ct(e.nextSibling),e===null))return null;return e}function Ct(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="F!"||t==="F")break;if(t==="/$")return null}}return e}function Ng(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"){if(t===0)return e;t--}else n==="/$"&&t++}e=e.previousSibling}return null}function $v(e,t,n){switch(t=Eu(n),e){case"html":if(e=t.documentElement,!e)throw Error(O(452));return e;case"head":if(e=t.head,!e)throw Error(O(453));return e;case"body":if(e=t.body,!e)throw Error(O(454));return e;default:throw Error(O(451))}}var xt=new Map,zg=new Set;function Cu(e){return typeof e.getRootNode=="function"?e.getRootNode():e.ownerDocument}var dn=re.d;re.d={f:HA,r:LA,D:VA,C:$A,L:qA,m:GA,X:XA,S:YA,M:kA};function HA(){var e=dn.f(),t=oo();return e||t}function LA(e){var t=Ra(e);t!==null&&t.tag===5&&t.type==="form"?$y(t):dn.r(e)}var Ca=typeof document>"u"?null:document;function qv(e,t,n){var l=Ca;if(l&&typeof t=="string"&&t){var a=pt(t);a='link[rel="'+e+'"][href="'+a+'"]',typeof n=="string"&&(a+='[crossorigin="'+n+'"]'),zg.has(a)||(zg.add(a),e={rel:e,crossOrigin:n,href:t},l.querySelector(a)===null&&(t=l.createElement("link"),Te(t,"link",e),Re(t),l.head.appendChild(t)))}}function VA(e){dn.D(e),qv("dns-prefetch",e,null)}function $A(e,t){dn.C(e,t),qv("preconnect",e,t)}function qA(e,t,n){dn.L(e,t,n);var l=Ca;if(l&&e&&t){var a='link[rel="preload"][as="'+pt(t)+'"]';t==="image"&&n&&n.imageSrcSet?(a+='[imagesrcset="'+pt(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(a+='[imagesizes="'+pt(n.imageSizes)+'"]')):a+='[href="'+pt(e)+'"]';var i=a;switch(t){case"style":i=va(e);break;case"script":i=_a(e)}xt.has(i)||(e=ue({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),xt.set(i,e),l.querySelector(a)!==null||t==="style"&&l.querySelector(Pi(i))||t==="script"&&l.querySelector(Ii(i))||(t=l.createElement("link"),Te(t,"link",e),Re(t),l.head.appendChild(t)))}}function GA(e,t){dn.m(e,t);var n=Ca;if(n&&e){var l=t&&typeof t.as=="string"?t.as:"script",a='link[rel="modulepreload"][as="'+pt(l)+'"][href="'+pt(e)+'"]',i=a;switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":i=_a(e)}if(!xt.has(i)&&(e=ue({rel:"modulepreload",href:e},t),xt.set(i,e),n.querySelector(a)===null)){switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(Ii(i)))return}l=n.createElement("link"),Te(l,"link",e),Re(l),n.head.appendChild(l)}}}function YA(e,t,n){dn.S(e,t,n);var l=Ca;if(l&&e){var a=Pl(l).hoistableStyles,i=va(e);t=t||"default";var r=a.get(i);if(!r){var u={loading:0,preload:null};if(r=l.querySelector(Pi(i)))u.loading=5;else{e=ue({rel:"stylesheet",href:e,"data-precedence":t},n),(n=xt.get(i))&&ld(e,n);var o=r=l.createElement("link");Re(o),Te(o,"link",e),o._p=new Promise(function(s,c){o.onload=s,o.onerror=c}),o.addEventListener("load",function(){u.loading|=1}),o.addEventListener("error",function(){u.loading|=2}),u.loading|=4,Vr(r,t,l)}r={type:"stylesheet",instance:r,count:1,state:u},a.set(i,r)}}}function XA(e,t){dn.X(e,t);var n=Ca;if(n&&e){var l=Pl(n).hoistableScripts,a=_a(e),i=l.get(a);i||(i=n.querySelector(Ii(a)),i||(e=ue({src:e,async:!0},t),(t=xt.get(a))&&ad(e,t),i=n.createElement("script"),Re(i),Te(i,"link",e),n.head.appendChild(i)),i={type:"script",instance:i,count:1,state:null},l.set(a,i))}}function kA(e,t){dn.M(e,t);var n=Ca;if(n&&e){var l=Pl(n).hoistableScripts,a=_a(e),i=l.get(a);i||(i=n.querySelector(Ii(a)),i||(e=ue({src:e,async:!0,type:"module"},t),(t=xt.get(a))&&ad(e,t),i=n.createElement("script"),Re(i),Te(i,"link",e),n.head.appendChild(i)),i={type:"script",instance:i,count:1,state:null},l.set(a,i))}}function Ug(e,t,n,l){var a=(a=_n.current)?Cu(a):null;if(!a)throw Error(O(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=va(n.href),n=Pl(a).hoistableStyles,l=n.get(t),l||(l={type:"style",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=va(n.href);var i=Pl(a).hoistableStyles,r=i.get(e);if(r||(a=a.ownerDocument||a,r={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},i.set(e,r),(i=a.querySelector(Pi(e)))&&!i._p&&(r.instance=i,r.state.loading=5),xt.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},xt.set(e,n),i||QA(a,e,n,r.state))),t&&l===null)throw Error(O(528,""));return r}if(t&&l!==null)throw Error(O(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=_a(n),n=Pl(a).hoistableScripts,l=n.get(t),l||(l={type:"script",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};default:throw Error(O(444,e))}}function va(e){return'href="'+pt(e)+'"'}function Pi(e){return'link[rel="stylesheet"]['+e+"]"}function Gv(e){return ue({},e,{"data-precedence":e.precedence,precedence:null})}function QA(e,t,n,l){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?l.loading=1:(t=e.createElement("link"),l.preload=t,t.addEventListener("load",function(){return l.loading|=1}),t.addEventListener("error",function(){return l.loading|=2}),Te(t,"link",n),Re(t),e.head.appendChild(t))}function _a(e){return'[src="'+pt(e)+'"]'}function Ii(e){return"script[async]"+e}function Fg(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var l=e.querySelector('style[data-href~="'+pt(n.href)+'"]');if(l)return t.instance=l,Re(l),l;var a=ue({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return l=(e.ownerDocument||e).createElement("style"),Re(l),Te(l,"style",a),Vr(l,n.precedence,e),t.instance=l;case"stylesheet":a=va(n.href);var i=e.querySelector(Pi(a));if(i)return t.state.loading|=4,t.instance=i,Re(i),i;l=Gv(n),(a=xt.get(a))&&ld(l,a),i=(e.ownerDocument||e).createElement("link"),Re(i);var r=i;return r._p=new Promise(function(u,o){r.onload=u,r.onerror=o}),Te(i,"link",l),t.state.loading|=4,Vr(i,n.precedence,e),t.instance=i;case"script":return i=_a(n.src),(a=e.querySelector(Ii(i)))?(t.instance=a,Re(a),a):(l=n,(a=xt.get(i))&&(l=ue({},n),ad(l,a)),e=e.ownerDocument||e,a=e.createElement("script"),Re(a),Te(a,"link",l),e.head.appendChild(a),t.instance=a);case"void":return null;default:throw Error(O(443,t.type))}else t.type==="stylesheet"&&!(t.state.loading&4)&&(l=t.instance,t.state.loading|=4,Vr(l,n.precedence,e));return t.instance}function Vr(e,t,n){for(var l=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),a=l.length?l[l.length-1]:null,i=a,r=0;r<l.length;r++){var u=l[r];if(u.dataset.precedence===t)i=u;else if(i!==a)break}i?i.parentNode.insertBefore(e,i.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function ld(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function ad(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var $r=null;function Bg(e,t,n){if($r===null){var l=new Map,a=$r=new Map;a.set(n,l)}else a=$r,l=a.get(n),l||(l=new Map,a.set(n,l));if(l.has(e))return l;for(l.set(e,null),n=n.getElementsByTagName(e),a=0;a<n.length;a++){var i=n[a];if(!(i[xi]||i[je]||e==="link"&&i.getAttribute("rel")==="stylesheet")&&i.namespaceURI!=="http://www.w3.org/2000/svg"){var r=i.getAttribute(t)||"";r=e+r;var u=l.get(r);u?u.push(i):l.set(r,[i])}}return l}function Hg(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function ZA(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function Yv(e){return!(e.type==="stylesheet"&&!(e.state.loading&3))}var Ti=null;function KA(){}function JA(e,t,n){if(Ti===null)throw Error(O(475));var l=Ti;if(t.type==="stylesheet"&&(typeof n.media!="string"||matchMedia(n.media).matches!==!1)&&!(t.state.loading&4)){if(t.instance===null){var a=va(n.href),i=e.querySelector(Pi(a));if(i){e=i._p,e!==null&&typeof e=="object"&&typeof e.then=="function"&&(l.count++,l=_u.bind(l),e.then(l,l)),t.state.loading|=4,t.instance=i,Re(i);return}i=e.ownerDocument||e,n=Gv(n),(a=xt.get(a))&&ld(n,a),i=i.createElement("link"),Re(i);var r=i;r._p=new Promise(function(u,o){r.onload=u,r.onerror=o}),Te(i,"link",n),t.instance=i}l.stylesheets===null&&(l.stylesheets=new Map),l.stylesheets.set(t,e),(e=t.state.preload)&&!(t.state.loading&3)&&(l.count++,t=_u.bind(l),e.addEventListener("load",t),e.addEventListener("error",t))}}function PA(){if(Ti===null)throw Error(O(475));var e=Ti;return e.stylesheets&&e.count===0&&Tc(e,e.stylesheets),0<e.count?function(t){var n=setTimeout(function(){if(e.stylesheets&&Tc(e,e.stylesheets),e.unsuspend){var l=e.unsuspend;e.unsuspend=null,l()}},6e4);return e.unsuspend=t,function(){e.unsuspend=null,clearTimeout(n)}}:null}function _u(){if(this.count--,this.count===0){if(this.stylesheets)Tc(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var Au=null;function Tc(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,Au=new Map,t.forEach(IA,e),Au=null,_u.call(e))}function IA(e,t){if(!(t.state.loading&4)){var n=Au.get(e);if(n)var l=n.get(null);else{n=new Map,Au.set(e,n);for(var a=e.querySelectorAll("link[data-precedence],style[data-precedence]"),i=0;i<a.length;i++){var r=a[i];(r.nodeName==="LINK"||r.getAttribute("media")!=="not all")&&(n.set(r.dataset.precedence,r),l=r)}l&&n.set(null,l)}a=t.instance,r=a.getAttribute("data-precedence"),i=n.get(r)||l,i===l&&n.set(null,a),n.set(r,a),this.count++,l=_u.bind(this),a.addEventListener("load",l),a.addEventListener("error",l),i?i.parentNode.insertBefore(a,i.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(a,e.firstChild)),t.state.loading|=4}}var Di={$$typeof:Pt,Provider:null,Consumer:null,_currentValue:rl,_currentValue2:rl,_threadCount:0};function WA(e,t,n,l,a,i,r,u){this.tag=1,this.containerInfo=e,this.finishedWork=this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=Bo(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.finishedLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=Bo(0),this.hiddenUpdates=Bo(null),this.identifierPrefix=l,this.onUncaughtError=a,this.onCaughtError=i,this.onRecoverableError=r,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=u,this.incompleteTransitions=new Map}function Xv(e,t,n,l,a,i,r,u,o,s,c,f){return e=new WA(e,t,n,r,u,o,s,f),t=1,i===!0&&(t|=24),i=ht(3,null,null,t),e.current=i,i.stateNode=e,t=jf(),t.refCount++,e.pooledCache=t,t.refCount++,i.memoizedState={element:l,isDehydrated:n,cache:t},Qf(i),e}function kv(e){return e?(e=Yl,e):Yl}function Qv(e,t,n,l,a,i){a=kv(a),l.context===null?l.context=a:l.pendingContext=a,l=An(t),l.payload={element:n},i=i===void 0?null:i,i!==null&&(l.callback=i),n=On(e,l,t),n!==null&&(Ue(n,e,t),ui(n,e,t))}function Lg(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function id(e,t){Lg(e,t),(e=e.alternate)&&Lg(e,t)}function Zv(e){if(e.tag===13){var t=Fn(e,67108864);t!==null&&Ue(t,e,67108864),id(e,67108864)}}var Ou=!0;function eO(e,t,n,l){var a=G.T;G.T=null;var i=re.p;try{re.p=2,rd(e,t,n,l)}finally{re.p=i,G.T=a}}function tO(e,t,n,l){var a=G.T;G.T=null;var i=re.p;try{re.p=8,rd(e,t,n,l)}finally{re.p=i,G.T=a}}function rd(e,t,n,l){if(Ou){var a=Dc(l);if(a===null)cs(e,t,l,Tu,n),Vg(e,l);else if(lO(a,e,t,n,l))l.stopPropagation();else if(Vg(e,l),t&4&&-1<nO.indexOf(e)){for(;a!==null;){var i=Ra(a);if(i!==null)switch(i.tag){case 3:if(i=i.stateNode,i.current.memoizedState.isDehydrated){var r=Kn(i.pendingLanes);if(r!==0){var u=i;for(u.pendingLanes|=2,u.entangledLanes|=2;r;){var o=1<<31-et(r);u.entanglements[1]|=o,r&=~o}$t(i),!(ce&6)&&(Su=Ft()+500,Ji(0))}}break;case 13:u=Fn(i,2),u!==null&&Ue(u,i,2),oo(),id(i,2)}if(i=Dc(l),i===null&&cs(e,t,l,Tu,n),i===a)break;a=i}a!==null&&l.stopPropagation()}else cs(e,t,l,null,n)}}function Dc(e){return e=bf(e),ud(e)}var Tu=null;function ud(e){if(Tu=null,e=nl(e),e!==null){var t=wa(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=_0(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return Tu=e,null}function Kv(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch($C()){case O0:return 2;case T0:return 8;case iu:case qC:return 32;case D0:return 268435456;default:return 32}default:return 32}}var Mc=!1,Mn=null,jn=null,Nn=null,Mi=new Map,ji=new Map,wn=[],nO="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function Vg(e,t){switch(e){case"focusin":case"focusout":Mn=null;break;case"dragenter":case"dragleave":jn=null;break;case"mouseover":case"mouseout":Nn=null;break;case"pointerover":case"pointerout":Mi.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":ji.delete(t.pointerId)}}function La(e,t,n,l,a,i){return e===null||e.nativeEvent!==i?(e={blockedOn:t,domEventName:n,eventSystemFlags:l,nativeEvent:i,targetContainers:[a]},t!==null&&(t=Ra(t),t!==null&&Zv(t)),e):(e.eventSystemFlags|=l,t=e.targetContainers,a!==null&&t.indexOf(a)===-1&&t.push(a),e)}function lO(e,t,n,l,a){switch(t){case"focusin":return Mn=La(Mn,e,t,n,l,a),!0;case"dragenter":return jn=La(jn,e,t,n,l,a),!0;case"mouseover":return Nn=La(Nn,e,t,n,l,a),!0;case"pointerover":var i=a.pointerId;return Mi.set(i,La(Mi.get(i)||null,e,t,n,l,a)),!0;case"gotpointercapture":return i=a.pointerId,ji.set(i,La(ji.get(i)||null,e,t,n,l,a)),!0}return!1}function Jv(e){var t=nl(e.target);if(t!==null){var n=wa(t);if(n!==null){if(t=n.tag,t===13){if(t=_0(n),t!==null){e.blockedOn=t,PC(e.priority,function(){if(n.tag===13){var l=tt(),a=Fn(n,l);a!==null&&Ue(a,n,l),id(n,l)}});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function qr(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=Dc(e.nativeEvent);if(n===null){n=e.nativeEvent;var l=new n.constructor(n.type,n);Ks=l,n.target.dispatchEvent(l),Ks=null}else return t=Ra(n),t!==null&&Zv(t),e.blockedOn=n,!1;t.shift()}return!0}function $g(e,t,n){qr(e)&&n.delete(t)}function aO(){Mc=!1,Mn!==null&&qr(Mn)&&(Mn=null),jn!==null&&qr(jn)&&(jn=null),Nn!==null&&qr(Nn)&&(Nn=null),Mi.forEach($g),ji.forEach($g)}function Rr(e,t){e.blockedOn===t&&(e.blockedOn=null,Mc||(Mc=!0,we.unstable_scheduleCallback(we.unstable_NormalPriority,aO)))}var Er=null;function qg(e){Er!==e&&(Er=e,we.unstable_scheduleCallback(we.unstable_NormalPriority,function(){Er===e&&(Er=null);for(var t=0;t<e.length;t+=3){var n=e[t],l=e[t+1],a=e[t+2];if(typeof l!="function"){if(ud(l||n)===null)continue;break}var i=Ra(n);i!==null&&(e.splice(t,3),t-=3,ic(i,{pending:!0,data:a,method:n.method,action:l},l,a))}}))}function Ni(e){function t(o){return Rr(o,e)}Mn!==null&&Rr(Mn,e),jn!==null&&Rr(jn,e),Nn!==null&&Rr(Nn,e),Mi.forEach(t),ji.forEach(t);for(var n=0;n<wn.length;n++){var l=wn[n];l.blockedOn===e&&(l.blockedOn=null)}for(;0<wn.length&&(n=wn[0],n.blockedOn===null);)Jv(n),n.blockedOn===null&&wn.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(l=0;l<n.length;l+=3){var a=n[l],i=n[l+1],r=a[Ge]||null;if(typeof i=="function")r||qg(n);else if(r){var u=null;if(i&&i.hasAttribute("formAction")){if(a=i,r=i[Ge]||null)u=r.formAction;else if(ud(a)!==null)continue}else u=r.action;typeof u=="function"?n[l+1]=u:(n.splice(l,3),l-=3),qg(n)}}}function od(e){this._internalRoot=e}fo.prototype.render=od.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(O(409));var n=t.current,l=tt();Qv(n,l,e,t,null,null)};fo.prototype.unmount=od.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;e.tag===0&&aa(),Qv(e.current,2,null,e,null,null),oo(),t[ba]=null}};function fo(e){this._internalRoot=e}fo.prototype.unstable_scheduleHydration=function(e){if(e){var t=F0();e={blockedOn:null,target:e,priority:t};for(var n=0;n<wn.length&&t!==0&&t<wn[n].priority;n++);wn.splice(n,0,e),n===0&&Jv(e)}};var Gg=w0.version;if(Gg!=="19.0.0")throw Error(O(527,Gg,"19.0.0"));re.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(O(188)):(e=Object.keys(e).join(","),Error(O(268,e)));return e=HC(t),e=e!==null?A0(e):null,e=e===null?null:e.stateNode,e};var iO={bundleType:0,version:"19.0.0",rendererPackageName:"react-dom",currentDispatcherRef:G,findFiberByHostInstance:nl,reconcilerVersion:"19.0.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var Cr=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!Cr.isDisabled&&Cr.supportsFiber)try{$i=Cr.inject(iO),We=Cr}catch{}}Pu.createRoot=function(e,t){if(!b0(e))throw Error(O(299));var n=!1,l="",a=Qy,i=Zy,r=Ky,u=null;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(l=t.identifierPrefix),t.onUncaughtError!==void 0&&(a=t.onUncaughtError),t.onCaughtError!==void 0&&(i=t.onCaughtError),t.onRecoverableError!==void 0&&(r=t.onRecoverableError),t.unstable_transitionCallbacks!==void 0&&(u=t.unstable_transitionCallbacks)),t=Xv(e,1,!1,null,null,n,l,a,i,r,u,null),e[ba]=t.current,nd(e.nodeType===8?e.parentNode:e),new od(t)};Pu.hydrateRoot=function(e,t,n){if(!b0(e))throw Error(O(299));var l=!1,a="",i=Qy,r=Zy,u=Ky,o=null,s=null;return n!=null&&(n.unstable_strictMode===!0&&(l=!0),n.identifierPrefix!==void 0&&(a=n.identifierPrefix),n.onUncaughtError!==void 0&&(i=n.onUncaughtError),n.onCaughtError!==void 0&&(r=n.onCaughtError),n.onRecoverableError!==void 0&&(u=n.onRecoverableError),n.unstable_transitionCallbacks!==void 0&&(o=n.unstable_transitionCallbacks),n.formState!==void 0&&(s=n.formState)),t=Xv(e,1,!0,t,n??null,l,a,i,r,u,o,s),t.context=kv(null),n=t.current,l=tt(),a=An(l),a.callback=null,On(n,a,l),t.current.lanes=l,Gi(t,l),$t(t),e[ba]=t.current,nd(e),new fo(t)};Pu.version="19.0.0";function Pv(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(Pv)}catch(e){console.error(e)}}Pv(),v0.exports=Pu;var rO=v0.exports;const uO=zi(rO),oO=new u1;uO.createRoot(document.getElementById("root")).render(g.jsx(ye.StrictMode,{children:g.jsx(o1,{client:oO,children:g.jsx(DC,{})})}));
//# sourceMappingURL=index-CjsUuief.js.map
